#include <c6x.h>
#include <csl.h> /* CSL library */
#include <csl_dma.h> /* DMA_SUPPORT */
#include <csl_edma.h> /* EDMA_SUPPORT */
#include <csl_irq.h> /* IRQ_SUPPORT */
#include <csl_mcbsp.h> /* MCBSP_SUPPORT */
#include "FPGA_interface.h"
#include <emif.h>
#include <Timers.h>
#include <TVG.h>

volatile tvgIn_s tvgIn = { 0, 0, 0, 0, 0, 0 };

volatile tvgOut_s tvgOut =  {
     0xA0B0C0D0,
     0xDEADBEEF,
     0xABCD, 0xEFEF,
     0x1234, 0x4567,
     0x6789, 0x1313,
     {7,4,
      0,
      0,0,0,
      0,
      0,
      0,
      1,
      0},
      0x1212,
      0x1111,
      0x2222,
      0x3333
      };



void read_FPGA_data (volatile tvgIn_s *p)
{
    uint32 adr = start_adr_FPGA + TVG_IO_READ_ADDR;
    p->Xs = (*((uint32 *)adr + TVG_IO_XS_OFFSET) & 0xFFFF) | ((*((int32 *)adr + TVG_IO_XS_OFFSET + 1) & 0xFFFF) << 16);   //Int32 Xs
    p->Xc = (*((uint32 *)adr + TVG_IO_XC_OFFSET) & 0xFFFF) | ((*((int32 *)adr + TVG_IO_XC_OFFSET + 1) & 0xFFFF) << 16);   //Int32 Xc
    p->Ys = (*((uint32 *)adr + TVG_IO_YS_OFFSET) & 0xFFFF) | ((*((int32 *)adr + TVG_IO_YS_OFFSET + 1) & 0xFFFF) << 16);   //Int32 Ys
    p->Yc = (*((uint32 *)adr + TVG_IO_YC_OFFSET) & 0xFFFF) | ((*((int32 *)adr + TVG_IO_YC_OFFSET + 1) & 0xFFFF) << 16);   //Int32 Yc

    p->regINVERS = (int16)(*((uint32 *)adr + TVG_IO_REG_INVERS_OFFSET));   //Int16 REG_INVERS
}


void write_FPGA_data (volatile tvgOut_s *p)
{
    uint32 addr = start_adr_FPGA + TVG_IO_WRITE_ADDR;

    *((uint32 *)addr + TVG_IO_PLL_OFFSET) = (p->PLL & 0xFFFF);             //uint32 PLL;
    *((uint32 *)addr + TVG_IO_PLL_OFFSET + 1) = ((p->PLL >> 16) & 0xFFFF); //uint32 PLL;

    *((uint32 *)addr + TVG_IO_NRX_OFFSET) = p->Nrx;                        //int16 Nrx;
    *((uint32 *)addr + TVG_IO_NRY_OFFSET) = p->Nry;                        //int16 Nry;
    *((uint32 *)addr + TVG_IO_NPX_OFFSET) = p->Npx;                        //int16 Npx;
    *((uint32 *)addr + TVG_IO_NPY_OFFSET) = p->Npy;                        //int16 Npy;
    *((uint32 *)addr + TVG_IO_NAX_OFFSET) = p->Nax;                        //int16 Nax;
    *((uint32 *)addr + TVG_IO_NAY_OFFSET) = p->Nay;                        //int16 Nay;

    *((uint32 *)addr + TVG_IO_R1_OFFSET) = *((uint32 *)&p->R1);            //TTVGR1

    *((uint32 *)addr + TVG_IO_NRPWM_OFFSET) = p->NrPWM;                    //int16 NrPWM;
    *((uint32 *)addr + TVG_IO_NPPWM_OFFSET) = p->NpPWM;                    //int16 NpPWM;
    *((uint32 *)addr + TVG_IO_NAPWM_OFFSET) = p->NaPWM;                    //int16 NaPWM;
}


void FPGA_data_exchange (volatile tvgIn_s *in,volatile  tvgOut_s *out)
{
    read_FPGA_data (in);
    write_FPGA_data (out);
}

void tvgISR (void)
{
     uint32 Time0;

     Time0 = TIMER_getCount(Timer0);
     read_FPGA_data (&tvgIn);
     Program_Flag.flagISRPLIS = 1;
     Tims.LastFPGATim = Tims.CurFPGATim;
     Tims.CurFPGATim = Time0;
     Tims.CiklFPGA = Time0 - Tims.LastFPGATim;
}

