/******************************************************************************\
*           Copyright (C) 2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl.c
* DATE CREATED.. 06/13/2000  
* LAST MODIFIED. 08/02/2004  Adding support for C6418
*                07/24/2004 - Re-introduced BIOS dependency due to compatibility issues
*                02/05/2004  Modified _CSL_init()definition to remove bios component 
*                06/17/2003  Added CHIP_6712C
*                05/28/2003  Added CHIP_6711C
*                02/12/2002  _TIMER_hBios capture
*                11/05/2001  addition 6412, DM642 CSL libraries
*                10/04/2001  addition 6712, 6713, 6414, 6415, 6415 
\******************************************************************************/
#define _CSL_MOD_

#include "csl_irq.h"
#include "csl_timer.h"
#include "csl.h"
#include "csl_edma.h"

/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static char dateStamp[] = "DATE STAMP ["__DATE__"] ["__TIME__"]";

static int initialized = 0;


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/


/******************************************************************************\
* global function definitions
\******************************************************************************/
void _CSL_init(_CSL_Config *config) {

  Uint32 gie;
  TIMER_Handle hTimer;
  Uint32 biosPresent;
  _IRQ_Dispatch *dispatchTable;
  Uint32 timerUsed;
  Uint32 timerNum;

  UNREFERENCED_PARAMETER(dateStamp[0]);

  gie = IRQ_globalDisable(); 

  if (!initialized) {
    #if (EDMA_SUPPORT)
      /* intialize the EDMA NULL parameter table */
      EDMA_configArgs(EDMA_hNull,0,0,0,0,0,0);
    #endif
    initialized = 1;
  }

  if (config != INV) {
    hTimer         = (TIMER_Handle)INV;
    biosPresent    = config->args.biosPresent;
    dispatchTable  = config->args.dispatchTable;
    timerUsed      = config->args.timerUsed;
    timerNum       = config->args.timerNum;
    _IRQ_init(biosPresent, dispatchTable);
    

    if (timerUsed) {
        hTimer = TIMER_open(timerNum,0);  
        _TIMER_hBios = hTimer; 
    }
    config->ret.hTimer = hTimer;
    config->ret.event2IntTbl = (Uint32*)_IRQ_eventTable;
    config->ret.int2EventTbl = (Uint32*)_IRQ_intTable;
  } else {
    _IRQ_init(0,INV);
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/

  /* Here we will define a function whose name matches the library name  */
  /* It will be called by CSL_init() and is used to detect when the      */
  /* incorrect library is linked in.                                     */

#if (CHIP_6201)
  void CSL6201_LIB_() {}

#elif (CHIP_6202)
  void CSL6202_LIB_() {}

#elif (CHIP_6203)
  void CSL6203_LIB_() {}

#elif (CHIP_6204)
  void CSL6204_LIB_() {}

#elif (CHIP_6205)
  void CSL6205_LIB_() {}

#elif (CHIP_6211)
  void CSL6211_LIB_() {}

#elif (CHIP_6701)
  void CSL6701_LIB_() {}

#elif (CHIP_6711)
  void CSL6711_LIB_() {}

#elif (CHIP_6712)
  void CSL6712_LIB_() {}

#elif (CHIP_6713)
  void CSL6713_LIB_() {} 

#elif (CHIP_DA610)
  void CSLDA610_LIB_() {} 

#elif (CHIP_DM642)
  void CSLDM642_LIB_() {}

#elif (CHIP_DM640)
  void CSLDM640_LIB_() {}

#elif (CHIP_DM641)
  void CSLDM641_LIB_() {}

#elif (CHIP_6412)
  void CSL6412_LIB_() {}

#elif (CHIP_6414)
  void CSL6414_LIB_() {}

#elif (CHIP_6415)
  void CSL6415_LIB_() {}

#elif (CHIP_6416)
  void CSL6416_LIB_() {}

#elif (CHIP_6711C)
  void CSL6711C_LIB_() {}

#elif (CHIP_6712C)
  void CSL6712C_LIB_() {}

#elif (CHIP_6411)
  void CSL6411_LIB_() {}

/* next two libaries are DRI300 variants */
#elif (CHIP_6410)
  void CSL6410_LIB_() {}

#elif (CHIP_6413)
  void CSL6413_LIB_() {}

#elif (CHIP_6418)
  void CSL6418_LIB_() {}

#endif

/*----------------------------------------------------------------------------*/


/******************************************************************************\
* End of csl.c
\******************************************************************************/

