/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_cache.c
* DATE CREATED.. 06/11/1999
* LAST MODIFIED. 08/02/2004 - Adding support for C6418
*                05/31/2002
*------------------------------------------------------------------------------
* DESCRIPTION:  (body file for the CACHE module)
*
*   Supported Cache Operations:
*                      +-----+-----+-----+
*                      | L2  | L1P | L1D |
*   +------------------+-----+-----+-----+
*   | invalidate range |     |  X  |     |
*   | invalidate all   |     |  X  |  X  |
*   +------------------+-----+-----+-----+
*   | flush range      |  X  |     |  X  |
*   | flush all        |  X  |     |     |
*   +------------------+-----+-----+-----+
*   | clean range      |  X  |     |     |
*   | clean all        |  X  |     |     |
*   +------------------+-----+-----+-----+
*
*   invalidate range L1P - use L1PFBAR/L1PFWC
*   invalidate all L1P   - use CCFG.IP
*   invalidate all L1D   - use CCFG.ID
*   flush range L2       - use L2FBAR/L2FWC
*   flush range L1D      - use L1DFBAR/L1DFWC
*   flush all L2         - use L2FLUSH
*   clean range L2       - use L2CBAR/L2CWC
*   clean all L2         - use L2CLEAN
*
*   L2 Cache Mode Switching Guidelines
*     Decrease L2 SRAM:
*       1) Perform flush of L2 cache (all). Spin on completion.
*       2) Write new L2 mode / Read back.
*       3) NOP 8
*
*
*     Increase L2 SRAM:
*       1) Perform flush of L2 cache (all). Spin on completion.
*       2) Write new L2 mode / Read back.
*       3) NOP 8
*
\******************************************************************************/
#define _CACHE_MOD_
#include "csl_cache.h"
#include "csl_edma.h"


#if (CACHE_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/

static Uint32 busyMaskCache = 0x00000000;

/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/


/******************************************************************************\
* global function definitions
\******************************************************************************/

void CACHE_reset() {

  #if (L2CACHE_SUPPORT && C64_SUPPORT)
      volatile Uint32 addr;
      #if (CHIP_6414|CHIP_6415|CHIP_6416)
      /* Reset EMIFB regions */
       for (addr=0x01848180u; addr<0x018481C0; addr+=4)
            REG32(addr) = REG32(addr)&0xFFFFFFFE;
      #endif
      /* Reset EMIFA regions */
       for (addr=0x01848200u; addr<0x01848300; addr+=4)
            REG32(addr) = REG32(addr)&0xFFFFFFFE;

       /* Set entire cache region as SRAM */
       CACHE_setL2Mode(0);
  #endif

  /* Invalidate Program and Data cache */
  CHIP_FSET(CSR,PCC,0);
  CHIP_FSET(CSR,DCC,0);

  #if (L2CACHE_SUPPORT && !C64_SUPPORT)
    CACHE_RSET(MAR0,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR1,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR2,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR3,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR4,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR5,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR6,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR7,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR8,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR9,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR10,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR11,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR12,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR13,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR14,CACHE_MAR_DEFAULT);
    CACHE_RSET(MAR15,CACHE_MAR_DEFAULT);
    //CACHE_RSET(CCFG,CACHE_CCFG_DEFAULT);
  #endif /* (L2CACHE_SUPPORT) */
}
/*----------------------------------------------------------------------------*/
#if (L2CACHE_SUPPORT && (CHIP_6414|CHIP_6415|CHIP_6416))
void CACHE_resetEmifb() {

    volatile Uint32 addr;

    for (addr=0x01848180u; addr<0x018481C0; addr+=4) {
      REG32(addr) = REG32(addr)&0xFFFFFFFE;
    }
}
#endif
/*----------------------------------------------------------------------------*/
void CACHE_resetEmifa() {

  volatile Uint32 addr;

   #if (L2CACHE_SUPPORT && (CHIP_6414|CHIP_6415|CHIP_6416|CHIP_DM642|CHIP_DM641|CHIP_DM640|CHIP_6412|CHIP_6411|CHIP_6410|CHIP_6413|CHIP_6418))
      for (addr=0x01848200u; addr<0x01848300; addr+=4) {
          REG32(addr) = REG32(addr)&0xFFFFFFFE;
      }
  #endif
}
/*----------------------------------------------------------------------------*/
CACHE_L2Mode CACHE_getL2Mode() {

  #if (L2CACHE_SUPPORT)
    return (CACHE_L2Mode)CACHE_FGET(CCFG,L2MODE);
  #else
    return (CACHE_L2Mode)CACHE_0KCACHE;
  #endif
}

/*----------------------------------------------------------------------------*/
CACHE_L2Mode CACHE_setL2Mode(CACHE_L2Mode newMode) {

  #if (L2CACHE_SUPPORT)
  Uint32 gie;
  Uint32 base_address;
  Uint32 byte_count;
  Uint32 curMode = CACHE_FGET(CCFG,L2MODE);
  #if (CHIP_6211 | CHIP_6711 | CHIP_6712 | CHIP_6711C | CHIP_6712C) //....
  Uint32 start_of_cache[] =
  {
    0x00010000,         /* 000 =   64K SRAM,   0K cache   */
    0x0000C000,         /* 001 =   48K SRAM,  16K cache   */
    0x00008000,         /* 010 =   32K SRAM,  32K cache   */
    0x00004000,         /* 011 =   16K SRAM,  48K cache   */
    0x00000000,         /* 100 -- undefined, treat as 111 */
    0x00000000,         /* 101 -- undefined, treat as 111 */
    0x00000000,         /* 110 -- undefined, treat as 111 */
    0x00000000          /* 111 =    0K SRAM,  64K cache   */
  };
  #endif

  #if (CHIP_6713 | CHIP_DA610)
  Uint32 start_of_cache[] =
  {
    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
    0x0003C000,         /* 001 =  240K SRAM,  16K cache   */
    0x00038000,         /* 010 =  224K SRAM,  32K cache   */
    0x00034000,         /* 011 =  208K SRAM,  48K cache   */
    0x00030000,         /* 100 -- undefined, treat as 111 */
    0x00030000,         /* 101 -- undefined, treat as 111 */
    0x00030000,         /* 110 -- undefined, treat as 111 */
    0x00030000          /* 111 =  192K SRAM,  64K cache   */
  };
  #endif

  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
  Uint32 start_of_cache[] =
  {
    0x00100000,         /* 000 = 1024K SRAM,   0K cache   */
    0x000F8000,         /* 001 =  992K SRAM,  32K cache   */
    0x000F0000,         /* 010 =  960K SRAM,  64K cache   */
    0x000E0000,         /* 011 =  896K SRAM, 128K cache   */
    0x000C0000,         /* 100 -- undefined, treat as 111 */
    0x000C0000,         /* 101 -- undefined, treat as 111 */
    0x000C0000,         /* 110 -- undefined, treat as 111 */
    0x000C0000          /* 111 =  768K SRAM, 256K cache   */
  };
  #endif

 #if (CHIP_DM642 | CHIP_6412 | CHIP_6411)
  Uint32 start_of_cache[] =
  {
    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
    0x00038000,         /* 001 =  224K SRAM,  32K cache   */
    0x00030000,         /* 010 =  192K SRAM,  64K cache   */
    0x00020000,         /* 011 =  128K SRAM, 128K cache   */
    0x00000000,         /* 100 -- undefined, treat as 111 */
    0x00000000,         /* 101 -- undefined, treat as 111 */
    0x00000000,         /* 110 -- undefined, treat as 111 */
    0x00000000          /* 111 =    0K SRAM, 256K cache   */
  };
 #endif

 #if (CHIP_6418)
  Uint32 start_of_cache[] =
  {
    0x00080000,         /* 000 =  512K SRAM,   0K cache   */
    0x00078000,         /* 001 =  480K SRAM,  32K cache   */
    0x00070000,         /* 010 =  448K SRAM,  64K cache   */
    0x00060000,         /* 011 =  384K SRAM, 128K cache   */
    0x00040000,         /* 100 -- undefined, treat as 111 */
    0x00040000,         /* 101 -- undefined, treat as 111 */
    0x00040000,         /* 110 -- undefined, treat as 111 */
    0x00040000          /* 111 =  256K SRAM, 256K cache   */
  };
 #endif

 #if (CHIP_6413)
  Uint32 start_of_cache[] =
  {
    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
    0x00038000,         /* 001 =  224K SRAM,  32K cache   */
    0x00030000,         /* 010 =  192K SRAM,  64K cache   */
    0x00020000,         /* 011 =  128K SRAM, 128K cache   */
    0x00000000,         /* 100 -- undefined, treat as 111 */
    0x00000000,         /* 101 -- undefined, treat as 111 */
    0x00000000,         /* 110 -- undefined, treat as 111 */
    0x00000000          /* 111 =  256K cache              */
  };
 #endif
  
 #if (CHIP_6410 | CHIP_DM641 | CHIP_DM640)
  Uint32 start_of_cache[] =
  {
    0x00020000,         /* 000 =  128K SRAM,   0K cache   */
    0x00018000,         /* 001 =  96K SRAM,   32K cache   */
    0x00010000,         /* 010 =  64K SRAM,   64K cache   */
    0x00000000,         /* 011 =  128K cache              */
    0x00000000,         /* 100 -- undefined, treat as 011 */
    0x00000000,         /* 101 -- undefined, treat as 011 */
    0x00000000,         /* 110 -- undefined, treat as 011 */
    0x00000000          /* 111 =  undefined, treat as 011 */
  };
 #endif

  gie = IRQ_globalDisable();


    /* switch cache modes and flush as needed */
    
    /* To take care of the invalid L2MODE entries, map them to the mode with highest cache configuration   *
     * This is just so that no unnecessary cache clean is done if an invalide mode is given.               */

  #if (!(CHIP_6410|CHIP_DM640|CHIP_DM641))
    if (newMode & 0x4) {
       newMode = 0x7; 
    }
  #else
    if (newMode & 0x4) {
        newMode = 0x3; /* max cache size is 128 K */
    }
  #endif

    if (newMode < curMode) {    /* increase SRAM size */

      /* Clean all L2 and wait for completion */
     CACHE_clean(CACHE_L2ALL, 0, 0);

      /* Switch L2 mode */
      CACHE_FSET(CCFG,L2MODE,newMode);
      CACHE_FGET(CCFG,L2MODE);
      asm(" NOP 4");
      asm(" NOP 4");

    } else if (newMode > curMode) {   /* decrease SRAM size */

      /* Writeback Invalidate L1D */
        base_address = start_of_cache[newMode];
        byte_count   = start_of_cache[curMode] - start_of_cache[newMode];

		/* If 256K Cache is to be configured */
		if (byte_count == 0x40000)
		{
                    /* wbInvL1d 0xFFFF words from base_address */
	   	    CACHE_wbInvL1d((void *)base_address, 0x3FFFC, CACHE_WAIT);
		    /* wbInvL1d last word */
		    CACHE_wbInvL1d((void *)((Uint32)base_address + 0x3FFFC), 0x4, CACHE_WAIT);
		} else
                    CACHE_wbInvL1d((void *)base_address, byte_count, CACHE_WAIT);

	  //      CACHE_invalidate(CACHE_L1DALL, 0, 0);
      /* Clean L2 and wait for completion */
      CACHE_clean(CACHE_L2ALL, 0, 0);
      /* Switch L2 mode */
      CACHE_FSET(CCFG,L2MODE,newMode);
      CACHE_FGET(CCFG,L2MODE);
      asm(" NOP 4");
      asm(" NOP 4");
    }
    IRQ_globalRestore(gie);

    return (CACHE_L2Mode)(curMode);

  #else
    UNREFERENCED_PARAMETER(newMode);
    return (CACHE_L2Mode)0;
  #endif /* (L2CACHE_SUPPORT) */
}

/*----------------------------------------------------------------------------*/
CACHE_Pcc CACHE_setPccMode(CACHE_Pcc newMode) {

  #if !(L2CACHE_SUPPORT)
    Uint32 curMode = CHIP_FGET(CSR,PCC);
    CHIP_FSET(CSR,PCC,newMode);
    return (CACHE_Pcc)curMode;
  #else
    Uint32 curMode = CHIP_FGET(CSR,PCC);
    UNREFERENCED_PARAMETER(newMode);
    return (CACHE_Pcc)curMode;
  #endif
}

/*----------------------------------------------------------------------------*/
void CACHE_flush(CACHE_Region region, void *addr, Uint32 wordCnt) {

  #if (L2CACHE_SUPPORT)

    /* flush specified region of cache then wait for completion */
    switch (region) {
      case CACHE_L2:
      {
        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

        CACHE_RSET(L2FBAR,addr);
        CACHE_RSET(L2FWC,wordCnt);

        IRQ_globalRestore(gie);

        while (CACHE_RGET(L2FWC))
                    ;
        break;
      }

      case CACHE_L2ALL:
      {
        int i = 0;

        CACHE_FSET(L2FLUSH,F,1);
        while (CACHE_FGET(L2FLUSH,F))
                  i++;

    #if     C64_SUPPORT == 0
        UNREFERENCED_PARAMETER(i);
    #else
        /* -------------------------------------------------------------- */
        /*  Workaround for L2FLUSH errata on C641x Rev 1.03 and earlier.  */
        /*  See CACHE_clean() for description of the workaround.          */
        /* -------------------------------------------------------------- */
        if (i < 4)
        {
        CACHE_FSET(L2FLUSH,F,1);
          while (CACHE_FGET(L2FLUSH,F))
                        ;
        }
   #endif
        break;
      }

      case CACHE_L1D:
      {
        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

        CACHE_RSET(L1DFBAR,addr);
        CACHE_RSET(L1DFWC,wordCnt);

        IRQ_globalRestore(gie);

        while (CACHE_RGET(L1DFWC))
                    ;
        break;
    }
    }

  #else
    UNREFERENCED_PARAMETER(region);
    UNREFERENCED_PARAMETER(addr);
    UNREFERENCED_PARAMETER(wordCnt);
  #endif /* (L2CACHE_SUPPORT) */
}

/*----------------------------------------------------------------------------*/
void CACHE_clean(CACHE_Region region, void *addr, Uint32 wordCnt) {

  #if (L2CACHE_SUPPORT)

    /* clean specified region of cache then wait for completion */
    switch (region) {
      case CACHE_L2:
      {
        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

        CACHE_RSET(L2CBAR,addr);
        asm(" NOP");
        CACHE_RSET(L2CWC,wordCnt);

        IRQ_globalRestore(gie);

        while (CACHE_RGET(L2CWC))
                   ;
        break;
      }

      case CACHE_L2ALL:
      {
        int i = 0;

        CACHE_FSET(L2CLEAN,C,1);
        while (CACHE_FGET(L2CLEAN,C))  /*if executed,L2CLEAN takes a minimum of 5500 cys*/
                  i++;                 /*if broken, L2CLEAN takes 500 cys.CPU is stalled, won't iterate*/

      #if     C64_SUPPORT == 0
        UNREFERENCED_PARAMETER(i);
      #else
        /* ---------------------------------------------------------------- */
        /*  Workaround for L2CLEAN errata on C641x Rev 1.03 and earlier:    */
        /*  Every second L2CLEAN doesn't clean L2, and thus the poll        */
        /*  loop above won't iterate more than once or twice.  We           */
        /*  only issue the extra clean if we detect L2CLEAN didn't work.    */
        /*  This allows the same code to operate correctly on Rev <= 1.03   */
        /*  and Rev > 1.03 parts with the same object code, and without     */
        /*  unnecessary cleans.  A similar fix is required for L2FLUSH.     */
        /*                                                                  */
        /*  Compiler aside:  We don't need to #ifdef the i++ as the         */
        /*  compiler will deadcode it for us.                               */
        /* ---------------------------------------------------------------- */
        if (i < 4)
        {
          CACHE_FSET(L2CLEAN,C,1);
          while (CACHE_FGET(L2CLEAN,C))
                       ;
        }
     #endif
        break;
    }
    }
  #else
    UNREFERENCED_PARAMETER(region);
    UNREFERENCED_PARAMETER(addr);
    UNREFERENCED_PARAMETER(wordCnt);
  #endif /* (L2CACHE_SUPPORT) */
}

/*----------------------------------------------------------------------------*/
void CACHE_invalidate(CACHE_Region region, void *addr, Uint32 wordCnt) {

  #if (L2CACHE_SUPPORT)

    /* invalidate specified region of cache then wait for completion */
    switch (region) {
      case CACHE_L1P:
    {
        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

        CACHE_RSET(L1PFBAR,addr);
        CACHE_RSET(L1PFWC,wordCnt);

        IRQ_globalRestore(gie);

        while (CACHE_RGET(L1PFWC))
                     ;
        break;
    }

      case CACHE_L1PALL:
     {
        CACHE_FSET(CCFG,IP,1);
//        while (CACHE_FGET(CCFG,IP));
        break;
     }

/*
      case CACHE_L1DALL:
     {
        CACHE_FSET(CCFG,ID,1);
        while (CACHE_FGET(CCFG,ID));
        break;
    }
*/
  }
  #else
    UNREFERENCED_PARAMETER(region);
    UNREFERENCED_PARAMETER(addr);
    UNREFERENCED_PARAMETER(wordCnt);
  #endif /* (L2CACHE_SUPPORT) */
}

/*----------------------------------------------------------------------------*/
Uint32 CACHE_getL2SramSize() {

  #if (L2CACHE_SUPPORT)

    Uint32 curMode = CACHE_FGET(CCFG,L2MODE);
    Uint32 size;

   #if C11_SUPPORT
    switch (curMode) {
    case 7:   size = 0x00000000; break;
    case 3:   size = 0x00004000; break;
    case 2:   size = 0x00008000; break;
    case 1:   size = 0x0000C000; break;
    case 0:   size = 0x00010000; break;
    }
   #endif

    #if (CHIP_6414|CHIP_6415|CHIP_6416)
    switch (curMode) {
    case 7:   size = 0x000C0000; break;
    case 3:   size = 0x000E0000; break;
    case 2:   size = 0x000F0000; break;
    case 1:   size = 0x000F8000; break;
    case 0:   size = 0x00100000; break;
    }
   #endif

    #if (CHIP_6713 | CHIP_DA610)
      switch (curMode) {
    case 7:   size = 0x00030000; break; /* + 192K SRAM OFFSET*/
    case 3:   size = 0x00034000; break;
    case 2:   size = 0x00038000; break;
    case 1:   size = 0x0003C000; break;
    case 0:   size = 0x00040000; break;
    }
    #endif

    #if (CHIP_DM642 | CHIP_6412 | CHIP_6411)
      switch (curMode) {
    case 7:   size = 0x00000000; break; /* 64Kbyte Bank */
    case 3:   size = 0x00020000; break;
    case 2:   size = 0x00030000; break;
    case 1:   size = 0x00038000; break;
    case 0:   size = 0x00040000; break;
    }
   #endif

   #if (CHIP_6418)
      switch (curMode) {
    case 7:   size = 0x00040000; break; /* 256Kbyte Offset */
    case 3:   size = 0x00060000; break;
    case 2:   size = 0x00070000; break;
    case 1:   size = 0x00078000; break; 
    case 0:   size = 0x00080000; break;
    }
   #endif

   #if (CHIP_6413)
      switch (curMode) {
    case 7:   size = 0x00000000; break; /* 256Kbyte Offset */
    case 3:   size = 0x00020000; break;
    case 2:   size = 0x00030000; break;
    case 1:   size = 0x00038000; break; 
    case 0:   size = 0x00040000; break; 
    }
   #endif

   #if (CHIP_6410 | CHIP_DM641 | CHIP_DM640)
      switch (curMode) {
    case 3:   size = 0x00000000; break;
    case 2:   size = 0x00010000; break;
    case 1:   size = 0x00018000; break; 
    case 0:   size = 0x00020000; break; 
    }
   #endif

    return size;

  #else
    return (Uint32)0;
  #endif
}

/*----------------------------------------------------------------------------*/
void CACHE_wbL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

      CACHE_RSET(L2WBAR,blockPtr);
      CACHE_RSET(L2WWC,(byteCnt+3)>>2);

	  if (wait)
             while (CACHE_RGET(L2WWC))
                        ;
	  else
	     busyMaskCache = CACHE_WAIT_L2WB;
          
      IRQ_globalRestore(gie);
      
  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
/*----------------------------------------------------------------------------*/
#if (C64_SUPPORT)
void CACHE_invL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */


      CACHE_RSET(L2IBAR,blockPtr);
      CACHE_RSET(L2IWC,(byteCnt+3)>>2);

	  if (wait)
             while (CACHE_RGET(L2IWC))
                          ;
	  else
             busyMaskCache = CACHE_WAIT_L2INV;

       IRQ_globalRestore(gie);

  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
#endif /* C64_SUPPORT */
/*----------------------------------------------------------------------------*/
void CACHE_wbInvL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){


  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */


      CACHE_RSET(L2WIBAR,blockPtr);
      CACHE_RSET(L2WIWC,(byteCnt+3)>>2);

          if (wait)
             while (CACHE_RGET(L2WIWC))
                          ;
	  else
	     busyMaskCache = CACHE_WAIT_L2WBINV;
          
      IRQ_globalRestore(gie);

  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
/*----------------------------------------------------------------------------*/
void CACHE_wbAllL2(CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

      int i = 0;
	  Uint32 gie = IRQ_globalDisable();

      CACHE_FSET(L2WB,C,1);

      if (wait){
         while (CACHE_FGET(L2WB,C))
                i++;

    #if     C64_SUPPORT == 0
        UNREFERENCED_PARAMETER(i);
    #else
        /* -------------------------------------------------------------- */
        /*  Workaround for L2FLUSH errata on C641x Rev 1.03 and earlier.  */
        /*  See CACHE_clean() for description of the workaround.          */
        /* -------------------------------------------------------------- */
        if (i < 4)
        {
           CACHE_FSET(L2WB,C,1);
           while (CACHE_FGET(L2WB,C))
                        ;
        }
    #endif

      }
      else
        busyMaskCache = CACHE_WAIT_L2WBALL;

   IRQ_globalRestore(gie);

  #else
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
/*----------------------------------------------------------------------------*/
void CACHE_wbInvAllL2(CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

      int i = 0;

	  Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

      CACHE_FSET(L2WBINV,C,1);

      if (wait){
         while (CACHE_FGET(L2WBINV,C))
		  i++;

      #if     C64_SUPPORT == 0
        UNREFERENCED_PARAMETER(i);
      #else
        /* ---------------------------------------------------------------- */
        /*  Workaround for L2CLEAN errata on C641x Rev 1.03 and earlier:    */
        /*  Every second L2CLEAN doesn't clean L2, and thus the poll        */
        /*  loop above won't iterate more than once or twice.  We           */
        /*  only issue the extra clean if we detect L2CLEAN didn't work.    */
        /*  This allows the same code to operate correctly on Rev <= 1.03   */
        /*  and Rev > 1.03 parts with the same object code, and without     */
        /*  unnecessary cleans.  A similar fix is required for L2FLUSH.     */
        /*                                                                  */
        /*  Compiler aside:  We don't need to #ifdef the i++ as the         */
        /*  compiler will deadcode it for us.                               */
        /* ---------------------------------------------------------------- */
        if (i < 4)
        {
          CACHE_FSET(L2WBINV,C,1);
          while (CACHE_FGET(L2WBINV,C))
             ;
        }
     #endif
      }
       else
         busyMaskCache = CACHE_WAIT_L2WBINVALL;

     IRQ_globalRestore(gie);

  #else
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
/*----------------------------------------------------------------------------*/
#if (C64_SUPPORT)
void CACHE_invL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

      CACHE_RSET(L1DIBAR,blockPtr);
      CACHE_RSET(L1DIWC,(byteCnt+3)>>2);

      if (wait)
         while (CACHE_RGET(L1DIWC))
                      ;
      else
        busyMaskCache = CACHE_WAIT_L1DINV;

      IRQ_globalRestore(gie);
      
  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT) */

}
#endif /* C64_SUPPORT */
/*----------------------------------------------------------------------------*/
void CACHE_wbInvL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

      CACHE_RSET(L1DWIBAR,blockPtr);
      CACHE_RSET(L1DWIWC,(byteCnt+3)>>2);

      if (wait)
         while (CACHE_RGET(L1DWIWC))
                      ;
      else
        busyMaskCache = CACHE_WAIT_L1DWBINV;
      
      IRQ_globalRestore(gie);
      
  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT && C64_SUPPORT) */

}
/*----------------------------------------------------------------------------*/
/*
void CACHE_invAllL1d(){

  #if (L2CACHE_SUPPORT)

      Uint32 gie = IRQ_globalDisable();

      CACHE_FSET(CCFG,ID,1);

      IRQ_globalRestore(gie);

  #endif

}
*/
/*----------------------------------------------------------------------------*/
void CACHE_invL1p(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){

  #if (L2CACHE_SUPPORT)

	  Uint32 gie;

	  CACHE_wait();
	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */

      CACHE_RSET(L1PIBAR,blockPtr);
      CACHE_RSET(L1PIWC,(byteCnt+3)>>2);

      if (wait)
         while (CACHE_RGET(L1PIWC))
                      ;
      else
         busyMaskCache = CACHE_WAIT_L1PINV;
      
      IRQ_globalRestore(gie);
      
  #else
    UNREFERENCED_PARAMETER(blockPtr);
    UNREFERENCED_PARAMETER(byteCnt);
    UNREFERENCED_PARAMETER(wait);
  #endif /* (L2CACHE_SUPPORT && C64_SUPPORT) */


}
/*----------------------------------------------------------------------------*/
void CACHE_invAllL1p(){

  #if (L2CACHE_SUPPORT)

      Uint32 gie = IRQ_globalDisable();

      CACHE_FSET(CCFG,IP,1);

      IRQ_globalRestore(gie);
  #endif

}
/*----------------------------------------------------------------------------*/
void CACHE_wait(){

  #if (L2CACHE_SUPPORT)

    switch (busyMaskCache) {
      case CACHE_WAIT_L2WB:
	  {
        while(CACHE_RGET(L2WWC));

        break;
	  }
    #if(C64_SUPPORT)
	  case CACHE_WAIT_L2INV:
	  {
             while(CACHE_RGET(L2IWC))
                         ;
             break;
	  }
    #endif /* C64_SUPPORT */
	  case CACHE_WAIT_L2WBINV:
	  {
             while(CACHE_RGET(L2WIWC))
                         ;
             break;
	  }
      case CACHE_WAIT_L2WBALL:
	  {
             while(CACHE_RGET(L2WB))
                         ;
             break;
	  }
      case CACHE_WAIT_L2WBINVALL:
	  {
             while(CACHE_RGET(L2WBINV))
                         ;
             break;
	  }
    #if (C64_SUPPORT)
      case CACHE_WAIT_L1DINV:
	  {
             while(CACHE_RGET(L1DIWC))
                         ;
             break;
	  }
    #endif /* C64_SUPPORT */
      case CACHE_WAIT_L1DWBINV:
	  {
             while(CACHE_RGET(L1DWIWC))
                         ;
             break;
	  }
      case CACHE_WAIT_L1PINV:
	  {
             while(CACHE_RGET(L1PIWC))
                         ;
             break;
	  }
   }

	busyMaskCache = 0;

  #endif /* (L2CACHE_SUPPORT) */

}

/*----------------------------------------------------------------------------*/

void CACHE_wbInvL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize){

  #if(L2CACHE_SUPPORT)
    Uint32 i, loopCnt, wordCnt, remCnt;
    Uint32 gie;

    CACHE_wait();
 
    /* convert to words, round up to next word */ 

    blockSize = (blockSize + 3) >> 2;       
    wordCnt   = (byteCnt   + 3) >> 2;       

    /* number of full blocks to operate on */

    loopCnt = wordCnt / blockSize; 

    /* number of words in last block  */    

    remCnt = wordCnt - blockSize*loopCnt;           

    if (remCnt) loopCnt++;

    for(i = 0; i < loopCnt; i++) {

       /* treat BAR/WC as critical section */

       gie = IRQ_globalDisable();

       /* calculate the starting address for each consecutive block */

       CACHE_RSET(L2WIBAR,(void *)((Uint32)blockPtr + 4*blockSize*i));  

       if (remCnt && (i==(loopCnt-1)))
          CACHE_RSET(L2WIWC, remCnt);
       else
          CACHE_RSET(L2WIWC, blockSize);
                
       IRQ_globalRestore(gie);
        
       while (CACHE_RGET(L2WIWC));

    }
  #else 
      UNREFERENCED_PARAMETER(blockPtr);
      UNREFERENCED_PARAMETER(byteCnt);
  #endif /* (L2CACHE_SUPPORT) */

}


/*----------------------------------------------------------------------------*/

void CACHE_wbL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize){

  #if (L2CACHE_SUPPORT)

    Uint32 i, loopCnt, wordCnt, remCnt;
    Uint32 gie;

    CACHE_wait();
 
    /* convert to words, round up to next word */ 

    blockSize = (blockSize + 3) >> 2;       
    wordCnt   = (byteCnt   + 3) >> 2;       

    /* number of full blocks to operate on */

    loopCnt = wordCnt / blockSize; 

    /* number of words in last block  */    

    remCnt = wordCnt - blockSize*loopCnt;           

    if (remCnt) loopCnt++;

    for(i = 0; i < loopCnt; i++) {

       /* treat BAR/WC as critical section */

       gie = IRQ_globalDisable();

       /* calculate the starting address for each consecutive block */

       CACHE_RSET(L2WBAR,(void *)((Uint32)blockPtr + 4*blockSize*i));  

       if (remCnt && (i==(loopCnt-1)))
          CACHE_RSET(L2WWC, remCnt);
       else
          CACHE_RSET(L2WWC, blockSize);
                
       IRQ_globalRestore(gie);
        
       while (CACHE_RGET(L2WWC));

    }
  #else 
      UNREFERENCED_PARAMETER(blockPtr);
      UNREFERENCED_PARAMETER(byteCnt);
  #endif /* (L2CACHE_SUPPORT) */

}

/*----------------------------------------------------------------------------*/

#endif /* CACHE_SUPPORT */
/******************************************************************************\
* End of csl_cache.c
\******************************************************************************/

