/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_dma.c
* DATE CREATED.. 06/11/1999 
* LAST MODIFIED. 09/19/2000
*
\******************************************************************************/
#define _DMA_MOD_
#include "csl_dma.h"


#if (DMA_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/
#define CHANNEL_ENTRY(chaNum) { \
  FALSE, \
  IRQ_EVT_DMAINT##chaNum##,\
  (volatile Uint32*)_DMA_BASE_CHA##chaNum##, \
}

/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static Uint32 gblAllocMask = 0x00000000;

static DMA_Obj channelTbl[DMA_CHA_CNT] = {
  CHANNEL_ENTRY(0),
  CHANNEL_ENTRY(1),
  CHANNEL_ENTRY(2),
  CHANNEL_ENTRY(3)
};

static Uint32 globalAllocMask = 0x00000000;


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/
Uint32 _DMA_gblRegTbl[_DMA_GBLREG_CNT] = {
  /* [0x00] DMA_GBL_ADDRRLD 0 */ 0xFFFFFFFFu,
  /* [0x01] DMA_GBL_ADDRRLD 1 */ DMA_ADDR(GBLADDRB),
  /* [0x02] DMA_GBL_ADDRRLD 2 */ DMA_ADDR(GBLADDRC),
  /* [0x03] DMA_GBL_ADDRRLD 3 */ DMA_ADDR(GBLADDRD),
  /* [0x04] DMA_GBL_INDEX   0 */ DMA_ADDR(GBLIDXA),
  /* [0x05] DMA_GBL_INDEX   1 */ DMA_ADDR(GBLIDXB),
  /* [0x06] DMA_GBL_INDEX   2 */ 0xFFFFFFFFu,
  /* [0x07] DMA_GBL_INDEX   3 */ 0xFFFFFFFFu,
  /* [0x08] DMA_GBL_CNTRLD  0 */ DMA_ADDR(GBLCNTA),
  /* [0x09] DMA_GBL_CNTRLD  1 */ DMA_ADDR(GBLCNTB),
  /* [0x0A] DMA_GBL_CNTRLD  2 */ 0xFFFFFFFFu,
  /* [0x0B] DMA_GBL_CNTRLD  3 */ 0xFFFFFFFFu,
  /* [0x0C] DMA_GBL_SPLIT   0 */ 0xFFFFFFFFu,
  /* [0x0D] DMA_GBL_SPLIT   1 */ DMA_ADDR(GBLADDRA),
  /* [0x0E] DMA_GBL_SPLIT   2 */ DMA_ADDR(GBLADDRB),
  /* [0x0F] DMA_GBL_SPLIT   3 */ DMA_ADDR(GBLADDRC)
};

/* Predefined channel handles, these are only here for legacy    */
/* purposes and shoul not be used.                               */
DMA_Handle _DMA_hCha0 = (DMA_Handle)&channelTbl[0];
DMA_Handle _DMA_hCha1 = (DMA_Handle)&channelTbl[1];
DMA_Handle _DMA_hCha2 = (DMA_Handle)&channelTbl[2];
DMA_Handle _DMA_hCha3 = (DMA_Handle)&channelTbl[3];


/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
void DMA_reset(DMA_Handle hDma) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hDma == INV) {
    DMA_reset(&(channelTbl[0]));
    DMA_reset(&(channelTbl[1]));
    DMA_reset(&(channelTbl[2]));
    DMA_reset(&(channelTbl[3]));
  } else {
    IRQ_disable(hDma->eventId);
    DMA_RSETH(hDma,PRICTL,DMA_PRICTL_DEFAULT);
    DMA_RSETH(hDma,SECCTL,
       DMA_SECCTL_DEFAULT
      |DMA_FMK(SECCTL,WSYNCCLR,DMA_SECCTL_WSYNCCLR_CLEAR)
      |DMA_FMK(SECCTL,RSYNCCLR,DMA_SECCTL_RSYNCCLR_CLEAR)
    );
    DMA_RSETH(hDma,SRC,DMA_SRC_DEFAULT);
    DMA_RSETH(hDma,DST,DMA_DST_DEFAULT);
    DMA_RSETH(hDma,XFRCNT,DMA_XFRCNT_DEFAULT);
    DMA_RSETH(hDma,PRICTL,DMA_PRICTL_DEFAULT);
    IRQ_clear(hDma->eventId);
  }
  IRQ_globalRestore(gie);
  return;
}

/*----------------------------------------------------------------------------*/
DMA_Handle DMA_open(int chaNum, Uint32 flags) {

  DMA_Handle hDma = (DMA_Handle)INV;
  Uint32 cha,gie;

  gie = IRQ_globalDisable();

  if (chaNum==-1) {
    for (cha=0; cha<DMA_CHA_CNT; cha++) {
      if (!channelTbl[cha].allocated) {
        hDma = (DMA_Handle)&(channelTbl[cha]);
        break;
      }
    }
  } else if (!channelTbl[chaNum].allocated) {
    hDma = (DMA_Handle)&(channelTbl[chaNum]);
  }

  if (hDma != INV) {
    hDma->allocated = TRUE;
    if (flags & DMA_OPEN_RESET) {
      DMA_reset(hDma);
    }
  }

  IRQ_globalRestore(gie);

  return hDma;
}

/*----------------------------------------------------------------------------*/
void DMA_close(DMA_Handle hDma) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  DMA_reset(hDma);
  hDma->allocated = FALSE;
  IRQ_globalRestore(gie);
  return;
}

/*----------------------------------------------------------------------------*/
Uint32 DMA_allocGlobalReg(DMA_Gbl regType, Uint32 initVal) {

  Uint32 gie;
  Uint32 regId = 0x00000000;
  Uint32 regNum0=0,regNum1=0,regNum=0;
  Uint32 mask = 0x00000000;

  gie = IRQ_globalDisable();

  switch (regType) {
    case DMA_GBL_ADDRRLD: regNum0=0x01; regNum1=0x03; mask = 0x20000000; break;
    case DMA_GBL_INDEX:   regNum0=0x04; regNum1=0x05; mask = 0x01000000; break;
    case DMA_GBL_CNTRLD:  regNum0=0x08; regNum1=0x09; mask = 0x00100000; break;
    case DMA_GBL_SPLIT:   regNum0=0x0D; regNum1=0x0F; mask = 0x10000000; break;
  }

  for (regNum=regNum0; regNum<=regNum1; regNum++) {
    if ((gblAllocMask & mask)==0) {
      gblAllocMask |= mask;
      regId = mask | regNum;
      DMA_setGlobalReg(regId,initVal);
      break;
    }
    mask = mask << 1;
  }
 
  IRQ_globalRestore(gie);

  return regId;
}

/*----------------------------------------------------------------------------*/
void DMA_freeGlobalReg(Uint32 regId) {

  Uint32 gie = IRQ_globalDisable();
  gblAllocMask &= ~(regId & 0xFFFF0000);
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
Uint32 DMA_globalAlloc(Uint32 regs) {
  
  Uint32 mask,gie;
  gie = IRQ_globalDisable();
  mask = (globalAllocMask^regs)&regs;
  if (mask == regs) {
    globalAllocMask = globalAllocMask | mask;
  }
  IRQ_globalRestore(gie);
  return mask;
}

/*----------------------------------------------------------------------------*/
void DMA_globalFree(Uint32 regs) {

  Uint32 gie;
  gie = IRQ_globalDisable();
  globalAllocMask = globalAllocMask & ~regs;
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/


#endif /* DMA_SUPPORT */
/******************************************************************************\
* End of csl_dma.c
\******************************************************************************/

