/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_edma.c
* DATE CREATED.. 06/11/1999
* LAST MODIFIED. 08/13/2004 Modified tccAllocTable type from static to global. 
*                06/17/2003 Added support for 6712C
*                05/28/2003 Added support for 6711C
*                12/05/2000
\******************************************************************************/
#define _EDMA_MOD_
#include "csl_edma.h"

/* We need to use the runtime library intrinsic _lmbd() function but */
/* want to declare it as far.                                        */
extern far Uint32 _lmbd(Uint32 src1, Uint32 src2);


#if (EDMA_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
#if (C64_SUPPORT)
  static Uint32 allocMaskCL = 0x00000000;
  static Uint32 allocMaskCH = 0x00000000;
#else
  static Uint32 allocMaskC  = 0x00000000;
#endif

#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
static Uint32 allocMaskT[5] = {0,0,0,0,0};
#else
static Uint32 allocMaskT[4] = {0,0,0,0};
#endif


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/
Uint8 tccAllocTable[64] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

/******************************************************************************\
* global function definitions
\******************************************************************************/
void EDMA_reset(EDMA_Handle hEdma) {

  Uint32 gie = IRQ_globalDisable();

  if (hEdma != EDMA_HINV) {
    EDMA_disableChannel(hEdma);
    EDMA_clearChannel(hEdma);
    EDMA_RSETH(hEdma,OPT,EDMA_OPT_DEFAULT);
    EDMA_RSETH(hEdma,SRC,EDMA_SRC_DEFAULT);
    EDMA_RSETH(hEdma,DST,EDMA_DST_DEFAULT);
    EDMA_RSETH(hEdma,CNT,EDMA_CNT_DEFAULT);
    EDMA_RSETH(hEdma,RLD,EDMA_RLD_DEFAULT);
    EDMA_RSETH(hEdma,IDX,EDMA_IDX_DEFAULT);
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void EDMA_resetAll() {

 //int gie;
 int gie,x;

 Uint32 *pram = (Uint32*)_EDMA_PRAM_START;

 gie = IRQ_globalDisable();

  /* we're going to reset the entire EDMA system */

  /* start by disabling and clearing EDMA channels */
  #if (C64_SUPPORT)
    EDMA_RSET(EERL,  EDMA_EERL_DEFAULT);
    EDMA_RSET(EERH,  EDMA_EERH_DEFAULT);
    EDMA_RSET(CCERL, EDMA_CCERL_DEFAULT);
    EDMA_RSET(CCERH, EDMA_CCERH_DEFAULT);
    EDMA_RSET(ECRL,  0xFFFFFFFF);
    EDMA_RSET(ECRH,  0xFFFFFFFF);
  #endif

  #if(CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
    EDMA_RSET(EER,  EDMA_EER_DEFAULT);
    EDMA_RSET(CCER, EDMA_CCER_DEFAULT);
    EDMA_RSET(ECR,  0xFFFFFFFF);
	EDMA_RSET(ESEL0, EDMA_ESEL0_DEFAULT);
	EDMA_RSET(ESEL1, EDMA_ESEL1_DEFAULT);
    /* ESEL2 reserved */
	EDMA_RSET(ESEL3, EDMA_ESEL3_DEFAULT);
  #endif

  #if(CHIP_6201|CHIP_6202|CHIP_6203|CHIP_6204|CHIP_6205|CHIP_6211| \
      CHIP_6701|CHIP_6711|CHIP_6712)
    EDMA_RSET(EER,  EDMA_EER_DEFAULT);
    EDMA_RSET(CCER, EDMA_CCER_DEFAULT);
    EDMA_RSET(ECR,  0xFFFFFFFF);
  #endif

  /* now disable and clear all interrupts */
  #if (C64_SUPPORT)
    EDMA_RSET(CIERL, EDMA_CIERL_DEFAULT);
    EDMA_RSET(CIERH, EDMA_CIERH_DEFAULT);
    EDMA_RSET(CIPRL, 0xFFFFFFFF);
    EDMA_RSET(CIPRH, 0xFFFFFFFF);
  #else
    EDMA_RSET(CIER, EDMA_CIER_DEFAULT);
    EDMA_RSET(CIPR, 0xFFFFFFFF);
  #endif

  /* reset the EDMA CPU interrupt */
  IRQ_reset(IRQ_EVT_EDMAINT);

  /* now let's clear out the parameter RAM */
  //EDMA_clearPram(0x00000000);

  for (x=0; x<_EDMA_PRAM_ERASE>>2; x++) {
    pram[x] = 0x00000000;
  }
  
#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
 for (x=0; x<5; ++x) {
     allocMaskT[x] = 0;
  }
#else
  for (x=0; x<4; ++x) {
     allocMaskT[x] = 0;
  }
#endif
  
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
EDMA_Handle EDMA_open(int channel, Uint32 flags) {

  Uint32 gie,addr;
  EDMA_Handle hEdma = EDMA_HINV;
  int cha;

  gie = IRQ_globalDisable();

  if (channel==EDMA_CHA_ANY) {
    for (cha=0; cha<EDMA_CHA_CNT; cha++) {
      #if (C64_SUPPORT)
        if (cha < 32) {
          if (!(allocMaskCL & (1<<cha))) {
            allocMaskCL |= (1<<cha);
            addr = (cha*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
            hEdma=_EDMA_MK_HANDLE(addr,cha,_EDMA_TYPE_C);
            break;
          }
        } else {
          if (!(allocMaskCH & (1<<(cha-32)))) {
            allocMaskCH |= (1<<(cha-32));
            addr = (cha*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
            hEdma=_EDMA_MK_HANDLE(addr,cha,_EDMA_TYPE_C);
            break;
          }
        }
      #else
        if (!(allocMaskC & (1<<cha))) {
          allocMaskC |= (1<<cha);
          addr = (cha*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
          hEdma=_EDMA_MK_HANDLE(addr,cha,_EDMA_TYPE_C);
          break;
        }
      #endif
    }
  } else {
    #if (C64_SUPPORT)
      if (channel < 32) {
        if (!(allocMaskCL & (1<<channel))) {
          allocMaskCL |= (1<<channel);
          addr = (channel*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
          hEdma=_EDMA_MK_HANDLE(addr,channel,_EDMA_TYPE_C);
        }
      } else {
        if (!(allocMaskCH & (1<<(channel-32)))) {
          allocMaskCH |= (1<<(channel-32));
          addr = (channel*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
          hEdma=_EDMA_MK_HANDLE(addr,channel,_EDMA_TYPE_C);
        }
      }
    #else
      if (!(allocMaskC & (1<<channel))) {
        allocMaskC |= (1<<channel);
        addr = (channel*_EDMA_ENTRY_SIZE)+_EDMA_PRAM_START;
        hEdma=_EDMA_MK_HANDLE(addr,channel,_EDMA_TYPE_C);
      }
    #endif
  }

  IRQ_globalRestore(gie);

  if (hEdma != (EDMA_Handle)INV) {
    if (flags & EDMA_OPEN_RESET) {
      EDMA_reset(hEdma);
    }

    if (flags & EDMA_OPEN_ENABLE) {
      EDMA_enableChannel(hEdma);
    }
  }

  return hEdma;
}

/*----------------------------------------------------------------------------*/
void EDMA_close(EDMA_Handle hEdma) {

  Uint32 gie,cha;

  gie = IRQ_globalDisable();
  if (hEdma & _EDMA_TYPE_C) {
    cha  = ((((Uint32)hEdma)&0x00FF0000)>>16);
    #if (C64_SUPPORT)
      if (cha < 32) allocMaskCL &= ~(1<<cha);
      else allocMaskCH &= ~(1<<(cha-32));
    #else
      allocMaskC &= ~(1<<cha);
    #endif
    EDMA_reset(hEdma);
  }
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
EDMA_Handle EDMA_allocTable(int tableNum) {

  Uint32 gie,addr;
  EDMA_Handle hEdma = EDMA_HINV;
  int tbl,major,minor;

  gie = IRQ_globalDisable();

  if (tableNum==EDMA_ALLOC_ANY) {
    for (tbl=0; tbl<EDMA_TABLE_CNT; tbl++) {
      major = ((tbl)&(~0x1F))>>5;
      minor = (tbl)&(0x1F);
      if (!(allocMaskT[major] & (1<<minor))) {
        allocMaskT[major] |= (1<<minor);
        addr = (tbl*_EDMA_ENTRY_SIZE)+_EDMA_LINK_START;
        hEdma=_EDMA_MK_HANDLE(addr,tbl,_EDMA_TYPE_T);
        break;
      }
    }
  } else {
   if ((tableNum < EDMA_TABLE_CNT)) {
    major = ((tableNum)&(~0x1F))>>5;
    minor = (tableNum)&(0x1F);
    if (!(allocMaskT[major] & (1<<minor))) {
      allocMaskT[major] |= (1<<minor);
      addr = (tableNum*_EDMA_ENTRY_SIZE)+_EDMA_LINK_START;
      hEdma=_EDMA_MK_HANDLE(addr,tableNum,_EDMA_TYPE_T);
    }
   }
  }

  IRQ_globalRestore(gie);

  return hEdma;
}

/*----------------------------------------------------------------------------*/
void EDMA_freeTable(EDMA_Handle hEdma) {

  Uint32 gie,tbl,major,minor;

  gie = IRQ_globalDisable();
  if (hEdma & _EDMA_TYPE_T) {
    tbl = ((((Uint32)hEdma)&0x00FF0000)>>16);
    major = ((tbl)&(~0x1F))>>5;
    minor = (tbl)&(0x1F);
    allocMaskT[major] &= ~(1<<minor);
  }
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
int EDMA_allocTableEx(int cnt, EDMA_Handle *array) {

  int gie,x,numAllocated=0;
  EDMA_Handle hEdma;

  gie = IRQ_globalDisable();
  for (x=0; x<cnt; x++) {
    hEdma = EDMA_allocTable(EDMA_ALLOC_ANY);
    if (hEdma != EDMA_HINV) {
      numAllocated++;
      array[x] = hEdma;
    } else {
      EDMA_freeTableEx(numAllocated,array);
      numAllocated = 0;
    }
  }
  IRQ_globalRestore(gie);
  return numAllocated;
}

/*----------------------------------------------------------------------------*/
void EDMA_freeTableEx(int cnt, EDMA_Handle *array) {

  int gie,x;
  gie = IRQ_globalDisable();
  for (x=0; x<cnt; x++) {
    EDMA_freeTable(array[x]);
    array[x] = EDMA_HINV;
  }
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void EDMA_clearPram(Uint32 val) {

  int gie,x;

  Uint32 *pram = (Uint32*)_EDMA_PRAM_START;

  gie = IRQ_globalDisable();

  for (x=0; x<_EDMA_PRAM_SIZE>>2; x++) {
    pram[x] = val;
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
int EDMA_intAlloc(int tcc) {

  int gie;
  int tccReturn = -1;

  gie = IRQ_globalDisable();
  if (tcc == -1) {
    for (tcc=0; tcc<EDMA_CHA_CNT; tcc++) {
      if (tccAllocTable[tcc] == 0) {
        tccAllocTable[tcc] = 1;
        tccReturn = tcc;
        break;
      }
    }
  } else {
    if (tccAllocTable[tcc] == 0) {
      tccAllocTable[tcc] = 1;
      tccReturn = tcc;
    }
  }
  IRQ_globalRestore(gie);

  return tccReturn;
}

/*----------------------------------------------------------------------------*/
void EDMA_intFree(int tcc) {

  int gie;
  gie = IRQ_globalDisable();
  tccAllocTable[tcc] = 0;
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
int EDMA_map(int eventNum,int chaNum){
  int chaSelected = -1;

  if(chaNum == 0){
	  EDMA_FSET(ESEL0,EVTSEL0,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 1){
	  EDMA_FSET(ESEL0,EVTSEL1,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 2){
	  EDMA_FSET(ESEL0,EVTSEL2,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 3){
	  EDMA_FSET(ESEL0,EVTSEL3,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 4){
	  EDMA_FSET(ESEL1,EVTSEL4,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 5){
	  EDMA_FSET(ESEL1,EVTSEL5,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 6){
	  EDMA_FSET(ESEL1,EVTSEL6,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 7){
	  EDMA_FSET(ESEL1,EVTSEL7,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 12){
	  EDMA_FSET(ESEL3,EVTSEL12,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 13){
	  EDMA_FSET(ESEL3,EVTSEL13,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 14){
	  EDMA_FSET(ESEL3,EVTSEL14,eventNum);
      chaSelected = chaNum;
  }
  else if(chaNum == 15){
	  EDMA_FSET(ESEL3,EVTSEL15,eventNum);
      chaSelected = chaNum;
  }

  return chaSelected;
}
#endif
/*----------------------------------------------------------------------------*/
#endif /* EDMA_SUPPORT */
/******************************************************************************\
* End of csl_edma.c
\******************************************************************************/

