/******************************************************************************\
*      Copyright (C) 2000-2001 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_gpio.c
* DATE CREATED.. 12/04/2000
* LAST MODIFIED. 08/02/2004 - Adding support for C6418
*                06/17/2003   - Added support for 6712C
*                06/09/2003   - Added support for 6711C
*                02/11/2002
*
\******************************************************************************/

#define _GPIO_MOD_

#include "csl_gpio.h"

#if (GPIO_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/**********************************************\
*  GPIO static macro declarations
\**********************************************/
#if (CHIP_DA610 || CHIP_6713 || CHIP_6416 || CHIP_6415 || CHIP_6414 || CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412 || \
     CHIP_6411 || CHIP_6410 || CHIP_6413 || CHIP_6418)
#define _PIN_ALLOCATION_MASK  0x0000FFFFu
#elif (CHIP_6711C || CHIP_6712C)
#define _PIN_ALLOCATION_MASK  0x000000F4u
#endif

#if (CHIP_DA610)
#define _GPIO_DEV_CNT  2
#else
  #define _GPIO_DEV_CNT  1
#endif


#define DEVICE_ENTRY(devNum) { \
  FALSE,\
 (volatile Uint32*)_GPIO_BASE_ADDR##devNum##,\
  _PIN_ALLOCATION_MASK \
}



/**********************************************\
*  GPIO static typedef declarations
\**********************************************/

/**********************************************\
*  GPIO static function declarations
\**********************************************/

/**********************************************\
*  GPIO static variable definitions
\**********************************************/
static GPIO_Obj _GPIO_deviceTable[_GPIO_DEV_CNT] = {
  DEVICE_ENTRY(0)
   #if (CHIP_DA610)
    ,DEVICE_ENTRY(1)
  #endif
};


/**********************************************\
*  GPIO static function definitions
\**********************************************/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                    G L O B A L   S E C T I O N
\******************************************************************************/

/**********************************************\
* Step  GPIO global variable definitions
\**********************************************/
  Uint32 _GPIO_intTable[GPIO_INT_CNT] = {
  GPIO_MASK_00,
  GPIO_MASK_04,
  GPIO_MASK_05,
  GPIO_MASK_06,
  GPIO_MASK_07
};

GPIO_Handle _hGpioDev0 = (GPIO_Handle) (&(_GPIO_deviceTable[0]));

/**********************************************\
* Step  GPIO global function definitions
\**********************************************/

/*----------------------------------------------------------------------------*/
IDEF void GPIO_reset(GPIO_Handle hGpio) {
   Uint32 gie;

    gie = IRQ_globalDisable();

     GPIO_RSETH(hGpio,GPGC,GPIO_GPGC_DEFAULT);
     GPIO_RSETH(hGpio,GPDIR,GPIO_GPDIR_DEFAULT);
     GPIO_RSETH(hGpio,GPPOL,GPIO_GPPOL_DEFAULT);
     GPIO_RSETH(hGpio,GPHM,GPIO_GPHM_DEFAULT);
     GPIO_RSETH(hGpio,GPLM,GPIO_GPLM_DEFAULT);
     GPIO_RSETH(hGpio,GPEN,GPIO_GPEN_DEFAULT);  /* 0xF9 for 64x*/

     GPIO_RSETH(hGpio,GPDH,0xFFFFFFFF);
     GPIO_RSETH(hGpio,GPDL,0xFFFFFFFF);

     GPIO_RSETH(hGpio,GPVAL,GPIO_GPVAL_DEFAULT);

    IRQ_globalRestore(gie);

}


/*---------------------------------------------------------------------------*/

IDEF void GPIO_clear( GPIO_Handle hGpio ) {

    Uint32 gie;

    gie = IRQ_globalDisable();

     GPIO_RSETH(hGpio,GPDH,0xFFFFFFFF);
     GPIO_RSETH(hGpio,GPDL,0xFFFFFFFF);

   IRQ_globalRestore(gie);
}


/*----------------------------------------------------------------------------*/


GPIO_Handle GPIO_open(int devNum, Uint32 flags) {

  Uint32 gie;

  GPIO_Handle hGpio = INV;

  gie = IRQ_globalDisable();
  if (!_GPIO_deviceTable[devNum].allocated) {
    hGpio = &(_GPIO_deviceTable[devNum]);
    _GPIO_deviceTable[devNum].allocated = TRUE;

    if (hGpio != INV) {
      if (flags & GPIO_OPEN_RESET) {
        GPIO_reset(hGpio);
      }
    }
  }
  IRQ_globalRestore(gie);

  return hGpio;
}
/*----------------------------------------------------------------------------*/
void GPIO_close(GPIO_Handle hGpio) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hGpio != INV) {
    GPIO_reset(hGpio);
    hGpio->allocated = FALSE;
  }
  IRQ_globalRestore(gie);
  return;
}



#endif /* GPIO_SUPPORT */
/******************************************************************************\
* End of csl_gpio.c
\******************************************************************************/
