/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_i2c.c
* DATE CREATED.. 10/17/2001
* LAST MODIFIED. 08/02/2004 - Adding support for C6418
*                11/09/2001
\******************************************************************************/
#define _I2C_MOD_
#include <csl_i2c.h>


#if (I2C_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/
#define DEVICE_ENTRY(devNum) { \
  FALSE, \
  IRQ_EVT_I2CINT##devNum##, \
  (volatile Uint32*)_I2C_BASE_PORT##devNum##, \
  _I2C_I2CDRR##devNum##_ADDR, \
  _I2C_I2CDXR##devNum##_ADDR \
}


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static I2C_Obj _I2C_deviceTable[_I2C_PORT_CNT] = {
#if (CHIP_6713 | CHIP_DA610 | CHIP_6413 | CHIP_6418 | CHIP_6410)
   DEVICE_ENTRY(0)
  ,DEVICE_ENTRY(1)
#endif
#if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412)
   DEVICE_ENTRY(0)
#endif
  };




/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/

/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
void I2C_reset(I2C_Handle hI2c) {
  
  Uint32 gie;

  gie = IRQ_globalDisable();

  if (hI2c == INV) {
    I2C_reset((I2C_Handle)(&(_I2C_deviceTable[0])));
  #if (CHIP_6713 | CHIP_DA610 | CHIP_6413 | CHIP_6418 | CHIP_6410)
    I2C_reset((I2C_Handle)(&(_I2C_deviceTable[1])));
  #endif
  } else {
    I2C_RSETH(hI2c,I2COAR,I2C_I2COAR_DEFAULT);
    I2C_RSETH(hI2c,I2CIER,I2C_I2CIER_DEFAULT);
    I2C_RSETH(hI2c,I2CSTR,I2C_I2CSTR_DEFAULT);
    I2C_RSETH(hI2c,I2CCLKL,I2C_I2CCLKL_DEFAULT);
    I2C_RSETH(hI2c,I2CCLKH,I2C_I2CCLKH_DEFAULT);
    I2C_RSETH(hI2c,I2CCNT,I2C_I2CCNT_DEFAULT);
    I2C_RSETH(hI2c,I2CSAR,I2C_I2CSAR_DEFAULT);
    I2C_RSETH(hI2c,I2CPSC,I2C_I2CPSC_DEFAULT);
    I2C_RSETH(hI2c,I2CDXR,I2C_I2CDXR_DEFAULT);
    
    while(I2C_RGETH(hI2c,I2CISRC))  /* Reading I2CISRC clears the interrupt flag */
              ;

    I2C_RSETH(hI2c,I2CMDR,I2C_I2CMDR_DEFAULT); /* I2C is reset mode  IRS=0 */

#if (CHIP_6413 | CHIP_6418 | CHIP_6410)
    I2C_RSETH(hI2c,I2CEMDR,I2C_I2CEMDR_DEFAULT);
    I2C_RSETH(hI2c,I2CPFUNC,I2C_I2CPFUNC_DEFAULT);
    I2C_RSETH(hI2c,I2CPDIR,I2C_I2CPDIR_DEFAULT);
    I2C_RSETH(hI2c,I2CPDOUT,I2C_I2CPDOUT_DEFAULT);
#endif    
	IRQ_reset(hI2c->eventId);
  }
  
  IRQ_globalRestore(gie);
  
  return;
}
/*----------------------------------------------------------------------------*/
void I2C_resetAll() {
  I2C_reset(INV);
}
/*----------------------------------------------------------------------------*/
I2C_Handle I2C_open(int devNum, Uint32 flags) {

  Uint32 gie;

  I2C_Handle hI2c = INV;

  gie = IRQ_globalDisable();
  if (!_I2C_deviceTable[devNum].allocated) {
    hI2c = &(_I2C_deviceTable[devNum]);
    _I2C_deviceTable[devNum].allocated = TRUE;

    if (hI2c != INV) {
      if (flags & I2C_OPEN_RESET) {
        I2C_reset(hI2c);
      }
    }
  }

  IRQ_globalRestore(gie);

  return hI2c;
}
/*----------------------------------------------------------------------------*/
void I2C_close(I2C_Handle hI2c) {

  Uint32 gie;

  gie = IRQ_globalDisable();

  if (hI2c != INV) {
    I2C_reset(hI2c);
    hI2c->allocated = FALSE;
  }
  
  IRQ_globalRestore(gie);
  
  return;
}
/*----------------------------------------------------------------------------*/

#endif /* I2C_SUPPORT */
/******************************************************************************\
* End of csl_i2c.c
\******************************************************************************/

