/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_irq.c
* DATE CREATED.. 06/09/1999 
* LAST MODIFIED. 07/24/2004 - Re-introduced BIOS dependency due to compatibility issues
*                02/05/2004    Removed bios related components
*                10/03/2001
*                - CHIP_RSET()/CHIP_RGET() replaced by CHIP_CRSET()/CHIP_CRGET()
\******************************************************************************/
#define _IRQ_MOD_
#include <csl_irq.h>
#include <csl_edmahal.h>

#if (IRQ_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/* We need to use the runtime library intrinsic _lmbd() function but */
extern far Uint32 _lmbd(Uint32 src1, Uint32 src2);


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/
static void mux2Tables();
static int getMux(int intNum);
static void setMux(int intNum, int val);


/******************************************************************************\
* static variable definitions
\******************************************************************************/

/* NOTE: _IRQ_init() must be called to initialize these items */
static Uint32 _IRQ_biosPresent = 0;

static char dateStamp[] = "DATE STAMP ["__DATE__"] ["__TIME__"]";

/******************************************************************************\
* static function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
static void mux2Tables() {

  int x;
  
  for (x=0; x<IRQ_EVENT_CNT+1; x++) {
    _IRQ_eventTable[x] = IRQ_MASK_NA;
  }
  
  for (x=4; x<IRQ_INT_CNT; x++) {
    _IRQ_intTable[x] = getMux(x);
    _IRQ_eventTable[_IRQ_intTable[x]] = (1<<x);
  }                              
}


/*----------------------------------------------------------------------------*/
static int getMux(int intNum) {

  Uint32 shift,mask,addr;
  int eventId = 0x1F;

  if ((intNum >= 4) && (intNum <= 9)) {
    addr  = _IRQ_MUXL_ADDR;
    shift = 5*(intNum-4)+(intNum>6 ? 1: 0);
  } else if ((intNum >= 10) && (intNum <= 15)) {
    addr  = _IRQ_MUXH_ADDR;
    shift = 5*(intNum-10)+(intNum>12 ? 1: 0);
  } else {
    addr = 0xFFFFFFFFu;
  }

  /* Go ahead and update the MUX register to reflect the new mapping */
  if (addr != 0xFFFFFFFFu) {
    mask  = 31<<shift;
    eventId = ((*(volatile Uint32*)addr) & mask) >> shift;
  }  
  return eventId;
}

/*----------------------------------------------------------------------------*/
static void setMux(int intNum, int eventId) {

  Uint32 shift,mask,addr,v;

  if ((intNum >= 4) && (intNum <= 9)) {
    addr  = _IRQ_MUXL_ADDR;
    shift = 5*(intNum-4)+(intNum>6 ? 1: 0);
  } else if ((intNum >= 10) && (intNum <= 15)) {
    addr  = _IRQ_MUXH_ADDR;
    shift = 5*(intNum-10)+(intNum>12 ? 1: 0);
  } else {
    addr = 0xFFFFFFFFu;
  }

  /* Go ahead and update the MUX register to reflect the new mapping */
  if (addr != 0xFFFFFFFFu) {
    mask  = 31<<shift;
    v = *(volatile Uint32*)addr;
    v = v & ~mask;
    v = v | ((eventId<<shift)&mask);
    *(volatile Uint32*)addr = v;
  }  
}

/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/

Uint32 _IRQ_eventTable[IRQ_EVENT_CNT+1] = {
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA,
  IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA, IRQ_MASK_NA
};

Uint32 _IRQ_intTable[IRQ_INT_CNT] = {
  IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA,
  IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA,
  IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA,
  IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA, IRQ_EVT_NA
};

_IRQ_Dispatch *_IRQ_dispatchTable = (_IRQ_Dispatch*)INV; 


/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
void _IRQ_init(Uint32 biosPresent, _IRQ_Dispatch *dispatchTable) {

  int gie;
  static int initialized = 0;

  UNREFERENCED_PARAMETER(dateStamp[0]);

  gie = IRQ_globalDisable(); 

  if (!initialized) {
    initialized = 1;
    _IRQ_biosPresent   = biosPresent;
    _IRQ_dispatchTable = dispatchTable;
  }  

  mux2Tables();

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void IRQ_map(Uint32 eventId, Uint32 intNum) {

  int gie,x,i;
  int oldIntNum;

  gie = IRQ_globalDisable(); 
  
  /* Find out which interrupt eventId is currently mapped to then */
  /* wipe out it's entry in the MUX registers and the eventTable. */
  oldIntNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);
  if (oldIntNum >= 0) {
    _IRQ_intTable[oldIntNum] = IRQ_EVT_NA;
    setMux(oldIntNum, 0x1F);
  }
  
  /* Also need to unmap any other events that are mapped to this interrupt */
  for (x=0; x<IRQ_EVENT_CNT; x++) {
    i = 31-_lmbd(1,_IRQ_eventTable[x]);
    if (i == intNum) {
      _IRQ_eventTable[x] = IRQ_MASK_NA;
    } 
  }
  
  /* Now it's ok to go ahead and map the new eventId to intNum */
  _IRQ_intTable[intNum] = eventId;
  _IRQ_eventTable[eventId] = 1<<intNum;
  setMux(intNum, eventId);
  
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
Uint32 IRQ_biosPresent() {
  return _IRQ_biosPresent;
}

/*----------------------------------------------------------------------------*/
void IRQ_config(Uint32 eventId, IRQ_Config *config) {

  /* This is a dispatcher only function */

  Uint32 gie,ie_mask;
  int intNum;

  gie = IRQ_globalDisable();

  if (_IRQ_dispatchTable != INV) {

    intNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);

    if (intNum >= 0) {

      if (config->ieMask & IRQ_IEMASK_SELF) {
        ie_mask = _IRQ_eventTable[eventId];
      } else {
        ie_mask = config->ieMask;
      }
  
      _IRQ_dispatchTable[intNum].funcAddr = config->funcAddr;
      _IRQ_dispatchTable[intNum].funcArg  = config->funcArg;
      _IRQ_dispatchTable[intNum].ccMask   = config->ccMask;
      _IRQ_dispatchTable[intNum].ieMask   = ie_mask;
    }
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void IRQ_configArgs(Uint32 eventId, void *funcAddr, Uint32 funcArg, 
  Uint32 ccMask, Uint32 ieMask) {

  /* This is a dispatcher only function */

  Uint32 gie,ie_mask;
  int intNum;

  gie = IRQ_globalDisable();

  if (_IRQ_dispatchTable != INV) {

    intNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);

    if (intNum >= 0) {

      if (ieMask & IRQ_IEMASK_SELF) {
        ie_mask = _IRQ_eventTable[eventId];
      } else {
        ie_mask = ieMask;
      }
  
      _IRQ_dispatchTable[intNum].funcAddr = funcAddr;
      _IRQ_dispatchTable[intNum].funcArg  = funcArg;
      _IRQ_dispatchTable[intNum].ccMask   = ccMask;
      _IRQ_dispatchTable[intNum].ieMask   = ie_mask;
    }
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void IRQ_getConfig(Uint32 eventId, IRQ_Config *config) {

  /* This is a dispatcher only function */

  Uint32 gie;
  int intNum;

  gie = IRQ_globalDisable();

  if (_IRQ_dispatchTable != INV) {

    intNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);

    if (intNum >= 0) {
      config->funcAddr = _IRQ_dispatchTable[intNum].funcAddr;
      config->funcArg  = _IRQ_dispatchTable[intNum].funcArg;
      config->ccMask   = _IRQ_dispatchTable[intNum].ccMask;
      config->ieMask   = _IRQ_dispatchTable[intNum].ieMask;
    }
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
Uint32 IRQ_getArg(Uint32 eventId) {

  /* This is a dispatcher only function */

  Uint32 gie;
  int intNum;
  Uint32 arg = 0x00000000;

  gie = IRQ_globalDisable();

  if (_IRQ_dispatchTable != INV) {

    intNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);

    if (intNum >= 0) {
      arg = _IRQ_dispatchTable[intNum].funcArg;
    }
  }

  IRQ_globalRestore(gie);

  return arg;
}

/*----------------------------------------------------------------------------*/
void IRQ_setArg(Uint32 eventId, Uint32 arg) {

  /* This is a dispatcher only function */

  Uint32 gie;
  int intNum;

  gie = IRQ_globalDisable();

  if (_IRQ_dispatchTable != INV) {

    intNum = 31-_lmbd(1,_IRQ_eventTable[eventId]);

    if (intNum >= 0) {
      _IRQ_dispatchTable[intNum].funcArg = arg;
    }
  }

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void *IRQ_setVecs(void *vecs) {

  void *oldVecs;

  oldVecs = (void*)(CHIP_CRGET(ISTP) & _CHIP_ISTP_ISTB_MASK);
  CHIP_CRSET(ISTP,(Uint32)vecs & _CHIP_ISTP_ISTB_MASK);

  return oldVecs;
}

/*----------------------------------------------------------------------------*/
extern far void _IRQ_hookFetchPacket();

  asm(" .global __IRQ_hookFetchPacket");
  asm("__IRQ_hookFetchPacket:");
  asm("  STW  B0, *--B15");
  asm("  MVKL 0x00000000, B0");
  asm("  MVKH 0x00000000, B0");
  asm("  B    B0");
  asm("  LDW  *B15++, B0");
  asm("  NOP  4");
  asm("  NOP");
  asm("  NOP");

/*----------------------------------------------------------------------------*/
void IRQ_hook(int intNum, void *func) {
  
  int gie;
  Uint32 vector;
  Uint32 *work,*fetchPacket;

  gie = IRQ_globalDisable();

  #if (C11_SUPPORT || C64_SUPPORT)
    work = (Uint32*)_EDMA_RSVD_PARAM;
    fetchPacket = (Uint32*)_IRQ_hookFetchPacket;

    /* copy the fixed assembled fetch packet over to the work area */
    work[0] = fetchPacket[0];
    work[1] = fetchPacket[1];
    work[2] = fetchPacket[2];
    work[3] = fetchPacket[3];
    work[4] = fetchPacket[4];
    work[5] = fetchPacket[5];

    /* patch in the function address directly into the MVKL/H opcodes */
    work[1] = (work[1] & 0xFF80007F) | (((Uint32)func & 0x0000FFFF) << 7);
    work[2] = (work[2] & 0xFF80007F) | (((Uint32)func & 0xFFFF0000) >> 9);
    
    /* this is where the fetch packet will be copied to in the IST */
    vector = (CHIP_CRGET(ISTP) & _CHIP_ISTP_ISTB_MASK) + 8*4*intNum;
  
    /* copy the fetch packet using QDMA in two steps */
    EDMA_RSET(CIER, EDMA_RGET(CIER) & ~0x00000001);

    EDMA_RSET(CIPR, 0x00000001);
    EDMA_RSET(QSRC, work);
    EDMA_RSET(QDST, vector);
    EDMA_RSET(QCNT, 6);
    EDMA_RSET(QIDX, 4);
    EDMA_RSET(QSOPT, 0x41300001);
    while (!EDMA_RGET(CIPR) & 0x00000001);

    work[0] = fetchPacket[6];
    work[1] = fetchPacket[7];
    EDMA_RSET(CIPR, 0x00000001);
    EDMA_RSET(QSRC, work);
    EDMA_RSET(QDST, vector+4*6);
    EDMA_RSET(QCNT, 2);
    EDMA_RSET(QIDX, 4);
    EDMA_RSET(QSOPT, 0x41300001);
    while (!EDMA_RGET(CIPR) & 0x00000001);
    EDMA_RSET(CIPR, 0x00000001);
  #else
    UNREFERENCED_PARAMETER(vector);
    UNREFERENCED_PARAMETER(work);
    UNREFERENCED_PARAMETER(fetchPacket);
  #endif

  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/


#endif /* IRQ_SUPPORT */
/******************************************************************************\
* End of csl_irq.c
\******************************************************************************/

