/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_mcasp.c
* DATE CREATED.. 06/10/2001 
* LAST MODIFIED. 04/16/2004 - Fixed MCASP_getPins() bug
*                02/28/2002 - Zeno update
*                02/19/2002 - addition _enableHclk()
*                           - clear RSTAT/XSTAT register in reset function  
*                           - set all the XDTM/RDTM bits to 1 in ditConfig()
*                             correction of XROT value
*                11/27/2001 - addition MCASP_ditConfig()
\******************************************************************************/
#define _MCASP_MOD_
#include <csl_mcasp.h>

#if (MCASP_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/******************************************************************************\
* static macro declarations
\******************************************************************************/

/*This should match the Mcasp_Obj structure in csl_mcasp.h*/

//NOTE:  currently, the MCASP event IDs are being used!
//This needs to be changed to MCASP event IDs once they are finalized
//Please remove this comment when the new event IDs are put in

#define DEVICE_ENTRY(devNum) { \
  FALSE, \
  IRQ_EVT_AXINT##devNum##, \
  IRQ_EVT_ARINT##devNum##, \
  (volatile Uint32*)_MCASP_BASE_PORT##devNum##, \
  _MCASP_XBUF##devNum##_ADDR, \
  _MCASP_XBUF0##devNum##_ADDR, \
  _MCASP_RBUF##devNum##_ADDR, \
  _MCASP_RBUF0##devNum##_ADDR, \
  _MCASP_DITCSR##devNum##_ADDR, \
  _MCASP_DITUDR##devNum##_ADDR \
}


/******************************************************************************\
* static typedef declarations
\******************************************************************************/

/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static MCASP_Obj _MCASP_deviceTable[_MCASP_PORT_CNT] = {
   DEVICE_ENTRY(0)
 #if (_MCASP_PORT_CNT == 2)
  ,DEVICE_ENTRY(1)
#endif
};

/* table to set transmit mask data - DIT mode - see MCASP_ditConfig() */
static Uint32 _MCASP_xmaskIntTable[9]= {
      0x0000FFFF,0x0001FFFF,0x0003FFFF,0x0007FFFF,0x000FFFFF,
      0x001FFFFF,0x003FFFFF,0x007FFFFF,0x00FFFFFF};

static Uint32 _MCASP_xmaskQTable[9]= {
      0xFFFF0000u,0xFFFF1000u,0xFFFF3000u,0xFFFF7000u,0xFFFFF000u,
      0xFFFFF100u,0xFFFFF300u,0xFFFFF700u,0xFFFFFF00u };


/******************************************************************************\
* static function definitions
\******************************************************************************/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/******************************************************************************\
* global variable definitions
\******************************************************************************/



/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
void MCASP_reset(MCASP_Handle hMcasp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hMcasp == INV) {
    MCASP_reset((MCASP_Handle)(&(_MCASP_deviceTable[0])));

    #if (_MCASP_PORT_CNT==2)
    MCASP_reset((MCASP_Handle)(&(_MCASP_deviceTable[1])));
    #endif

  } else {
    
    MCASP_RSETH(hMcasp,PFUNC,   	MCASP_PFUNC_DEFAULT);
    MCASP_RSETH(hMcasp,PDIR,    	MCASP_PDIR_DEFAULT);          
    MCASP_RSETH(hMcasp,AMUTE,   	MCASP_AMUTE_DEFAULT);          
    MCASP_RSETH(hMcasp,DLBCTL,   	MCASP_DLBCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,DITCTL, 	        MCASP_DITCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,RMASK,   	MCASP_RMASK_DEFAULT);          
    MCASP_RSETH(hMcasp,RFMT, MCASP_RFMT_DEFAULT);          
    MCASP_RSETH(hMcasp,AFSRCTL, MCASP_AFSRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,ACLKRCTL,MCASP_ACLKRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,AHCLKRCTL,MCASP_AHCLKRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,RTDM,MCASP_RTDM_DEFAULT);          
    MCASP_RSETH(hMcasp,RINTCTL,MCASP_RINTCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,RCLKCHK,MCASP_RCLKCHK_DEFAULT);          
    MCASP_RSETH(hMcasp,XMASK, MCASP_XMASK_DEFAULT);     
    MCASP_RSETH(hMcasp,XFMT,MCASP_XFMT_DEFAULT);          
    MCASP_RSETH(hMcasp,AFSXCTL,MCASP_AFSXCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,ACLKXCTL,MCASP_ACLKXCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,AHCLKXCTL,MCASP_AHCLKXCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,XTDM,MCASP_XTDM_DEFAULT);          
    MCASP_RSETH(hMcasp,XINTCTL,MCASP_XINTCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,XCLKCHK,MCASP_XCLKCHK_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL0,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL1,MCASP_SRCTL_DEFAULT);     
    MCASP_RSETH(hMcasp,SRCTL2,MCASP_SRCTL_DEFAULT);     
    MCASP_RSETH(hMcasp,SRCTL3,MCASP_SRCTL_DEFAULT);
 /* Entries for _MCASP_CHANNEL_CNT == 4 end here*/

 #if (_MCASP_CHANNEL_CNT > 4)
    MCASP_RSETH(hMcasp,SRCTL4,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL5,MCASP_SRCTL_DEFAULT);
 #endif /* _MCASP_CHANNEL_CNT == 6 */

 #if (_MCASP_CHANNEL_CNT > 6)
    MCASP_RSETH(hMcasp,SRCTL6,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL7,MCASP_SRCTL_DEFAULT);          
 #endif /* _MCASP_CHANNEL_CNT == 8 */

 #if (_MCASP_CHANNEL_CNT == 16)
    MCASP_RSETH(hMcasp,SRCTL8,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL9,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL10,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL11,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL12,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL13,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL14,MCASP_SRCTL_DEFAULT);          
    MCASP_RSETH(hMcasp,SRCTL15,MCASP_SRCTL_DEFAULT);  
 #endif 
 
    MCASP_RSETH(hMcasp,RSTAT,0x01F7);
    MCASP_RSETH(hMcasp,XSTAT,0x01F7);
    MCASP_RSETH(hMcasp,GBLCTL,MCASP_GBLCTL_DEFAULT);
    IRQ_reset(hMcasp->xmtEventId);
    IRQ_reset(hMcasp->rcvEventId);
  }                    
  IRQ_globalRestore(gie);
  return;              
}                      

/*----------------------------------------------------------------------------*/
void MCASP_resetAll() 
{
  MCASP_reset(INV);
}

/*----------------------------------------------------------------------------*/
MCASP_Handle MCASP_open(int devNum, Uint32 flags) {
                       
  Uint32 gie;          
                       
  MCASP_Handle hMcasp = INV;
                       
  gie = IRQ_globalDisable();
  if (!_MCASP_deviceTable[devNum].allocated) {
    hMcasp = &(_MCASP_deviceTable[devNum]);
    _MCASP_deviceTable[devNum].allocated = TRUE;
                       
    if (hMcasp != INV) {
      if (flags & MCASP_OPEN_RESET) {
        MCASP_reset(hMcasp);
      }                
    }                  
  }
  IRQ_globalRestore(gie);

  return hMcasp;
}
/*----------------------------------------------------------------------------*/
MCASP_Handle MCASP_getHandle(int devNum) {
                       
  Uint32 gie;          
                       
  MCASP_Handle hMcasp = INV;
                       
  gie = IRQ_globalDisable();
  if (_MCASP_deviceTable[devNum].allocated) {
     hMcasp = &(_MCASP_deviceTable[devNum]);
  } 
  
 IRQ_globalRestore(gie);
  
  return hMcasp;
}

/*----------------------------------------------------------------------------*/
void MCASP_close(MCASP_Handle hMcasp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hMcasp != INV) {
    MCASP_reset(hMcasp);
    hMcasp->allocated = FALSE;
  }
  IRQ_globalRestore(gie);
  return;
}

/*----------------------------------------------------------------------------*/
Uint32 MCASP_getPins(MCASP_Handle hMcasp) {

  return MCASP_RGETH(hMcasp,PDIN);
}

/*----------------------------------------------------------------------------*/
void MCASP_setPins(MCASP_Handle hMcasp, Uint32 pins) {


#if (_MCASP_CHANNEL_CNT == 16) 
  MCASP_RSETH(hMcasp,PDSET, 
	(
       MCASP_FMK(PDSET, AXR0,  pins>>_MCASP_PDSET_AXR0_SHIFT)
      |MCASP_FMK(PDSET, AXR1,  pins>>_MCASP_PDSET_AXR1_SHIFT)                
      |MCASP_FMK(PDSET, AXR2,  pins>>_MCASP_PDSET_AXR2_SHIFT)                
      |MCASP_FMK(PDSET, AXR3,  pins>>_MCASP_PDSET_AXR3_SHIFT)                
      |MCASP_FMK(PDSET, AXR4,  pins>>_MCASP_PDSET_AXR4_SHIFT)                
      |MCASP_FMK(PDSET, AXR5,  pins>>_MCASP_PDSET_AXR5_SHIFT)                
      |MCASP_FMK(PDSET, AXR6,  pins>>_MCASP_PDSET_AXR6_SHIFT)                
      |MCASP_FMK(PDSET, AXR7,  pins>>_MCASP_PDSET_AXR7_SHIFT)                
      |MCASP_FMK(PDSET, AXR8,  pins>>_MCASP_PDSET_AXR8_SHIFT)                
      |MCASP_FMK(PDSET, AXR9,  pins>>_MCASP_PDSET_AXR9_SHIFT)                
      |MCASP_FMK(PDSET, AXR10,  pins>>_MCASP_PDSET_AXR10_SHIFT)             
      |MCASP_FMK(PDSET, AXR11,  pins>>_MCASP_PDSET_AXR11_SHIFT)                
      |MCASP_FMK(PDSET, AXR12,  pins>>_MCASP_PDSET_AXR12_SHIFT)                
      |MCASP_FMK(PDSET, AXR13,  pins>>_MCASP_PDSET_AXR13_SHIFT)                
      |MCASP_FMK(PDSET, AXR14,  pins>>_MCASP_PDSET_AXR14_SHIFT)                
      |MCASP_FMK(PDSET, AXR15,  pins>>_MCASP_PDSET_AXR15_SHIFT)    
      |MCASP_FMK(PDSET, AMUTE,    pins>>_MCASP_PDSET_AMUTE_SHIFT)  
      |MCASP_FMK(PDSET, ACLKX,    pins>>_MCASP_PDSET_ACLKX_SHIFT)                            
      |MCASP_FMK(PDSET, AHCLKX,   pins>>_MCASP_PDSET_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDSET, AFSX,     pins>>_MCASP_PDSET_AFSX_SHIFT)
      |MCASP_FMK(PDSET, ACLKR,    pins>>_MCASP_PDSET_ACLKR_SHIFT)                            
      |MCASP_FMK(PDSET, AHCLKR,   pins>>_MCASP_PDSET_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDSET, AFSR,     pins>>_MCASP_PDSET_AFSR_SHIFT)                                                      
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 16 */
  
#if (_MCASP_CHANNEL_CNT == 8)

MCASP_RSETH(hMcasp,PDSET, 
	(
       MCASP_FMK(PDSET, AXR0,  pins>>_MCASP_PDSET_AXR0_SHIFT)
      |MCASP_FMK(PDSET, AXR1,  pins>>_MCASP_PDSET_AXR1_SHIFT)                
      |MCASP_FMK(PDSET, AXR2,  pins>>_MCASP_PDSET_AXR2_SHIFT)                
      |MCASP_FMK(PDSET, AXR3,  pins>>_MCASP_PDSET_AXR3_SHIFT)                
      |MCASP_FMK(PDSET, AXR4,  pins>>_MCASP_PDSET_AXR4_SHIFT)                
      |MCASP_FMK(PDSET, AXR5,  pins>>_MCASP_PDSET_AXR5_SHIFT)                
      |MCASP_FMK(PDSET, AXR6,  pins>>_MCASP_PDSET_AXR6_SHIFT)                
      |MCASP_FMK(PDSET, AXR7,  pins>>_MCASP_PDSET_AXR7_SHIFT)                
      |MCASP_FMK(PDSET, AMUTE,    pins>>_MCASP_PDSET_AMUTE_SHIFT)  
      |MCASP_FMK(PDSET, ACLKX,    pins>>_MCASP_PDSET_ACLKX_SHIFT)                            
      |MCASP_FMK(PDSET, AHCLKX,   pins>>_MCASP_PDSET_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDSET, AFSX,     pins>>_MCASP_PDSET_AFSX_SHIFT)
      |MCASP_FMK(PDSET, ACLKR,    pins>>_MCASP_PDSET_ACLKR_SHIFT)                            
      |MCASP_FMK(PDSET, AHCLKR,   pins>>_MCASP_PDSET_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDSET, AFSR,     pins>>_MCASP_PDSET_AFSR_SHIFT)                                                      
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 8 */

#if (_MCASP_CHANNEL_CNT == 6)

MCASP_RSETH(hMcasp,PDSET, 
     (
       MCASP_FMK(PDSET, AXR0,  pins>>_MCASP_PDSET_AXR0_SHIFT)
      |MCASP_FMK(PDSET, AXR1,  pins>>_MCASP_PDSET_AXR1_SHIFT)
      |MCASP_FMK(PDSET, AXR2,  pins>>_MCASP_PDSET_AXR2_SHIFT)
      |MCASP_FMK(PDSET, AXR3,  pins>>_MCASP_PDSET_AXR3_SHIFT)
      |MCASP_FMK(PDSET, AXR4,  pins>>_MCASP_PDSET_AXR4_SHIFT)
      |MCASP_FMK(PDSET, AXR5,  pins>>_MCASP_PDSET_AXR5_SHIFT)
      |MCASP_FMK(PDSET, AMUTE,    pins>>_MCASP_PDSET_AMUTE_SHIFT)
      |MCASP_FMK(PDSET, ACLKX,    pins>>_MCASP_PDSET_ACLKX_SHIFT)
      |MCASP_FMK(PDSET, AHCLKX,   pins>>_MCASP_PDSET_AHCLKX_SHIFT)
      |MCASP_FMK(PDSET, AFSX,     pins>>_MCASP_PDSET_AFSX_SHIFT) 
      |MCASP_FMK(PDSET, ACLKR,    pins>>_MCASP_PDSET_ACLKR_SHIFT)
      |MCASP_FMK(PDSET, AHCLKR,   pins>>_MCASP_PDSET_AHCLKR_SHIFT)
      |MCASP_FMK(PDSET, AFSR,     pins>>_MCASP_PDSET_AFSR_SHIFT)
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 6 */

#if (_MCASP_CHANNEL_CNT == 4)

MCASP_RSETH(hMcasp,PDSET, 
     (
       MCASP_FMK(PDSET, AXR0,  pins>>_MCASP_PDSET_AXR0_SHIFT)
      |MCASP_FMK(PDSET, AXR1,  pins>>_MCASP_PDSET_AXR1_SHIFT)
      |MCASP_FMK(PDSET, AXR2,  pins>>_MCASP_PDSET_AXR2_SHIFT)
      |MCASP_FMK(PDSET, AXR3,  pins>>_MCASP_PDSET_AXR3_SHIFT)
      |MCASP_FMK(PDSET, AMUTE,    pins>>_MCASP_PDSET_AMUTE_SHIFT)
      |MCASP_FMK(PDSET, ACLKX,    pins>>_MCASP_PDSET_ACLKX_SHIFT)
      |MCASP_FMK(PDSET, AHCLKX,   pins>>_MCASP_PDSET_AHCLKX_SHIFT)
      |MCASP_FMK(PDSET, AFSX,     pins>>_MCASP_PDSET_AFSX_SHIFT) 
      |MCASP_FMK(PDSET, ACLKR,    pins>>_MCASP_PDSET_ACLKR_SHIFT)
      |MCASP_FMK(PDSET, AHCLKR,   pins>>_MCASP_PDSET_AHCLKR_SHIFT)
      |MCASP_FMK(PDSET, AFSR,     pins>>_MCASP_PDSET_AFSR_SHIFT)
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 4 */

}

/*----------------------------------------------------------------------------*/
void MCASP_clearPins(MCASP_Handle hMcasp, Uint32 pins) {


#if (_MCASP_CHANNEL_CNT == 16)
  MCASP_RSETH(hMcasp,PDCLR, 
	(
       MCASP_FMK(PDCLR, AXR0,  pins>>_MCASP_PDCLR_AXR0_SHIFT)
      |MCASP_FMK(PDCLR, AXR1,  pins>>_MCASP_PDCLR_AXR1_SHIFT)                
      |MCASP_FMK(PDCLR, AXR2,  pins>>_MCASP_PDCLR_AXR2_SHIFT)                
      |MCASP_FMK(PDCLR, AXR3,  pins>>_MCASP_PDCLR_AXR3_SHIFT)                
      |MCASP_FMK(PDCLR, AXR4,  pins>>_MCASP_PDCLR_AXR4_SHIFT)                
      |MCASP_FMK(PDCLR, AXR5,  pins>>_MCASP_PDCLR_AXR5_SHIFT)                
      |MCASP_FMK(PDCLR, AXR6,  pins>>_MCASP_PDCLR_AXR6_SHIFT)                
      |MCASP_FMK(PDCLR, AXR7,  pins>>_MCASP_PDCLR_AXR7_SHIFT)                
      |MCASP_FMK(PDCLR, AXR8,  pins>>_MCASP_PDCLR_AXR8_SHIFT)                
      |MCASP_FMK(PDCLR, AXR9,  pins>>_MCASP_PDCLR_AXR9_SHIFT)                
      |MCASP_FMK(PDCLR, AXR10,  pins>>_MCASP_PDCLR_AXR10_SHIFT)             
      |MCASP_FMK(PDCLR, AXR11,  pins>>_MCASP_PDCLR_AXR11_SHIFT)                
      |MCASP_FMK(PDCLR, AXR12,  pins>>_MCASP_PDCLR_AXR12_SHIFT)                
      |MCASP_FMK(PDCLR, AXR13,  pins>>_MCASP_PDCLR_AXR13_SHIFT)                
      |MCASP_FMK(PDCLR, AXR14,  pins>>_MCASP_PDCLR_AXR14_SHIFT)                
      |MCASP_FMK(PDCLR, AXR15,  pins>>_MCASP_PDCLR_AXR15_SHIFT)   
      |MCASP_FMK(PDCLR, AMUTE,    pins>>_MCASP_PDCLR_AMUTE_SHIFT)  
      |MCASP_FMK(PDCLR, ACLKX,    pins>>_MCASP_PDCLR_ACLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKX,   pins>>_MCASP_PDCLR_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSX,     pins>>_MCASP_PDCLR_AFSX_SHIFT)
      |MCASP_FMK(PDCLR, ACLKR,    pins>>_MCASP_PDCLR_ACLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKR,   pins>>_MCASP_PDCLR_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSR,     pins>>_MCASP_PDCLR_AFSR_SHIFT)                                                                   
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 16 */

#if (_MCASP_CHANNEL_CNT == 8)

  MCASP_RSETH(hMcasp,PDCLR, 
	(
       MCASP_FMK(PDCLR, AXR0,  pins>>_MCASP_PDCLR_AXR0_SHIFT)
      |MCASP_FMK(PDCLR, AXR1,  pins>>_MCASP_PDCLR_AXR1_SHIFT)                
      |MCASP_FMK(PDCLR, AXR2,  pins>>_MCASP_PDCLR_AXR2_SHIFT)                
      |MCASP_FMK(PDCLR, AXR3,  pins>>_MCASP_PDCLR_AXR3_SHIFT)                
      |MCASP_FMK(PDCLR, AXR4,  pins>>_MCASP_PDCLR_AXR4_SHIFT)                
      |MCASP_FMK(PDCLR, AXR5,  pins>>_MCASP_PDCLR_AXR5_SHIFT)                
      |MCASP_FMK(PDCLR, AXR6,  pins>>_MCASP_PDCLR_AXR6_SHIFT)                
      |MCASP_FMK(PDCLR, AXR7,  pins>>_MCASP_PDCLR_AXR7_SHIFT)                
      |MCASP_FMK(PDCLR, AMUTE,    pins>>_MCASP_PDCLR_AMUTE_SHIFT)  
      |MCASP_FMK(PDCLR, ACLKX,    pins>>_MCASP_PDCLR_ACLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKX,   pins>>_MCASP_PDCLR_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSX,     pins>>_MCASP_PDCLR_AFSX_SHIFT)
      |MCASP_FMK(PDCLR, ACLKR,    pins>>_MCASP_PDCLR_ACLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKR,   pins>>_MCASP_PDCLR_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSR,     pins>>_MCASP_PDCLR_AFSR_SHIFT)                                                                   
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 8 */

#if (_MCASP_CHANNEL_CNT == 6)
  MCASP_RSETH(hMcasp,PDCLR, 
     (
       MCASP_FMK(PDCLR, AXR0,  pins>>_MCASP_PDCLR_AXR0_SHIFT)
      |MCASP_FMK(PDCLR, AXR1,  pins>>_MCASP_PDCLR_AXR1_SHIFT)                
      |MCASP_FMK(PDCLR, AXR2,  pins>>_MCASP_PDCLR_AXR2_SHIFT)                
      |MCASP_FMK(PDCLR, AXR3,  pins>>_MCASP_PDCLR_AXR3_SHIFT)                
      |MCASP_FMK(PDCLR, AXR4,  pins>>_MCASP_PDCLR_AXR4_SHIFT)                
      |MCASP_FMK(PDCLR, AXR5,  pins>>_MCASP_PDCLR_AXR5_SHIFT)                
      |MCASP_FMK(PDCLR, AMUTE,    pins>>_MCASP_PDCLR_AMUTE_SHIFT)  
      |MCASP_FMK(PDCLR, ACLKX,    pins>>_MCASP_PDCLR_ACLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKX,   pins>>_MCASP_PDCLR_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSX,     pins>>_MCASP_PDCLR_AFSX_SHIFT)
      |MCASP_FMK(PDCLR, ACLKR,    pins>>_MCASP_PDCLR_ACLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKR,   pins>>_MCASP_PDCLR_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSR,     pins>>_MCASP_PDCLR_AFSR_SHIFT)                                                                   
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 6 */

#if (_MCASP_CHANNEL_CNT == 4)
  MCASP_RSETH(hMcasp,PDCLR, 
     (
       MCASP_FMK(PDCLR, AXR0,  pins>>_MCASP_PDCLR_AXR0_SHIFT)
      |MCASP_FMK(PDCLR, AXR1,  pins>>_MCASP_PDCLR_AXR1_SHIFT)                
      |MCASP_FMK(PDCLR, AXR2,  pins>>_MCASP_PDCLR_AXR2_SHIFT)                
      |MCASP_FMK(PDCLR, AXR3,  pins>>_MCASP_PDCLR_AXR3_SHIFT)                
      |MCASP_FMK(PDCLR, AMUTE,    pins>>_MCASP_PDCLR_AMUTE_SHIFT)  
      |MCASP_FMK(PDCLR, ACLKX,    pins>>_MCASP_PDCLR_ACLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKX,   pins>>_MCASP_PDCLR_AHCLKX_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSX,     pins>>_MCASP_PDCLR_AFSX_SHIFT)
      |MCASP_FMK(PDCLR, ACLKR,    pins>>_MCASP_PDCLR_ACLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AHCLKR,   pins>>_MCASP_PDCLR_AHCLKR_SHIFT)                            
      |MCASP_FMK(PDCLR, AFSR,     pins>>_MCASP_PDCLR_AFSR_SHIFT)                                                                   
    )
  );
#endif /* _MCASP_CHANNEL_CNT == 4 */


}
/*----------------------------------------------------------------------------*/
void MCASP_enableClk(MCASP_Handle hMcasp, Uint32 direction)
{
	Uint32 rsrgr = 0;
	Uint32 xsrgr = 0;
	
	if (direction & MCASP_RCV)
	{
	  rsrgr = (MCASP_FMKS(GBLCTL,RCLKRST,ACTIVE));
	}
	if (direction & MCASP_XMT)
	{
	  xsrgr = (MCASP_FMKS(GBLCTL,XCLKRST,ACTIVE));
	}	  	
	if (direction){
	  MCASP_RSETH(hMcasp,GBLCTL,(MCASP_RGETH(hMcasp,GBLCTL)|rsrgr|xsrgr));
	  }
}
/*----------------------------------------------------------------------------*/
void MCASP_enableHclk(MCASP_Handle hMcasp, Uint32 direction)
{
	Uint32 rsrgr = 0;
	Uint32 xsrgr = 0;
	
	if (direction & MCASP_RCV)
	{
		rsrgr = ( MCASP_FMKS(GBLCTL,RHCLKRST,ACTIVE)) ;
      }

	if (direction & MCASP_XMT)
	{
		xsrgr = (MCASP_FMKS(GBLCTL,XHCLKRST,ACTIVE) );
	}
	
	if (direction){
	  MCASP_RSETH(hMcasp,GBLCTL,(MCASP_RGETH(hMcasp,GBLCTL)|rsrgr|xsrgr));
	 }
}


/*----------------------------------------------------------------------------*/
void MCASP_enableSers(MCASP_Handle hMcasp, Uint32 direction)
{
	Uint32 rsers = 0;
	Uint32 xsers = 0;
	
	if (direction & MCASP_RCV)
	{
		rsers = (MCASP_FMKS(GBLCTL,RSRCLR,ACTIVE));
	}

	if (direction & MCASP_XMT)
	{
		xsers = (MCASP_FMKS(GBLCTL,XSRCLR,ACTIVE));
	}
	
	if (direction)
	  MCASP_RSETH(hMcasp,GBLCTL,(MCASP_RGETH(hMcasp,GBLCTL)|rsers| xsers));
	  
}

/*----------------------------------------------------------------------------*/
void MCASP_enableSm(MCASP_Handle hMcasp, Uint32 direction)
{
	Uint32 rsmchn = 0;
	Uint32 xsmchn = 0;
	
	if (direction & MCASP_RCV)
	{
		rsmchn = (MCASP_FMKS(GBLCTL,RSMRST,ACTIVE));
	}

	if (direction & MCASP_XMT)
	{
		xsmchn = (MCASP_FMKS(GBLCTL,XSMRST,ACTIVE));
	}
	
	if (direction)
	  MCASP_RSETH(hMcasp,GBLCTL,(MCASP_RGETH(hMcasp,GBLCTL)|rsmchn|xsmchn));
}

/*----------------------------------------------------------------------------*/
void MCASP_enableFsync(MCASP_Handle hMcasp, Uint32 direction)
{
	Uint32 rfsync = 0;
	Uint32 xfsync = 0;
	
	if (direction & MCASP_RCV)
	{
		rfsync = (MCASP_FMKS(GBLCTL,RFRST,ACTIVE));
	}

	if (direction & MCASP_XMT)
	{
		xfsync = (MCASP_FMKS(GBLCTL,XFRST,ACTIVE));
	}
	
	if (direction)
	  MCASP_RSETH(hMcasp,GBLCTL,(MCASP_RGETH(hMcasp,GBLCTL)|rfsync|xfsync));
}

/*----------------------------------------------------------------------------*/
void MCASP_setupClk(MCASP_Handle hMcasp,MCASP_SetupClk *setupclk, 
                    Uint32 direction) 
{
 Uint32 gie;
 
 gie = IRQ_globalDisable();
    if (direction & MCASP_XMT) 
    {
        MCASP_RSETH(hMcasp,ACLKXCTL,
                    MCASP_ACLKXCTL_RMK(setupclk->xclkpol,
                                       setupclk->syncmode, 
                                       setupclk->xclksrc, 
                                       (setupclk->xclkdiv)-1));
}
/* asynchronous mode */
    if (direction & MCASP_RCV) 
    {
        MCASP_RSETH(hMcasp,ACLKRCTL,
                    MCASP_ACLKRCTL_RMK(setupclk->rclkpol, 
                                       setupclk->rclksrc, 
                                       (setupclk->rclkdiv)-1));
 }
 IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void MCASP_setupHclk(MCASP_Handle hMcasp,MCASP_SetupHclk *setuphclk, 
                     Uint32 direction) 
{
 
 Uint32 gie;
 
 gie = IRQ_globalDisable();
    if (direction & MCASP_XMT) 
    {
        MCASP_RSETH(hMcasp,AHCLKXCTL,
                    MCASP_AHCLKXCTL_RMK(setuphclk->xhclksrc,
                                        setuphclk->xhclkpol, 
                                        (setuphclk->xhclkdiv)-1));
 }
    if (direction & MCASP_RCV) 
    {
        MCASP_RSETH(hMcasp,AHCLKRCTL,
                    MCASP_AHCLKRCTL_RMK(setuphclk->rhclksrc,
                                        setuphclk->rhclkpol, 
                                       (setuphclk->rhclkdiv)-1));
 }
 
 IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void MCASP_setupFsync(MCASP_Handle hMcasp,MCASP_SetupFsync *setupfsync,
                      Uint32 direction) 
{
 Uint32 xslotsize,rslotsize,gie;
 
 gie = IRQ_globalDisable();
 
 if (direction & MCASP_XMT) {
 if (setupfsync->xmode == MCASP_MODE_BURST) { 
     xslotsize  =    0;
 } else {
     xslotsize  =   setupfsync->xslotsize;
 }
 
        MCASP_RSETH(hMcasp,AFSXCTL,
                    MCASP_AFSXCTL_RMK(xslotsize,setupfsync->fxwid,
                                      setupfsync->xfssrc,setupfsync-> xfspol));
}
if (direction & MCASP_RCV) {
  if (setupfsync->rmode == MCASP_MODE_BURST) { 
     rslotsize  =    0;
  } else {
     rslotsize  =   setupfsync->rslotsize;
  }
        MCASP_RSETH(hMcasp,AFSRCTL,
                    MCASP_AFSRCTL_RMK(rslotsize,setupfsync->frwid, 
                                      setupfsync->rfssrc,setupfsync-> rfspol));
 }
 IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void MCASP_setupFormat(MCASP_Handle hMcasp,MCASP_SetupFormat *setupformat, 
                       Uint32 direction) 
{
 Uint32 gie,rrot,xrot;
 Uint32 xslotval,rslotval;
 Uint32 xrotval,rrotval;
 
 gie = IRQ_globalDisable();


/*** Transmitter format **************************************************/
if (direction & MCASP_XMT) {

 if (setupformat->xdsprep == MCASP_DSP_Q31) {
  if (    setupformat->xorder == MCASP_FORMAT_MSB) { 
       if (setupformat->xalign == MCASP_FORMAT_LEFT  ) {  
                  xrot = 0;                         /* 0 */ 
          } else { /* right aligned - (SLOT - WORD) */
                  xrot = (setupformat->xslotsize - setupformat->xwordsize);
          }
     }else { /* LSB first */                
          if (setupformat->xalign == MCASP_FORMAT_LEFT  ) {  
                  xrot =(32 - setupformat->xwordsize); /* 32-WORD */
          } else { /* right aligned */
                  xrot = (32-setupformat->xslotsize); /* 32-SLOT */
          }
     }     
  }else  { /* DSP Integer Representation */    
     if (  setupformat->xorder == MCASP_FORMAT_MSB) { 
       if (setupformat->xalign == MCASP_FORMAT_LEFT  ) {  
                  xrot = (setupformat->xwordsize); /* WORD */ 
          } else { /* right aligned  */
                  xrot = (setupformat->xslotsize); /* SLOT */ 
          }
     }else { /* LSB first */                
          if (setupformat->xalign == MCASP_FORMAT_LEFT  ) {  
                  xrot = 0; 
          } else { /* right aligned (32-(slot-word))%32 */
                  xrot = (32 -(setupformat->xslotsize-setupformat->xwordsize))%32; 
          }
     }
  }       
/* Translation of rslotsize/xslotsize              *\
   MCASP_XFMT_XSSZ_8BITS            	0x00000003u
   MCASP_XFMT_XSSZ_12BITS            	0x00000005u
   MCASP_XFMT_XSSZ_16BITS            	0x00000007u
   MCASP_XFMT_XSSZ_20BITS            	0x00000009u
   MCASP_XFMT_XSSZ_24BITS            	0x0000000Bu
   MCASP_XFMT_XSSZ_28BITS            	0x0000000Du
\* MCASP_XFMT_XSSZ_32BITS                 0x0000000Fu */
   xslotval =((setupformat->xslotsize-8)/2) + 3;
      
           
 /* MCASP_XFMT_XROT_4BITS            	0x00000001u*\
    MCASP_XFMT_XROT_8BITS            	0x00000002u
    MCASP_XFMT_XROT_12BITS            	0x00000003u
    MCASP_XFMT_XROT_16BITS            	0x00000004u
    MCASP_XFMT_XROT_20BITS            	0x00000005u
    MCASP_XFMT_XROT_24BITS            	0x00000006u
 \* MCASP_XFMT_XROT_28BITS            	0x00000007u*/
   xrotval=((xrot-4)/4)+1; 

 MCASP_RSETH(hMcasp,XFMT,
  MCASP_FMK(XFMT,XDATDLY,setupformat->xdelay) |
  MCASP_FMK(XFMT,XRVRS,setupformat->xorder) |
  MCASP_FMK(XFMT,XPAD,setupformat->xpad) |
  MCASP_FMK(XFMT,XPBIT,setupformat->xpbit) |
  MCASP_FMK(XFMT,XSSZ,xslotval) |
  MCASP_FMK(XFMT,XBUSEL,setupformat->xbusel) |
  MCASP_FMK(XFMT,XROT,xrotval)
  );
 } /* end of transmitter */
 
 
  /****** Receiver format *************************************************/
 if (direction & MCASP_RCV) {
     
 if (setupformat->rdsprep == MCASP_DSP_Q31) {
  if (    setupformat->rorder == MCASP_FORMAT_MSB) { 
       if (setupformat->ralign == MCASP_FORMAT_LEFT  ) {  
                rrot = (setupformat->rslotsize); /* SLOT */ 
          } else { /* right aligned - (WORD) */
                rrot = setupformat->rwordsize;
          }
     }else { /* LSB first */                
          if (setupformat->xalign == MCASP_FORMAT_LEFT  ) {  
                /* (32-(slot-word))%32 */
                rrot = (32 -(setupformat->rslotsize-setupformat->rwordsize))%32; 
     
          } else { /* right aligned */
                rrot = 0;
          }
     }     
  }else  { /* DSP Integer Representation */    
     if (  setupformat->rorder == MCASP_FORMAT_MSB) { 
       if (setupformat->ralign == MCASP_FORMAT_LEFT  ) {  
                  rrot = (setupformat->rslotsize-setupformat->rwordsize); 
     
          } else { /* right aligned  */
                  rrot = 0 ;
          }
     }else { /* LSB first */                
          if (setupformat->ralign == MCASP_FORMAT_LEFT  ) {  
                  rrot = (32 -setupformat->rslotsize);
          } else { /* right aligned (32-(slot-word))%32 */
                  rrot = (32 -setupformat->rwordsize); 
          }
     }
  }               
  
/* Translation of rslotsize/xslotsize              *\
   MCASP_XFMT_XSSZ_8BITS            	0x00000003u
   MCASP_XFMT_XSSZ_12BITS            	0x00000005u
   MCASP_XFMT_XSSZ_16BITS            	0x00000007u
   MCASP_XFMT_XSSZ_20BITS            	0x00000009u
   MCASP_XFMT_XSSZ_24BITS            	0x0000000Bu
   MCASP_XFMT_XSSZ_28BITS            	0x0000000Du
\* MCASP_XFMT_XSSZ_32BITS               0x0000000Fu */
  
  rslotval =((setupformat->rslotsize-8)/2) + 3; 
  
  /* MCASP_XFMT_XROT_4BITS            	0x00000001u*\
   MCASP_XFMT_XROT_8BITS            	0x00000002u
   MCASP_XFMT_XROT_12BITS            	0x00000003u
   MCASP_XFMT_XROT_16BITS            	0x00000004u
   MCASP_XFMT_XROT_20BITS            	0x00000005u
   MCASP_XFMT_XROT_24BITS            	0x00000006u
\* MCASP_XFMT_XROT_28BITS            	0x00000007u*/

  rrotval=((rrot-4)/4)+1;


  MCASP_RSETH(hMcasp,RFMT,
  MCASP_FMK(RFMT,RDATDLY,setupformat->rdelay) |
  MCASP_FMK(RFMT,RRVRS,setupformat->rorder) |
  MCASP_FMK(RFMT,RPAD,setupformat->rpad) |
  MCASP_FMK(RFMT,RPBIT,setupformat->rpbit) |
  MCASP_FMK(RFMT,RSSZ,rslotval) |
  MCASP_FMK(RFMT,RBUSEL,setupformat->rbusel) |
  MCASP_FMK(RFMT,RROT,rrotval)
  );
  
} /* end of rcv format */

 IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void MCASP_configDit(MCASP_Handle hMcasp,MCASP_Dsprep dsprep, Uint32 datalen){
 Uint32 maskval;

 maskval=datalen-16;

/* required configuration */
/*****************************************\
  * XMOD  =   384 tdm slot                
  * FSWID= 0 (bit duration 1 bit) 
  * FSXM=  1 (internal)
  * FSXP = 0 (polarity rising edge)
 \****************************************/
 
  MCASP_RSETH(hMcasp,AFSXCTL,MCASP_AFSXCTL_RMK(0x180u,0,1,0));
  MCASP_FSETH(hMcasp,ACLKXCTL,ASYNC,1);
  
/* Transmit DIT and Receiver DIR and TDM mode */
  MCASP_RSETH(hMcasp,XTDM,0xFFFFFFFF);
  //MCASP_RSETH(hMcasp,RTDM,MCASP_FMKS(RTDM,RTDMS0,ACTIVE)| MCASP_FMKS(RTDM,RTDMS1,ACTIVE));

/*  Only 24 bits data - XFORMAT unit : ROT / ORD / MASK */
  if ( dsprep == 0) {
 /* set data format 0x000000F0 */
  MCASP_RSETH(hMcasp,XFMT,\
         MCASP_FMKS(XFMT,XRVRS,LSBFIRST) |
         MCASP_FMKS(XFMT,XPAD,ZERO) |
         MCASP_FMKS(XFMT,XROT,NONE) | 
         MCASP_FMKS(XFMT,XSSZ,32BITS) \
           );

 
 /* set mask */
  MCASP_RSETH(hMcasp,XMASK,_MCASP_xmaskIntTable[maskval]);

  } else  {  /* Q31 data */
   /* set data format 0x000000F3*/
    MCASP_RSETH(hMcasp,XFMT,\
         MCASP_FMKS(XFMT,XRVRS,LSBFIRST) |
         MCASP_FMKS(XFMT,XPAD,ZERO) |
         MCASP_FMKS(XFMT,XROT,8BITS) | 
         MCASP_FMKS(XFMT,XSSZ,32BITS) \
        );
    /* set mask */
 
   MCASP_RSETH(hMcasp,XMASK,_MCASP_xmaskQTable[maskval]);
  }
}

#endif /* MCASP_SUPPORT */
/******************************************************************************\
* End of csl_mcasp.c
\******************************************************************************/


