/******************************************************************************\
*           Copyright (C) 1999-2001 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_mcbsp.c
* DATE CREATED.. 06/11/1999 
* LAST MODIFIED. 09/07/2001 
*                    - MCBSP_start() added
\******************************************************************************/
#define _MCBSP_MOD_

#include <csl_mcbsp.h>


#if (MCBSP_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/
#define DEVICE_ENTRY(devNum) { \
  FALSE, \
  IRQ_EVT_XINT##devNum##, \
  IRQ_EVT_RINT##devNum##, \
  (volatile Uint32*)_MCBSP_BASE_PORT##devNum##, \
  _MCBSP_DRR##devNum##_ADDR, \
  _MCBSP_DXR##devNum##_ADDR \
}

#define PIN_MASK (Uint32)(\
  _MCBSP_PCR_CLKXP_MASK| \
  _MCBSP_PCR_FSXP_MASK| \
  _MCBSP_PCR_DXSTAT_MASK| \
  _MCBSP_PCR_CLKRP_MASK| \
  _MCBSP_PCR_FSRP_MASK| \
  _MCBSP_PCR_DRSTAT_MASK| \
  _MCBSP_PCR_CLKSSTAT_MASK \
)


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static MCBSP_Obj _MCBSP_deviceTable[_MCBSP_PORT_CNT] = {
  DEVICE_ENTRY(0),
  DEVICE_ENTRY(1)
  #if (_MCBSP_PORT_CNT==3)
    ,DEVICE_ENTRY(2)
  #endif
};


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/
/* predefined handles for legacy - should not be used */
MCBSP_Handle _MCBSP_hDev0=(MCBSP_Handle)(&(_MCBSP_deviceTable[0]));
MCBSP_Handle _MCBSP_hDev1=(MCBSP_Handle)(&(_MCBSP_deviceTable[1]));
#if (_MCBSP_PORT_CNT == 3)
  MCBSP_Handle _MCBSP_hDev2=(MCBSP_Handle)(&(_MCBSP_deviceTable[2]));
#endif



/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
void MCBSP_reset(MCBSP_Handle hMcbsp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hMcbsp == INV) {
    MCBSP_reset((MCBSP_Handle)(&(_MCBSP_deviceTable[0])));
    MCBSP_reset((MCBSP_Handle)(&(_MCBSP_deviceTable[1])));
    #if (_MCBSP_PORT_CNT==3)
      MCBSP_reset((MCBSP_Handle)(&(_MCBSP_deviceTable[2])));
    #endif
  } else {
    MCBSP_RSETH(hMcbsp,SPCR,MCBSP_SPCR_DEFAULT);
    MCBSP_RSETH(hMcbsp,RCR, MCBSP_RCR_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCR, MCBSP_XCR_DEFAULT);
    MCBSP_RSETH(hMcbsp,SRGR,MCBSP_SRGR_DEFAULT);
    MCBSP_RSETH(hMcbsp,MCR, MCBSP_MCR_DEFAULT);
    #if (!C64_SUPPORT) /* ?? added C64_SUPPORT */
    MCBSP_RSETH(hMcbsp,RCER,MCBSP_RCER_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCER,MCBSP_XCER_DEFAULT);
    #else /*if(C64_SUPPORT)*/
    MCBSP_RSETH(hMcbsp,RCERE0,MCBSP_RCERE0_DEFAULT);
    MCBSP_RSETH(hMcbsp,RCERE1,MCBSP_RCERE1_DEFAULT);
    MCBSP_RSETH(hMcbsp,RCERE2,MCBSP_RCERE2_DEFAULT);
    MCBSP_RSETH(hMcbsp,RCERE3,MCBSP_RCERE3_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCERE0,MCBSP_XCERE0_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCERE1,MCBSP_XCERE1_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCERE2,MCBSP_XCERE2_DEFAULT);
    MCBSP_RSETH(hMcbsp,XCERE3,MCBSP_XCERE3_DEFAULT);
    #endif
    MCBSP_RSETH(hMcbsp,PCR, MCBSP_PCR_DEFAULT);
    IRQ_reset(hMcbsp->xmtEventId);
    IRQ_reset(hMcbsp->rcvEventId);
  }
  IRQ_globalRestore(gie);
  return;
}

/*----------------------------------------------------------------------------*/
void MCBSP_resetAll() {

  MCBSP_reset(INV);
}

/*----------------------------------------------------------------------------*/
MCBSP_Handle MCBSP_open(int devNum, Uint32 flags) {

  Uint32 gie;

  MCBSP_Handle hMcbsp = INV;

  gie = IRQ_globalDisable();
  if (!_MCBSP_deviceTable[devNum].allocated) {
    hMcbsp = &(_MCBSP_deviceTable[devNum]);
    _MCBSP_deviceTable[devNum].allocated = TRUE;

    if (hMcbsp != INV) {
      if (flags&MCBSP_OPEN_RESET) {
        MCBSP_reset(hMcbsp);
      }
    }
  }
  IRQ_globalRestore(gie);

  return hMcbsp;
}

/*----------------------------------------------------------------------------*/
void MCBSP_close(MCBSP_Handle hMcbsp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hMcbsp != INV) {
    MCBSP_reset(hMcbsp);
    hMcbsp->allocated = FALSE;
  }
  IRQ_globalRestore(gie);
  return;
}

/*----------------------------------------------------------------------------*/
void MCBSP_start(MCBSP_Handle hMcbsp, Uint32 startMask, Uint32 sampleratedelay) {

  /* compiler should not optimize delay loop after setting GRST under -o3 switch */
  volatile Uint32 count = sampleratedelay;
  

  if (startMask & MCBSP_SRGR_START) {
    MCBSP_FSETSH(hMcbsp,SPCR,GRST,NO);
    while(count > 0) {
        --count;
    }
  }

  if (startMask & MCBSP_RCV_START) {
    MCBSP_FSETSH(hMcbsp,SPCR,RRST,NO);
  }
  
  if (startMask & MCBSP_XMIT_START) {
    MCBSP_FSETSH(hMcbsp,SPCR,XRST,NO);
  }

  if (startMask & MCBSP_SRGR_FRAMESYNC) {
     MCBSP_FSETSH(hMcbsp,SPCR,FRST,NO);
  }
  
}


/*----------------------------------------------------------------------------*/

Uint32 MCBSP_getPins(MCBSP_Handle hMcbsp) {

  return (MCBSP_RGETH(hMcbsp,PCR)& PIN_MASK);
}

/*----------------------------------------------------------------------------*/
void MCBSP_setPins(MCBSP_Handle hMcbsp, Uint32 pins) {

  MCBSP_RSETH(hMcbsp,PCR, 
    (MCBSP_RGETH(hMcbsp,PCR)&~PIN_MASK)
    |(
       MCBSP_FMK(PCR, CLKXP,    pins>>_MCBSP_PCR_CLKXP_SHIFT)
      |MCBSP_FMK(PCR, FSXP,     pins>>_MCBSP_PCR_FSXP_SHIFT)
      |MCBSP_FMK(PCR, DXSTAT,   pins>>_MCBSP_PCR_DXSTAT_SHIFT)
      |MCBSP_FMK(PCR, CLKRP,    pins>>_MCBSP_PCR_CLKRP_SHIFT)
      |MCBSP_FMK(PCR, FSRP,     pins>>_MCBSP_PCR_FSRP_SHIFT)
      |MCBSP_FMK(PCR, DRSTAT,   pins>>_MCBSP_PCR_DRSTAT_SHIFT)
      |MCBSP_FMK(PCR, CLKSSTAT, pins>>_MCBSP_PCR_CLKSSTAT_SHIFT)
    )
  );
}

/*----------------------------------------------------------------------------*/


#endif /* MCBSP_SUPPORT */
/******************************************************************************\
* End of csl_mcbsp.c
\******************************************************************************/

