/******************************************************************************\
*           Copyright (C) 2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_pci.c
* DATE CREATED.. Mon 06/12/2000 
* LAST MODIFIED. Fri 10/13/2000 
*
\******************************************************************************/
#define _PCI_MOD_
#include "csl_pci.h"

#if (PCI_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/****************************************\
* PCI static macro declarations
\****************************************/

/****************************************\
* PCI static typedef declarations
\****************************************/

/****************************************\
* PCI static function declarations
\****************************************/

/****************************************\
* PCI static variable definitions
\****************************************/

/****************************************\
* PCI static function definitions
\****************************************/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/****************************************\
* PCI global variable definitions
\****************************************/
  Uint32 _PCI_maskTable[13] = {
 _PCI_PCIIEN_PWRMGMT_MASK,
 _PCI_PCIIEN_PCITARGET_MASK,
 _PCI_PCIIEN_PCIMASTER_MASK,
 _PCI_PCIIEN_HOSTSW_MASK,
 _PCI_PCIIEN_PWRLH_MASK,
 _PCI_PCIIEN_PWRHL_MASK,
 _PCI_PCIIEN_MASTEROK_MASK,
 _PCI_PCIIEN_CFGDONE_MASK,
 _PCI_PCIIEN_CFGERR_MASK,
 _PCI_PCIIEN_EERDY_MASK, 
  0,
 _PCI_PCIIEN_PRST_MASK,   
 _PCI_PCIIS_DMAHALTED_MASK  
 };
  
  
/****************************************\
* PCI global function definitions
\****************************************/
/*----------------------------------------------------------------------------*/
Uint32 PCI_eepromWrite(Uint32 eeaddr,Uint16 eedata) {
  
  Uint32 success = 0;
 
 /* Enable write */  
    while ( _PCI_eepromEnableWrite() == 0);
     
  
/* Write */ 
  if ( PCI_FGET(EECTL,EESZ)!=0)
   {
    while( !(PCI_FGET(RSTSRC,CFGDONE)));       /* wait for EEPROM ready */
    PCI_FSET(EEADD,EEADD,eeaddr);             /* address EEprom into EEADD */ 
    PCI_FSET(EEDAT,EEDAT,eedata);             /* 16-bit data into EEDAT  */
    PCI_FSET(EECTL,EECNT,0x1);                 /* Op-code for WRITE command  */
    while( PCI_FGET(EECTL,READY) == 0);        /* wait for READY is to 1*/
 
/* Read back the data */
    if ( PCI_eepromRead(eeaddr) == eedata) 
         { 
          success= 1;
         }
   } 
  
  /* Disable Write */
  PCI_FSET(EEADD,EEADD,0x0);                 /* address 0x0 into EEADD */ 
  PCI_FSET(EECTL,EECNT,0x0);                /* Op-code for Disable 0x0  */
  while( PCI_FGET(EECTL,READY) == 0); 
       
  return (success);
}      

/*----------------------------------------------------------------------------*/
Uint32 PCI_eepromWriteAll(Uint16 eedata) {
  
  Uint32 success = 0;
  Uint32 DATASZ,eeaddr;
 
 /* Enable Write ALL*/  
   while ( _PCI_eepromEnableWrite() == 0);
 
 /* Select the eeaddr  */  
   DATASZ=PCI_FGET(EECTL,EESZ);
   if ( DATASZ == 1) { eeaddr=0x010;}
   else if ( DATASZ == 2) { eeaddr=0x080;} 
   else if ( DATASZ == 3) { eeaddr=0x040;}
   else if ( DATASZ == 4) { eeaddr = 0x100;}
   else { return 0;}
     
         
 /* Write all with eedata value */      
  if ( PCI_FGET(EECTL,EESZ)!=0)
   {
    while( !(PCI_FGET(RSTSRC,CFGDONE)));          /* wait for READY is to 1*/
    PCI_FSET(EEADD,EEADD,eeaddr);                /* address EEprom into EEADD 0x10 */ 
    PCI_FSET(EEDAT,EEDAT,eedata);               /* 16-bit data into EEDAT  */
    PCI_FSET(EECTL,EECNT,0x00);                 /* Op-code for WRITE ALL command  */
    while( PCI_FGET(EECTL,READY) == 0);         /* wait for READY is to 1*/
    success= 1;
   } 
  
  /* Disable Write ALL */
  PCI_FSET(EEADD,EEADD,0x0);                /* address EEprom into EEADD 0x0*/ 
  PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for Disable 0x0  */
  while( PCI_FGET(EECTL,READY) == 0); 
 

   return (success);
}
/*----------------------------------------------------------------------------*/
Uint16 PCI_eepromRead(Uint32 eeaddr) {
  if ( PCI_FGET(EECTL,EESZ)!=0)
   {
    while( !(PCI_FGET(RSTSRC,CFGDONE)));
    PCI_FSET(EEADD,EEADD,eeaddr);             /* address EEprom into EEADD */                 PCI_FSET(EECTL,EECNT,0x2);               /* Op-code for READ command  0x2*/
    while( PCI_FGET(EECTL,READY) == 0);   /* Wait for READY is set to 1*/
    return(PCI_FGET(EEDAT,EEDAT));  
   }
  else {
  return(0xFFFF);
  }
}
/*----------------------------------------------------------------------------*/
Uint32 PCI_eepromErase(Uint32 eeaddr) {
  Uint32 success=0;  
  
  /* Enable Erase */  
   while ( _PCI_eepromEnableWrite() == 0);
   
   /* Erase Data */
  if ( PCI_FGET(EECTL,EESZ)!=0)
   {
   
    PCI_FSET(EEADD,EEADD,eeaddr);             /* address EEprom into EEADD */             
    while( !(PCI_FGET(RSTSRC,CFGDONE)));     /* Wait the EEPROM is ready for next command */ 
    PCI_FSET(EECTL,EECNT,0x3);               /* Op-code for ERASE command 0x3 */
    while( !(PCI_FGET(EECTL,READY)));        /* Wait for READY is set to 1*/
    success = 1;
   }    
   
  /* Disable Write */
   PCI_FSET(EEADD,EEADD,0x0);                /* address EEprom into EEADD 0x0 */ 
   PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for Disable command 0x0 */
   while( PCI_FGET(EECTL,READY) == 0); 
   
   return(success);
}    
/*----------------------------------------------------------------------------*/
Uint32 PCI_eepromEraseAll() {
  Uint32 success=0; 
  Uint32 DATASZ,eeaddr; 
  
  /* Enable Write (Erase) */   
   while ( _PCI_eepromEnableWrite() == 0);     
   
  /* Select address */ 
   DATASZ=PCI_FGET(EECTL,EESZ);
   if ( DATASZ == 1) { eeaddr=0x020;}
   else if ( DATASZ == 2) { eeaddr=0x100;} 
   else if ( DATASZ == 3) { eeaddr=0x080;}
   else if ( DATASZ == 4) { eeaddr = 0x200;}
   else { return 0;}
     
 


  /* Erase All setup */     
  if ( PCI_FGET(EECTL,EESZ)!=0)
   {
    PCI_FSET(EEADD,EEADD,eeaddr);             /* address EEprom into EEADD 0x20*/ 
    PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for ERAL command 0x0 */
    while( PCI_FGET(EECTL,READY) == 0);                  
     success = 1;
   }    
   
  /* Disable Erase ALL */
   PCI_FSET(EEADD,EEADD,0x0);             /* address EEprom into EEADD */ 
   PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for WRITE command  */
   while( PCI_FGET(EECTL,READY) == 0); 
   
   return(success);
}
/*----------------------------------------------------------------------------*/
 Uint32 _PCI_eepromEnableWrite() {
 Uint32 success = 1;
 volatile Uint32 DATASZ; 
 
 DATASZ=PCI_FGET(EECTL,EESZ);
 

if (DATASZ ==  1 )
      {  
       PCI_FSET(EEADD,EEADD,0x030);             /* 1K: address EEprom into EEADD 0x30*/ 
       PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for EWEN command 0x0 */
       while( PCI_FGET(EECTL,READY) == 0);  
      }
else if (DATASZ ==  2)
      {  
       PCI_FSET(EEADD,EEADD,0x0180);             /*  2K: address EEprom into EEADD 0x180*/ 
       PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for EWEN command 0x0 */
       while( PCI_FGET(EECTL,READY) == 0);  
      }
else if (DATASZ ==  3)
      {  
       PCI_FSET(EEADD,EEADD,0x0C0);             /* 4K: address EEprom into EEADD 0x180*/ 
       PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for EWEN command 0x0 */
       while( PCI_FGET(EECTL,READY) == 0);  
      }
else if (DATASZ ==  4)
      {  
       PCI_FSET(EEADD,EEADD,0x300);             /* 16k: address EEprom into EEADD 0x300*/ 
       PCI_FSET(EECTL,EECNT,0x0);                 /* Op-code for EWEN command 0x0 */
       while( PCI_FGET(EECTL,READY) == 0);  
      }                
else { 
      success = 0;
      }
  
  return success;
}    
/*----------------------------------------------------------------------------*/
 
#endif /* PCI_SUPPORT */
/******************************************************************************\
* End of csl_pci.c
\******************************************************************************/

