/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_tcp.c
* DATE CREATED.. 02/23/2001 
* LAST MODIFIED. 05/30/2001
\******************************************************************************/
#define _TCP_MOD_
#include "csl_tcp.h"

#if (TCP_SUPPORT)
#ifndef OPT_LE
 #ifndef OPT_BE
  #ifndef OPT_BE32
   #define NO_OPT
  #endif
 #endif
#endif

/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/


/******************************************************************************\
* global function definitions
\******************************************************************************/

void TCP_tailConfig(TCP_Standard standard,
        TCP_Mode mode,
        TCP_Map map,
        TCP_Rate rate,
        TCP_UserData *restrict xabData,
        TCP_ConfigIc *restrict configIc) {

  if (standard == TCP_STANDARD_3GPP) TCP_tailConfig3GPP(mode, map, xabData,
                    configIc);
  else TCP_tailConfigIs2000(mode, map, rate, xabData, configIc);
  
} /* end TCP_tailConfig() */
/*----------------------------------------------------------------------------*/

void TCP_tailConfig3GPP(TCP_Mode mode,
        TCP_Map map,
        TCP_UserData *restrict xabData,
        TCP_ConfigIc *restrict configIc) {
  Int32 gie;
  Uint32 tail1, tail2, tail3, tail4, tail5, tail6;

  gie = IRQ_globalDisable();

  if ((mode == TCP_MODE_SP) && (map == TCP_MAP_MAP2)){
    /* tail1 = 0, x_t_2, x_t_1, x_t_0 */
    /* tail2 = 0, a_t_2, a_t_1, a_t_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0,   0  ,   0  ,   0   */
    /* tail5 = 0,   0  ,   0  ,   0   */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
    tail2 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
    tail3 = 0;
    tail4 = 0;
    tail5 = 0;
    tail6 = 0;
  } /* end if SPMAP2 */
  else if (mode == TCP_MODE_SP){
    /* tail1 = 0, xt_2, xt_1, xt_0 */
    /* tail2 = 0, at_2, at_1, at_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0,   0  ,   0  ,   0   */
    /* tail5 = 0,   0  ,   0  ,   0   */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
    tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
    tail3 = 0;
    tail4 = 0;
    tail5 = 0;
    tail6 = 0;
  } /* end if SPMAP1 */
  else { /* if (mode == TCP_MODE_SA) */
    /* tail1 = 0,  xt_2,  xt_1,  xt_0 */
    /* tail2 = 0,  at_2,  at_1,  at_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0, x_t_2, x_t_1, x_t_0 */
    /* tail5 = 0, a_t_2, a_t_1, a_t_0 */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
    tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
    tail3 = 0;
    tail4 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
    tail5 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
    tail6 = 0;
  } /* end if SA */
  configIc->ic6  = tail1;
  configIc->ic7  = tail2;
  configIc->ic8  = tail3;
  configIc->ic9  = tail4;
  configIc->ic10 = tail5;
  configIc->ic11 = tail6;

  IRQ_globalRestore(gie);
} /* end TCP_tailConfig3GPP */
/*----------------------------------------------------------------------------*/

void TCP_tailConfigIs2000(TCP_Mode mode,
        TCP_Map map,
        TCP_Rate rate,
        TCP_UserData *restrict xabData,
        TCP_ConfigIc *restrict configIc) {

  Int32 gie;
  Uint32 tail1, tail2, tail3, tail4, tail5, tail6;

  gie = IRQ_globalDisable();

  if ((mode == TCP_MODE_SP) && (map == TCP_MAP_MAP2)) {
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail2 = 0, a_t_2, a_t_1, a_t_0 */
        tail1 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail2 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[15]), \
                                (xabData[12]), (xabData[9]));
        tail2 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       b_t_2     ,       b_t_1     ,       b_t_0      */
        /* tail3 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[20]), \
                                (xabData[16]), (xabData[12]));
        tail2 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        tail3 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch rate */
  } /* end if mode == TCP_MODE_SPMAP2 */
  /*--------------------------------------------------------------------------*/
  else if (mode == TCP_MODE_SP) { /*  && (map == TCP_MAP1) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, xt_2, xt_1, xt_0 */
        /* tail2 = 0, at_2, at_1, at_0 */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[6]), \
                                (xabData[3]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (xt_2 + _xt_2), (xt_1 + _xt_1), (xt_0 + _xt_0) */
        /* tail2 = 0,      at_2     ,      at_1     ,      at_0      */
        /* tail3 = 0,      bt_2     ,      bt_1     ,      bt_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[8]), \
                                (xabData[4]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SPMAP1 */
  /*--------------------------------------------------------------------------*/
  else { /* (mode == TCP_MODE_SA) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0,  xt_2,  xt_1,  xt_0 */
        /* tail2 = 0,  at_2,  at_1,  at_0 */
        /* tail3 = 0,   0  ,   0  ,   0   */
        /* tail4 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail5 = 0, a_t_2, a_t_1, a_t_0 */
        /* tail6 = 0,   0  ,   0  ,   0   */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail5 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,        at_2     ,        at_1     ,        at_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        tail1 = TCP_makeTailArgs(        0               , (xabData[6]), \
                                (xabData[3]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(        0                 , (xabData[15]), \
                                (xabData[12]), (xabData[9]));
        tail5 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,       at_2      ,       at_1      ,       at_0       */
        /* tail3 = 0,       bt_2      ,       bt_1      ,       bt_0       */
        /* tail4 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail5 = 0,      a_t_2      ,      a_t_1      ,      a_t_0       */
        /* tail6 = 0,      b_t_2      ,      b_t_1      ,      b_t_0       */
        tail1 = TCP_makeTailArgs(        0               , (xabData[8]), \
                                (xabData[4]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = TCP_makeTailArgs(        0                 , (xabData[20]), \
                                (xabData[16]), (xabData[12]));
        tail5 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail6 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SA */
  /*--------------------------------------------------------------------------*/
  
  configIc->ic6  = tail1;
  configIc->ic7  = tail2;
  configIc->ic8  = tail3;
  configIc->ic9  = tail4;
  configIc->ic10 = tail5;
  configIc->ic11 = tail6;

  IRQ_globalRestore(gie);
} /* end TCP_tailConfig */

/*----------------------------------------------------------------------------*/
void TCP_setParams(TCP_Params *restrict configParms,
        TCP_ConfigIc *restrict configIc) {

  Uint32 gie;
  //TCP_Standard standard  = configParms->standard;
  TCP_Mode mode          = configParms->mode;
  TCP_Map  map           = configParms->map;
  TCP_Rate rate          = configParms->rate - 1;
  Uint32 intFlag         = configParms->intFlag;
  Uint32 outParmFlag     = configParms->outParmFlag;
  Uint32 frameLen        = configParms->frameLen;
  Uint32 subFrameLen     = configParms->subFrameLen;
  Uint32 relLen          = configParms->relLen;
  Uint32 relLenLast      = configParms->relLenLast;
  Uint32 prologSize      = configParms->prologSize;
  Uint32 numSubBlock     = configParms->numSubBlock;
  Uint32 numSubBlockLast = configParms->numSubBlockLast;
  Uint32 maxIter         = configParms->maxIter;
  Uint32 snr             = configParms->snr;
  Uint32 numInter        = configParms->numInter;
  Uint32 numSysPar       = configParms->numSysPar;
  Uint32 numApriori      = configParms->numApriori;
  Uint32 numExt          = configParms->numExt;
  Uint32 numHd           = configParms->numHd;
  Uint32 ic0, ic1, ic2, ic3, ic4, ic5;

  gie = IRQ_globalDisable();

  if (mode) mode = (mode << 2) + map;
  
  ic0 = TCP_IC0_RMK(frameLen,outParmFlag,intFlag,rate,mode);
  ic1 = TCP_IC1_RMK(relLenLast,relLen,subFrameLen);
  ic2 = TCP_IC2_RMK(snr,maxIter,numSubBlockLast,numSubBlock,prologSize);
  ic3 = TCP_IC3_RMK(numSysPar,numInter);
  ic4 = TCP_IC4_RMK(numExt,numApriori);
  ic5 = TCP_IC5_RMK(numHd);

  configIc->ic0 = ic0;
  configIc->ic1 = ic1;
  configIc->ic2 = ic2;
  configIc->ic3 = ic3;
  configIc->ic4 = ic4;
  configIc->ic5 = ic5;
  
  IRQ_globalRestore(gie);
} /* end TCP_genParamIc */

/*----------------------------------------------------------------------------*/

void TCP_genIc(TCP_Params *restrict configParms,
               TCP_UserData *restrict xabData,
               TCP_ConfigIc *restrict configIc) {

  TCP_Standard standard = configParms->standard;
  TCP_Mode mode         = configParms->mode;
  TCP_Map  map          = configParms->map;
  TCP_Rate rate         = configParms->rate;

  TCP_setParams(configParms, configIc);
  if (standard == TCP_STANDARD_3GPP) TCP_tailConfig3GPP(mode, map, xabData, configIc);
  else TCP_tailConfig(standard, mode, map, rate, xabData, configIc);
} /* end TCP_genIc */

/*----------------------------------------------------------------------------*/

Uint32 TCP_genParams(TCP_BaseParams *configBase, TCP_Params *configParms)
{
Uint32 numSubFrame = 0;
Uint32 frameLen = configBase->frameLen;

    configParms->standard    = configBase->standard;
    configParms->rate        = configBase->rate;
    configParms->intFlag     = configBase->intFlag;
    configParms->outParmFlag = configBase->outParmFlag;
    configParms->frameLen    = frameLen;
    configParms->prologSize  = configBase->prologSize;
    configParms->maxIter     = configBase->maxIter;
    configParms->snr         = configBase->snr;

    if (frameLen <= 5114){             /* Configure for Standalone processing */
      configParms->mode = TCP_MODE_SA;
      TCP_calcSubBlocksSA(configParms);
      TCP_calcCountsSA(configParms);
    } /* end if standalone */
    else {                             /* Configure for Shared processing     */
      configParms->mode = TCP_MODE_SP;
      numSubFrame = TCP_calcSubBlocksSP(configParms);
      TCP_calcCountsSP(configParms);
    } /* end if shared processing */

    /* Return the number of sub frames */
    return(numSubFrame);
    
} /* end TCP_genParams() */
    
/*----------------------------------------------------------------------------*/

void TCP_calcSubBlocksSA(TCP_Params *configParms)
{
Uint16 frameLen   = configParms->frameLen;
//Uint8  prologSize = configParms->prologSize;  - not used
Uint16 winSize;
Uint16 numSlidingWindow;
Uint16 relLen;
Uint16 numSubBlock;

    if (frameLen <= 128) numSlidingWindow = 1;
    else if (frameLen <= 256) numSlidingWindow = 2;
    else numSlidingWindow = 4;
    
    winSize = frameLen;
    winSize = TCP_normalCeil(winSize, numSlidingWindow);
    numSubBlock = TCP_normalCeil(winSize, TCP_RLEN_MAX);
    relLen = TCP_normalCeil(winSize, numSubBlock) - 1;
    
    if(relLen < 39) relLen = 39;
    
    configParms->relLen          = relLen;
    configParms->numSubBlock     = numSubBlock;
    configParms->subFrameLen     = 0;
    configParms->relLenLast      = 0;
    configParms->numSubBlockLast = 0;
    
} /* end TCP_calcSubBlocksSA() */

/*----------------------------------------------------------------------------*/

void TCP_calcCountsSA(TCP_Params *configParms)
{
Uint16 frameLen = configParms->frameLen;
TCP_Rate rate   = configParms->rate;
Uint16 numSysPar;
Uint16 numInter;
Uint16 numHd;

    numSysPar = TCP_ceil((frameLen * rate), 4);
    numInter  = TCP_ceil( frameLen        , 1);
    numHd     = TCP_ceil( frameLen        , 5);
	
    configParms->numSysPar  = numSysPar;
    configParms->numInter   = numInter;
    configParms->numHd      = numHd;	
    configParms->numApriori = 0;
    configParms->numExt     = 0;

} /* end TCP_calcCountsSA() */

/*----------------------------------------------------------------------------*/


Uint32 TCP_calcSubBlocksSP(TCP_Params *configParms)
{
Uint16 frameLen   = configParms->frameLen;
Uint8  prologSize = configParms->prologSize;
TCP_Rate rate     = configParms->rate;
Uint16 numSubFrame;
Uint16 subFrameLen;
Uint16 subFrameLenLast;
Uint16 winSize;
Uint16 numSlidingWindow;
Uint16 relLen;
Uint16 relLenLast;
Uint16 numSubBlock;
Uint16 numSubBlockLast;

    numSubFrame = (frameLen / (TCP_FLEN_MAX - 2 * prologSize)) + 1;
    subFrameLen = frameLen / numSubFrame;
    if (rate == TCP_RATE_1_4) subFrameLen = TCP_ceil(subFrameLen, 4) << 4;
    else subFrameLen = TCP_ceil(subFrameLen, 3) << 3;
    subFrameLenLast = frameLen - (numSubFrame - 1) * subFrameLen;

    if (subFrameLen <= 128) numSlidingWindow = 1;
    else if (subFrameLen <= 256) numSlidingWindow = 2;
    else numSlidingWindow = 4;
    
    winSize = subFrameLen;
    subFrameLen += 2 * prologSize;
    winSize = TCP_normalCeil(winSize, numSlidingWindow);
    numSubBlock = TCP_normalCeil(winSize, TCP_RLEN_MAX);
    relLen = TCP_normalCeil(winSize, numSubBlock) - 1;
    if(relLen < 39) relLen = 39;
    
    if (subFrameLenLast <= 128) numSlidingWindow = 1;
    else if (subFrameLenLast <= 256) numSlidingWindow = 2;
    else numSlidingWindow = 4;
    
    winSize = subFrameLenLast; 
    subFrameLenLast += 2 * prologSize;
    winSize = TCP_normalCeil(winSize, numSlidingWindow);
    numSubBlockLast = TCP_normalCeil(winSize, TCP_RLEN_MAX);
    relLenLast = TCP_normalCeil(winSize, numSubBlockLast) - 1;
    if (relLenLast < 39) relLenLast = 39;
    
    configParms->subFrameLen     = subFrameLen;
    configParms->relLen          = relLen;
    configParms->relLenLast      = relLenLast;
    configParms->numSubBlock     = numSubBlock;
    configParms->numSubBlockLast = numSubBlockLast;
    
    return(numSubFrame);

} /* end TCP_calcSubBlocksSP() */

/*----------------------------------------------------------------------------*/

void TCP_calcCountsSP(TCP_Params *configParms)
{
Uint32 numApriori = configParms->numApriori;
Uint32 numSysPar = configParms->numSysPar;
Uint32 numExt = configParms->numExt;
Uint32 numHd = configParms->numHd;
Uint32 subFrameLen = configParms->subFrameLen;
Uint32 prologSize = configParms->prologSize;
Uint32 frameLen = configParms->frameLen;

    if (configParms->rate == 4){
      numSysPar = TCP_ceil((subFrameLen * 10), 4);
    } /* end if rate == 4 */
    else { /* if rate != 4 */
      numSysPar = TCP_ceil(subFrameLen, 1);
    } /* end if rate != 4 */
	
	numApriori = TCP_ceil(subFrameLen, 2);
	numExt = TCP_ceil(subFrameLen - 2 * prologSize, 2);
	numHd = TCP_ceil(frameLen, 5);

    configParms->numApriori = numApriori;
    configParms->numSysPar  = numSysPar;
    configParms->numExt     = numExt;
    configParms->numHd      = numHd;
    configParms->numInter   = 0;

} /* end TCP_calcCountsSP() */

/*----------------------------------------------------------------------------*/
#ifdef NO_OPT

void TCP_interleaveExt(TCP_ExtrinsicData *restrict aprioriMap2, 
                              const TCP_ExtrinsicData *restrict extrinsicsMap1, 
                              const Uint16 *restrict interleaverTable, 
                              Uint32  numExt)
{   
Uint32 i;
    
    /* interleaving extrinsics */
    for (i=0; i<numExt; i++)
    {
      aprioriMap2[i]=*(extrinsicsMap1 + interleaverTable[i]);
    }
} /* end TCP_interleaveExt() */

/*----------------------------------------------------------------------------*/

void TCP_deinterleaveExt(TCP_ExtrinsicData *restrict aprioriMap1, 
                                const TCP_ExtrinsicData *restrict extrinsicsMap2, 
                                const Uint16 *restrict interleaverTable, 
                                 Uint32  numExt)
{
Uint32 i;
    
    /* deinterleave extrinsics */
    for (i=0; i<numExt; i++)
    {
      aprioriMap1[interleaverTable[i]]=*(extrinsicsMap2 + i);
    }
} /* end TCP_deinterleaveExt() */

/*----------------------------------------------------------------------------*/

void TCP_calculateHd(const TCP_ExtrinsicData *restrict extrinsicsMap1, 
                     const TCP_ExtrinsicData *restrict apriori, 
                     const TCP_UserData *restrict channel_data, 
                     Uint32 *restrict hardDecisions,
                     Uint16 numExt,
                     Uint8  rate)
{
Uint32 i;
Int32  extInt;
Int32  apInt;
Int32  inputInt;
Int32  softDecision;
Uint32 wordCount = 0;
Uint32 hdCount = 0;
    
    hardDecisions[0] = 0;
    
    for (i = 0; i < numExt; i++) 
    { 
      if((extrinsicsMap1[i]<<1)&0x80) 
        extInt = (extrinsicsMap1[i]<<1) | 0xffffff00; 
      else 
        extInt = (Int32)(extrinsicsMap1[i]<<1); 
             
      if((apriori[i]<<1)&0x80) 
        apInt = (apriori[i]<<1) | 0xffffff00; 
      else 
        apInt = (Int32)(apriori[i]<<1); 
  
      inputInt = (Int32)channel_data[i*rate];
          
      if(inputInt & 0x80) inputInt |= 0xFFFFFF00;
                   
      softDecision = inputInt + extInt + apInt; 
              
      if(hdCount == 32)
        {    
          hdCount = 0;
          wordCount++;
          hardDecisions[wordCount] = 0;
        }
          
      if((softDecision & 0x200) == 0)
        hardDecisions[wordCount] |= 1 << hdCount; 
         
      hdCount++; 
          
    } 
} /* end TCP_calculateHd() */

/*----------------------------------------------------------------------------*/

void TCP_demuxInput(Uint32 rate,
                        Uint32 frameLen,
                        const TCP_UserData *restrict input,
                        const Uint16 *restrict interleaver,
                        TCP_ExtrinsicData *restrict nonInterleaved,
                        TCP_ExtrinsicData *restrict interleaved)
{
Int32 i,j,k;
Uint32 tmp0, tmp1;
Uint32* output0 = (Uint32 *)nonInterleaved;
Uint32* output1 = (Uint32 *)interleaved;

/*
'interleaver' is a buffer of frameLen 16-bit indices
'input' is a buffer of frameLen * rate 8-bit elements
'output[2]' is the 2 de-multiplexed buffers :
            .out[0] buffer which is the TCP input buffer for LOG-MAP 1 decoding
            .out[1] buffer which is the TCP input buffer for LOG-MAP 2 decoding
*/
    
    switch(rate)
    {
      case 2:
/*
rate 1/2: 
input are taken by set of 8 symbols
input[...] :
00000000h    X0
00000001h    A0
00000002h    X1
00000003h    A'1
00000004h    X2
00000005h    A2
00000006h    X3
00000007h    A'3

Output are created by set of 8 symbols ordered as follow:
output[0][...]=
00000000h    X0
00000001h    X1
00000002h    X2
00000003h    X3
00000004h    A0
00000005h    A2
00000006h    0
00000007h    0

output[1][...]=
00000000h    X'0
00000001h    X'1
00000002h    X'2
00000003h    X'3
00000004h    A'1
00000005h    A'3
00000006h    0
00000007h    0
*/
      j=0;
      k=0;
        
      for (i=0; i < frameLen*rate; i+=8)
      {
        /* X0 */
        tmp0=input[i]; 
        /* X1 */
        tmp0|=input[i+2]<<8;
        /* X2 */
        tmp0|=input[i+4]<<16;
        /* X3 */
        tmp0|=input[i+6]<<24;
        /* tmp0 contains systematics X3 X2 X1 X0 */
        output0[j] = tmp0;

        /* A0 */
        tmp0=input[i+1];
        /* A2 */
        tmp0|=input[i+5]<<8;
        /* tmp0 contains parities 0 0 A2 A0*/
        output0[j+1] = tmp0;

        /* X'0 */
        tmp1=input[interleaver[k]*rate];
        /* X'1 */
        tmp1|=input[interleaver[k+1]*rate]<<8;
        /* X'2 */
        tmp1|=input[interleaver[k+2]*rate]<<16;
        /* X'3 */
        tmp1|=input[interleaver[k+3]*rate]<<24;
        /* tmp1 contains interleaved systematics X'3 X'2 X'1 X'0*/
        output1[j] = tmp1;

        k+=4;
        /* A'1 */
        tmp1=input[i+3];    
        /* A'3 */
        tmp1|=input[i+7]<<8;
        /* tmp1 contains interleaved parities 0 0 A'3 A'1 */
        output1[j+1] = tmp1;
       
        j+=2;
      }  
     
      break;
     
     
      case 3:
/*
rate 1/3:  
input are taken by set of 12 symbols
input[...] :
00000000h    X0
00000001h    A0
00000002h    A'0
00000003h    X1
00000004h    A1
00000005h    A'1
00000006h    X2
00000007h    A2
00000008h    A'2
00000009h    X3
0000000Ah    A3
0000000Bh    A'3

Output are created by set of 8 symbols ordered as follow:
output[0][...]=
00000000h    X0
00000001h    X1
00000002h    X2
00000003h    X3
00000004h    A0
00000005h    A1
00000006h    A2
00000007h    A3

output[1][...]=
00000000h    X'0
00000001h    X'1
00000002h    X'2
00000003h    X'3
00000004h    A'0
00000005h    A'1
00000006h    A'2
00000007h    A'3

*/
      j = 0;
      k=0;
     
      for (i=0;i<frameLen*rate;i+=12)
      {
        /* X0 */
        tmp0=input[i];
        /* X1 */
        tmp0|=input[i+3]<<8;
        /* X2 */
        tmp0|=input[i+6]<<16;
        /* X3 */
        tmp0|=input[i+9]<<24;
        /* tmp0 contains systematics X3 X2 X1 X0 */
        output0[j] = tmp0;

        /* A0 */
        tmp0=input[i+1];
        /* A1 */
        tmp0|=input[i+4]<<8;
        /* A2 */
        tmp0|=input[i+7]<<16;
        /* A3 */
        tmp0|=input[i+10]<<24;
        /* tmp0 contains parities A3 A2 A1 A0*/
        output0[j+1] = tmp0;
    
        /* X'0 */
        tmp1=input[interleaver[k]*rate];
        /* X'1 */
        tmp1|=input[interleaver[k+1]*rate]<<8;
        /* X'2 */
        tmp1|=input[interleaver[k+2]*rate]<<16;
        /* X'3 */
        tmp1|=input[interleaver[k+3]*rate]<<24;
        /* tmp1 contains interleaved systematics X'3 X'2 X'1 X'0 */
        output1[j] = tmp1;
    
        k+=4;

        /* A'0 */
        tmp1=input[i+2];
        /* A'1 */
        tmp1|=input[i+5]<<8;
        /* A'2 */
        tmp1|=input[i+8]<<16;
        /* A'3 */
        tmp1|=input[i+11]<<24;
        /* tmp1 contains interleaved parities A'3 A'2 A'1 A'0 */
        output1[j+1] = tmp1;
        
        j+=2;
      }  
     
      break;
     
     
      case 4:
/*
rate 1/4:
input are taken by set of 32 symbols   
input[...] :
00000000h    X0
00000001h    A0
00000002h    B0
00000003h    B'0
00000004h    X1
00000005h    A1
00000006h    A'1
00000007h    B'1
00000008h    X2
00000009h    A2
0000000Ah    B2
0000000Bh    B'2
0000000Ch    X3
0000000Dh    A3
0000000Eh    A'3
0000000Fh    B'3

Output are created by set of 10 symbols ordered as follow:
output[0][...]=
00000000h    X0
00000001h    X1
00000002h    X2
00000003h    X3
00000004h    A0
00000005h    A1
00000006h    A2
00000007h    A3
00000008h    B0
00000009h    B2

output[1][...]=
00000000h    X'0
00000001h    X'1
00000002h    X'2
00000003h    X'3
00000004h    A'1
00000005h    A'3
00000006h    B'0
00000007h    B'1
00000008h    B'2
00000009h    B'3

*/
      j = 0;
      k=0;
     
      for (i=0;i<frameLen*rate;i+=32)
      {
        /* X0 */
        tmp0=input[i];
        /* X1 */
        tmp0|=input[i+4]<<8;
        /* X2 */
        tmp0|=input[i+8]<<16;
        /* X3 */
        tmp0|=input[i+12]<<24;
        /* tmp0 contains systematics X3 X2 X1 X0 */
        output0[j] = tmp0;
        
        /* A0 */
        tmp0=input[i+1];
        /* A1 */
        tmp0|=input[i+5]<<8;
        /* A2 */
        tmp0|=input[i+9]<<16;
        /* A3 */
        tmp0|=input[i+13]<<24;
        /* tmp0 contains parities A3 A2 A1 A0 */
        output0[j+1] = tmp0;
        
        /* B0 */
        tmp0=input[i+2];
        /* B2 */
        tmp0|=input[i+10]<<8;
        /* X4 */
        tmp0|=input[i+16]<<16;  
        /* X5 */
        tmp0|=input[i+4+16]<<24;
        /* tmp0 contains systematics and parities X5 X4 B2 B0 */
        output0[j+2] = tmp0;
        
        /* X6 */
        tmp0=input[i+8+16];
        /* X7 */
        tmp0|=input[i+12+16]<<8;
        /* A4 */
        tmp0|=input[i+1+16]<<16;
        /* A5 */
        tmp0|=input[i+5+16]<<24;
        /* temp 0 contains systematics and parities A5 A4 X7 X6 */
        output0[j+3] = tmp0;
        
        /* A6 */
        tmp0=input[i+9+16];
        /* A7 */
        tmp0|=input[i+13+16]<<8;
        /* B4 */
        tmp0|=input[i+2+16]<<16;
        /* B6 */
        tmp0|=input[i+10+16]<<24;
        /* tmp0 contains parities B6 B4 A7 A6 */
        output0[j+4] = tmp0;

        /* X'0 */
        tmp1=input[interleaver[k]*rate];
        /* X'1 */
        tmp1|=input[interleaver[k+1]*rate]<<8;
        /* X'2 */
        tmp1|=input[interleaver[k+2]*rate]<<16;
        /* X'3 */
        tmp1|=input[interleaver[k+3]*rate]<<24;
        /* tmp1 contains systematics X'3 X'2 X'1 X'0 */
        output1[j] = tmp1;
        
        k+=4;

        /* B'0 */
        tmp1=input[i+3];
        /* B'1 */
        tmp1|=input[i+7]<<8;
        /* B'2 */
        tmp1|=input[i+11]<<16;
        /* B'3 */
        tmp1|=input[i+15]<<24;
        /* tmp1 contains parities B'3 B'2 B'1 B'0 */
        output1[j+1] = tmp1;
        
        /* A'1 */
        tmp1=input[i+6];
        /* A'3 */
        tmp1|=input[i+14]<<8;
        /* X'4 */
        tmp1|=input[interleaver[k]*rate]<<16;
        /* X'5 */
        tmp1|=input[interleaver[k+1]*rate]<<24;
        /* tmp1 contains parities X'5 X'4 A'3 A'1 */
        output1[j+2] = tmp1;
        
        k+=2;

        /* X'6 */
        tmp1=input[interleaver[k]*rate];
        /* X'7 */
        tmp1|=input[interleaver[k+1]*rate]<<8;
        /* B'4 */
        tmp1|=input[i+3+16]<<16;
        /* B'5 */
        tmp1|=input[i+7+16]<<24;
        /* tmp1 contains parities B'5 B'4 X'7 X'6 */
        output1[j+3] = tmp1;

        k+=2;

        /* B'6 */
        tmp1=input[i+11+16];
        /* B'7 */
        tmp1|=input[i+15+16]<<8;
        /* A'5 */
        tmp1|=input[i+6+16]<<16;
        /* A'7 */
        tmp1|=input[i+14+16]<<24;
        /* tmp1 contains parities A'7 A'5 B'7 B'6 */
        output1[j+4] = tmp1;

        j+=5;
      }
         
    /*break;
    default:
    return(-1);*/
    } 

  /*return(0);*/

} /* end TCP_demuxInput() */

#else
 #ifdef OPT_LE

/******************************************************************************\
 * TCP_demuxInput() function definition for SP mode
 * Optimized for LITTLE ENDIAN data
 * interleaver table will be read in half-words
 * input channel data in words or double words depending on the rate
 * apriori will be written in words
 * demux buffers will be written as double-words
 * demux buffer size in bytes must therefore be a multiple of 8
 *
 * Coding Rate    Code Size    Number of Cycles
 *     1/2           2448            14713
 *     1/3           2448            25037
 *     1/4           2448            35214
 *
\******************************************************************************/
void TCP_demuxInput(Uint32 rate,
                        Uint32 frameLen,
                        const TCP_UserData *restrict input,
                        const Uint16 *restrict interleaver,
                        TCP_ExtrinsicData *restrict nonInterleaved,
                        TCP_ExtrinsicData *restrict interleaved)                      
{
 
Int32 i,k;
double* out0 = (double *)nonInterleaved;
double* out1 = (double *)interleaved;
Uint8 *out0rate4=nonInterleaved;
Uint8 *out1rate4=interleaved;
const Uint8 *inputForInterleave=input;

 
    double ap3x3a2x2ap1x1a0x0, z00a2a0x3x2x1x0,z00ap3ap1xp3xp2xp1xp0;
    Uint32 ap3x3a2x2,ap1x1a0x0,x3x2x1x0,ap3a2ap1a0,z00a2a0,z00ap3ap1,xp3xp2xp1xp0;
      
    Uint32  x1ap0a0x0, x2ap1a1x1, x3ap2a2x2, yap3a3x3, a1x1a0x0, a3x3a2x2,  \
            x2ap1x1ap0, yap3x3ap2, a3a2a1a0, ap3ap2ap1ap0;
    double a3a2a1a0x3x2x1x0,ap3ap2ap1ap0xp3xp2xp1xp0;
 
    Uint32 bp1ap1bp0b0, bp3ap3bp2b2, bp3bp2bp1bp0, \
               bp2b2bp0b0, bp3ap3bp1ap1, ap3ap1b2b0, bp1ap1a1x1, bp0b0a0x0, \
               bp3ap3a3x3, bp2b2a2x2;
    double bp3bp2bp1bp0xp3xp2xp1xp0;
    double bp1ap1a1x1bp0b0a0x0, bp3ap3a3x3bp2b2a2x2;
 

/*
'interleaver' is a buffer of frameLen 16-bit indices
'input' is a buffer of frameLen * rate 8-bit elements
'interleaved' and 'nonInterleaved' are the 2 de-multiplexed buffers :
                  .nonInterleaved buffer which is the TCP input buffer for LOG-MAP 1 decoding
                  .interleaved buffer which is the TCP input buffer for LOG-MAP 2 decoding
*/

      
    switch(rate)
    {
            
      /*
      rate 1/2:                          
      input are taken by set of 8 symbols
      Output are created by set of 8 symbols ordered as follow:
            
      input[...] :          out0[...]=                   out1[...]=
      00000000h      X0        00000000h      X0          00000000h      X'0
      00000001h      A0        00000001h      X1          00000001h      X'1
      00000002h      X1        00000002h      X2          00000002h      X'2
      00000003h      A'1       00000003h      X3          00000003h      X'3
      00000004h      X2        00000004h      A0          00000004h      A'1
      00000005h      A2        00000005h      A2          00000005h      A'3
      00000006h      X3        00000006h      0           00000006h      0
      00000007h      A'3       00000007h      0           00000007h      0
      */

      case 2:
        k=0;
            
        for (i=0;i<frameLen*rate;i+=8)
        {
          ap3x3a2x2ap1x1a0x0 = _memd8((void *)input); 
          input+=8;
            
          ap3x3a2x2   = _hi(ap3x3a2x2ap1x1a0x0);
          ap1x1a0x0   = _lo(ap3x3a2x2ap1x1a0x0);
            
          x3x2x1x0    = _packl4(ap3x3a2x2,ap1x1a0x0);
          ap3a2ap1a0  = _packh4(ap3x3a2x2,ap1x1a0x0);
            
          z00a2a0      = _packl4(0,ap3a2ap1a0);
          z00ap3ap1    = _packh4(0,ap3a2ap1a0);

          /* X'0 */
          xp3xp2xp1xp0 = inputForInterleave[interleaver[k]*rate];
          /* X'1 */
          xp3xp2xp1xp0 |=inputForInterleave[interleaver[k+1]*rate]<<8;
          /* X'2 */
          xp3xp2xp1xp0 |=inputForInterleave[interleaver[k+2]*rate]<<16;
          /* X'3 */
          xp3xp2xp1xp0 |=inputForInterleave[interleaver[k+3]*rate]<<24;
          k+=4;

          z00a2a0x3x2x1x0       = _itod(z00a2a0,x3x2x1x0);
          z00ap3ap1xp3xp2xp1xp0 = _itod(z00ap3ap1,xp3xp2xp1xp0);
            
          *out0++ = z00a2a0x3x2x1x0;         
          *out1++ = z00ap3ap1xp3xp2xp1xp0;
            
        }
       
        break;


      /*
      rate 1/3:  
      input are taken by set of 12 symbols
      Output are created by set of 8 symbols ordered as follow:
            
      input[...] :           out0[...]=        out1[...]=
      00000000h      X0         00000000h      X0        00000000h      X'0
      00000001h      A0         00000001h      X1        00000001h      X'1
      00000002h      A'0        00000002h      X2        00000002h      X'2
      00000003h      X1         00000003h      X3        00000003h      X'3
      00000004h      A1         00000004h      A0        00000004h      A'0
      00000005h      A'1        00000005h      A1        00000005h      A'1
      00000006h      X2         00000006h      A2        00000006h      A'2
      00000007h      A2         00000007h      A3        00000007h      A'3
      00000008h      A'2                               
      00000009h      X3
      0000000Ah      A3
      0000000Bh      A'3
            
      */
      case 3:
            
        k=0;
       
        for (i=0;i<frameLen*rate;i+=12)
        {
          x1ap0a0x0  = _mem4((void *)input); input+=3;
          x2ap1a1x1  = _mem4((void *)input); input+=3;
          x3ap2a2x2  = _mem4((void *)input); input+=3;
          yap3a3x3   = _mem4((void *)input); input+=3;
                          
          a1x1a0x0   = _pack2(x2ap1a1x1,x1ap0a0x0);
          a3x3a2x2   = _pack2(yap3a3x3,x3ap2a2x2);
          x2ap1x1ap0 = _packh2(x2ap1a1x1,x1ap0a0x0);
          yap3x3ap2  = _packh2(yap3a3x3,x3ap2a2x2);
                          
          x3x2x1x0   = _packl4(a3x3a2x2,a1x1a0x0);
          a3a2a1a0   = _packh4(a3x3a2x2,a1x1a0x0);
             
          ap3ap2ap1ap0 = _packl4(yap3x3ap2,x2ap1x1ap0);
             
          a3a2a1a0x3x2x1x0 = _itod(a3a2a1a0,x3x2x1x0);

          // X'0 
          xp3xp2xp1xp0=inputForInterleave[interleaver[k]*rate];
          // X'1 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+1]*rate]<<8;
          // X'2 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+2]*rate]<<16;
          // X'3 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+3]*rate]<<24;
          k+=4;
            
          ap3ap2ap1ap0xp3xp2xp1xp0 = _itod(ap3ap2ap1ap0,xp3xp2xp1xp0);
            
          *out0++ = a3a2a1a0x3x2x1x0;         
          *out1++ = ap3ap2ap1ap0xp3xp2xp1xp0;
       
        }
        break;

      /*
      rate 1/4:
      input are taken by set of 32 symbols   
      Output are created by set of 10 symbols ordered as follow:
            
      input[...] :            out0[...]=        out1[...]=
      00000000h      X0          00000000h      X0         00000000h      X'0
      00000001h      A0          00000001h      X1         00000001h      X'1
      00000002h      B0          00000002h      X2         00000002h      X'2
      00000003h      B'0         00000003h      X3         00000003h      X'3
      00000004h      X1          00000004h      A0         00000004h      B'0
      00000005h      A1          00000005h      A1         00000005h      B'1
      00000006h      A'1         00000006h      A2         00000006h      B'2
      00000007h      B'1         00000007h      A3         00000007h      B'3
      00000008h      X2          00000008h      B0         00000008h      A'1
      00000009h      A2          00000009h      B2         00000009h      A'3
      0000000Ah      B2                                              
      0000000Bh      B'2                                             
      0000000Ch      X3
      0000000Dh      A3
      0000000Eh      A'3
      0000000Fh      B'3
                  
      */        
      case 4:

        k=0;
             
      
        for (i=0;i<frameLen*rate;i+=16)
        {
      
          bp1ap1a1x1bp0b0a0x0 = _memd8((void *)input); input+=8;
          bp3ap3a3x3bp2b2a2x2 = _memd8((void *)input); input+=8;
                         
          bp1ap1a1x1 = _hi(bp1ap1a1x1bp0b0a0x0);
          bp0b0a0x0  = _lo(bp1ap1a1x1bp0b0a0x0);
                         
          bp3ap3a3x3 = _hi(bp3ap3a3x3bp2b2a2x2); 
          bp2b2a2x2  = _lo(bp3ap3a3x3bp2b2a2x2);
                         
          a1x1a0x0 = _pack2(bp1ap1a1x1,bp0b0a0x0);
          a3x3a2x2 = _pack2(bp3ap3a3x3,bp2b2a2x2);
          a3a2a1a0 = _packh4(a3x3a2x2,a1x1a0x0);
          x3x2x1x0 = _packl4(a3x3a2x2,a1x1a0x0);
                  
          a3a2a1a0x3x2x1x0 = _itod(a3a2a1a0,x3x2x1x0);
                  
          bp1ap1bp0b0  = _packh2(bp1ap1a1x1,bp0b0a0x0);
          bp3ap3bp2b2  = _packh2(bp3ap3a3x3,bp2b2a2x2);
          bp3bp2bp1bp0 = _packh4(bp3ap3bp2b2,bp1ap1bp0b0);
                  
          // X'0 
          xp3xp2xp1xp0=inputForInterleave[interleaver[k]*rate];
          // X'1 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+1]*rate]<<8;
          // X'2 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+2]*rate]<<16;
          // X'3 
          xp3xp2xp1xp0|=inputForInterleave[interleaver[k+3]*rate]<<24;
          // xp3xp2xp1xp0 contains systematics X'3 X'2 X'1 X'0 
          k+=4;
             
          bp3bp2bp1bp0xp3xp2xp1xp0 = _itod(bp3bp2bp1bp0,xp3xp2xp1xp0);
             
          //_memd8((void *)out0)=a3a2a1a0x3x2x1x0; out0+=10;
          //_memd8((void *)out1)=bp3bp2bp1bp0xp3xp2xp1xp0; out0+=10;
          _memd8((void *)out0rate4)=a3a2a1a0x3x2x1x0; 
          out0rate4+=8;
          _memd8((void *)out1rate4)=bp3bp2bp1bp0xp3xp2xp1xp0; 
          out1rate4+=8;
                         
                         
          bp2b2bp0b0 = _pack2(bp3ap3bp2b2,bp1ap1bp0b0);
          bp3ap3bp1ap1 = _packh2(bp3ap3bp2b2,bp1ap1bp0b0);
          ap3ap1b2b0 = _packl4(bp3ap3bp1ap1,bp2b2bp0b0);
             
             
          *(Uint16 *)out0rate4=(Uint16)ap3ap1b2b0;             
          out0rate4+=2;
          *(Uint16 *)out1rate4=(Uint16)(ap3ap1b2b0>>16);       
          out1rate4+=2;
             
             
        }
        break;
  
  
//      default:
//        return(-1);
      }

//      return(0);

} /* end TCP_demuxInput() */


/******************************************************************************\
 * TCP_interleaveExt() function definition for extrinsics in SP mode
 * Optimized for LITTLE ENDIAN data
 * interleaver table will be read in double-words - dummy indexes must be put
 * at the end of the interleaver table to make sure extra data must not
 * overwrite valid data
 * extrinsics will be read in bytes
 * apriori will be written in words 
 * will interleave ceil(frameLen/4) data
 * apriori buffer size in bytes must therefore be a multiple of 4
 *
 * Frame Size   Code Size   Incl. Total   Max Cycles   Min Cycles   Ave Cycles
 *   6138          436           3           7296         7214         7260
\******************************************************************************/
void TCP_interleaveExt(TCP_ExtrinsicData *restrict aprioriMap2, 
                              const TCP_ExtrinsicData *restrict extrinsicsMap1, 
                              const Uint16 *restrict interleaverTable, 
                              Uint32  numExt)
{
Uint32 i;  
const double *interleaver=(const double*)interleaverTable;
Uint32 *out=(Uint32 *)aprioriMap2;
Uint32 counter = TCP_ceil(numExt,2);
Uint32 temp,temp1;
      
    for (i=0;i<counter;i++)
    {
      temp   =extrinsicsMap1[_lo(interleaver[i])&0xffff];
      temp  |=_mpyu(extrinsicsMap1[_lo(interleaver[i])>>16],256);
      temp1  =extrinsicsMap1[_hi(interleaver[i])&0xffff];
      temp1 |=(extrinsicsMap1[_hi(interleaver[i])>>16]<<8);
      out[i] =_pack2(temp1,temp);
    } /* end for i */
      
} /* end TCP_interleaveExt() */
                         

/******************************************************************************\
 * TCP_deinterleaveExt() function definition for extrinsics in SP mode
 * Optimized for LITTLE ENDIAN data
 * tested for little endian mode - need modification for big endian
 * interleaver table will in double-words
 * extrinsics will be read in words
 * apriori will be written in bytes 
 * will deinterleave ceil(frameLen/4) data
 * apriori buffer size in bytes must therefore be a multiple of 4
 *
 * Frame Size   Code Size   Incl. Total   Max Cycles   Min Cycles   Ave Cycles
 *    6138         384           3           8628         8495         8557
\******************************************************************************/
void TCP_deinterleaveExt(TCP_ExtrinsicData *restrict aprioriMap1, 
                                const TCP_ExtrinsicData *restrict extrinsicsMap2, 
                                const Uint16 *restrict interleaverTable, 
                                 Uint32  numExt)
{
Uint32 i;  
const double *interleaver=(const double*)interleaverTable;
const Uint32 *in=(const Uint32 *)extrinsicsMap2;
Uint32 counter = TCP_ceil(numExt,2);
Uint32 temp,temp1,temp2;
Uint32 a,b,c,d;
      
    temp=in[0];
      
    for (i=0;i<counter;i++)
    {
            
      temp=in[i];
            
      temp1 = _lo(interleaver[i]);
      aprioriMap1[temp1&0xffff] = temp;
      a = _rotl(temp,24); ; 
      b = (temp1>>16)&0xFFFF; 
      aprioriMap1[b] = a;
            
      temp2 = _hi(interleaver[i]);
      aprioriMap1[temp2&0xffff] = temp>>16;
      c = _rotl(temp,8); ; 
      d =(temp2>>16)&0xFFFF; 
      aprioriMap1[d] = c;

    } /* end for i */
      
} /* end TCP_deinterleaveExt() */


/******************************************************************************\
 * TCP_calculateHd() function definition for HardDecisions in SP mode
 * Optimized for LITTLE ENDIAN data
 * tested for little endian mode - need modification for big endian/
 * extrinsics1 will be read in words
 * extrinsics2 will be read in words
 * systematics are read in bytes
 * hard decisions will be written in words
 * 
 * Frame Size   Code Size   Incl. Total   Max Cycles   Min Cycles   Ave Cycles
 *    6138         1256          3           25039        24373        24627
\******************************************************************************/
void TCP_calculateHd(const TCP_ExtrinsicData *restrict extrinsicsMap1, 
                     const TCP_ExtrinsicData *restrict apriori, 
                     const TCP_UserData *restrict channel_data, 
                     Uint32 *restrict hardDecisions,
                     Uint16 numExt,
                     Uint8  rate)
{
Uint32 i;
const Int32 *ext32MAP1=(const Int32*)extrinsicsMap1;
const Int32 *ext32MAP2=(const Int32*)apriori;
Uint32 nbWords;
Uint32 wordCounter=0;
Uint32 shift=0;
Int32 temp1,temp2,a,b,c,d;
Uint32 j=0;


    nbWords=TCP_ceil(numExt,2);
    hardDecisions[0]=0;

    for (i = 0; i < nbWords; i++) 
    { 

      temp1=ext32MAP1[i];
      temp2=ext32MAP2[i];
 
      a = ((0-_ext(channel_data[(j)*rate],24,24)-_ext(temp1,25,24)-_ext(temp2,25,24)));
          hardDecisions[wordCounter] |= ((a>>9)&0x1)<<(shift++);

      b = ((0-_ext(channel_data[(j+1)*rate],24,24)-_ext(temp1,17,24)-_ext(temp2,17,24)));
          hardDecisions[wordCounter] |= ((b>>9)&0x1)<<(shift++);

      c = ((0-_ext(channel_data[(j+2)*rate],24,24)-_ext(temp1,9,24)-_ext(temp2,9,24)));
          hardDecisions[wordCounter] |= ((c>>9)&0x1)<<(shift++);

      d = ((0-_ext(channel_data[(j+3)*rate],24,24)-_ext(temp1,1,24)-_ext(temp2,1,24)));
          hardDecisions[wordCounter] |= ((d>>9)&0x1)<<(shift++);

      j+=4;

      if ((shift & 0x1f) == 0)
      {
        hardDecisions[++wordCounter] = 0;
        shift = 0;                  
      } /* end if shift & 0x1f */

    } /* end for i */

} /* end TCP_calculateHd() */


 #endif /* OPT_LE */
#endif /* else if !NO_OPT */
 
/*******************************************************************************************\
*  TCP_init() Input parameters structure
*             set the register values from parameter structure init.
*             calls tailInitxxx functions for ic6-ic11 values
* 
\*******************************************************************************************/
/* void TCP_init(TCP_Init *Init) {

  Uint32 gie = IRQ_globalDisable();
  TCP_UserData  *restrict xabData =(Init->xabData);
  TCP_Params configParms; 
  TCP_BaseParams configBase={0,0,0,0,0,0,0,0}; 


    configParms.standard    = Init->standard;
    configParms.rate        = Init->rate;
    configParms.intFlag     = Init->intFlag;
    configParms.outParmFlag = Init->outParmFlag;
    configParms.frameLen    = Init->frameLen;
    configParms.prologSize  = Init->prologSize;
    configParms.maxIter     = Init->maxIter;
    configParms.snr         = Init->snr;

    configBase.standard = Init->standard;
    configBase.rate        = Init->rate;
    configBase.intFlag     = Init->intFlag;
    configBase.outParmFlag = Init->outParmFlag;
    configBase.frameLen    = Init->frameLen;
    configBase.prologSize  = Init->prologSize;
    configBase.maxIter     = Init->maxIter;
    configBase.snr         = Init->snr;


    TCP_genParams(&configBase, &configParms);

  
  if (configParms.mode) {  (configParms.mode) = (configParms.mode << 2) + (configParms.map); }

  
  TCP_RSET(IC0,TCP_IC0_RMK(Init->frameLen,Init->outParmFlag,Init->intFlag,Init->rate,configParms.mode));
  TCP_RSET(IC1,TCP_IC1_RMK(configParms.relLenLast,configParms.relLen,configParms.subFrameLen));
  TCP_RSET(IC2,TCP_IC2_RMK(Init->snr,Init->maxIter,configParms.numSubBlockLast,configParms.numSubBlock,Init->prologSize));
  TCP_RSET(IC3,TCP_IC3_RMK(configParms.numSysPar,configParms.numInter));
  TCP_RSET(IC4,TCP_IC4_RMK(configParms.numExt,configParms.numApriori));
  TCP_RSET(IC5,TCP_IC5_RMK(configParms.numHd));

  
  if (Init->standard == TCP_STANDARD_3GPP) {
          TCP_tailInit3GPP(configParms.mode, configParms.map, xabData);
 }  else  {
          TCP_tailInitIs2000(configParms.mode, configParms.map, Init->rate,xabData);
 } 

  IRQ_globalRestore(gie);

}*/

/****** functions set the ic6-ic11 registers based on parameter structure *****/

 void TCP_tailInit3GPP(TCP_Mode mode, TCP_Map map,TCP_UserData *restrict xabData) {
  register Uint32 tail1, tail2, tail3, tail4, tail5, tail6;
  Int32 gie;

  gie = IRQ_globalDisable();

  if ((mode == TCP_MODE_SP) && (map == TCP_MAP_MAP2)){
    /* tail1 = 0, x_t_2, x_t_1, x_t_0 */
    /* tail2 = 0, a_t_2, a_t_1, a_t_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0,   0  ,   0  ,   0   */
    /* tail5 = 0,   0  ,   0  ,   0   */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
    tail2 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
    tail3 = 0;
    tail4 = 0;
    tail5 = 0;
    tail6 = 0;
  } /* end if SPMAP2 */
  else if (mode == TCP_MODE_SP){
    /* tail1 = 0, xt_2, xt_1, xt_0 */
    /* tail2 = 0, at_2, at_1, at_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0,   0  ,   0  ,   0   */
    /* tail5 = 0,   0  ,   0  ,   0   */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
    tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
    tail3 = 0;
    tail4 = 0;
    tail5 = 0;
    tail6 = 0;
  } /* end if SPMAP1 */
  else { /* if (mode == TCP_MODE_SA) */
    /* tail1 = 0,  xt_2,  xt_1,  xt_0 */
    /* tail2 = 0,  at_2,  at_1,  at_0 */
    /* tail3 = 0,   0  ,   0  ,   0   */
    /* tail4 = 0, x_t_2, x_t_1, x_t_0 */
    /* tail5 = 0, a_t_2, a_t_1, a_t_0 */
    /* tail6 = 0,   0  ,   0  ,   0   */
    tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
    tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
    tail3 = 0;
    tail4 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
    tail5 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
    tail6 = 0;
  } /* end if SA */

  TCP_RSET(IC6,tail1);
  TCP_RSET(IC7,tail2);
  TCP_RSET(IC8,tail3);
  TCP_RSET(IC9,tail4);
  TCP_RSET(IC10,tail5);
  TCP_RSET(IC11,tail6);

  IRQ_globalRestore(gie);
}
 /* end TCP_tailInit3GPP */
/*----------------------------------------------------------------------------*/

void TCP_tailInitIs2000(TCP_Mode mode,TCP_Map map,TCP_Rate rate,TCP_UserData *restrict xabData) {

  Int32 gie;
  Uint32 tail1, tail2, tail3, tail4, tail5, tail6;

  gie = IRQ_globalDisable();

  if ((mode == TCP_MODE_SP) && (map == TCP_MAP_MAP2)) {
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail2 = 0, a_t_2, a_t_1, a_t_0 */
        tail1 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail2 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[15]), \
                                (xabData[12]), (xabData[9]));
        tail2 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       b_t_2     ,       b_t_1     ,       b_t_0      */
        /* tail3 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[20]), \
                                (xabData[16]), (xabData[12]));
        tail2 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        tail3 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch rate */
  } /* end if mode == TCP_MODE_SPMAP2 */
  /*--------------------------------------------------------------------------*/
  else if (mode == TCP_MODE_SP) { /*  && (map == TCP_MAP1) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, xt_2, xt_1, xt_0 */
        /* tail2 = 0, at_2, at_1, at_0 */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[6]), \
                                (xabData[3]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (xt_2 + _xt_2), (xt_1 + _xt_1), (xt_0 + _xt_0) */
        /* tail2 = 0,      at_2     ,      at_1     ,      at_0      */
        /* tail3 = 0,      bt_2     ,      bt_1     ,      bt_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[8]), \
                                (xabData[4]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SPMAP1 */
  /*--------------------------------------------------------------------------*/
  else { /* (mode == TCP_MODE_SA) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0,  xt_2,  xt_1,  xt_0 */
        /* tail2 = 0,  at_2,  at_1,  at_0 */
        /* tail3 = 0,   0  ,   0  ,   0   */
        /* tail4 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail5 = 0, a_t_2, a_t_1, a_t_0 */
        /* tail6 = 0,   0  ,   0  ,   0   */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail5 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,        at_2     ,        at_1     ,        at_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        tail1 = TCP_makeTailArgs(        0               , (xabData[6]), \
                                (xabData[3]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(        0                 , (xabData[15]), \
                                (xabData[12]), (xabData[9]));
        tail5 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,       at_2      ,       at_1      ,       at_0       */
        /* tail3 = 0,       bt_2      ,       bt_1      ,       bt_0       */
        /* tail4 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail5 = 0,      a_t_2      ,      a_t_1      ,      a_t_0       */
        /* tail6 = 0,      b_t_2      ,      b_t_1      ,      b_t_0       */
        tail1 = TCP_makeTailArgs(        0               , (xabData[8]), \
                                (xabData[4]), (xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = TCP_makeTailArgs(        0                 , (xabData[20]), \
                                (xabData[16]), (xabData[12]));
        tail5 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail6 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SA */
  /*--------------------------------------------------------------------------*/
  TCP_RSET(IC6,tail1);
  TCP_RSET(IC7,tail2);
  TCP_RSET(IC8,tail3);
  TCP_RSET(IC9,tail4);
  TCP_RSET(IC10,tail5);
  TCP_RSET(IC11,tail6);  


  IRQ_globalRestore(gie);
} 
/* end of TCP_tailInitIs2000() */

 
 
 
#endif /*TCP_SUPPORT*/

/******************************************************************************\
* End of csl_tcp.c
\******************************************************************************/

/*----------------------------------------------------------------------------*/
#if 0
void TCP_tailConfigIs2000(TCP_Mode mode,
        TCP_Map map,
        TCP_Rate rate,
        const TCP_UserData *restrict xabData,
        TCP_ConfigIc *restrict configIc) {

  Int32 gie;
  Uint32 tail1, tail2, tail3, tail4, tail5, tail6;

  gie = IRQ_globalDisable();

  if ((mode == TCP_MODE_SP) && (map == TCP_MAP_MAP2)) {
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail2 = 0, a_t_2, a_t_1, a_t_0 */
        tail1 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail2 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[16] + xabData[15]), \
                                (xabData[13] + xabData[12]), (xabData[10] + xabData[9]));
        tail2 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        /* tail3 = 0,       b_t_2     ,       b_t_1     ,       b_t_0      */
        tail1 = TCP_makeTailArgs(        0                 , (xabData[21] + xabData[20]), \
                                (xabData[17] + xabData[16]), (xabData[13] + xabData[12]));
        tail2 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail3 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch rate */
  } /* end if mode == TCP_MODE_SPMAP2 */
  /*--------------------------------------------------------------------------*/
  else if (mode == TCP_MODE_SP) { /*  && (map == TCP_MAP1) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0, xt_2, xt_1, xt_0 */
        /* tail2 = 0, at_2, at_1, at_0 */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[7] + xabData[6]), \
                                (xabData[4] + xabData[3]), (xabData[1] + xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, (xt_2 + _xt_2), (xt_1 + _xt_1), (xt_0 + _xt_0) */
        /* tail2 = 0,      at_2     ,      at_1     ,      at_0      */
        /* tail3 = 0,      bt_2     ,      bt_1     ,      bt_0      */
        tail1 = TCP_makeTailArgs(        0               , (xabData[9] + xabData[8]), \
                                (xabData[5] + xabData[4]), (xabData[1] + xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = 0;
        tail5 = 0;
        tail6 = 0;
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SPMAP1 */
  /*--------------------------------------------------------------------------*/
  else { /* (mode == TCP_MODE_SA) */
    switch (rate) {
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_2:
        /* tail1 = 0,  xt_2,  xt_1,  xt_0 */
        /* tail2 = 0,  at_2,  at_1,  at_0 */
        /* tail3 = 0,   0  ,   0  ,   0   */
        /* tail4 = 0, x_t_2, x_t_1, x_t_0 */
        /* tail5 = 0, a_t_2, a_t_1, a_t_0 */
        /* tail6 = 0,   0  ,   0  ,   0   */
        tail1 = TCP_makeTailArgs(0, xabData[4], xabData[2], xabData[0]);
        tail2 = TCP_makeTailArgs(0, xabData[5], xabData[3], xabData[1]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(0, xabData[10], xabData[8], xabData[6]);
        tail5 = TCP_makeTailArgs(0, xabData[11], xabData[9], xabData[7]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_3:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,        at_2     ,        at_1     ,        at_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        /* tail1 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail2 = 0,       a_t_2     ,       a_t_1     ,       a_t_0      */
        /* tail3 = 0,         0       ,         0       ,         0        */
        tail1 = TCP_makeTailArgs(        0               , (xabData[7] + xabData[6]), \
                                (xabData[4] + xabData[3]), (xabData[1] + xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[8], xabData[5], xabData[2]);
        tail3 = 0;
        tail4 = TCP_makeTailArgs(        0                 , (xabData[16] + xabData[15]), \
                                (xabData[13] + xabData[12]), (xabData[10] + xabData[9]));
        tail5 = TCP_makeTailArgs(0, xabData[17], xabData[14], xabData[11]);
        tail6 = 0;
        break;
    /*------------------------------------------------------------------------*/
      case TCP_RATE_1_4:
        /* tail1 = 0, ( xt_2 +  _xt_2), ( xt_1 +  _xt_1), ( xt_0 +  _xt_0) */
        /* tail2 = 0,       at_2      ,       at_1      ,       at_0       */
        /* tail3 = 0,       bt_2      ,       bt_1      ,       bt_0       */
        /* tail4 = 0, (x_t_2 + _x_t_2), (x_t_1 + _x_t_1), (x_t_0 + _x_t_0) */
        /* tail5 = 0,      a_t_2      ,      a_t_1      ,      a_t_0       */
        /* tail6 = 0,      b_t_2      ,      b_t_1      ,      b_t_0       */
        tail1 = TCP_makeTailArgs(        0               , (xabData[9] + xabData[8]), \
                                (xabData[5] + xabData[4]), (xabData[1] + xabData[0]));
        tail2 = TCP_makeTailArgs(0, xabData[10], xabData[6], xabData[2]);
        tail3 = TCP_makeTailArgs(0, xabData[11], xabData[7], xabData[3]);
        tail4 = TCP_makeTailArgs(        0                 , (xabData[21] + xabData[20]), \
                                (xabData[17] + xabData[16]), (xabData[13] + xabData[12]));
        tail5 = TCP_makeTailArgs(0, xabData[22], xabData[18], xabData[14]);
        tail6 = TCP_makeTailArgs(0, xabData[23], xabData[19], xabData[15]);
        break;
    } /* end switch (rate) */
  } /* end if mode == TCP_MODE_SA */
  /*--------------------------------------------------------------------------*/
  
  configIc->ic6  = tail1;
  configIc->ic7  = tail2;
  configIc->ic8  = tail3;
  configIc->ic9  = tail4;
  configIc->ic10 = tail5;
  configIc->ic11 = tail6;

  IRQ_globalRestore(gie);
} /* end TCP_tailConfigIs2000 */
#endif



