/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_timer.c
* DATE CREATED.. 06/20/1999 
* LAST MODIFIED. 07/24/2004 - Re-introduced BIOS dependency due to compatibility issues
*                Removed bios related items
*                02/12/2002 _TIMER_setBiosHandle() , TIMER_getBiosHandle()
*                12/18/2000
\******************************************************************************/
#define _TIMER_MOD_

#include <csl_timer.h>
#include <csl.h>


#if (TIMER_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/
#define DEVICE_TABLE_ENTRY(devNum) { \
  /* allocated */ FALSE, \
  /* eventId   */ (Uint32)IRQ_EVT_TINT##devNum,  \
  /* baseAddr  */ (volatile Uint32 *)_TIMER_BASE_DEV##devNum## \
}


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/

/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* global variable definitions
\******************************************************************************/
TIMER_Obj _TIMER_deviceTable[TIMER_DEVICE_CNT] = {
  DEVICE_TABLE_ENTRY(0),
  DEVICE_TABLE_ENTRY(1)
  #if (TIMER_DEVICE_CNT > 2)
    ,DEVICE_TABLE_ENTRY(2)
  #endif
};

TIMER_Handle _TIMER_hDev0 = (TIMER_Handle)(&_TIMER_deviceTable[0]);
TIMER_Handle _TIMER_hDev1 = (TIMER_Handle)(&_TIMER_deviceTable[1]);
TIMER_Handle _TIMER_hBios = INV;

/******************************************************************************\
* global function definitions
\******************************************************************************/
/*----------------------------------------------------------------------------*/
TIMER_Handle TIMER_getBiosHandle() {
      return _TIMER_hBios;
}

/*----------------------------------------------------------------------------*/
void TIMER_reset(TIMER_Handle hTimer) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hTimer == INV) {
    TIMER_reset(&_TIMER_deviceTable[0]);
    TIMER_reset(&_TIMER_deviceTable[1]);
    #if (TIMER_DEVICE_CNT > 2)
      TIMER_reset(&_TIMER_deviceTable[2]);
    #endif
  } else {
    TIMER_RSETH(hTimer,CTL,TIMER_CTL_DEFAULT);
    TIMER_RSETH(hTimer,PRD,TIMER_PRD_DEFAULT);
    TIMER_RSETH(hTimer,CNT,TIMER_CNT_DEFAULT);
    IRQ_reset(hTimer->eventId);
  }
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/
void TIMER_resetAll() {

  TIMER_reset(INV); 
}
/*----------------------------------------------------------------------------*/
TIMER_Handle TIMER_open(int devNum, Uint32 flags) {

  TIMER_Handle hTimer = INV;
  Uint32 dev,gie;

  gie = IRQ_globalDisable();

  if (devNum==-1) {
    for (dev=0; dev<TIMER_DEVICE_CNT; dev++) {
      if (!_TIMER_deviceTable[dev].allocated) {
        hTimer = (TIMER_Handle)&(_TIMER_deviceTable[dev]);
        break;
      }
    }
  } else if (!_TIMER_deviceTable[devNum].allocated) {
    hTimer = (TIMER_Handle)&(_TIMER_deviceTable[devNum]);
  }

  if (hTimer != INV) {
    hTimer->allocated = TRUE;
    if (flags & TIMER_OPEN_RESET) {
      TIMER_reset(hTimer);
    }
  }

  IRQ_globalRestore(gie);

  return hTimer;
}

/*----------------------------------------------------------------------------*/
void TIMER_close(TIMER_Handle hTimer) {
  
  Uint32 gie;

  gie = IRQ_globalDisable();
  hTimer->allocated = FALSE;
  TIMER_reset(hTimer);
  IRQ_globalRestore(gie);
}

/*----------------------------------------------------------------------------*/


#endif /* TIMER_SUPPORT */
/******************************************************************************\
* End of csl_timer.c
\******************************************************************************/

