/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_vcp.c
* DATE CREATED.. 04/09/2001 
* LAST MODIFIED. 04/09/2001
\******************************************************************************/
#define _VCP_MOD_
#include "csl_vcp.h"

#if (VCP_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/


/******************************************************************************\
* static macro declarations
\******************************************************************************/


/******************************************************************************\
* static typedef declarations
\******************************************************************************/


/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/


/******************************************************************************\
* static function definitions
\******************************************************************************/


/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/
void VCP_genParams(VCP_BaseParams *restrict baseParms, 
                   VCP_Params *restrict configParms){

VCP_Rate rate     = baseParms->rate;
Uint8  constLen   = baseParms->constLen;
Uint16 frameLen   = baseParms->frameLen;
Uint16 yamTh      = baseParms->yamTh;
Uint8  stateNum   = baseParms->stateNum;
Uint8  decision   = baseParms->decision;
Uint8  readFlag   = baseParms->readFlag;

Uint8        poly0    = 0;
Uint8        poly1    = 0;
Uint8        poly2    = 0;
Uint8        poly3    = 0;
Uint16       relLen   = 0;
Uint16       convDist = 0;
Uint16       maxSm    = 0x400; /* was 0xFA -changed to 0x400 on 01/08/2002 */
Uint16       minSm    = 0;
Uint8        bmBuffLen;
Uint8        decBuffLen;
Uint16       numBmFrames;
Uint16       numDecFrames;
Uint8        traceBack;

Uint16  polynomes[4];
Uint16  fMax     = 0;
Uint16  rcMax    = 0;
Uint16  multiple = 0;
Uint16  nsw      = 0;
Uint16  mask     = 0;
Uint16  length   = 0;
Uint8   i;

    if (rate == VCP_RATE_1_2){
      if (constLen == 9){
        polynomes[0] =   0561;  /*   1 + D^2 + D^3 + D^4 + D^8              */
        polynomes[1] =   0753;  /*   1 + D   + D^2 + D^3 + D^5 + D^7 + D^8  */
      } /* end if constLen == 9 */
      else if (constLen == 8){
        polynomes[0] =   0712;       /*  (712 = 111001010)   */
        polynomes[1] =   0476;       /*  (476 = 100111110)   */
      } /* end if constLen == 8 */
      else if (constLen == 7){
        polynomes[0] =   0554;       /*  (554 = 101101100)   */
        polynomes[1] =   0744;       /*  (744 = 111100100)   */
      } /* end if constLen == 7 */
      else if (constLen == 6){
        polynomes[0] =   0650;       /*  ( 65 = 110101000)   */
        polynomes[1] =   0570;       /*  ( 57 = 101111000)   */
      } /* end if constLen == 6 */
      else { /* (constLen == 5) */
        polynomes[0] =   0460;       /*  ( 46 = 100110000)   */
        polynomes[1] =   0720;       /*  ( 72 = 111010000)   */
      } /* end if constLen == 5 */
    } /* end if rate 1/2 */
    else { /* if rate 1/3 or 1/4 */  
      if (constLen == 9){
        polynomes[0] =   0557;       /*  (557 = 101101111)  */
        polynomes[1] =   0663;       /*  (663 = 110110011)  */
        polynomes[2] =   0711;       /*  (711 = 111001001)  */
        polynomes[3] =   0557;       /*  (557 = 101101111)  */
      } /* end if constLen == 9 */
      else if (constLen == 8){
        polynomes[0] =   0452;       /*  (452 = 100101010)   */
        polynomes[1] =   0662;       /*  (662 = 110110010)   */
        polynomes[2] =   0756;       /*  (756 = 111101110)   */
        polynomes[3] =   0452;       /*  (452 = 100101010)   */
      } /* end if constLen == 8 */
      else if (constLen == 7){
        polynomes[0] =   0544;       /*  (554 = 101101100)   */
        polynomes[1] =   0624;       /*  (624 = 110010100)   */
        polynomes[2] =   0764;       /*  (764 = 111110100)   */
        polynomes[3] =   0544;       /*  (554 = 101101100)   */
      } /* end if constLen == 7 */
      else if (constLen == 6){
        polynomes[0] =   0470;       /*  (47  = 100111000)   */
        polynomes[1] =   0530;       /*  (53  = 101011000)   */
        polynomes[2] =   0750;       /*  (75  = 111101000)   */
        polynomes[3] =   0470;       /*  (47  = 100111000)   */
      } /* end if constLen == 6 */
      else { /* (constLen == 5) */
        polynomes[0] =   0520;       /*  (52  = 101010000)   */
        polynomes[1] =   0660;       /*  (66  = 110110000)   */
        polynomes[2] =   0760;       /*  (76  = 111110000)   */
        polynomes[3] =   0520;       /*  (52  = 101010000)   */
      } /* end if constLen == 5 */
    } /* end if rate 1/3 or 1/4 */

    /* zero out any unneeded polynomials, based on the data rate */
    for (i = VCP_RATE_1_4; i > rate; i--){
      polynomes[i-1] = 0;
    }
    /* the polynome sent to the coprocessor has always the most
       significant bit masked */
    mask = 0x00FF;
    
    /* Polynomes coefficients */   
    poly0 = polynomes[0]&mask;
    poly1 = polynomes[1]&mask;
    poly2 = polynomes[2]&mask;
    poly3 = polynomes[3]&mask;

    /* Identify Fmax for a given constraint length and decision type */
    if (decision == VCP_DECISION_HARD){
      if (constLen == 9){
        fMax =  120;
        rcMax = 124;
        multiple = 4;
      } /* end if constLen == 9 */
      if (constLen == 8){
        fMax =  217;
        rcMax = 217;
        multiple = 7;
      } /* end if constLen == 8 */
      if (constLen == 7){
        fMax =  378;
        rcMax = 372;
        multiple = 6;
      } /* end if constLen == 7 */
      if (constLen == 6){
        fMax =  635;
        rcMax = 605;
        multiple = 5;
      } /* end if constLen == 6 */
      if (constLen == 5){
        fMax = 2044;
        rcMax = 1020;
        multiple = 4;
      } /* end if constLen == 5 */
    } /* end if hard decision */
    else { /* if soft decision */
      if (constLen == 9){
        fMax     =  24;
        relLen   =   4;
        convDist =  24;
      } /* end if constLen == 9 */
      if (constLen == 8){
        fMax     =  49;
        relLen   =  28;
        convDist =  21;
      } /* end if constLen == 8 */
      if (constLen == 7){
        fMax     =  90;
        relLen   =  60;
        convDist =  18;
      } /* end if constLen == 7 */
      if (constLen == 6){
        fMax     = 155;
        relLen   =  60;
        convDist =  15;
      } /* end if constLen == 6 */
      if (constLen == 5){
        fMax     = 508;
        relLen   =  60;
        convDist =  12;
      } /* end if constLen == 5 */
    } /* end if soft decision */
      
    if (frameLen > fMax) traceBack = VCP_TRACEBACK_MIXED;
    else traceBack = VCP_TRACEBACK_TAILED;
    
    if ((decision == VCP_DECISION_HARD) && (traceBack != VCP_TRACEBACK_TAILED)){
      convDist = 3 * (constLen - 1);
      nsw = VCP_normalCeil(frameLen, rcMax - convDist);
      relLen = multiple * VCP_normalCeil(frameLen, nsw * multiple);
    } /* end if mixed or convergent traceback modes */

    /* Setting number of symbol per XEVT and REVT depending on Rate
       (just multiple of 4/64)*/
  
    /* n. of symbols for XEVT always expressed as multiple of 4. Low 4 bits.*/
    bmBuffLen = 64 >> rate;             /* 16 for 1/2; 8 for 1/3; 4 for 1/4 */
    decBuffLen = 16;
    numDecFrames = 1;                   /* Default # frames                 */

    length = frameLen + constLen - 1;         /* calculate the frame length */
    numBmFrames = VCP_normalCeil(length, 4 * bmBuffLen);/* number of sub frames  */
    bmBuffLen = VCP_normalCeil(length, 4 * numBmFrames);/* optimal SYMX          */

    if (decision == VCP_DECISION_HARD){
      if (64 * decBuffLen < frameLen){
        numDecFrames = VCP_normalCeil(frameLen, 64 * decBuffLen);
        decBuffLen = VCP_normalCeil(frameLen, 64 * numDecFrames);
      } /* end if 64 * decBuffLen < frameLen */
      else { /* if frameLen < 64 * decBuffLen */
        numDecFrames = 1;
        decBuffLen = VCP_ceil(frameLen, 6);
      } /* end else if frameLen < 64 * decBuffLen */
    } /* end if Hard decisions  */
    else if (traceBack == VCP_TRACEBACK_TAILED) { /* Soft Dec and Tailed TB */
      if (decBuffLen < frameLen){
        numDecFrames = VCP_normalCeil(frameLen, 4 * decBuffLen);
        decBuffLen = VCP_normalCeil(frameLen, 4 * numDecFrames);
      } /* end if decBuffLen < frameLen */
      else { /* if frameLen < 64 * decBuffLen */
        numDecFrames = 1;
        decBuffLen = VCP_ceil(frameLen, 2);
      } /* end else if frameLen < 64 * decBuffLen */
    } /* end if Soft decisions and tailed traceback mode */
    else { /* Soft Dec and either convergent or mixed traceback */
      numDecFrames = VCP_normalCeil(frameLen, 4 * decBuffLen);
      decBuffLen = VCP_normalCeil(frameLen, 4 * numDecFrames);
    } /* end else if Soft decisions and convergent or mixed traceback */

    configParms->rate         = rate;
    configParms->constLen     = constLen;
    configParms->poly0        = poly0;
    configParms->poly1        = poly1;
    configParms->poly2        = poly2;
    configParms->poly3        = poly3;
    configParms->yamTh        = yamTh;
    configParms->frameLen     = frameLen;
    configParms->relLen       = relLen;
    configParms->convDist     = convDist;
    configParms->maxSm        = maxSm;
    configParms->minSm        = minSm;
    configParms->stateNum     = stateNum;
    configParms->bmBuffLen    = bmBuffLen;
    configParms->decBuffLen   = decBuffLen;
    configParms->traceBack    = traceBack;
    configParms->readFlag     = readFlag;
    configParms->decision     = decision;
    configParms->numBmFrames  = numBmFrames;
    configParms->numDecFrames = numDecFrames;

} /* end VCP_genParams() */


void VCP_genIc(VCP_Params *restrict configParms,
               VCP_ConfigIc *restrict configIc){

  Uint32 gie;
  Uint32 poly0      = configParms->poly0;
  Uint32 poly1      = configParms->poly1;
  Uint32 poly2      = configParms->poly2;
  Uint32 poly3      = configParms->poly3;
  Uint32 yamTh      = configParms->yamTh;
  Uint32 frameLen   = configParms->frameLen;
  Uint32 relLen     = configParms->relLen;
  Uint32 convDist   = configParms->convDist;
  Uint32 maxSm      = configParms->maxSm;
  Uint32 minSm      = configParms->minSm;
  Uint32 stateNum   = configParms->stateNum;
  Uint32 bmBuffLen  = configParms->bmBuffLen;
  Uint32 decBuffLen = configParms->decBuffLen;
  Uint32 traceBack  = configParms->traceBack;
  Uint32 readFlag   = configParms->readFlag;
  Uint32 decision   = configParms->decision;
  Uint32 ic0, ic1, ic2, ic3, ic4, ic5;

  gie = IRQ_globalDisable();

  ic0 = VCP_IC0_RMK(poly3,poly2,poly1,poly0);
  if (yamTh) ic1 = VCP_IC1_RMK(VCP_IC1_YAMEN_ENABLE, yamTh,VCP_IC1_ZERO_ZEROS);
  else ic1 = 0;
  ic2 = VCP_IC2_RMK(relLen,frameLen);
  ic3 = VCP_IC3_RMK(convDist);
  ic4 = VCP_IC4_RMK(minSm,maxSm);
  ic5 = VCP_IC5_RMK(decision,readFlag,traceBack,decBuffLen-1,bmBuffLen-1,stateNum);

  configIc->ic0 = ic0;
  configIc->ic1 = ic1;
  configIc->ic2 = ic2;
  configIc->ic3 = ic3;
  configIc->ic4 = ic4;
  configIc->ic5 = ic5;      
  

  IRQ_globalRestore(gie);
} /* end VCP_genIc */






/******************************************************************************\
* global variable definitions
\******************************************************************************/


/******************************************************************************\
* global function definitions
\******************************************************************************/


#endif /* VCP_SUPPORT */
/******************************************************************************\
* End of csl_vcp2.c
\******************************************************************************/


