/******************************************************************************\
*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
*                           All Rights Reserved
*------------------------------------------------------------------------------
* FILENAME...... csl_vp.c
* DATE CREATED.. 03/1/2002 
* LAST MODIFIED. 
* 
\******************************************************************************/
#define _VP_MOD_
#include <csl_vp.h>

#if (VP_SUPPORT)
/******************************************************************************\
*                         L O C A L   S E C T I O N
\******************************************************************************/

/******************************************************************************\
* static macro declarations
\******************************************************************************/

/*This should match the Vp_Obj structure in csl_vp.h*/
#if (CHIP_DM642)
 #define DEVICE_ENTRY(devNum) { \
  FALSE, \
  IRQ_EVT_VINT##devNum##, \
  (volatile Uint32*)_VP_BASE_PORT##devNum##, \
  _VP_YSRCA##devNum##_ADDR, \
  _VP_CBSRCA##devNum##_ADDR, \
  _VP_CRSRCA##devNum##_ADDR, \
  _VP_YDSTA##devNum##_ADDR, \
  _VP_CBDST##devNum##_ADDR, \
  _VP_CRDST##devNum##_ADDR, \
  _VP_YSRCB##devNum##_ADDR, \
  _VP_CBSRCB##devNum##_ADDR, \
  _VP_CRSRCB##devNum##_ADDR, \
  _VP_YDSTB##devNum##_ADDR \
 }
#else
 #define DEVICE_ENTRY(devNum) { \
  FALSE, \
  IRQ_EVT_VINT##devNum##, \
  (volatile Uint32*)_VP_BASE_PORT##devNum##, \
  _VP_YSRCA##devNum##_ADDR, \
  _VP_CBSRCA##devNum##_ADDR, \
  _VP_CRSRCA##devNum##_ADDR, \
  _VP_YDSTA##devNum##_ADDR, \
  _VP_CBDST##devNum##_ADDR, \
  _VP_CRDST##devNum##_ADDR, \
 }

#endif

/******************************************************************************\
* static typedef declarations
\******************************************************************************/

/******************************************************************************\
* static function declarations
\******************************************************************************/


/******************************************************************************\
* static variable definitions
\******************************************************************************/
static VP_Obj _VP_deviceTable[_VP_PORT_CNT] = {
  DEVICE_ENTRY(0)
  #if (_VP_PORT_CNT==2)
  ,DEVICE_ENTRY(1)
  #else
    #if (_VP_PORT_CNT==3)
    ,DEVICE_ENTRY(1)
    ,DEVICE_ENTRY(2)
    #endif
  #endif
};

/******************************************************************************\
* static function definitions
\******************************************************************************/

/******************************************************************************\
*                        G L O B A L   S E C T I O N
\******************************************************************************/

/******************************************************************************\
* global variable definitions
\******************************************************************************/


/******************************************************************************\
* global function definitions
\******************************************************************************/

/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
void VP_reset(VP_Handle hVp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hVp == INV) {
    VP_reset((VP_Handle)(&(_VP_deviceTable[0])));
    #if (_VP_PORT_CNT==2)
	  VP_reset((VP_Handle)(&(_VP_deviceTable[1])));
    #else
    #if (_VP_PORT_CNT==3)
      VP_reset((VP_Handle)(&(_VP_deviceTable[1])));
      VP_reset((VP_Handle)(&(_VP_deviceTable[2])));
    #endif
    #endif
  } else {

#if 1 

  /* Enable video port in PCR register if it is not */
  if(!VP_FGETH(hVp, PCR, PEREN))
  {
  	VP_FSETSH(hVp, PCR, PEREN, ENABLE);
  }

  VP_FSETSH(hVp,VPCTL,VPRST,RESET);

  /* Wait for reset to complete */
  while(VP_FGETH(hVp,VPCTL,VPRST));

#else
    VP_RSETH(hVp,PFUNC,VP_PFUNC_DEFAULT);
    VP_RSETH(hVp,PDIR,VP_PDIR_DEFAULT);
    VP_RSETH(hVp,PDOUT,VP_PDOUT_DEFAULT);
    VP_RSETH(hVp,PDSET,VP_PDSET_DEFAULT);
    VP_RSETH(hVp,PDCLR,VP_PDCLR_DEFAULT);
    VP_RSETH(hVp,PIEN,VP_PIEN_DEFAULT);
    VP_RSETH(hVp,PIPOL,VP_PIPOL_DEFAULT);
    VP_RSETH(hVp,PICLR,VP_PICLR_DEFAULT);
    VP_RSETH(hVp,VPCTL,VP_VPCTL_DEFAULT&0xFFFFBFFF);/* Reserved the VPHLT Default Value */
    VP_RSETH(hVp,VPIE,VP_VPIE_DEFAULT);
    VP_RSETH(hVp,VPIS,VP_VPIS_DEFAULT);
    VP_RSETH(hVp,VCACTL,VP_VCACTL_DEFAULT);
    VP_RSETH(hVp,VCASTRT1,VP_VCASTRT1_DEFAULT);
    VP_RSETH(hVp,VCASTOP1,VP_VCASTOP1_DEFAULT);
    VP_RSETH(hVp,VCASTRT2,VP_VCASTRT2_DEFAULT);
    VP_RSETH(hVp,VCASTOP2,VP_VCASTOP2_DEFAULT);
    VP_RSETH(hVp,VCAVINT,VP_VCAVINT_DEFAULT);
    VP_RSETH(hVp,VCATHRLD,VP_VCATHRLD_DEFAULT);
    VP_RSETH(hVp,VCAEVTCT,VP_VCAEVTCT_DEFAULT);
    VP_RSETH(hVp,VCBCTL,VP_VCBCTL_DEFAULT);
    VP_RSETH(hVp,VCBSTRT1,VP_VCBSTRT1_DEFAULT);
    VP_RSETH(hVp,VCBSTOP1,VP_VCBSTOP1_DEFAULT);
    VP_RSETH(hVp,VCBSTRT2,VP_VCBSTRT2_DEFAULT);
    VP_RSETH(hVp,VCBSTOP2,VP_VCBSTOP2_DEFAULT);
    VP_RSETH(hVp,VCBVINT,VP_VCBVINT_DEFAULT);
    VP_RSETH(hVp,VCBTHRLD,VP_VCBTHRLD_DEFAULT);
    VP_RSETH(hVp,VCBEVTCT,VP_VCBEVTCT_DEFAULT);
    VP_RSETH(hVp,TSICTL,VP_TSICTL_DEFAULT);
    VP_RSETH(hVp,TSICLKINITL,VP_TSICLKINITL_DEFAULT);
    VP_RSETH(hVp,TSICLKINITM,VP_TSICLKINITM_DEFAULT);
    VP_RSETH(hVp,TSISTCMPL,VP_TSISTCMPL_DEFAULT);
    VP_RSETH(hVp,TSISTCMPM,VP_TSISTCMPM_DEFAULT);
    VP_RSETH(hVp,TSISTMSKL,VP_TSISTMSKL_DEFAULT);
    VP_RSETH(hVp,TSISTMSKM,VP_TSISTMSKM_DEFAULT);
    VP_RSETH(hVp,TSITICKS,VP_TSITICKS_DEFAULT);
    VP_RSETH(hVp,VDCTL,VP_VDCTL_DEFAULT);
    VP_RSETH(hVp,VDFRMSZ,VP_VDFRMSZ_DEFAULT);
    VP_RSETH(hVp,VDHBLNK,VP_VDHBLNK_DEFAULT);
    VP_RSETH(hVp,VDVBLKS1,VP_VDVBLKS1_DEFAULT);
    VP_RSETH(hVp,VDVBLKE1,VP_VDVBLKE1_DEFAULT);
    VP_RSETH(hVp,VDVBLKS2,VP_VDVBLKS2_DEFAULT);
    VP_RSETH(hVp,VDVBLKE2,VP_VDVBLKE2_DEFAULT);
    VP_RSETH(hVp,VDIMGOFF1,VP_VDIMGOFF1_DEFAULT);
    VP_RSETH(hVp,VDIMGSZ1,VP_VDIMGSZ1_DEFAULT);
    VP_RSETH(hVp,VDIMGOFF2,VP_VDIMGOFF2_DEFAULT);
    VP_RSETH(hVp,VDIMGSZ2,VP_VDIMGSZ2_DEFAULT);
    VP_RSETH(hVp,VDFLDT1,VP_VDFLDT1_DEFAULT);
    VP_RSETH(hVp,VDFLDT2,VP_VDFLDT2_DEFAULT);
    VP_RSETH(hVp,VDTHRLD,VP_VDTHRLD_DEFAULT);
    VP_RSETH(hVp,VDHSYNC,VP_VDHSYNC_DEFAULT);
    VP_RSETH(hVp,VDVSYNS1,VP_VDVSYNS1_DEFAULT);
    VP_RSETH(hVp,VDVSYNE1,VP_VDVSYNE1_DEFAULT);
    VP_RSETH(hVp,VDVSYNS2,VP_VDVSYNS2_DEFAULT);
    VP_RSETH(hVp,VDVSYNE2,VP_VDVSYNE2_DEFAULT);
    VP_RSETH(hVp,VDRELOAD,VP_VDRELOAD_DEFAULT);
    VP_RSETH(hVp,VDDISPEVT,VP_VDDISPEVT_DEFAULT);
    VP_RSETH(hVp,VDCLIP,VP_VDCLIP_DEFAULT);
    VP_RSETH(hVp,VDVINT,VP_VDVINT_DEFAULT);
    VP_RSETH(hVp,VDVINT,VP_VDFBIT_DEFAULT);
    VP_RSETH(hVp,VDVINT,VP_VDVBIT1_DEFAULT);
    VP_RSETH(hVp,VDVINT,VP_VDVBIT2_DEFAULT);
#endif
	
	IRQ_reset(hVp->eventId);
  }                    
  IRQ_globalRestore(gie);
  return;              
}                      
/*----------------------------------------------------------------------------*/
void VP_resetAll() {

  VP_reset(INV);
}
/*----------------------------------------------------------------------------*/
void VP_resetCaptureChA(VP_Handle hVp) {

  Uint32 gie;

  gie = IRQ_globalDisable();

  VP_FSETSH(hVp,VCACTL,RSTCH,RESET);

  /* Wait for reset to complete */
  while(VP_FGETH(hVp, VCACTL, RSTCH));
  /* Clean up channel A related bit field in VPIE and VPIS */
  VP_FSETSH(hVp,VPIE,VIE,DISABLE);

  VP_FSETSH(hVp,VPIE,LFDA,DISABLE);
  VP_FSETSH(hVp,VPIE,SFDA,DISABLE);
  VP_FSETSH(hVp,VPIE,VINTA2,DISABLE);
  VP_FSETSH(hVp,VPIE,VINTA1,DISABLE);
  VP_FSETSH(hVp,VPIE,SERRA,DISABLE);
  VP_FSETSH(hVp,VPIE,CCMPA,DISABLE);
  VP_FSETSH(hVp,VPIE,COVRA,DISABLE);

  VP_FSETSH(hVp,VPIS,LFDA,CLEAR);
  VP_FSETSH(hVp,VPIS,SFDA,CLEAR);
  VP_FSETSH(hVp,VPIS,VINTA2,CLEAR);
  VP_FSETSH(hVp,VPIS,VINTA1,CLEAR);
  VP_FSETSH(hVp,VPIS,SERRA,CLEAR);
  VP_FSETSH(hVp,VPIS,CCMPA,CLEAR);
  VP_FSETSH(hVp,VPIS,COVRA,CLEAR);

  VP_FSETSH(hVp,VPIE,VIE,ENABLE);

  IRQ_reset(hVp->eventId);

  IRQ_globalRestore(gie);
  return;              
}
/*----------------------------------------------------------------------------*/
#if (CHIP_DM642)
 void VP_resetCaptureChB(VP_Handle hVp) {

  Uint32 gie;

  gie = IRQ_globalDisable();

  VP_FSETSH(hVp,VCBCTL,RSTCH,RESET);

  /* Wait for reset to complete */
  while(VP_FGETH(hVp, VCBCTL, RSTCH));

  /* Clean up channel B related bit field in VPIE and VPIS */
  VP_FSETSH(hVp,VPIE,VIE,DISABLE);

  VP_FSETSH(hVp,VPIE,LFDB,DISABLE);
  VP_FSETSH(hVp,VPIE,SFDB,DISABLE);
  VP_FSETSH(hVp,VPIE,VINTB2,DISABLE);
  VP_FSETSH(hVp,VPIE,VINTB1,DISABLE);
  VP_FSETSH(hVp,VPIE,SERRB,DISABLE);
  VP_FSETSH(hVp,VPIE,CCMPB,DISABLE);
  VP_FSETSH(hVp,VPIE,COVRB,DISABLE);

  VP_FSETSH(hVp,VPIS,LFDB,CLEAR);
  VP_FSETSH(hVp,VPIS,SFDB,CLEAR);
  VP_FSETSH(hVp,VPIS,VINTB2,CLEAR);
  VP_FSETSH(hVp,VPIS,VINTB1,CLEAR);
  VP_FSETSH(hVp,VPIS,SERRB,CLEAR);
  VP_FSETSH(hVp,VPIS,CCMPB,CLEAR);
  VP_FSETSH(hVp,VPIS,COVRB,CLEAR);

  VP_FSETSH(hVp,VPIE,VIE,ENABLE);

  IRQ_reset(hVp->eventId);

  IRQ_globalRestore(gie);
  return;              
 }
#endif
/*----------------------------------------------------------------------------*/
void VP_resetDisplay(VP_Handle hVp) {

  Uint32 gie;

  gie = IRQ_globalDisable();

  VP_FSETSH(hVp,VDCTL,RSTCH,RESET);

  /* Wait for reset to complete */
  while(VP_FGETH(hVp, VDCTL, RSTCH));

  /* Clean up display related bit field in VPIE and VPIS */
  VP_FSETSH(hVp,VPIE,VIE,DISABLE);

  VP_FSETSH(hVp,VPIE,DCMP,DISABLE);
  VP_FSETSH(hVp,VPIE,DUND,DISABLE);
  VP_FSETSH(hVp,VPIE,DCNA,DISABLE);

  VP_FSETSH(hVp,VPIS,DCMP,CLEAR);
  VP_FSETSH(hVp,VPIS,DUND,CLEAR);
  VP_FSETSH(hVp,VPIS,DCNA,CLEAR);

  VP_FSETSH(hVp,VPIE,VIE,ENABLE);

  IRQ_reset(hVp->eventId);


  IRQ_globalRestore(gie);
  return;              
}
/*----------------------------------------------------------------------------*/
VP_Handle VP_open(int devNum, Uint32 flags) {
                       
  Uint32 gie;          
                       
  VP_Handle hVp = INV;
                       
  gie = IRQ_globalDisable();
  if (!_VP_deviceTable[devNum].allocated) {
    hVp = &(_VP_deviceTable[devNum]);
    _VP_deviceTable[devNum].allocated = TRUE;
                       
    if (hVp != INV) {
      if (flags & VP_OPEN_RESET) {
        VP_reset(hVp);
      }                
    }                  
  }
  IRQ_globalRestore(gie);

  return hVp;
}
/*----------------------------------------------------------------------------*/
void VP_close(VP_Handle hVp) {

  Uint32 gie;

  gie = IRQ_globalDisable();
  if (hVp != INV) {
    VP_reset(hVp);
    hVp->allocated = FALSE;
  }
  IRQ_globalRestore(gie);
  return;
}
/*----------------------------------------------------------------------------*/
Uint32 VP_getPins(VP_Handle hVp) {

  return VP_RGETH(hVp,PDIN);
}
/*----------------------------------------------------------------------------*/
void VP_setPins(VP_Handle hVp, Uint32 val) {

  VP_RSETH(hVp,PDSET,val);
  return;
}
/*----------------------------------------------------------------------------*/
void VP_clearPins(VP_Handle hVp, Uint32 val) {

  VP_RSETH(hVp,PDCLR,val);
  return;
}
/*----------------------------------------------------------------------------*/

#endif /* VP_SUPPORT */
/******************************************************************************\
* End of csl_vp.c
\******************************************************************************/


