
/* Include files */
#include <c6x.h>
#include <csl.h> /* CSL library */
#include <csl_dma.h> /* DMA_SUPPORT */
#include <csl_edma.h> /* EDMA_SUPPORT */
#include <csl_irq.h> /* IRQ_SUPPORT */
#include <csl_mcbsp.h> /* MCBSP_SUPPORT */
#include <string.h>
/**/
#include <Timers.h>
#include "mcbsp_recv_slave.h"

#define BUFFER_SIZE 127 /* set same value as xmit */
#define SPI_STOP    0
#define SPI_IN_PROC 1
#define SPI_FIN     2

uint32_t dma_rxbuf[BUFFER_SIZE];
uint32_t dma_txbuf[BUFFER_SIZE];
volatile uint32_t dma_tx_len = 0;
volatile int spi_rx_stat = 0;
volatile int spi_tx_stat = 0;

typedef struct
{
    uint16_t crc;
    uint8_t e;
    uint8_t cmd;
    uint8_t len;
    uint8_t *data;
} spi_pack;

volatile spi_pack spack;
volatile int dmarx_counter = 0;
volatile int dmatx_counter = 0;
volatile int recv0_done = FALSE;
volatile int xmit0_done = FALSE;
/**/

#define reverse(x) ((x>>24)&0xff) | ((x<<8)&0xff0000) | ((x>>8)&0xff00) | ((x<<24)&0xff000000)
#define SWAP_UINT32(x) (((x) >> 24) | (((x) & 0x00FF0000) >> 8) | (((x) & 0x0000FF00) << 8) | ((x) << 24))

/* Declare CSL objects */
static MCBSP_Handle hdev; /* Handles for McBSP */
DMA_Handle hDma1;           /* Handles for DMA */
DMA_Handle hDma2;
/* External functions and function prototypes */
void init_mcbsp0_slave(void); /* Function prototypes */
void set_interrupts_dma(void);
/* Include the vector table to call the IRQ ISRs hookup */
extern far void vectors();
static volatile int val = 0;
void set_interrupts_dma(void);
void init_dma(uint32_t src, uint32_t dst, uint32_t rxlen, uint32_t txlen);


static MCBSP_Config ConfigSlaveSPI = {

  /* Serial Port Control Register (SPCR) */
  MCBSP_SPCR_RMK(

    MCBSP_SPCR_FRST_YES, /* Frame sync generator reset(FRST)
                              MCBSP_SPCR_FRST_YES /MCBSP_SPCR_FRST_0 - Frame sync
                                   generation logic is reset.
                              MCBSP_SPCR_FRST_NO /MCBSP_SPCR_FRST_1  - Frame sync
                                   signal is generated after eight CLKG clocks. */

    MCBSP_SPCR_GRST_YES, /* Sample rate generator reset(GRST)
                             MCBSP_SPCR_GRST_YES /MCBSP_SPCR_GRST_0  - Reset
                             MCBSP_SPCR_GRST_NO /MCBSP_SPCR_GRST_1 -Out of reset*/

    MCBSP_SPCR_XINTM_FRM,/* Transmit interrupt mode(XINTM)
                             MCBSP_SPCR_XINTM_XRDY - XINT driven by XRDY
                             MCBSP_SPCR_XINTM_EOS  - XINT generated by
                                   end-of-subframe in multichannel operation
                             MCBSP_SPCR_XINTM_FRM  - XINT generated by a new frame
                                   synchronization
                             MCBSP_SPCR_XINTM_XSYNCERR - XINT generated by XSYNCERR
                                                                                */
    MCBSP_SPCR_XSYNCERR_NO,/* Transmit synchronization error
                             MCBSP_SPCR_XSYNCERR_NO /MCBSP_SPCR_XSYNCERR_0  - No
                                   frame synchronization error
                             MCBSP_SPCR_XSYNCERR_YES /MCBSP_SPCR_XSYNCERR_1 - Frame
                                   synchronization error detected by McBSP       */

    MCBSP_SPCR_XRST_YES, /* Transmitter reset(XRST). This resets or enables
                            transmitter.
                             MCBSP_SPCR_XRST_YES /MCBSP_SPCR_XRST_0  - Serial port
                                   transmitter is disabled and is in reset state.
                             MCBSP_SPCR_XRST_NO /MCBSP_SPCR_XRST_1   - Serial port
                                   transmitter is enabled.                       */

    MCBSP_SPCR_DLB_OFF,   /* Digital loopback(DLB) mode
                             MCBSP_SPCR_DLB_OFF   - DLB Disabled
                             MCBSP_SPCR_DLB_ON    - DLB Enabled                  */

    MCBSP_SPCR_RJUST_RZF,/*Receive data sign-extension and justification mode(RJUST)
                             MCBSP_SPCR_RJUST_RZF - Right-justify and zero-fill
                                   MSBs in DRR.
                             MCBSP_SPCR_RJUST_RSE - Right-justify and sign-extend
                                   MSBs in DRR.
                             MCBSP_SPCR_RJUST_LZF - Left-justify and zero-fill
                                   LSBs in DRR.                                  */

    MCBSP_SPCR_CLKSTP_NODELAY,/* Clock stop(CLKSTP) mode
                             MCBSP_SPCR_CLKSTP_DISABLE - Disabled. Normal
                                  clocking enabled for non-SPI mode.
                             MCBSP_SPCR_CLKSTP_NODELAY -Clock starts without delay.
                             MCBSP_SPCR_CLKSTP_DELAY   - Clock starts with delay.*/

    MCBSP_SPCR_RINTM_RRDY,/* Receive interrupt(RINT) mode
                             MCBSP_SPCR_RINTM_RRDY - RINT driven by RRDY
                             MCBSP_SPCR_RINTM_EOS  - RINT generated by
                                  end-of-subframe in multichannel operation
                             MCBSP_SPCR_RINTM_FRM  - RINT generated by a new frame
                                  synchronization
                             MCBSP_SPCR_RINTM_RSYNCERR -RINT generated by RSYNCERR*/

    MCBSP_SPCR_RSYNCERR_NO,/* Receive synchronization error(RSYNCERR)
                             MCBSP_SPCR_RSYNCERR_NO /MCBSP_SPCR_RSYNCERR_0 - No
                                  frame synchronization error
                             MCBSP_SPCR_RSYNCERR_YES /MCBSP_SPCR_RSYNCERR_1 - Frame
                                  synchronization error detected by McBSP         */

    MCBSP_SPCR_RRST_YES /* Receiver reset(RRST). This resets or enables
                           receiver.
                             MCBSP_SPCR_RRST_YES /MCBSP_SPCR_RRST_0 - Serial
                                 port receiver is disabled and is in reset state.
                             MCBSP_SPCR_RRST_NO  /MCBSP_SPCR_RRST_1 - Serial
                                  port receiver is enabled.                    */

  ),

  /*  Receive Control Register (RCR) */
  MCBSP_RCR_RMK(

    MCBSP_RCR_RPHASE_SINGLE, /* Receive phases
                             MCBSP_RCR_RPHASE_SINGLE - Single phase frame
                             MCBSP_RCR_RPHASE_DUAL   - Dual phase frame        */

    MCBSP_RCR_RFRLEN2_OF(0), /* Receive frame length in phase 2(RFRLEN2)
                              000 0000b: 1 word per phase
                              000 0001b: 2 words per phase
                              . . . . . . . . . . . .
                              111 1111b: 128 words per phase                   */

    MCBSP_RCR_RWDLEN2_32BIT,/* Receive element length in phase 2(RWDLEN2)
                             MCBSP_RCR_RWDLEN2_8BIT  -  8  bits
                             MCBSP_RCR_RWDLEN2_12BIT -  12 bits
                             MCBSP_RCR_RWDLEN2_16BIT -  16 bits
                             MCBSP_RCR_RWDLEN2_20BIT -  20 bits
                             MCBSP_RCR_RWDLEN2_24BIT -  24 bits
                             MCBSP_RCR_RWDLEN2_32BIT -  32 bits                */

    MCBSP_RCR_RCOMPAND_MSB,/* Receive companding mode (RCOMPAND)
                             MCBSP_RCR_RCOMPAND_MSB  - No companding.Data
                                  transfer starts with MSB first.
                             MCBSP_RCR_RCOMPAND_8BITLSB - No companding,
                                  8-bit data. Transfer starts with LSB first.
                                  Applicable to 8-bit data or 32-bit data in
                                  data reversal mode.
                             MCBSP_RCR_RCOMPAND_ULAW -  Compand using m-law for
                                  receive data. Applicable to 8-bit data only
                             MCBSP_RCR_RCOMPAND_ALAW - Compand using A-law for
                                  receive data. Applicable to 8-bit data only  */

    MCBSP_RCR_RFIG_YES, /* Receive frame ignore(RFIG)
                             MCBSP_RCR_RFIG_NO  - Unexpected receive frame
                                  synchronization pulses restart the transfer.
                             MCBSP_RCR_RFIG_YES - Unexpected receive frame
                                  synchronization pulses are ignored.          */

    MCBSP_RCR_RDATDLY_0BIT,/* Receive data delay(RDATDLY)
                             MCBSP_RCR_RDATDLY_0BIT - 0 bit data delay
                             MCBSP_RCR_RDATDLY_1BIT - 1 bit data delay
                             MCBSP_RCR_RDATDLY_2BIT - 2 bit data delay         */

    MCBSP_RCR_RFRLEN1_OF(BUFFER_SIZE), /* Receive frame length in phase 1(RFRLEN1)
                             000 0000b: 1 word per phase
                             000 0001b: 2 words per phase
                             . . . . . . . . . . . .
                             111 1111b: 128 words per phase                    */

    MCBSP_RCR_RWDLEN1_32BIT/* Receive element length in phase 1(RWDLEN1)

                             MCBSP_RCR_RWDLEN1_8BIT  -  8 bits
                             MCBSP_RCR_RWDLEN1_12BIT - 12 bits
                             MCBSP_RCR_RWDLEN1_16BIT - 16 bits
                             MCBSP_RCR_RWDLEN1_20BIT - 20 bits
                             MCBSP_RCR_RWDLEN1_24BIT - 24 bits
                             MCBSP_RCR_RWDLEN1_32BIT - 32 bits                 */


  ),

  /* Transmit Control Register (XCR) */
  MCBSP_XCR_RMK(

    MCBSP_XCR_XPHASE_SINGLE,/* Transmit phases
                             MCBSP_XCR_XPHASE_SINGLE - Single phase frame
                             MCBSP_XCR_XPHASE_DUAL   - Dual phase frame       */

    MCBSP_XCR_XFRLEN2_OF(0),/* Transmit frame length in phase 2(XFRLEN2)
                             000 0000b: 1 word per phase
                             000 0001b: 2 words per phase
                             . . . . . . . . . . . .
                             111 1111b: 128 words per phase                   */

    MCBSP_XCR_XWDLEN2_32BIT, /*  Transmit element length in phase 2
                             MCBSP_XCR_XWDLEN2_8BIT    -  8  bits
                             MCBSP_XCR_XWDLEN2_12BIT   -  12 bits
                             MCBSP_XCR_XWDLEN2_16BIT   -  16 bits
                             MCBSP_XCR_XWDLEN2_20BIT   -  20 bits
                             MCBSP_XCR_XWDLEN2_24BIT   -  24 bits
                             MCBSP_XCR_XWDLEN2_32BIT   -  32 bits             */

    MCBSP_XCR_XCOMPAND_MSB, /* Transmit companding mode(XCOMPAND)
                             MCBSP_XCR_XCOMPAND_MSB      - No companding. Data
                                  transfer starts with MSB first.
                             MCBSP_XCR_XCOMPAND_8BITLSB  - No companding, 8-bit
                                   data. Transfer starts with LSB first.
                                   Applicable to 8-bit data,or 32-bit data in
                                   data reversal mode.
                             MCBSP_XCR_XCOMPAND_ULAW     - Compand using m-law
                                   for receive data.Applicable to 8-bit data only.
                             MCBSP_XCR_XCOMPAND_ALAW  - Compand using A-law for
                                   receive data.Applicable to 8-bit data only. */

    MCBSP_XCR_XFIG_YES, /* Transmit frame ignore(XFIG)
                             MCBSP_XCR_XFIG_NO  - Unexpected transmit frame
                                   synchronization pulses restart the transfer.
                             MCBSP_XCR_XFIG_YES - Unexpected transmit frame
                                   synchronization pulses are ignored.         */

    MCBSP_XCR_XDATDLY_0BIT, /*  Transmit data delay(XDATDLY)
                             MCBSP_XCR_XDATDLY_0BIT   - 0 bit data delay
                             MCBSP_XCR_XDATDLY_1BIT   - 1 bit data delay
                             MCBSP_XCR_XDATDLY_2BIT   - 2 bit data delay       */

    MCBSP_XCR_XFRLEN1_OF(127), /* Transmit frame length in phase 1(XFRLEN1)
                             000 0000b: 1 word per phase
                             000 0001b: 2 words per phase
                             . . . . . . . . . . . .
                             111 1111b: 128 words per phase                    */

    MCBSP_XCR_XWDLEN1_32BIT /* Transmit element length in phase 1(XWDLEN1)
                             MCBSP_XCR_XWDLEN1_8BIT  -   8 bits
                             MCBSP_XCR_XWDLEN1_12BIT -  12 bits
                             MCBSP_XCR_XWDLEN1_16BIT -  16 bits
                             MCBSP_XCR_XWDLEN1_20BIT -  20 bits
                             MCBSP_XCR_XWDLEN1_24BIT -  24 bits
                             MCBSP_XCR_XWDLEN1_32BIT -  32 bits                */


  ),

  /*serial port sample rate generator register(SRGR) */
  MCBSP_SRGR_RMK(

          MCBSP_SRGR_GSYNC_FREE,/* Sample rate generator clock synchronization(GSYNC).
                             MCBSP_SRGR_GSYNC_FREE - The sample rate generator
                                  clock CLKG) is free running.
                             MCBSP_SRGR_GSYNC_SYNC - (CLKG) is running but is
                                   resynchronized, and the frame sync signal
                                   (FSG)is generated only after the receive
                                   frame synchronization signal(FSR)is detected.
                                   Also,the frame period (FPER) is a dont care
                                   because the period is dictated by the external
                                   frame sync pulse.                            */

    MCBSP_SRGR_CLKSP_FALLING,/* CLKS polarity clock edge select(CLKSP)
                              MCBSP_SRGR_CLKSP_RISING  - The rising edge of CLKS
                                   generates CLKG and FSG.
                              MCBSP_SRGR_CLKSP_FALLING - The falling edge of CLKS
                                   generates CLKG and FSG.                      */
    MCBSP_SRGR_CLKSM_INTERNAL,/* MCBSP sample rate generator clock mode(CLKSM)
                              MCBSP_SRGR_CLKSM_CLKS   - The sample rate generator
                                   clock is derived from CLKS.
                              MCBSP_SRGR_CLKSM_INTERNAL - (Default value) The
                                   sample rate generator clock is derived from
                                   the internal clock source.                   */

    MCBSP_SRGR_FSGM_DXR2XSR,/*Sample rate generator transmit frame synchronization
                               mode.(FSGM)
                              MCBSP_SRGR_FSGM_DXR2XSR  - The transmit frame sync
                                   signal (FSX) is generated on every DXR to XSR
                                   copy.
                              MCBSP_SRGR_FSGM_FSG      - The transmit frame sync
                                   signal is driven by the sample rate generator
                                   frame sync signal, FSG.                      */

    MCBSP_SRGR_FPER_OF(63),/* Frame period(FPER)
                              Valid values: 0 to 4095                           */

    MCBSP_SRGR_FWID_OF(31),/* Frame width(FWID)
                              Valid values: 0 to 255                            */

    MCBSP_SRGR_CLKGDV_OF(7)/* Sample rate generator clock divider(CLKGDV)
                              Valid values: 0 to 255                            */

  ),

  MCBSP_MCR_DEFAULT, /* Using default value of MCR register */
  MCBSP_RCER_DEFAULT,/* Using default value of RCER register */
  MCBSP_XCER_DEFAULT,/* Using default value of XCER register */

  /* serial port pin control register(PCR) */
  MCBSP_PCR_RMK(

    MCBSP_PCR_XIOEN_SP, /* Transmitter in general-purpose I/O mode - only when
                           XRST = 0 in SPCR - (XIOEN)
                           MCBSP_PCR_XIOEN_SP    -  CLKS pin is not a general
                                purpose input. DX pin is not a general purpose
                                output.FSX and CLKX are not general-purpose I/Os.
                           MCBSP_PCR_XIOEN_GPIO  -  CLKS pin is a general-purpose
                                input. DX pin is a general-purpose output.
                                FSX and CLKX are general-purpose I/Os. These
                                serial port pins do not perform serial port
                                operation.                                     */
    MCBSP_PCR_RIOEN_SP, /* Receiver in general-purpose I/O mode - only when
                           RRST = 0 in SPCR -(RIOEN)
                           MCBSP_PCR_RIOEN_SP    - DR and CLKS pins are not
                                general-purpose inputs. FSR and CLKR are not
                                general-purpose I/Os and perform serial port
                                operation.
                           MCBSP_PCR_RIOEN_GPIO  - DR and CLKS pins are
                                general-purpose inputs. FSR and CLKR are
                                general-purpose I/Os. These serial port pins do
                                not perform serial port operation.            */
    MCBSP_PCR_FSXM_EXTERNAL, /* Transmit frame synchronization mode(FSXM)
                             MCBSP_PCR_FSXM_EXTERNAL - Frame synchronization
                                  signal is provided by an external source. FSX
                                  is an input pin.
                             MCBSP_PCR_FSXM_INTERNAL - Frame synchronization
                                  generation is determined by the sample rate
                                  generator frame synchronization mode bit FSGM
                                  in the SRGR.                                */

    MCBSP_PCR_FSRM_EXTERNAL, /* Receive frame synchronization mode (FSRM)
                             MCBSP_PCR_FSRM_EXTERNAL  - Frame synchronization
                                  signals are generated by an external device.
                                  FSR is an input pin.
                             MCBSP_PCR_FSRM_INTERNAL  - Frame synchronization
                                  signals are generated internally by the sample
                                  rate generator. FSR is an output pin except
                                  when GSYNC = 1 in SRGR.                     */

    MCBSP_PCR_CLKXM_INPUT, /* Transmitter clock mode (CLKXM)
                             MCBSP_PCR_CLKXM_INPUT    -  Transmitter clock is
                                  driven by an external clock with CLKX as an
                                  input pin.
                             MCBSP_PCR_CLKXM_OUTPUT   - CLKX is an output pin
                                   and is driven by the internal sample rate
                                   generator.

                             During SPI mode :
                             MCBSP_PCR_CLKXM_INPUT    -  McBSP is a slave and
                                  (CLKX) is driven by the SPI master in the
                                   system. CLKR is internally driven by CLKX.
                             MCBSP_PCR_CLKXM_OUTPUT   - McBSP is a master and
                                  generates the transmitter clock (CLKX) to
                                  drive its receiver clock (CLKR) and the shift
                                  clock of the SPI-compliant slaves in the
                                  system.                                     */
    MCBSP_PCR_CLKRM_INPUT, /* Receiver clock mode (CLKRM)

                              Case 1: Digital loopback mode not set in SPCR

                              MCBSP_PCR_CLKRM_INPUT - Receive clock (CLKR) is
                                   an input driven by an external clock.

                              MCBSP_PCR_CLKRM_OUTPUT -  CLKR is an output pin
                                   and is driven by the sample rate generator.

                              Case 2: Digital loopback mode set  in SPCR

                              MCBSP_PCR_CLKRM_INPUT - Receive clock  is driven
                                   by the transmit clock (CLKX), which is based
                                   on the CLKXM bit in PCR. CLKR is in high
                                   impedance.
                              MCBSP_PCR_CLKRM_INPUT - CLKR is an output pin and
                                   is driven by the transmit clock. The transmit
                                   clock is derived from CLKXM bit in the PCR.*/


    MCBSP_PCR_CLKSSTAT_0, /*  CLKS pin status(CLKSSTAT)
                              MCBSP_PCR_CLKSSTAT_0
                              MCBSP_PCR_CLKSSTAT_1                            */

    MCBSP_PCR_DXSTAT_0,   /*  DX pin status(DXSTAT)
                              MCBSP_PCR_DXSTAT_0
                              MCBSP_PCR_DXSTAT_1                              */

    MCBSP_PCR_FSXP_ACTIVELOW, /* Transmit frame synchronization polarity(FSXP)
                              MCBSP_PCR_FSXP_ACTIVEHIGH - Frame synchronization
                                       pulse FSX is active high
                              MCBSP_PCR_FSXP_ACTIVELOW  - Frame synchronization
                                   pulse FSX is active low                    */
    MCBSP_PCR_FSRP_ACTIVEHIGH, /* Receive frame synchronization polarity(FSRP)
                              MCBSP_PCR_FSRP_ACTIVEHIGH - Frame synchronization
                                   pulse FSR is active high
                              MCBSP_PCR_FSRP_ACTIVELOW  - Frame synchronization
                                   pulse FSR is active low                    */
    MCBSP_PCR_CLKXP_RISING, /* Transmit clock polarity(CLKXP)
                              MCBSP_PCR_CLKXP_RISING - Transmit data driven on
                                   rising edge of CLKX
                              MCBSP_PCR_CLKXP_FALLING - Transmit data driven on
                                    falling edge of CLKX                      */
    MCBSP_PCR_CLKRP_FALLING /* Receive clock polarity(CLKRP)
                              MCBSP_PCR_CLKRP_FALLING - Receive data sampled on
                                   falling edge of CLKR
                              MCBSP_PCR_CLKRP_RISING - Receive data sampled on
                                    rising edge of CLKR                       */
  )
};


void prepare_spi()
{
    hdev = MCBSP_open(MCBSP_DEV1, MCBSP_OPEN_RESET);

    MCBSP_config(hdev,&ConfigSlaveSPI);

    init_dma((uint32_t)&dma_txbuf, (uint32_t)&dma_rxbuf, BUFFER_SIZE, dma_tx_len+1);

    DMA_start(hDma1);       // rx
    DMA_start(hDma2);       // tx

    MCBSP_start (hdev, MCBSP_SRGR_START| MCBSP_RCV_START | MCBSP_XMIT_START | MCBSP_SRGR_FRAMESYNC, 0x10);

    IRQ_enable(IRQ_EVT_XINT1);
    IRQ_enable(IRQ_EVT_RINT1);
}

void stop_spi()
{
    IRQ_disable(IRQ_EVT_XINT1);
    IRQ_disable(IRQ_EVT_RINT1);
    spi_rx_stat == SPI_STOP;
    DMA_stop(hDma1);       // rx
    DMA_stop(hDma2);       // tx
    MCBSP_reset(INV);
    MCBSP_close(hdev);
    DMA_close(hDma1);
    DMA_close(hDma2);

}

void init_spi (void)
{
    dmarx_counter = 0;

    spi_rx_stat = spi_tx_stat = SPI_STOP;

    IRQ_map(IRQ_EVT_XINT1, 14);  // FSX interrupt!
    IRQ_map(IRQ_EVT_RINT1, 15);
    dma_tx_len = 0;

    prepare_spi();
     spi_rx_stat = SPI_IN_PROC;
} /* end main, program ends here */


/**/
/* set_interrupts_dma() */
/**/
void set_interrupts_dma (void) /* if the device supports DMA */ /* Set the interrupts */
{
    IRQ_nmiEnable();

    IRQ_disable(IRQ_EVT_DMAINT2); /* INT09 */
    IRQ_disable(IRQ_EVT_DMAINT1); /* INT09 */
    IRQ_clear(IRQ_EVT_DMAINT2);
    IRQ_clear(IRQ_EVT_DMAINT1);
    IRQ_enable(IRQ_EVT_DMAINT2);
    IRQ_enable(IRQ_EVT_DMAINT1);

    return;
}
/**/
/* DMA DATA TRANSFER COMPLETION ISRs */
/**/
void dma_compl(int rx)
{
    if (rx) {
        dmarx_counter++;
    }
    else
        dmatx_counter++;
    return;
}

void fsx_irq(int end_rx)
{
    IRQ_clear(IRQ_EVT_RINT1);
    IRQ_clear(IRQ_EVT_XINT1);
    static uint32_t flag = 0;

    if (flag == 0) {
        flag = 1;
        return;
    }

    setReset(1);

    if (end_rx) // FSX interrupt;
    {
        flag = 0;
        int dma_read_len = (DMA_RGET(DST1) - (uint32_t)&dma_rxbuf)/4 - 1;       // DMA   1  .
        stop_spi();

        spack.crc = ((uint16_t *)dma_rxbuf)[0];
        spack.e = (0x80 & ((uint8_t *)dma_rxbuf)[1]) && 1;
        spack.cmd = 0x7F & ((uint8_t *)dma_rxbuf)[1];
        spack.len = ((uint8_t *)dma_rxbuf)[3];
        spack.data = &((uint8_t *)dma_rxbuf)[4];


        /*! call commands */
        static int i = 0;
        memset(dma_txbuf, i, 40);
        dma_txbuf[0] = dma_read_len;
        dma_tx_len = 20;
        i++;



        prepare_spi();
    }

    setReset(0);
    return;
}

void init_dma(uint32_t src, uint32_t dst, uint32_t rxlen, uint32_t txlen)
{
    /* Channel 2 transmits the data */
        hDma2 = DMA_open(DMA_CHA2, DMA_OPEN_RESET);  /* Handle to DMA channel 2 */
        DMA_configArgs(hDma2,
                       DMA_PRICTL_RMK(
                               DMA_PRICTL_DSTRLD_DEFAULT,
                               DMA_PRICTL_SRCRLD_DEFAULT,
                               DMA_PRICTL_EMOD_DEFAULT,
                               DMA_PRICTL_FS_DEFAULT,
                               DMA_PRICTL_TCINT_ENABLE, /* TCINT =1 */
                               DMA_PRICTL_PRI_DMA,      /* DMA high priority */
                               DMA_PRICTL_WSYNC_XEVT1,  /* Set synchronization event XEVT0=01100 */
                               DMA_PRICTL_RSYNC_DEFAULT,
                               DMA_PRICTL_INDEX_DEFAULT,
                               DMA_PRICTL_CNTRLD_DEFAULT,
                               DMA_PRICTL_SPLIT_DEFAULT,
                               DMA_PRICTL_ESIZE_32BIT,  /* Element size 32 bits */
                               DMA_PRICTL_DSTDIR_DEFAULT,
                               DMA_PRICTL_SRCDIR_INC,   /* Increment source by element size */
                               DMA_PRICTL_START_DEFAULT
                       ),
                       DMA_SECCTL_RMK(

                               DMA_SECCTL_DMACEN_DEFAULT,
                               DMA_SECCTL_WSYNCCLR_DEFAULT,
                               DMA_SECCTL_WSYNCSTAT_DEFAULT,
                               DMA_SECCTL_RSYNCCLR_DEFAULT,
                               DMA_SECCTL_RSYNCSTAT_DEFAULT,
                               DMA_SECCTL_WDROPIE_DEFAULT,
                               DMA_SECCTL_WDROPCOND_DEFAULT,
                               DMA_SECCTL_RDROPIE_DEFAULT,
                               DMA_SECCTL_RDROPCOND_DEFAULT,
                               DMA_SECCTL_BLOCKIE_ENABLE,  /* Enables DMA channel interrupt */
                               DMA_SECCTL_BLOCKCOND_DEFAULT,
                               DMA_SECCTL_LASTIE_DEFAULT,
                               DMA_SECCTL_LASTCOND_DEFAULT,
                               DMA_SECCTL_FRAMEIE_DEFAULT,
                               DMA_SECCTL_FRAMECOND_DEFAULT,
                               DMA_SECCTL_SXIE_DEFAULT,
                               DMA_SECCTL_SXCOND_DEFAULT
                       ),
                       DMA_SRC_RMK((Uint32)src),    /* Set source to dmaOutbuff */
                       DMA_DST_RMK(MCBSP_ADDRH(hdev, DXR)), /* Set destination to DXR */
                       DMA_XFRCNT_RMK(
                               DMA_XFRCNT_FRMCNT_DEFAULT,
                               DMA_XFRCNT_ELECNT_OF(txlen)
                       )
        );

        /* Channel 1 receives the data */
        hDma1 = DMA_open(DMA_CHA1, DMA_OPEN_RESET);  /* Handle to DMA channel 1 */
        DMA_configArgs(hDma1,
                       DMA_PRICTL_RMK(
                               DMA_PRICTL_DSTRLD_DEFAULT,
                               DMA_PRICTL_SRCRLD_DEFAULT,
                               DMA_PRICTL_EMOD_DEFAULT,
                               DMA_PRICTL_FS_DEFAULT,
                               DMA_PRICTL_TCINT_ENABLE, /* TCINT =1 */
                               DMA_PRICTL_PRI_DMA,      /* DMA high priority */
                               DMA_PRICTL_WSYNC_DEFAULT,
                               DMA_PRICTL_RSYNC_REVT1,  /* Set synchronization event REVT0=01101 */
                               DMA_PRICTL_INDEX_DEFAULT,
                               DMA_PRICTL_CNTRLD_DEFAULT,
                               DMA_PRICTL_SPLIT_DEFAULT,
                               DMA_PRICTL_ESIZE_32BIT,  /* Element size 32 bits */
                               DMA_PRICTL_DSTDIR_INC,   /* Increment destination by element size */
                               DMA_PRICTL_SRCDIR_DEFAULT,
                               DMA_PRICTL_START_DEFAULT
                       ),
                       DMA_SECCTL_RMK(

                               DMA_SECCTL_DMACEN_DEFAULT,
                               DMA_SECCTL_WSYNCCLR_DEFAULT,
                               DMA_SECCTL_WSYNCSTAT_DEFAULT,
                               DMA_SECCTL_RSYNCCLR_DEFAULT,
                               DMA_SECCTL_RSYNCSTAT_DEFAULT,
                               DMA_SECCTL_WDROPIE_DEFAULT,
                               DMA_SECCTL_WDROPCOND_DEFAULT,
                               DMA_SECCTL_RDROPIE_DEFAULT,
                               DMA_SECCTL_RDROPCOND_DEFAULT,
                               DMA_SECCTL_BLOCKIE_ENABLE,  /* Enables DMA channel interrupt */
                               DMA_SECCTL_BLOCKCOND_DEFAULT,
                               DMA_SECCTL_LASTIE_DEFAULT,
                               DMA_SECCTL_LASTCOND_DEFAULT,
                               DMA_SECCTL_FRAMEIE_DEFAULT,
                               DMA_SECCTL_FRAMECOND_DEFAULT,
                               DMA_SECCTL_SXIE_DEFAULT,
                               DMA_SECCTL_SXCOND_DEFAULT
                       ),
                       DMA_SRC_RMK(MCBSP_ADDRH(hdev, DRR)), /* Set source to DRR */
                       DMA_DST_RMK((Uint32)dst),     /* Set destination to dmaInbuff */
                       DMA_XFRCNT_RMK(
                               DMA_XFRCNT_FRMCNT_DEFAULT,
                               DMA_XFRCNT_ELECNT_OF(rxlen)
                       )
        );
}



void c_int10(void)      /* DMA ch2 */
{
    xmit0_done = TRUE;
    dmatx_counter++;
    return;
}

void c_int09(void)        /* DMA ch1 */
{
    recv0_done = TRUE;
    dmarx_counter++;
    return;
}

