L 1 "../csl/csl_cache.c"
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_cache.c
N* DATE CREATED.. 06/11/1999
N* LAST MODIFIED. 08/02/2004 - Adding support for C6418
N*                05/31/2002
N*------------------------------------------------------------------------------
N* DESCRIPTION:  (body file for the CACHE module)
N*
N*   Supported Cache Operations:
N*                      +-----+-----+-----+
N*                      | L2  | L1P | L1D |
N*   +------------------+-----+-----+-----+
N*   | invalidate range |     |  X  |     |
N*   | invalidate all   |     |  X  |  X  |
N*   +------------------+-----+-----+-----+
N*   | flush range      |  X  |     |  X  |
N*   | flush all        |  X  |     |     |
N*   +------------------+-----+-----+-----+
N*   | clean range      |  X  |     |     |
N*   | clean all        |  X  |     |     |
N*   +------------------+-----+-----+-----+
N*
N*   invalidate range L1P - use L1PFBAR/L1PFWC
N*   invalidate all L1P   - use CCFG.IP
N*   invalidate all L1D   - use CCFG.ID
N*   flush range L2       - use L2FBAR/L2FWC
N*   flush range L1D      - use L1DFBAR/L1DFWC
N*   flush all L2         - use L2FLUSH
N*   clean range L2       - use L2CBAR/L2CWC
N*   clean all L2         - use L2CLEAN
N*
N*   L2 Cache Mode Switching Guidelines
N*     Decrease L2 SRAM:
N*       1) Perform flush of L2 cache (all). Spin on completion.
N*       2) Write new L2 mode / Read back.
N*       3) NOP 8
N*
N*
N*     Increase L2 SRAM:
N*       1) Perform flush of L2 cache (all). Spin on completion.
N*       2) Write new L2 mode / Read back.
N*       3) NOP 8
N*
N\******************************************************************************/
N#define _CACHE_MOD_
N#include "csl_cache.h"
L 1 "..\csl\csl_cache.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2001 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_cache.h
N* DATE CREATED.. 06/11/1999
N* LAST MODIFIED. 10/01/2001 - Addition of 192K L2 SRAM for 6713
N*                04/16/2004 - Modified CACHE_ROUND_TO_LINESIZE implementation
N\******************************************************************************/
N#ifndef _CSL_CACHE_H_
N#define _CSL_CACHE_H_
N
N#include <csl_chip.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_chip.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2001 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_chip.h
N* DATE CREATED.. 08/19/1999 
N* LAST MODIFIED. 08/02/2004 - Adding support for C6418
N*                06/17/2003   - Added support for 6712C
N*                06/09/2003   - Added support for 6711C
N*                12/03/2001   - CHIP_configArgs
N*                11/08/2001   - CHIP_getSiliconRevId()
N*                             - CHIP_config() / CHIP_getConfig  DM642
N\******************************************************************************/
N#ifndef _CSL_CHIP_H_
N#define _CSL_CHIP_H_
N
N#include <csl_stdinc.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_stdinc.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_stdinc.h
N* DATE CREATED.. 03/11/1999 
N* LAST MODIFIED. 10/03/2000
N*                01/06/2005    Added _TI_STD_TYPES  directive to protect 
N*                              multiple definition of datatypes.
N\******************************************************************************/
N#ifndef _CSL_STDINC_H_
N#define _CSL_STDINC_H_
N
N#include <csl_stdinchal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_stdinchal.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_stdinchal.h
N* DATE CREATED.. 03/11/1999 
N* LAST MODIFIED. 03/23/2001
N\******************************************************************************/
N#ifndef _CSL_STDINCHAL_H_
N#define _CSL_STDINCHAL_H_
N
N
N/******************************************************************************\
N* HAL macro definitions
X
N\******************************************************************************/
N
N  #define _VALUEOF(x) ((Uint32)(x))
N
N  /*******************************************************/
N  /* generic macros for field manipulation               */
N  /*******************************************************/
N
N  #define _PER_FSHIFT(PER,REG,FIELD) \
N    _##PER##_##REG##_##FIELD##_SHIFT
X  #define _PER_FSHIFT(PER,REG,FIELD)     _##PER##_##REG##_##FIELD##_SHIFT
N
N  #define _PER_FMASK(PER,REG,FIELD) \
N    _##PER##_##REG##_##FIELD##_MASK
X  #define _PER_FMASK(PER,REG,FIELD)     _##PER##_##REG##_##FIELD##_MASK
N
N  #define _PER_FSYM(PER,REG,FIELD,SYM)\
N    PER##_##REG##_##FIELD##_##SYM
X  #define _PER_FSYM(PER,REG,FIELD,SYM)    PER##_##REG##_##FIELD##_##SYM
N
N  #define _PER_FDEFAULT(PER,REG,FIELD) \
N    ((##PER##_##REG##_##FIELD##_DEFAULT << _##PER##_##REG##_##FIELD##_SHIFT) \
N    & _##PER##_##REG##_##FIELD##_MASK)
X  #define _PER_FDEFAULT(PER,REG,FIELD)     ((##PER##_##REG##_##FIELD##_DEFAULT << _##PER##_##REG##_##FIELD##_SHIFT)     & _##PER##_##REG##_##FIELD##_MASK)
N
N  #define _PER_FMK(PER,REG,FIELD,x) (\
N    (((Uint32)(x))<<_PER_FSHIFT(##PER,##REG,##FIELD))\
N    &_PER_FMASK(##PER,##REG,##FIELD)\
N  )
X  #define _PER_FMK(PER,REG,FIELD,x) (    (((Uint32)(x))<<_PER_FSHIFT(##PER,##REG,##FIELD))    &_PER_FMASK(##PER,##REG,##FIELD)  )
N
N  #define _PER_FMKS(PER,REG,FIELD,SYM) (\
N    (_PER_FSYM(##PER,##REG,##FIELD,##SYM)<<_PER_FSHIFT(##PER,##REG,##FIELD))\
N    &_PER_FMASK(##PER,##REG,##FIELD)\
N  )
X  #define _PER_FMKS(PER,REG,FIELD,SYM) (    (_PER_FSYM(##PER,##REG,##FIELD,##SYM)<<_PER_FSHIFT(##PER,##REG,##FIELD))    &_PER_FMASK(##PER,##REG,##FIELD)  )
N
N  #define _PER_FEXTRACT(PER,REG,FIELD,reg) (Uint32)(\
N    (((Uint32)(reg)&_PER_FMASK(##PER,##REG,##FIELD))\
N    >>_PER_FSHIFT(##PER,##REG,##FIELD))\
N  )
X  #define _PER_FEXTRACT(PER,REG,FIELD,reg) (Uint32)(    (((Uint32)(reg)&_PER_FMASK(##PER,##REG,##FIELD))    >>_PER_FSHIFT(##PER,##REG,##FIELD))  )
N
N  #define _PER_FINSERT(PER,REG,FIELD,reg,field) (Uint32)(\
N    (((Uint32)(reg)&~_PER_FMASK(##PER,##REG,##FIELD))|\
N    (((Uint32)(field)<<_PER_FSHIFT(##PER,##REG,##FIELD))\
N    &_PER_FMASK(##PER,##REG,##FIELD)))\
N  )
X  #define _PER_FINSERT(PER,REG,FIELD,reg,field) (Uint32)(    (((Uint32)(reg)&~_PER_FMASK(##PER,##REG,##FIELD))|    (((Uint32)(field)<<_PER_FSHIFT(##PER,##REG,##FIELD))    &_PER_FMASK(##PER,##REG,##FIELD)))  )
N
N  /*******************************************************/
N  /* macros for memmory mapped registers                 */
N  /*******************************************************/
N
N  #define _PER_RAOI(addr,PER,REG,and,or,inv)\
N    (*(volatile Uint32*)(addr))=(\
N      ((((*(volatile Uint32*)(addr))\
N      &((Uint32)(and)))\
N      |((Uint32)(or)))\
N      ^((Uint32)(inv)))\
N    )
X  #define _PER_RAOI(addr,PER,REG,and,or,inv)    (*(volatile Uint32*)(addr))=(      ((((*(volatile Uint32*)(addr))      &((Uint32)(and)))      |((Uint32)(or)))      ^((Uint32)(inv)))    )
N
N  #define _PER_RGET(addr,PER,REG) \
N    (*(volatile Uint32*)(addr))
X  #define _PER_RGET(addr,PER,REG)     (*(volatile Uint32*)(addr))
N
N  #define _PER_RSET(addr,PER,REG,x) \
N    (*(volatile Uint32*)(addr))=((Uint32)(x))
X  #define _PER_RSET(addr,PER,REG,x)     (*(volatile Uint32*)(addr))=((Uint32)(x))
N
N  #define _PER_FGET(addr,PER,REG,FIELD) \
N    _PER_FEXTRACT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG))
X  #define _PER_FGET(addr,PER,REG,FIELD)     _PER_FEXTRACT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG))
N
N  #define _PER_FSET(addr,PER,REG,FIELD,field)\
N    _PER_RSET(addr,##PER,##REG,\
N    _PER_FINSERT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG),field))
X  #define _PER_FSET(addr,PER,REG,FIELD,field)    _PER_RSET(addr,##PER,##REG,    _PER_FINSERT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG),field))
N
N  #define _PER_FSETS(addr,PER,REG,FIELD,SYM)\
N    _PER_RSET(addr,##PER,##REG,_PER_FINSERT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG),\
N    _PER_FSYM(##PER,##REG,##FIELD,##SYM)))
X  #define _PER_FSETS(addr,PER,REG,FIELD,SYM)    _PER_RSET(addr,##PER,##REG,_PER_FINSERT(##PER,##REG,##FIELD,_PER_RGET(addr,##PER,##REG),    _PER_FSYM(##PER,##REG,##FIELD,##SYM)))
N
N  /*******************************************************/
N  /* macros for CPU control registers                    */
N  /*******************************************************/
N
N  #define _PER_CRGET(PER,REG) \
N    REG
X  #define _PER_CRGET(PER,REG)     REG
N
N  #define _PER_CRSET(PER,REG,reg) \
N    REG=((Uint32)(reg))
X  #define _PER_CRSET(PER,REG,reg)     REG=((Uint32)(reg))
N
N  #define _PER_CFGET(PER,REG,FIELD) \
N    _PER_FEXTRACT(##PER,##REG,##FIELD,_PER_CRGET(##PER,##REG))
X  #define _PER_CFGET(PER,REG,FIELD)     _PER_FEXTRACT(##PER,##REG,##FIELD,_PER_CRGET(##PER,##REG))
N
N  #define _PER_CFSET(PER,REG,FIELD,field)\
N    _PER_CRSET(##PER,##REG,\
N    _PER_FINSERT(##PER,##REG,##FIELD,_PER_CRGET(##PER,##REG),field))
X  #define _PER_CFSET(PER,REG,FIELD,field)    _PER_CRSET(##PER,##REG,    _PER_FINSERT(##PER,##REG,##FIELD,_PER_CRGET(##PER,##REG),field))
N
N  #define _PER_CFSETS(PER,REG,FIELD,SYM)\
N    _PER_CRSET(##PER,##REG,\
N    _PER_FINSERT(##PER,##REG,FIELD,_PER_CRGET(##PER,##REG),\
N    _PER_FSYM(##PER,##REG,##FIELD,##SYM)))
X  #define _PER_CFSETS(PER,REG,FIELD,SYM)    _PER_CRSET(##PER,##REG,    _PER_FINSERT(##PER,##REG,FIELD,_PER_CRGET(##PER,##REG),    _PER_FSYM(##PER,##REG,##FIELD,##SYM)))
N
N/*----------------------------------------------------------------------------*/
N
N#endif /* _CSL_STDINCHAL_H_ */
N/******************************************************************************\
N* End of csl_stdinchal.h
X
N\******************************************************************************/
N
L 15 "E:/TI_workspase/c6701_t/csl/csl_stdinc.h" 2
N
N
N/******************************************************************************\
N* macro declarations
X
N\******************************************************************************/
N
N/* a few standard constants */
N#ifndef TRUE
N  #define TRUE   1
N#endif
N#ifndef FALSE
N  #define FALSE  0
N#endif
N
N/* Invalid Pointer                                               */
N/* In many instances, it is better to use INV as opposed to NULL */
N/* because 0 may actually be a valid pointer                     */
N#ifndef INV
N  #define INV    ((void*)(-1))
N#endif
N
N#ifndef UNREFERENCED_PARAMETER
N  #define UNREFERENCED_PARAMETER(P)    ((P)=(P))
N#endif
N
N#ifndef REG32
N  #define REG32(addr) (*(volatile unsigned int*)(addr))
N  #define REG16(addr) (*(volatile unsigned short*)(addr))
N  #define REG8(addr)  (*(volatile unsigned char*)(addr))
N#endif
N
N/******************************************************************************\
N* typedef declarations
X
N\******************************************************************************/
N
N#include <tistdtypes.h> /* use typedefs provided with CCS*/
L 1 "E:/TI_workspase/c6701_t/csl/tistdtypes.h" 1
N/*
N *  ======== tistdtypes.h ========
N */
N
N/*
N * These types are also defined by other TI components.  They are bracketed
N * with _TI_STD_TYPES to avoid warnings for duplicate definition.
N *
N * You may get warnings about duplicate type definitions when using this
N * header file with earlier versions of DSP/BIOS and CSL.
N *
N * You can use the '-pds303' compiler option to suppress these warnings.
N */
N#ifndef _TI_STD_TYPES
N#define _TI_STD_TYPES
N
N/*
N * This '#ifndef STD_' is needed to protect from duplicate definitions
N * of Int, Uns, etc. in DSP/BIOS v4.x (e.g. 4.90, 4.80) since these versions
N * of DSP/BIOS did not contain the '#ifndef_TI_STD_TYPES' logic.
N */
N#ifndef STD_
N
N/*
N * Aliases for standard C types
N */
Ntypedef int			Int;
Ntypedef unsigned		Uns;
Ntypedef char			Char;
N
N/* pointer to null-terminated character sequence */
Ntypedef char			*String;
N
Ntypedef void			*Ptr;		/* pointer to arbitrary type */
N
Ntypedef unsigned short		Bool;		/* boolean */
N
N#endif /* STD_ */
N
N/*
N * Uint8, Uint16, Uint32, etc are defined to be "smallest unit of
N * available storage that is large enough to hold unsigned or integer
N * of specified size".
N */
N
N/* Handle the 6x ISA */
N#if defined(_TMS320C6X)
X#if 1L
N    /* Unsigned integer definitions (32bit, 16bit, 8bit) follow... */
N    typedef unsigned int	Uint32;
N    typedef unsigned short	Uint16;
N    typedef unsigned char	Uint8;
N
N    /* Signed integer definitions (32bit, 16bit, 8bit) follow... */
N    typedef int			Int32;
N    typedef short		Int16;
N    typedef char		Int8;
N
N/* Handle the 54x, 55x and 28x ISAs */
N#elif defined(_TMS320C5XX) || defined(__TMS320C55X__) || defined(_TMS320C28X)
S    /* Unsigned integer definitions (32bit, 16bit, 8bit) follow... */
S    typedef unsigned long	Uint32;
S    typedef unsigned short	Uint16;
S    typedef unsigned char	Uint8;
S
S    /* Signed integer definitions (32bit, 16bit, 8bit) follow... */
S    typedef long		Int32;
S    typedef short		Int16;
S    typedef char		Int8;
S
S#else
S    /* Other ISAs not supported */
S    #error <tistdtypes.h> is not supported for this target
N#endif  /* defined(_6x_) */
N
N#endif  /* _TI_STD_TYPES */
L 51 "E:/TI_workspase/c6701_t/csl/csl_stdinc.h" 2
N
N/******************************************************************************\
N* variable declarations
X
N\******************************************************************************/
N
N
N typedef long             Int40;
N typedef unsigned long    Uint40;
N/******************************************************************************\
N* function declarations
X
N\******************************************************************************/
N
N
N#endif /* _CSL_STDINC_H_ */
N/******************************************************************************\
N* End of csl_stdinc.h
X
N\******************************************************************************/
N
L 18 "E:/TI_workspase/c6701_t/csl/csl_chip.h" 2
N
N#include <csl_chiphal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_chiphal.h" 1
N/******************************************************************************\
N*           Copyright (C) 2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_chiphal.h
N* DATE CREATED.. 14 Aug 2000
N* LAST MODIFIED. 14 Jan 2004 Adding support for DRI300 versions (6410, 6413)
N*                05 Aug 2003 Removing external control cregisters EM,ER,IN,OUT and DIER.
N*                26 Jun 2003 Added support for 6411
N*                17 Jun 2003 Added support for 6712C
N*                28 May 2003 Added support for 6711C
N*                05 Nov 2001 DM642 , 6411 remove 6400
N*                03 Oct 2001 - CHIP_6713 - MCASP_SUPPORT - IIC_SUPPORT
N*                           - PERCFG register
N*                           - redefinition of CHIP_RSET() / CHIP_RGET()
N*                           - new CHIP_CRSET() / CHIP_CRGET() => modification of csl_irq.h
N*                04 Apr 2004-  Removing external control cregisters EM,ER,IN,OUT and DIER.
N*                12 Jan 2005-  Removing external control cregisters FMCR,FADCR,FAUCR and GFPGFR
N*                06 Apr 2005-  Removing the macros ATLEN,ATLMEN and ADIV according to data manual 
N*                              tms320c6410(13)-sprs247 dated: Feb26 2004 specifications. 
N*				 26 Jul 2005-  Added C++ support.
N*------------------------------------------------------------------------------
N* REGISTERS
N*
N* CSR    - control/status register
N* IFR    - interrupt flag register
N* ISR    - interrupt set register
N* ICR    - interrupt clear register
N* IER    - interrupt enable register
N* ISTP   - interrupt service table pointer register
N* IRP    - interrupt return pointer
N* NRP    - non-maskable interrupt return pointer
N* AMR    - addressing mode reister
N* PERCFG - Device Configuration register  (4)
N* DEVSTAT - Device Status Register (5)
N* JTAGID - JTAG ID register (5)
N*
N* (1) only supported on 67xx
N* (2) only supported on floating point devices
N* (3) only supported on 6411/14/15/16 devices
N* (4) only supported on 6713/DA610/DM642/6412/6711C/6712C devices
N* (5) only supported on DM642/6412/6410/6413 devices
N*
N\******************************************************************************/
N#ifndef _CSL_CHIPHAL_H_
N#define _CSL_CHIPHAL_H_
N
N#include <csl_stdinc.h>
N
N#ifdef __cplusplus
Sextern "C" {
N#endif
N
N
N/******************************************************************************\
N* CHIP identification section
X
N\******************************************************************************/
N#ifdef CHIP_BASELINE
S  #undef CHIP_BASELINE
S  #define CHIP_BASELINE 1
N#else
N  #define CHIP_BASELINE 0
N#endif
N
N#if (CHIP_BASELINE)
X#if (0)
S  #define CHIP_6201 1
N#endif
N
N#ifdef CHIP_6201
S  #undef CHIP_6201
S  #define CHIP_6201 1
N#else
N  #define CHIP_6201 0
N#endif
N
N#ifdef CHIP_6202
S  #undef CHIP_6202
S  #define CHIP_6202 1
N#else
N  #define CHIP_6202 0
N#endif
N
N#ifdef CHIP_6203
S  #undef CHIP_6203
S  #define CHIP_6203 1
N#else
N  #define CHIP_6203 0
N#endif
N
N#ifdef CHIP_6204
S  #undef CHIP_6204
S  #define CHIP_6204 1
N#else
N  #define CHIP_6204 0
N#endif
N
N#ifdef CHIP_6205
S  #undef CHIP_6205
S  #define CHIP_6205 1
N#else
N  #define CHIP_6205 0
N#endif
N
N#ifdef CHIP_6211
S  #undef CHIP_6211
S  #define CHIP_6211 1
N#else
N  #define CHIP_6211 0
N#endif
N
N#ifdef CHIP_6701
N  #undef CHIP_6701
N  #define CHIP_6701 1
N#else
S  #define CHIP_6701 0
N#endif
N
N#ifdef CHIP_6711
S  #undef CHIP_6711
S  #define CHIP_6711 1
N#else
N  #define CHIP_6711 0
N#endif
N
N#ifdef CHIP_6712
S  #undef CHIP_6712
S  #define CHIP_6712 1
N#else
N  #define CHIP_6712 0
N#endif
N
N
N#ifdef CHIP_6713
S  #undef CHIP_6713
S  #define CHIP_6713 1
N#else
N  #define CHIP_6713 0
N#endif
N
N#ifdef CHIP_DA610
S  #undef CHIP_DA610
S  #define CHIP_DA610 1
N#else
N  #define CHIP_DA610 0
N#endif
N
N#ifdef CHIP_DM642
S  #undef CHIP_DM642
S  #define CHIP_DM642 1
N#else
N  #define CHIP_DM642 0
N#endif
N
N#ifdef CHIP_DM641
S  #undef CHIP_DM641
S  #define CHIP_DM641 1
N#else
N  #define CHIP_DM641 0
N#endif
N
N#ifdef CHIP_DM640
S  #undef CHIP_DM640
S  #define CHIP_DM640 1
N#else
N  #define CHIP_DM640 0
N#endif
N
N#ifdef CHIP_6412
S  #undef CHIP_6412
S  #define CHIP_6412 1
N#else
N  #define CHIP_6412 0
N#endif
N
N#ifdef CHIP_6414
S  #undef CHIP_6414
S  #define CHIP_6414 1
N#else
N  #define CHIP_6414 0
N#endif
N
N#ifdef CHIP_6415
S  #undef CHIP_6415
S  #define CHIP_6415 1
N#else
N  #define CHIP_6415 0
N#endif
N
N#ifdef CHIP_6416
S  #undef CHIP_6416
S  #define CHIP_6416 1
N#else
N  #define CHIP_6416 0
N#endif
N
N#ifdef CHIP_6711C
S  #undef CHIP_6711C
S  #define CHIP_6711C 1
N#else
N  #define CHIP_6711C 0
N#endif
N
N#ifdef CHIP_6712C
S  #undef CHIP_6712C
S  #define CHIP_6712C 1
N#else
N  #define CHIP_6712C 0
N#endif
N
N#ifdef CHIP_6411
S  #undef CHIP_6411
S  #define CHIP_6411 1
N#else
N  #define CHIP_6411 0
N#endif
N
N/* next two are DRI300 versions */
N#ifdef CHIP_6410
S  #undef CHIP_6410
S  #define CHIP_6410 1
N#else
N  #define CHIP_6410 0
N#endif
N
N#ifdef CHIP_6413
S  #undef CHIP_6413
S  #define CHIP_6413 1
N#else
N  #define CHIP_6413 0
N#endif
N
N#ifdef CHIP_6418
S  #undef CHIP_6418
S  #define CHIP_6418 1
N#else
N  #define CHIP_6418 0
N#endif
N
N#define CHIP_OROFALL (\
N  CHIP_6201 | \
N  CHIP_6202 | \
N  CHIP_6203 | \
N  CHIP_6204 | \
N  CHIP_6205 | \
N  CHIP_6211 | \
N  CHIP_6701 | \
N  CHIP_6711 | \
N  CHIP_6712 | \
N  CHIP_6713 | \
N  CHIP_DA610 | \
N  CHIP_DM642 | \
N  CHIP_DM641 | \
N  CHIP_DM640 | \
N  CHIP_6412 | \
N  CHIP_6414 | \
N  CHIP_6415 | \
N  CHIP_6416 |  \
N  CHIP_6711C | \
N  CHIP_6712C | \
N  CHIP_6411 |\
N  CHIP_6410 |\
N  CHIP_6413 |\
N  CHIP_6418 \
N)
X#define CHIP_OROFALL (  CHIP_6201 |   CHIP_6202 |   CHIP_6203 |   CHIP_6204 |   CHIP_6205 |   CHIP_6211 |   CHIP_6701 |   CHIP_6711 |   CHIP_6712 |   CHIP_6713 |   CHIP_DA610 |   CHIP_DM642 |   CHIP_DM641 |   CHIP_DM640 |   CHIP_6412 |   CHIP_6414 |   CHIP_6415 |   CHIP_6416 |    CHIP_6711C |   CHIP_6712C |   CHIP_6411 |  CHIP_6410 |  CHIP_6413 |  CHIP_6418 )
N
N#if (CHIP_OROFALL==0)
X#if (( 0 | 0 | 0 | 0 | 0 | 0 | 1 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 )==0)
S  #error NO CHIP DEFINED (use -dCHIP_XXXX where XXXX is chip number, i.e. 6201)
N#endif
N
N#define CHIP_NONE 0
N
N#define CHIP_SUPPORT(c0,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,c24) ( \
N  ( c0*CHIP_6201)  | \
N  ( c1*CHIP_6202)  | \
N  ( c2*CHIP_6203)  | \
N  ( c3*CHIP_6204)  | \
N  ( c4*CHIP_6205)  | \
N  ( c5*CHIP_6211)  | \
N  ( c6*CHIP_6701)  | \
N  ( c7*CHIP_6711)  | \
N  ( c8*CHIP_6712)  | \
N  ( c9*CHIP_6713)  | \
N  ( c10*CHIP_DA610)| \
N  ( c11*CHIP_DM642)| \
N  ( c12*CHIP_DM641)| \
N  ( c13*CHIP_DM640)| \
N  ( c14*CHIP_6412) | \
N  ( c15*CHIP_6414) | \
N  ( c16*CHIP_6415) | \
N  ( c17*CHIP_6416) | \
N  ( c18*CHIP_6711C) | \
N  ( c19*CHIP_6712C) | \
N  ( c20*CHIP_6411) | \
N  ( c21*CHIP_6410) | \
N  ( c22*CHIP_6413) | \
N  ( c23*CHIP_6418) | \
N  ( c24*CHIP_NONE)   \
N )
X#define CHIP_SUPPORT(c0,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20,c21,c22,c23,c24) (   ( c0*CHIP_6201)  |   ( c1*CHIP_6202)  |   ( c2*CHIP_6203)  |   ( c3*CHIP_6204)  |   ( c4*CHIP_6205)  |   ( c5*CHIP_6211)  |   ( c6*CHIP_6701)  |   ( c7*CHIP_6711)  |   ( c8*CHIP_6712)  |   ( c9*CHIP_6713)  |   ( c10*CHIP_DA610)|   ( c11*CHIP_DM642)|   ( c12*CHIP_DM641)|   ( c13*CHIP_DM640)|   ( c14*CHIP_6412) |   ( c15*CHIP_6414) |   ( c16*CHIP_6415) |   ( c17*CHIP_6416) |   ( c18*CHIP_6711C) |   ( c19*CHIP_6712C) |   ( c20*CHIP_6411) |   ( c21*CHIP_6410) |   ( c22*CHIP_6413) |   ( c23*CHIP_6418) |   ( c24*CHIP_NONE)    )
N
N/*---------------------------------------------------------------------------------------*/
N/*                                     6 6 6 6 6 6 6 6 6 6 D D D D 6 6 6 6 6 6 6 6 6 6 N */
N/*                                     2 2 2 2 2 2 7 7 7 7 A M M M 4 4 4 4 7 7 4 4 4 4 O */
N/*                                     0 0 0 0 0 1 0 1 1 1 6 6 6 6 1 1 1 1 1 1 1 1 1 1 N */
N/*                                     1 2 3 4 5 1 1 1 2 3 1 4 4 4 2 4 5 6 1 2 1 0 3 8 E */
N/*                                                         0 2 1 0         C C           */
N/*---------------------------------------------------------------------------------------*/
N#define CACHE_SUPPORT     CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define DMA_SUPPORT       CHIP_SUPPORT(1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
N#define EDMA_SUPPORT      CHIP_SUPPORT(0,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define EMIF_SUPPORT      CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,0,0,0,0,0)
N#define EMIFA_SUPPORT     CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,1,1,1,1,0)
N#define EMIFB_SUPPORT     CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0)
N#define GPIO_SUPPORT      CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define HPI_SUPPORT       CHIP_SUPPORT(1,0,0,0,0,1,1,1,0,1,1,1,1,0,1,1,1,1,1,0,1,1,1,1,0)
N#define I2C_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,1,1,1,0)
N#define IRQ_SUPPORT       CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define MCASP_SUPPORT     CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,0)
N#define MCBSP_SUPPORT     CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define PLL_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,1,1,0,0,0,0,0)
N#define TIMER_SUPPORT     CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define XBUS_SUPPORT      CHIP_SUPPORT(0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
N#define PCI_SUPPORT       CHIP_SUPPORT(0,0,0,0,1,0,0,0,0,0,0,1,0,0,1,0,1,1,0,0,1,0,0,0,0)
N/*---------------------------------------------------------------------------------------*/
N/*                                     6 6 6 6 6 6 6 6 6 6 D D D D 6 6 6 6 6 6 6 6 6 6 N */
N/*                                     2 2 2 2 2 2 7 7 7 7 A M M M 4 4 4 4 7 7 4 4 4 4 O */
N/*                                     0 0 0 0 0 1 0 1 1 1 6 6 6 6 1 1 1 1 1 1 1 1 1 1 N */
N/*                                     1 2 3 4 5 1 1 1 2 3 1 4 4 4 2 4 5 6 1 2 1 0 3 8 E */
N/*                                                         0 2 1 0         C C           */
N/*---------------------------------------------------------------------------------------*/
N#define VP_SUPPORT        CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0)
N#define VIC_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0)
N#define DAT_SUPPORT       CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define PWR_SUPPORT       CHIP_SUPPORT(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define UTOP_SUPPORT      CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0)
N#define TCP_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0)
N#define VCP_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,1,0)
N#define EMAC_SUPPORT      CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0)
N#define MDIO_SUPPORT      CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0)
N#define EMU_SUPPORT	  CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0)
N                                                                                       
N#define L2CACHE_SUPPORT   CHIP_SUPPORT(0,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define TC_SUPPORT        CHIP_SUPPORT(0,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0)
N#define FPU_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,1,1,0,0,0,0,0)
N#define C01_SUPPORT       CHIP_SUPPORT(1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
N#define C11_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1,1,0,0,0,0,0)
N#define C64_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,1,1,1,1,0)
N#define ATL_SUPPORT       CHIP_SUPPORT(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0)
N
N#define CACHE_L2_SUPPORT  L2CACHE_SUPPORT
N/*----------------------------------------------------------------------*/
N
N
N/******************************************************************************\
N* module level register/field access macros
X
N\******************************************************************************/
N
N  /* ----------------- */
N  /* FIELD MAKE MACROS */
N  /* ----------------- */
N
N  #define CHIP_FMK(REG,FIELD,x)\
N    _PER_FMK(CHIP,##REG,##FIELD,x)
X  #define CHIP_FMK(REG,FIELD,x)    _PER_FMK(CHIP,##REG,##FIELD,x)
N
N  #define CHIP_FMKS(REG,FIELD,SYM)\
N    _PER_FMKS(CHIP,##REG,##FIELD,##SYM)
X  #define CHIP_FMKS(REG,FIELD,SYM)    _PER_FMKS(CHIP,##REG,##FIELD,##SYM)
N
N
N  /* -------------------------------- */
N  /* RAW REGISTER/FIELD ACCESS MACROS */
N  /* -------------------------------- */
N
N  #define CHIP_CRGET(REG)\
N    _PER_CRGET(CHIP,##REG)
X  #define CHIP_CRGET(REG)    _PER_CRGET(CHIP,##REG)
N
N  #define CHIP_CRSET(REG,x)\
N    _PER_CRSET(CHIP,##REG,x)
X  #define CHIP_CRSET(REG,x)    _PER_CRSET(CHIP,##REG,x)
N
N  #define CHIP_RGET(REG)\
N   _PER_RGET(_CHIP_##REG##_ADDR,CHIP,##REG)
X  #define CHIP_RGET(REG)   _PER_RGET(_CHIP_##REG##_ADDR,CHIP,##REG)
N
N  #define CHIP_RSET(REG,x)\
N     _PER_RSET(_CHIP_##REG##_ADDR,CHIP,##REG,x)
X  #define CHIP_RSET(REG,x)     _PER_RSET(_CHIP_##REG##_ADDR,CHIP,##REG,x)
N
N
N  #define CHIP_FGET(REG,FIELD)\
N    _CHIP_##REG##_FGET(##FIELD)
X  #define CHIP_FGET(REG,FIELD)    _CHIP_##REG##_FGET(##FIELD)
N
N  #define CHIP_FSET(REG,FIELD,x)\
N    _CHIP_##REG##_FSET(##FIELD,x)
X  #define CHIP_FSET(REG,FIELD,x)    _CHIP_##REG##_FSET(##FIELD,x)
N
N  #define CHIP_FSETS(REG,FIELD,SYM)\
N    _CHIP_##REG##_FSETS(##FIELD,##SYM)
X  #define CHIP_FSETS(REG,FIELD,SYM)    _CHIP_##REG##_FSETS(##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  C S R            |
N* |___________________|
N*
N* CSR - control/status register
N*
N* FIELDS (msb -> lsb)
N* (r)  CPUID
N* (r)  REVID
N* (rw) PWRD
N* (rc) SAT
N* (r)  EN
N* (rw) PCC
N* (rw) DCC
N* (rw) PGIE
N* (rw) GIE
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int CSR;
N
N  #define _CHIP_CSR_CPUID_MASK         0xFF000000u
N  #define _CHIP_CSR_CPUID_SHIFT        0x00000018u
N  #define  CHIP_CSR_CPUID_DEFAULT      0x00000000u
N  #define  CHIP_CSR_CPUID_OF(x)        _VALUEOF(x)
N  #define  CHIP_CSR_CPUID_C62X         0x00000000u
N  #define  CHIP_CSR_CPUID_C67X         0x00000002u
N  #define  CHIP_CSR_CPUID_C64X         0x00000004u
N
N  #define _CHIP_CSR_REVID_MASK         0x00FF0000u
N  #define _CHIP_CSR_REVID_SHIFT        0x00000010u
N  #define  CHIP_CSR_REVID_DEFAULT      0x00000000u
N  #define  CHIP_CSR_REVID_OF(x)        _VALUEOF(x)
N  #define  CHIP_CSR_REVID_620120       0x00000001u
N  #define  CHIP_CSR_REVID_620121       0x00000001u
N  #define  CHIP_CSR_REVID_620130       0x00000002u
N  #define  CHIP_CSR_REVID_670100       0x00000201u
N  #define  CHIP_CSR_REVID_670110       0x00000202u
N  #define  CHIP_CSR_REVID_621110       0x00000002u
N  #define  CHIP_CSR_REVID_640010       0x00000801u
N  #define  CHIP_CSR_REVID_6202         0x00000002u
N  #define  CHIP_CSR_REVID_6202B        0x00000003u
N  #define  CHIP_CSR_REVID_6711         0x00000002u
N  #define  CHIP_CSR_REVID_6711C        0x00000003u
N  #define  CHIP_CSR_REVID_6712         0x00000002u
N  #define  CHIP_CSR_REVID_6712C        0x00000003u
N
N  #define _CHIP_CSR_PWRD_MASK          0x0000FC00u
N  #define _CHIP_CSR_PWRD_SHIFT         0x0000000Au
N  #define  CHIP_CSR_PWRD_DEFAULT       0x00000000u
N  #define  CHIP_CSR_PWRD_OF(x)         _VALUEOF(x)
N  #define  CHIP_CSR_PWRD_NONE          0x00000000u
N  #define  CHIP_CSR_PWRD_PD1A          0x00000009u
N  #define  CHIP_CSR_PWRD_PD1B          0x00000011u
N  #define  CHIP_CSR_PWRD_PD2           0x0000001Au
N  #define  CHIP_CSR_PWRD_PD3           0x0000001Cu
N
N  #define _CHIP_CSR_SAT_MASK           0x00000200u
N  #define _CHIP_CSR_SAT_SHIFT          0x00000009u
N  #define  CHIP_CSR_SAT_DEFAULT        0x00000000u
N  #define  CHIP_CSR_SAT_OF(x)          _VALUEOF(x)
N  #define  CHIP_CSR_SAT_0              0x00000000u
N  #define  CHIP_CSR_SAT_1              0x00000001u
N
N  #define _CHIP_CSR_EN_MASK            0x00000100u
N  #define _CHIP_CSR_EN_SHIFT           0x00000008u
N  #define  CHIP_CSR_EN_DEFAULT         0x00000000u
N  #define  CHIP_CSR_EN_OF(x)           _VALUEOF(x)
N  #define  CHIP_CSR_EN_BIG             0x00000000u
N  #define  CHIP_CSR_EN_LITTLE          0x00000001u
N
N  #define _CHIP_CSR_PCC_MASK           0x000000E0u
N  #define _CHIP_CSR_PCC_SHIFT          0x00000005u
N  #define  CHIP_CSR_PCC_DEFAULT        0x00000000u
N  #define  CHIP_CSR_PCC_OF(x)          _VALUEOF(x)
N  #define  CHIP_CSR_PCC_MAPPED         0x00000000u
N  #define  CHIP_CSR_PCC_ENABLE         0x00000002u
N  #define  CHIP_CSR_PCC_FREEZE         0x00000003u
N  #define  CHIP_CSR_PCC_BYPASS         0x00000004u
N
N  #define _CHIP_CSR_DCC_MASK           0x0000001Cu
N  #define _CHIP_CSR_DCC_SHIFT          0x00000002u
N  #define  CHIP_CSR_DCC_DEFAULT        0x00000000u
N  #define  CHIP_CSR_DCC_OF(x)          _VALUEOF(x)
N  #define  CHIP_CSR_DCC_MAPPED         0x00000000u
N  #define  CHIP_CSR_DCC_ENABLE         0x00000002u
N  #define  CHIP_CSR_DCC_FREEZE         0x00000003u
N  #define  CHIP_CSR_DCC_BYPASS         0x00000004u
N
N  #define _CHIP_CSR_PGIE_MASK          0x00000002u
N  #define _CHIP_CSR_PGIE_SHIFT         0x00000001u
N  #define  CHIP_CSR_PGIE_DEFAULT       0x00000000u
N  #define  CHIP_CSR_PGIE_OF(x)         _VALUEOF(x)
N  #define  CHIP_CSR_PGIE_0             0x00000000u
N  #define  CHIP_CSR_PGIE_1             0x00000001u
N
N  #define _CHIP_CSR_GIE_MASK           0x00000001u
N  #define _CHIP_CSR_GIE_SHIFT          0x00000000u
N  #define  CHIP_CSR_GIE_DEFAULT        0x00000000u
N  #define  CHIP_CSR_GIE_OF(x)          _VALUEOF(x)
N  #define  CHIP_CSR_GIE_0              0x00000000u
N  #define  CHIP_CSR_GIE_1              0x00000001u
N
N  #define  CHIP_CSR_OF(x)              _VALUEOF(x)
N
N  #define CHIP_CSR_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,CSR,CPUID) \
N    |_PER_FDEFAULT(CHIP,CSR,REVID) \
N    |_PER_FDEFAULT(CHIP,CSR,PWRD) \
N    |_PER_FDEFAULT(CHIP,CSR,SAT) \
N    |_PER_FDEFAULT(CHIP,CSR,EN) \
N    |_PER_FDEFAULT(CHIP,CSR,PCC) \
N    |_PER_FDEFAULT(CHIP,CSR,DCC) \
N    |_PER_FDEFAULT(CHIP,CSR,PGIE) \
N    |_PER_FDEFAULT(CHIP,CSR,GIE) \
N  )
X  #define CHIP_CSR_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,CSR,CPUID)     |_PER_FDEFAULT(CHIP,CSR,REVID)     |_PER_FDEFAULT(CHIP,CSR,PWRD)     |_PER_FDEFAULT(CHIP,CSR,SAT)     |_PER_FDEFAULT(CHIP,CSR,EN)     |_PER_FDEFAULT(CHIP,CSR,PCC)     |_PER_FDEFAULT(CHIP,CSR,DCC)     |_PER_FDEFAULT(CHIP,CSR,PGIE)     |_PER_FDEFAULT(CHIP,CSR,GIE)   )
N
N  #define CHIP_CSR_RMK(pwrd,pcc,dcc,pgie,gie) (Uint32)( \
N     _PER_FMK(CHIP,CSR,PWRD,pwrd) \
N    |_PER_FMK(CHIP,CSR,PCC,pcc) \
N    |_PER_FMK(CHIP,CSR,DCC,dcc) \
N    |_PER_FMK(CHIP,CSR,PGIE,pgie) \
N    |_PER_FMK(CHIP,CSR,GIE,gie) \
N  )
X  #define CHIP_CSR_RMK(pwrd,pcc,dcc,pgie,gie) (Uint32)(      _PER_FMK(CHIP,CSR,PWRD,pwrd)     |_PER_FMK(CHIP,CSR,PCC,pcc)     |_PER_FMK(CHIP,CSR,DCC,dcc)     |_PER_FMK(CHIP,CSR,PGIE,pgie)     |_PER_FMK(CHIP,CSR,GIE,gie)   )
N
N  #define _CHIP_CSR_FGET(FIELD)\
N    _PER_CFGET(CHIP,CSR,##FIELD)
X  #define _CHIP_CSR_FGET(FIELD)    _PER_CFGET(CHIP,CSR,##FIELD)
N
N  #define _CHIP_CSR_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,CSR,##FIELD,field)
X  #define _CHIP_CSR_FSET(FIELD,field)    _PER_CFSET(CHIP,CSR,##FIELD,field)
N
N  #define _CHIP_CSR_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,CSR,##FIELD,##SYM)
X  #define _CHIP_CSR_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,CSR,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I F R            |
N* |___________________|
N*
N* IFR - interruppt flag register
N*
N* FIELDS (msb -> lsb)
N* (rw) IF
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int IFR;
N
N  #define _CHIP_IFR_IF_MASK          0x0000FFFFu
N  #define _CHIP_IFR_IF_SHIFT         0x00000000u
N  #define  CHIP_IFR_IF_DEFAULT       0x00000000u
N  #define  CHIP_IFR_IF_OF(x)         _VALUEOF(x)
N
N  #define  CHIP_IFR_OF(x)            _VALUEOF(x)
N
N  #define CHIP_IFR_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,IFR,IF)\
N  )
X  #define CHIP_IFR_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,IFR,IF)  )
N
N  #define CHIP_IFR_RMK(if) (Uint32)( \
N     _PER_FMK(CHIP,IFR,IF,if)\
N  )
X  #define CHIP_IFR_RMK(if) (Uint32)(      _PER_FMK(CHIP,IFR,IF,if)  )
N
N  #define _CHIP_IFR_FGET(FIELD)\
N    _PER_CFGET(CHIP,IFR,##FIELD)
X  #define _CHIP_IFR_FGET(FIELD)    _PER_CFGET(CHIP,IFR,##FIELD)
N
N  #define _CHIP_IFR_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,IFR,##FIELD,field)
X  #define _CHIP_IFR_FSET(FIELD,field)    _PER_CFSET(CHIP,IFR,##FIELD,field)
N
N  #define _CHIP_IFR_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,IFR,##FIELD,##SYM)
X  #define _CHIP_IFR_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,IFR,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I S R            |
N* |___________________|
N*
N* ISR - interruppt set register
N*
N* FIELDS (msb -> lsb)
N* (w) IS
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int ISR;
N
N  #define _CHIP_ISR_IS_MASK          0x0000FFFFu
N  #define _CHIP_ISR_IS_SHIFT         0x00000000u
N  #define  CHIP_ISR_IS_DEFAULT       0x00000000u
N  #define  CHIP_ISR_IS_OF(x)         _VALUEOF(x)
N
N  #define  CHIP_ISR_OF(x)            _VALUEOF(x)
N
N  #define CHIP_ISR_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,ISR,IS)\
N  )
X  #define CHIP_ISR_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,ISR,IS)  )
N
N  #define CHIP_ISR_RMK(is) (Uint32)( \
N     _PER_FMK(CHIP,ISR,IS,is)\
N  )
X  #define CHIP_ISR_RMK(is) (Uint32)(      _PER_FMK(CHIP,ISR,IS,is)  )
N
N  #define _CHIP_ISR_FGET(FIELD)\
N    _PER_CFGET(CHIP,ISR,##FIELD)
X  #define _CHIP_ISR_FGET(FIELD)    _PER_CFGET(CHIP,ISR,##FIELD)
N
N  #define _CHIP_ISR_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,ISR,##FIELD,field)
X  #define _CHIP_ISR_FSET(FIELD,field)    _PER_CFSET(CHIP,ISR,##FIELD,field)
N
N  #define _CHIP_ISR_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,ISR,##FIELD,##SYM)
X  #define _CHIP_ISR_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,ISR,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I C R            |
N* |___________________|
N*
N* ICR - interruppt clear register
N*
N* FIELDS (msb -> lsb)
N* (w) IC
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int ICR;
N
N  #define _CHIP_ICR_IC_MASK          0x0000FFFFu
N  #define _CHIP_ICR_IC_SHIFT         0x00000000u
N  #define  CHIP_ICR_IC_DEFAULT       0x00000000u
N  #define  CHIP_ICR_IC_OF(x)         _VALUEOF(x)
N
N  #define  CHIP_ICR_OF(x)            _VALUEOF(x)
N
N  #define CHIP_ICR_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,ICR,IC)\
N  )
X  #define CHIP_ICR_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,ICR,IC)  )
N
N  #define CHIP_ICR_RMK(ic) (Uint32)( \
N     _PER_FMK(CHIP,ICR,IC,ic)\
N  )
X  #define CHIP_ICR_RMK(ic) (Uint32)(      _PER_FMK(CHIP,ICR,IC,ic)  )
N
N  #define _CHIP_ICR_FGET(FIELD)\
N    _PER_CFGET(CHIP,ICR,##FIELD)
X  #define _CHIP_ICR_FGET(FIELD)    _PER_CFGET(CHIP,ICR,##FIELD)
N
N  #define _CHIP_ICR_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,ICR,##FIELD,field)
X  #define _CHIP_ICR_FSET(FIELD,field)    _PER_CFSET(CHIP,ICR,##FIELD,field)
N
N  #define _CHIP_ICR_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,ICR,##FIELD,##SYM)
X  #define _CHIP_ICR_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,ICR,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I E R            |
N* |___________________|
N*
N* IER - interruppt enable register
N*
N* FIELDS (msb -> lsb)
N* (rw) IE
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int IER;
N
N  #define _CHIP_IER_IE_MASK          0x0000FFFFu
N  #define _CHIP_IER_IE_SHIFT         0x00000000u
N  #define  CHIP_IER_IE_DEFAULT       0x00000000u
N  #define  CHIP_IER_IE_OF(x)         _VALUEOF(x)
N
N  #define  CHIP_IER_OF(x)            _VALUEOF(x)
N
N  #define CHIP_IER_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,IER,IE)\
N  )
X  #define CHIP_IER_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,IER,IE)  )
N
N  #define CHIP_IER_RMK(ie) (Uint32)( \
N     _PER_FMK(CHIP,IER,IE,ie)\
N  )
X  #define CHIP_IER_RMK(ie) (Uint32)(      _PER_FMK(CHIP,IER,IE,ie)  )
N
N  #define _CHIP_IER_FGET(FIELD)\
N    _PER_CFGET(CHIP,IER,##FIELD)
X  #define _CHIP_IER_FGET(FIELD)    _PER_CFGET(CHIP,IER,##FIELD)
N
N  #define _CHIP_IER_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,IER,##FIELD,field)
X  #define _CHIP_IER_FSET(FIELD,field)    _PER_CFSET(CHIP,IER,##FIELD,field)
N
N  #define _CHIP_IER_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,IER,##FIELD,##SYM)
X  #define _CHIP_IER_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,IER,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I S T P          |
N* |___________________|
N*
N* ISTP - interrupt service table pointer
N*
N* FIELDS (msb -> lsb)
N* (r)   HPEINT
N* (rw)  ISTB
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int ISTP;
N
N  #define _CHIP_ISTP_ISTB_MASK         0xFFFFFC00u
N  #define _CHIP_ISTP_ISTB_SHIFT        0x0000000Au
N  #define  CHIP_ISTP_ISTB_DEFAULT      0x00000000u
N  #define  CHIP_ISTP_ISTB_OF(x)        _VALUEOF(x)
N
N  #define _CHIP_ISTP_HPEINT_MASK       0x000003E0u
N  #define _CHIP_ISTP_HPEINT_SHIFT      0x00000005u
N  #define  CHIP_ISTP_HPEINT_DEFAULT    0x00000000u
N  #define  CHIP_ISTP_HPEINT_OF(x)      _VALUEOF(x)
N
N  #define  CHIP_ISTP_OF(x)             _VALUEOF(x)
N
N  #define CHIP_ISTP_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,ISTP,ISTB)\
N    |_PER_FDEFAULT(CHIP,ISTP,HPEINT)\
N  )
X  #define CHIP_ISTP_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,ISTP,ISTB)    |_PER_FDEFAULT(CHIP,ISTP,HPEINT)  )
N
N  #define CHIP_ISTP_RMK(istb) (Uint32)( \
N     _PER_FMK(CHIP,ISTP,ISTB,istb)\
N  )
X  #define CHIP_ISTP_RMK(istb) (Uint32)(      _PER_FMK(CHIP,ISTP,ISTB,istb)  )
N
N  #define _CHIP_ISTP_FGET(FIELD)\
N    _PER_CFGET(CHIP,ISTP,##FIELD)
X  #define _CHIP_ISTP_FGET(FIELD)    _PER_CFGET(CHIP,ISTP,##FIELD)
N
N  #define _CHIP_ISTP_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,ISTP,##FIELD,field)
X  #define _CHIP_ISTP_FSET(FIELD,field)    _PER_CFSET(CHIP,ISTP,##FIELD,field)
N
N  #define _CHIP_ISTP_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,ISTP,##FIELD,##SYM)
X  #define _CHIP_ISTP_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,ISTP,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  I R P            |
N* |___________________|
N*
N* IRP - interrupt return pointer
N*
N* FIELDS (msb -> lsb)
N* (rw) IRP
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int IRP;
N
N  #define _CHIP_IRP_IRP_MASK           0xFFFFFFFFu
N  #define _CHIP_IRP_IRP_SHIFT          0x00000000u
N  #define  CHIP_IRP_IRP_DEFAULT        0x00000000u
N  #define  CHIP_IRP_IRP_OF(x)          _VALUEOF(x)
N
N  #define  CHIP_IRP_OF(x)              _VALUEOF(x)
N
N  #define CHIP_IRP_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,IRP,IRP)\
N  )
X  #define CHIP_IRP_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,IRP,IRP)  )
N
N  #define CHIP_IRP_RMK(irp) (Uint32)( \
N     _PER_FMK(CHIP,IRP,IRP,irp)\
N  )
X  #define CHIP_IRP_RMK(irp) (Uint32)(      _PER_FMK(CHIP,IRP,IRP,irp)  )
N
N  #define _CHIP_IRP_FGET(FIELD)\
N    _PER_CFGET(CHIP,IRP,##FIELD)
X  #define _CHIP_IRP_FGET(FIELD)    _PER_CFGET(CHIP,IRP,##FIELD)
N
N  #define _CHIP_IRP_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,IRP,##FIELD,field)
X  #define _CHIP_IRP_FSET(FIELD,field)    _PER_CFSET(CHIP,IRP,##FIELD,field)
N
N  #define _CHIP_IRP_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,IRP,##FIELD,##SYM)
X  #define _CHIP_IRP_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,IRP,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  N R P            |
N* |___________________|
N*
N* NRP - non-maskable interrupt return pointer
N*
N* FIELDS (msb -> lsb)
N* (rw) NRP
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int NRP;
N
N  #define _CHIP_NRP_NRP_MASK           0xFFFFFFFFu
N  #define _CHIP_NRP_NRP_SHIFT          0x00000000u
N  #define  CHIP_NRP_NRP_DEFAULT        0x00000000u
N  #define  CHIP_NRP_NRP_OF(x)          _VALUEOF(x)
N
N  #define  CHIP_NRP_OF(x)              _VALUEOF(x)
N
N  #define CHIP_NRP_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,NRP,NRP)\
N  )
X  #define CHIP_NRP_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,NRP,NRP)  )
N
N  #define CHIP_NRP_RMK(nrp) (Uint32)( \
N     _PER_FMK(CHIP,NRP,NRP,nrp)\
N  )
X  #define CHIP_NRP_RMK(nrp) (Uint32)(      _PER_FMK(CHIP,NRP,NRP,nrp)  )
N
N  #define _CHIP_NRP_FGET(FIELD)\
N    _PER_CFGET(CHIP,NRP,##FIELD)
X  #define _CHIP_NRP_FGET(FIELD)    _PER_CFGET(CHIP,NRP,##FIELD)
N
N  #define _CHIP_NRP_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,NRP,##FIELD,field)
X  #define _CHIP_NRP_FSET(FIELD,field)    _PER_CFSET(CHIP,NRP,##FIELD,field)
N
N  #define _CHIP_NRP_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,NRP,##FIELD,##SYM)
X  #define _CHIP_NRP_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,NRP,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  A M R            |
N* |___________________|
N*
N* AMR - addressing mode register
N*
N* FIELDS (msb -> lsb)
N* (rw) BK1
N* (rw) BK0
N* (rw) B7MODE
N* (rw) B6MODE
N* (rw) B5MODE
N* (rw) B4MODE
N* (rw) A7MODE
N* (rw) A6MODE
N* (rw) A5MODE
N* (rw) A4MODE
N*
N\******************************************************************************/
N  extern far cregister volatile unsigned int AMR;
N
N  #define _CHIP_AMR_BK1_MASK           0x02E00000u
N  #define _CHIP_AMR_BK1_SHIFT          0x00000015u
N  #define  CHIP_AMR_BK1_DEFAULT        0x00000000u
N  #define  CHIP_AMR_BK1_OF(x)          _VALUEOF(x)
N  #define  CHIP_AMR_BK1_2              0x00000000u
N  #define  CHIP_AMR_BK1_4              0x00000001u
N  #define  CHIP_AMR_BK1_8              0x00000002u
N  #define  CHIP_AMR_BK1_16             0x00000003u
N  #define  CHIP_AMR_BK1_32             0x00000004u
N  #define  CHIP_AMR_BK1_64             0x00000005u
N  #define  CHIP_AMR_BK1_128            0x00000006u
N  #define  CHIP_AMR_BK1_256            0x00000007u
N  #define  CHIP_AMR_BK1_512            0x00000008u
N  #define  CHIP_AMR_BK1_1K             0x00000009u
N  #define  CHIP_AMR_BK1_2K             0x0000000Au
N  #define  CHIP_AMR_BK1_4K             0x0000000Bu
N  #define  CHIP_AMR_BK1_8K             0x0000000Cu
N  #define  CHIP_AMR_BK1_16K            0x0000000Du
N  #define  CHIP_AMR_BK1_32K            0x0000000Eu
N  #define  CHIP_AMR_BK1_64K            0x0000000Fu
N  #define  CHIP_AMR_BK1_128K           0x00000010u
N  #define  CHIP_AMR_BK1_256K           0x00000011u
N  #define  CHIP_AMR_BK1_512K           0x00000012u
N  #define  CHIP_AMR_BK1_1M             0x00000013u
N  #define  CHIP_AMR_BK1_2M             0x00000014u
N  #define  CHIP_AMR_BK1_4M             0x00000015u
N  #define  CHIP_AMR_BK1_8M             0x00000016u
N  #define  CHIP_AMR_BK1_16M            0x00000017u
N  #define  CHIP_AMR_BK1_32M            0x00000018u
N  #define  CHIP_AMR_BK1_64M            0x00000019u
N  #define  CHIP_AMR_BK1_128M           0x0000001Au
N  #define  CHIP_AMR_BK1_256M           0x0000001Bu
N  #define  CHIP_AMR_BK1_512M           0x0000001Cu
N  #define  CHIP_AMR_BK1_1G             0x0000001Du
N  #define  CHIP_AMR_BK1_2G             0x0000001Eu
N  #define  CHIP_AMR_BK1_4G             0x0000001Fu
N
N  #define _CHIP_AMR_BK0_MASK           0x001F0000u
N  #define _CHIP_AMR_BK0_SHIFT          0x00000010u
N  #define  CHIP_AMR_BK0_DEFAULT        0x00000000u
N  #define  CHIP_AMR_BK0_OF(x)          _VALUEOF(x)
N  #define  CHIP_AMR_BK0_2              0x00000000u
N  #define  CHIP_AMR_BK0_4              0x00000001u
N  #define  CHIP_AMR_BK0_8              0x00000002u
N  #define  CHIP_AMR_BK0_16             0x00000003u
N  #define  CHIP_AMR_BK0_32             0x00000004u
N  #define  CHIP_AMR_BK0_64             0x00000005u
N  #define  CHIP_AMR_BK0_128            0x00000006u
N  #define  CHIP_AMR_BK0_256            0x00000007u
N  #define  CHIP_AMR_BK0_512            0x00000008u
N  #define  CHIP_AMR_BK0_1K             0x00000009u
N  #define  CHIP_AMR_BK0_2K             0x0000000Au
N  #define  CHIP_AMR_BK0_4K             0x0000000Bu
N  #define  CHIP_AMR_BK0_8K             0x0000000Cu
N  #define  CHIP_AMR_BK0_16K            0x0000000Du
N  #define  CHIP_AMR_BK0_32K            0x0000000Eu
N  #define  CHIP_AMR_BK0_64K            0x0000000Fu
N  #define  CHIP_AMR_BK0_128K           0x00000010u
N  #define  CHIP_AMR_BK0_256K           0x00000011u
N  #define  CHIP_AMR_BK0_512K           0x00000012u
N  #define  CHIP_AMR_BK0_1M             0x00000013u
N  #define  CHIP_AMR_BK0_2M             0x00000014u
N  #define  CHIP_AMR_BK0_4M             0x00000015u
N  #define  CHIP_AMR_BK0_8M             0x00000016u
N  #define  CHIP_AMR_BK0_16M            0x00000017u
N  #define  CHIP_AMR_BK0_32M            0x00000018u
N  #define  CHIP_AMR_BK0_64M            0x00000019u
N  #define  CHIP_AMR_BK0_128M           0x0000001Au
N  #define  CHIP_AMR_BK0_256M           0x0000001Bu
N  #define  CHIP_AMR_BK0_512M           0x0000001Cu
N  #define  CHIP_AMR_BK0_1G             0x0000001Du
N  #define  CHIP_AMR_BK0_2G             0x0000001Eu
N  #define  CHIP_AMR_BK0_4G             0x0000001Fu
N
N
N  #define _CHIP_AMR_B7MODE_MASK        0x0000C000u
N  #define _CHIP_AMR_B7MODE_SHIFT       0x0000000Eu
N  #define  CHIP_AMR_B7MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_B7MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_B7MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_B7MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_B7MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_B6MODE_MASK        0x00003000u
N  #define _CHIP_AMR_B6MODE_SHIFT       0x0000000Cu
N  #define  CHIP_AMR_B6MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_B6MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_B6MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_B6MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_B6MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_B5MODE_MASK        0x00000C00u
N  #define _CHIP_AMR_B5MODE_SHIFT       0x0000000Au
N  #define  CHIP_AMR_B5MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_B5MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_B5MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_B5MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_B5MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_B4MODE_MASK        0x00000300u
N  #define _CHIP_AMR_B4MODE_SHIFT       0x00000008u
N  #define  CHIP_AMR_B4MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_B4MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_B4MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_B4MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_B4MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_A7MODE_MASK        0x000000C0u
N  #define _CHIP_AMR_A7MODE_SHIFT       0x00000006u
N  #define  CHIP_AMR_A7MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_A7MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_A7MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_A7MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_A7MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_A6MODE_MASK        0x00000030u
N  #define _CHIP_AMR_A6MODE_SHIFT       0x00000004u
N  #define  CHIP_AMR_A6MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_A6MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_A6MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_A6MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_A6MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_A5MODE_MASK        0x0000000Cu
N  #define _CHIP_AMR_A5MODE_SHIFT       0x00000002u
N  #define  CHIP_AMR_A5MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_A5MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_A5MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_A5MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_A5MODE_CIRCULAR1   0x00000002u
N
N  #define _CHIP_AMR_A4MODE_MASK        0x00000003u
N  #define _CHIP_AMR_A4MODE_SHIFT       0x00000000u
N  #define  CHIP_AMR_A4MODE_DEFAULT     0x00000000u
N  #define  CHIP_AMR_A4MODE_OF(x)       _VALUEOF(x)
N  #define  CHIP_AMR_A4MODE_LINEAR      0x00000000u
N  #define  CHIP_AMR_A4MODE_CIRCULAR0   0x00000001u
N  #define  CHIP_AMR_A4MODE_CIRCULAR1   0x00000002u
N
N  #define  CHIP_AMR_OF(x)              _VALUEOF(x)
N
N  #define CHIP_AMR_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(CHIP,AMR,BK1)\
N    |_PER_FDEFAULT(CHIP,AMR,BK0)\
N    |_PER_FDEFAULT(CHIP,AMR,B7MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,B6MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,B5MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,B4MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,A7MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,A6MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,A5MODE)\
N    |_PER_FDEFAULT(CHIP,AMR,A4MODE)\
N  )
X  #define CHIP_AMR_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,AMR,BK1)    |_PER_FDEFAULT(CHIP,AMR,BK0)    |_PER_FDEFAULT(CHIP,AMR,B7MODE)    |_PER_FDEFAULT(CHIP,AMR,B6MODE)    |_PER_FDEFAULT(CHIP,AMR,B5MODE)    |_PER_FDEFAULT(CHIP,AMR,B4MODE)    |_PER_FDEFAULT(CHIP,AMR,A7MODE)    |_PER_FDEFAULT(CHIP,AMR,A6MODE)    |_PER_FDEFAULT(CHIP,AMR,A5MODE)    |_PER_FDEFAULT(CHIP,AMR,A4MODE)  )
N
N  #define CHIP_AMR_RMK(bk1,bk0,b7mode,b6mode,b5mode,b4mode,a7,ode,\
N    a6mode,a5mode,a4mode) (Uint32)( \
N     _PER_FMK(CHIP,AMR,BK1,bk1)\
N    |_PER_FMK(CHIP,AMR,BK0,bk0)\
N    |_PER_FMK(CHIP,AMR,B7MODE,b7mode)\
N    |_PER_FMK(CHIP,AMR,B6MODE,b6mode)\
N    |_PER_FMK(CHIP,AMR,B5MODE,b5mode)\
N    |_PER_FMK(CHIP,AMR,B4MODE,b4mode)\
N    |_PER_FMK(CHIP,AMR,A7MODE,a7mode)\
N    |_PER_FMK(CHIP,AMR,A6MODE,a6mode)\
N    |_PER_FMK(CHIP,AMR,A5MODE,a5mode)\
N    |_PER_FMK(CHIP,AMR,A4MODE,a4mode)\
N  )
X  #define CHIP_AMR_RMK(bk1,bk0,b7mode,b6mode,b5mode,b4mode,a7,ode,    a6mode,a5mode,a4mode) (Uint32)(      _PER_FMK(CHIP,AMR,BK1,bk1)    |_PER_FMK(CHIP,AMR,BK0,bk0)    |_PER_FMK(CHIP,AMR,B7MODE,b7mode)    |_PER_FMK(CHIP,AMR,B6MODE,b6mode)    |_PER_FMK(CHIP,AMR,B5MODE,b5mode)    |_PER_FMK(CHIP,AMR,B4MODE,b4mode)    |_PER_FMK(CHIP,AMR,A7MODE,a7mode)    |_PER_FMK(CHIP,AMR,A6MODE,a6mode)    |_PER_FMK(CHIP,AMR,A5MODE,a5mode)    |_PER_FMK(CHIP,AMR,A4MODE,a4mode)  )
N
N  #define _CHIP_AMR_FGET(FIELD)\
N    _PER_CFGET(CHIP,AMR,##FIELD)
X  #define _CHIP_AMR_FGET(FIELD)    _PER_CFGET(CHIP,AMR,##FIELD)
N
N  #define _CHIP_AMR_FSET(FIELD,field)\
N    _PER_CFSET(CHIP,AMR,##FIELD,field)
X  #define _CHIP_AMR_FSET(FIELD,field)    _PER_CFSET(CHIP,AMR,##FIELD,field)
N
N  #define _CHIP_AMR_FSETS(FIELD,SYM)\
N    _PER_CFSETS(CHIP,AMR,##FIELD,##SYM)
X  #define _CHIP_AMR_FSETS(FIELD,SYM)    _PER_CFSETS(CHIP,AMR,##FIELD,##SYM)
N
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  D E V C F G      |
N* |___________________|
N*
N* PERCFG - Device Configuration register (1)
N*
N* FIELDS (msb -> lsb)  CHIP_6713/CHIP_DA610
N* (rw) EKSRC
N* (rw) TOUT1SEL
N* (rw) TOUT0SEL
N* (rw) MCBSP0DIS
N* (rw) MCBSP1DIS
N* (rw) GPIO1EN (only for CHIP_DA610)
N*
N* FIELDS (msb -> lsb)  CHIP_DM642
N* (rw) VP2EN
N* (rw) VP1EN
N* (rw) VP0EN
N* (rw) I2C0EN
N* (rw) MCBSP1EN
N* (rw) MCBSP0EN
N* (rw) MCASP0EN
N*
N* FIELDS (msb -> lsb)  CHIP_6412
N* (rw) I2C0EN
N* (rw) MCBSP1EN
N* (rw) MCBSP0EN
N*
N* FIELDS (msb -> lsb)  CHIP_6711C/CHIP_6712C
N* (rw) EKSRC
N*
N* FIELDS (msb -> lsb)  CHIP_6410/CHIP_6413/CHIP_6418
N* (rw) AFCMUX
N* (rw) MCASP1EN
N* (rw) I2C1EN
N* (rw) I2C0EN
N* (r) MCBSP1EN
N* (r) MCBSP0EN
N* (rw) MCASP0EN
N*
N\******************************************************************************/
N
N#if (CHIP_DA610)
X#if (0)
S
S  #define   _CHIP_DEVCFG_ADDR              0x019C0200u
S  #define   _CHIP_DEVCFG_OFFSET             0
S
S  #define _CHIP_DEVCFG_GPIO1EN_MASK        0x00010000u
S  #define _CHIP_DEVCFG_GPIO1EN_SHIFT       0x0000000Fu
S  #define  CHIP_DEVCFG_GPIO1EN_DEFAULT     0x00000000u
S  #define  CHIP_DEVCFG_GPIO1EN_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_GPIO1EN_0           0x00000000u
S  #define  CHIP_DEVCFG_GPIO1EN_1           0x00000001u
S
S  #define _CHIP_DEVCFG_EKSRC_MASK          0x00000010u
S  #define _CHIP_DEVCFG_EKSRC_SHIFT         0x00000004u
S  #define  CHIP_DEVCFG_EKSRC_DEFAULT       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVCFG_EKSRC_SYSCLK3       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_ECLKIN        0x00000001u
S
S  #define _CHIP_DEVCFG_TOUT1SEL_MASK       0x00000008u
S  #define _CHIP_DEVCFG_TOUT1SEL_SHIFT      0x00000003u
S  #define  CHIP_DEVCFG_TOUT1SEL_DEFAULT    0x00000000u
S  #define  CHIP_DEVCFG_TOUT1SEL_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_TOUT1SEL_TOUT1PIN   0x00000000u
S  #define  CHIP_DEVCFG_TOUT1SEL_MCASPPIN   0x00000001u
S
S  #define _CHIP_DEVCFG_TOUT0SEL_MASK       0x00000004u
S  #define _CHIP_DEVCFG_TOUT0SEL_SHIFT      0x00000002u
S  #define  CHIP_DEVCFG_TOUT0SEL_DEFAULT    0x00000000u
S  #define  CHIP_DEVCFG_TOUT0SEL_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_TOUT0SEL_TOUT0PIN   0x00000000u
S  #define  CHIP_DEVCFG_TOUT0SEL_MCASPPIN   0x00000001u
S
S  #define _CHIP_DEVCFG_MCBSP0DIS_MASK       0x00000002u
S  #define _CHIP_DEVCFG_MCBSP0DIS_SHIFT      0x00000001u
S  #define  CHIP_DEVCFG_MCBSP0DIS_DEFAULT    0x00000000u
S  #define  CHIP_DEVCFG_MCBSP0DIS_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_MCBSP0DIS_0          0x00000000u
S  #define  CHIP_DEVCFG_MCBSP0DIS_1          0x00000001u
S
S  #define _CHIP_DEVCFG_MCBSP1DIS_MASK         0x00000001u
S  #define _CHIP_DEVCFG_MCBSP1DIS_SHIFT        0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_DEFAULT      0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_OF(x)        _VALUEOF(x)
S  #define  CHIP_DEVCFG_MCBSP1DIS_0            0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_1            0x00000001u
S
S
S  #define  CHIP_DEVCFG_OF(x)              _VALUEOF(x)
S
S  #define CHIP_DEVCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,DEVCFG,EKSRC) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,TOUT1SEL) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,TOUT0SEL) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP0DIS) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP1DIS) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,GPIO1EN) \
S )
X  #define CHIP_DEVCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,DEVCFG,EKSRC)     |_PER_FDEFAULT(CHIP,DEVCFG,TOUT1SEL)     |_PER_FDEFAULT(CHIP,DEVCFG,TOUT0SEL)     |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP0DIS)     |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP1DIS)     |_PER_FDEFAULT(CHIP,DEVCFG,GPIO1EN)  )
S
S  #define CHIP_DEVCFG_RMK(eksrc,tout1sel,tout0sel,mcbsp0dis,mcbsp1dis,\
S                          gpio1en ) (Uint32)( \
S      _PER_FMK(CHIP,DEVCFG,EKSRC,eksrc) \
S    |_PER_FMK(CHIP,DEVCFG,TOUT1SEL,tout1sel) \
S    |_PER_FMK(CHIP,DEVCFG,TOUT0SEL,tout0sel) \
S    |_PER_FMK(CHIP,DEVCFG,MCBSP0DIS,mcbsp0dis) \
S    |_PER_FMK(CHIP,DEVCFG,MCBSP1DIS,mcbsp1dis) \
S    |_PER_FMK(CHIP,DEVCFG,GPIO1EN,gpio1en) \
S)
X  #define CHIP_DEVCFG_RMK(eksrc,tout1sel,tout0sel,mcbsp0dis,mcbsp1dis,                          gpio1en ) (Uint32)(       _PER_FMK(CHIP,DEVCFG,EKSRC,eksrc)     |_PER_FMK(CHIP,DEVCFG,TOUT1SEL,tout1sel)     |_PER_FMK(CHIP,DEVCFG,TOUT0SEL,tout0sel)     |_PER_FMK(CHIP,DEVCFG,MCBSP0DIS,mcbsp0dis)     |_PER_FMK(CHIP,DEVCFG,MCBSP1DIS,mcbsp1dis)     |_PER_FMK(CHIP,DEVCFG,GPIO1EN,gpio1en) )
S#elif (CHIP_6713)
X#elif (0)
S  #define   _CHIP_DEVCFG_ADDR              0x019C0200u
S  #define   _CHIP_DEVCFG_OFFSET             0
S
S  #define _CHIP_DEVCFG_EKSRC_MASK          0x00000010u
S  #define _CHIP_DEVCFG_EKSRC_SHIFT         0x00000004u
S  #define  CHIP_DEVCFG_EKSRC_DEFAULT       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVCFG_EKSRC_SYSCLK3       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_ECLKIN        0x00000001u
S
S  #define _CHIP_DEVCFG_TOUT1SEL_MASK       0x00000008u
S  #define _CHIP_DEVCFG_TOUT1SEL_SHIFT      0x00000003u
S  #define  CHIP_DEVCFG_TOUT1SEL_DEFAULT    0x00000000u
S  #define  CHIP_DEVCFG_TOUT1SEL_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_TOUT1SEL_TOUT1PIN   0x00000000u
S  #define  CHIP_DEVCFG_TOUT1SEL_MCASPPIN   0x00000001u
S
S  #define _CHIP_DEVCFG_TOUT0SEL_MASK       0x00000004u
S  #define _CHIP_DEVCFG_TOUT0SEL_SHIFT      0x00000002u
S  #define  CHIP_DEVCFG_TOUT0SEL_DEFAULT    0x00000000u
S  #define  CHIP_DEVCFG_TOUT0SEL_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVCFG_TOUT0SEL_TOUT0PIN   0x00000000u
S  #define  CHIP_DEVCFG_TOUT0SEL_MCASPPIN   0x00000001u
S
S  #define _CHIP_DEVCFG_MCBSP0DIS_MASK      0x00000002u
S  #define _CHIP_DEVCFG_MCBSP0DIS_SHIFT     0x00000001u
S  #define  CHIP_DEVCFG_MCBSP0DIS_DEFAULT   0x00000000u
S  #define  CHIP_DEVCFG_MCBSP0DIS_OF(x)     _VALUEOF(x)
S  #define  CHIP_DEVCFG_MCBSP0DIS_0         0x00000000u
S  #define  CHIP_DEVCFG_MCBSP0DIS_1         0x00000001u
S
S  #define _CHIP_DEVCFG_MCBSP1DIS_MASK      0x00000001u
S  #define _CHIP_DEVCFG_MCBSP1DIS_SHIFT     0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_DEFAULT   0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_OF(x)     _VALUEOF(x)
S  #define  CHIP_DEVCFG_MCBSP1DIS_0         0x00000000u
S  #define  CHIP_DEVCFG_MCBSP1DIS_1         0x00000001u
S
S
S  #define  CHIP_DEVCFG_OF(x)              _VALUEOF(x)
S
S  #define CHIP_DEVCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,DEVCFG,EKSRC) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,TOUT1SEL) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,TOUT0SEL) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP0DIS) \
S    |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP1DIS) \
S )
X  #define CHIP_DEVCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,DEVCFG,EKSRC)     |_PER_FDEFAULT(CHIP,DEVCFG,TOUT1SEL)     |_PER_FDEFAULT(CHIP,DEVCFG,TOUT0SEL)     |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP0DIS)     |_PER_FDEFAULT(CHIP,DEVCFG,MCBSP1DIS)  )
S
S  #define CHIP_DEVCFG_RMK(eksrc,tout1sel,tout0sel,mcbsp0dis,mcbsp1dis\
S                          ) (Uint32)( \
S      _PER_FMK(CHIP,DEVCFG,EKSRC,eksrc) \
S    |_PER_FMK(CHIP,DEVCFG,TOUT1SEL,tout1sel) \
S    |_PER_FMK(CHIP,DEVCFG,TOUT0SEL,tout0sel) \
S    |_PER_FMK(CHIP,DEVCFG,MCBSP0DIS,mcbsp0dis) \
S    |_PER_FMK(CHIP,DEVCFG,MCBSP1DIS,mcbsp1dis) \
S)
X  #define CHIP_DEVCFG_RMK(eksrc,tout1sel,tout0sel,mcbsp0dis,mcbsp1dis                          ) (Uint32)(       _PER_FMK(CHIP,DEVCFG,EKSRC,eksrc)     |_PER_FMK(CHIP,DEVCFG,TOUT1SEL,tout1sel)     |_PER_FMK(CHIP,DEVCFG,TOUT0SEL,tout0sel)     |_PER_FMK(CHIP,DEVCFG,MCBSP0DIS,mcbsp0dis)     |_PER_FMK(CHIP,DEVCFG,MCBSP1DIS,mcbsp1dis) )
S
N#endif
N
N#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412)
X#if (0 || 0 || 0 || 0)
S
S  #define   _CHIP_PERCFG_ADDR                0x01B3F000u
S  #define   _CHIP_PERCFG_OFFSET              0
S
S#if (CHIP_DM642)
S  #define _CHIP_PERCFG_VP2EN_MASK            0x00000040u
S  #define _CHIP_PERCFG_VP2EN_SHIFT           0x00000006u
S  #define  CHIP_PERCFG_VP2EN_DEFAULT         0x00000000u
S  #define  CHIP_PERCFG_VP2EN_OF(x)           _VALUEOF(x)
S  #define  CHIP_PERCFG_VP2EN_DISABLE         0x00000000u
S  #define  CHIP_PERCFG_VP2EN_ENABLE          0x00000001u
S#endif
S
S#if (CHIP_DM642 | CHIP_DM641)                          
S  #define _CHIP_PERCFG_VP1EN_MASK            0x00000020u
S  #define _CHIP_PERCFG_VP1EN_SHIFT           0x00000005u
S  #define  CHIP_PERCFG_VP1EN_DEFAULT         0x00000000u
S  #define  CHIP_PERCFG_VP1EN_OF(x)           _VALUEOF(x)
S  #define  CHIP_PERCFG_VP1EN_DISABLE         0x00000000u
S  #define  CHIP_PERCFG_VP1EN_ENABLE          0x00000001u
S#endif
S
S#if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640)                          
S  #define _CHIP_PERCFG_VP0EN_MASK            0x00000010u
S  #define _CHIP_PERCFG_VP0EN_SHIFT           0x00000004u
S  #define  CHIP_PERCFG_VP0EN_DEFAULT         0x00000000u
S  #define  CHIP_PERCFG_VP0EN_OF(x)           _VALUEOF(x)
S  #define  CHIP_PERCFG_VP0EN_DISABLE         0x00000000u
S  #define  CHIP_PERCFG_VP0EN_ENABLE          0x00000001u
S#endif
S
S  #define _CHIP_PERCFG_I2C0EN_MASK           0x00000008u
S  #define _CHIP_PERCFG_I2C0EN_SHIFT          0x00000003u
S  #define  CHIP_PERCFG_I2C0EN_DEFAULT        0x00000000u
S  #define  CHIP_PERCFG_I2C0EN_OF(x)          _VALUEOF(x)
S  #define  CHIP_PERCFG_I2C0EN_DISABLE        0x00000000u
S  #define  CHIP_PERCFG_I2C0EN_ENABLE         0x00000001u
S
S  #define _CHIP_PERCFG_MCBSP1EN_MASK         0x00000004u
S  #define _CHIP_PERCFG_MCBSP1EN_SHIFT        0x00000002u
S  #define  CHIP_PERCFG_MCBSP1EN_DEFAULT      0x00000001u
S  #define  CHIP_PERCFG_MCBSP1EN_OF(x)        _VALUEOF(x)
S  #define  CHIP_PERCFG_MCBSP1EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCBSP1EN_ENABLE       0x00000001u
S
S  #define _CHIP_PERCFG_MCBSP0EN_MASK         0x00000002u
S  #define _CHIP_PERCFG_MCBSP0EN_SHIFT        0x00000001u
S  #define  CHIP_PERCFG_MCBSP0EN_DEFAULT      0x00000001u
S  #define  CHIP_PERCFG_MCBSP0EN_OF(x)        _VALUEOF(x)
S  #define  CHIP_PERCFG_MCBSP0EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCBSP0EN_ENABLE       0x00000001u
S
S#if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640)
S  #define _CHIP_PERCFG_MCASP0EN_MASK         0x00000001u
S  #define _CHIP_PERCFG_MCASP0EN_SHIFT        0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_DEFAULT      0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_OF(x)         _VALUEOF(x)
S  #define  CHIP_PERCFG_MCASP0EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_ENABLE       0x00000001u
S#endif
S
S
S  #define  CHIP_PERCFG_OF(x)              _VALUEOF(x)
S
S#if (CHIP_DM642)
S  #define CHIP_PERCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,PERCFG,VP2EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,VP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,VP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN) \
S   )
X  #define CHIP_PERCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,PERCFG,VP2EN)     |_PER_FDEFAULT(CHIP,PERCFG,VP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,VP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN)    )
S
S
S #define CHIP_PERCFG_RMK(vp2en,vp1en,vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)( \
S     _PER_FMK(CHIP,PERCFG,VP2EN,vp2en) \
S    |_PER_FMK(CHIP,PERCFG,VP1EN,vp1en) \
S    |_PER_FMK(CHIP,PERCFG,VP0EN,vp0en) \
S    |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en) \
S    |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en) \
S  )
X #define CHIP_PERCFG_RMK(vp2en,vp1en,vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)(      _PER_FMK(CHIP,PERCFG,VP2EN,vp2en)     |_PER_FMK(CHIP,PERCFG,VP1EN,vp1en)     |_PER_FMK(CHIP,PERCFG,VP0EN,vp0en)     |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en)     |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en)     |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en)     |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en)   )
S#endif
S
S#if (CHIP_DM641)
S  #define CHIP_PERCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,PERCFG,VP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,VP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN) \
S   )
X  #define CHIP_PERCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,PERCFG,VP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,VP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN)    )
S
S
S #define CHIP_PERCFG_RMK(vp1en,vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)( \
S     _PER_FMK(CHIP,PERCFG,VP1EN,vp1en) \
S    |_PER_FMK(CHIP,PERCFG,VP0EN,vp0en) \
S    |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en) \
S    |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en) \
S  )
X #define CHIP_PERCFG_RMK(vp1en,vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)(      _PER_FMK(CHIP,PERCFG,VP1EN,vp1en)     |_PER_FMK(CHIP,PERCFG,VP0EN,vp0en)     |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en)     |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en)     |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en)     |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en)   )
S#endif
S
S#if (CHIP_DM640)
S  #define CHIP_PERCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,PERCFG,VP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN) \
S   )
X  #define CHIP_PERCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,PERCFG,VP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN)    )
S
S
S #define CHIP_PERCFG_RMK(vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)( \
S     _PER_FMK(CHIP,PERCFG,VP0EN,vp0en) \
S    |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en) \
S    |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en) \
S  )
X #define CHIP_PERCFG_RMK(vp0en,i2c0en,mcbsp1en,mcbsp0en,mcasp0en) (Uint32)(      _PER_FMK(CHIP,PERCFG,VP0EN,vp0en)     |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en)     |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en)     |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en)     |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en)   )
S#endif
S
S#if (CHIP_6412)
S  #define CHIP_PERCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,PERCFG,I2C0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN) \
S   )
X  #define CHIP_PERCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,PERCFG,I2C0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN)    )
S
S
S #define CHIP_PERCFG_RMK(i2c0en,mcbsp1en,mcbsp0en) (Uint32)( \
S    |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en) \
S    |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en) \
S  )
X #define CHIP_PERCFG_RMK(i2c0en,mcbsp1en,mcbsp0en) (Uint32)(     |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en)     |_PER_FMK(CHIP,PERCFG,MCBSP1EN,mcbsp1en)     |_PER_FMK(CHIP,PERCFG,MCBSP0EN,mcbsp0en)   )
S#endif
S
N#endif /* CHIP_DM642 || CHIP_6412  */
N
N#if (CHIP_6410 || CHIP_6413 || CHIP_6418)
X#if (0 || 0 || 0)
S
S  #define   _CHIP_PERCFG_ADDR                0x01B3F000u
S  #define   _CHIP_PERCFG_OFFSET              0
S  
S  #define _CHIP_PERCFG_AFCMUX_MASK           0x00000600u
S  #define _CHIP_PERCFG_AFCMUX_SHIFT          0x00000009u
S  #define  CHIP_PERCFG_AFCMUX_DEFAULT        0x00000000u
S  #define  CHIP_PERCFG_AFCMUX_OF(x)           _VALUEOF(x)
S  #define  CHIP_PERCFG_AFCMUX_PIN0           0x00000000u
S  #define  CHIP_PERCFG_AFCMUX_PIN1           0x00000001u
S  #define  CHIP_PERCFG_AFCMUX_PIN2           0x00000002u
S  #define  CHIP_PERCFG_AFCMUX_PIN3           0x00000003u
S 
S  #define _CHIP_PERCFG_MCASP1EN_MASK         0x00000100u
S  #define _CHIP_PERCFG_MCASP1EN_SHIFT        0x00000008u
S  #define  CHIP_PERCFG_MCASP1EN_DEFAULT      0x00000000u
S  #define  CHIP_PERCFG_MCASP1EN_OF(x)         _VALUEOF(x)
S  #define  CHIP_PERCFG_MCASP1EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCASP1EN_ENABLE       0x00000001u
S
S  #define _CHIP_PERCFG_I2C1EN_MASK           0x00000080u
S  #define _CHIP_PERCFG_I2C1EN_SHIFT          0x00000007u
S  #define  CHIP_PERCFG_I2C1EN_DEFAULT        0x00000000u
S  #define  CHIP_PERCFG_I2C1EN_OF(x)          _VALUEOF(x)
S  #define  CHIP_PERCFG_I2C1EN_DISABLE        0x00000000u
S  #define  CHIP_PERCFG_I2C1EN_ENABLE         0x00000001u
S
S  #define _CHIP_PERCFG_I2C0EN_MASK           0x00000008u
S  #define _CHIP_PERCFG_I2C0EN_SHIFT          0x00000003u
S  #define  CHIP_PERCFG_I2C0EN_DEFAULT        0x00000000u
S  #define  CHIP_PERCFG_I2C0EN_OF(x)          _VALUEOF(x)
S  #define  CHIP_PERCFG_I2C0EN_DISABLE        0x00000000u
S  #define  CHIP_PERCFG_I2C0EN_ENABLE         0x00000001u
S
S  #define _CHIP_PERCFG_MCBSP1EN_MASK         0x00000004u
S  #define _CHIP_PERCFG_MCBSP1EN_SHIFT        0x00000002u
S  #define  CHIP_PERCFG_MCBSP1EN_DEFAULT      0x00000001u
S  #define  CHIP_PERCFG_MCBSP1EN_OF(x)        _VALUEOF(x)
S  #define  CHIP_PERCFG_MCBSP1EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCBSP1EN_ENABLE       0x00000001u
S 
S  #define _CHIP_PERCFG_MCBSP0EN_MASK         0x00000002u
S  #define _CHIP_PERCFG_MCBSP0EN_SHIFT        0x00000001u
S  #define  CHIP_PERCFG_MCBSP0EN_DEFAULT      0x00000001u
S  #define  CHIP_PERCFG_MCBSP0EN_OF(x)        _VALUEOF(x)
S  #define  CHIP_PERCFG_MCBSP0EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCBSP0EN_ENABLE       0x00000001u
S
S  #define _CHIP_PERCFG_MCASP0EN_MASK         0x00000001u
S  #define _CHIP_PERCFG_MCASP0EN_SHIFT        0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_DEFAULT      0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_OF(x)         _VALUEOF(x)
S  #define  CHIP_PERCFG_MCASP0EN_DISABLE      0x00000000u
S  #define  CHIP_PERCFG_MCASP0EN_ENABLE       0x00000001u
S
S  #define  CHIP_PERCFG_OF(x)              _VALUEOF(x)
S
S  #define CHIP_PERCFG_DEFAULT (Uint32)( \
S    _PER_FDEFAULT(CHIP,PERCFG,AFCMUX) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCASP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,I2C1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN) \
S    |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN) \
S   )  
X  #define CHIP_PERCFG_DEFAULT (Uint32)(     _PER_FDEFAULT(CHIP,PERCFG,AFCMUX)     |_PER_FDEFAULT(CHIP,PERCFG,MCASP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,I2C1EN)     |_PER_FDEFAULT(CHIP,PERCFG,I2C0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP1EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCBSP0EN)     |_PER_FDEFAULT(CHIP,PERCFG,MCASP0EN)    )  
S  
S
S #define CHIP_PERCFG_RMK(afcmux,mcasp1en,i2c1en,i2c0en,mcasp0en) (Uint32)( \
S    _PER_FMK(CHIP,PERCFG,AFCMUX,afcmux) \
S    |_PER_FMK(CHIP,PERCFG,MCASP1EN,mcasp1en) \
S    |_PER_FMK(CHIP,PERCFG,I2C1EN,i2c1en) \
S    |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en) \
S    |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en) \
S  )  
X #define CHIP_PERCFG_RMK(afcmux,mcasp1en,i2c1en,i2c0en,mcasp0en) (Uint32)(     _PER_FMK(CHIP,PERCFG,AFCMUX,afcmux)     |_PER_FMK(CHIP,PERCFG,MCASP1EN,mcasp1en)     |_PER_FMK(CHIP,PERCFG,I2C1EN,i2c1en)     |_PER_FMK(CHIP,PERCFG,I2C0EN,i2c0en)     |_PER_FMK(CHIP,PERCFG,MCASP0EN,mcasp0en)   )  
S
N#endif /* CHIP_6410 || CHIP_6413 || CHIP_6418 */
N
N#if (CHIP_6711C || CHIP_6712C)
X#if (0 || 0)
S  #define   _CHIP_DEVCFG_ADDR              0x019C0200u
S  #define   _CHIP_DEVCFG_OFFSET            0
S
S  #define _CHIP_DEVCFG_EKSRC_MASK          0x00000010u
S  #define _CHIP_DEVCFG_EKSRC_SHIFT         0x00000004u
S  #define  CHIP_DEVCFG_EKSRC_DEFAULT       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVCFG_EKSRC_SYSCLK3       0x00000000u
S  #define  CHIP_DEVCFG_EKSRC_ECLKIN        0x00000001u
S
S  #define  CHIP_DEVCFG_OF(x)              _VALUEOF(x)
S
S  #define CHIP_DEVCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CHIP,DEVCFG,EKSRC) \
S )
X  #define CHIP_DEVCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CHIP,DEVCFG,EKSRC)  )
S
S  #define CHIP_PERCFG_RMK(eksrc) (Uint32)( \
S      _PER_FMK(CHIP,PERCFG,EKSRC,eksrc) \
S)
X  #define CHIP_PERCFG_RMK(eksrc) (Uint32)(       _PER_FMK(CHIP,PERCFG,EKSRC,eksrc) )
S
N#endif /* CHIP_6711C || CHIP_6712C */
N
N#if (CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713) 
X#if (0 || 0 || 0 || 0) 
S  #define _CHIP_DEVCFG_FGET(FIELD)\
S    _PER_FGET(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD)
X  #define _CHIP_DEVCFG_FGET(FIELD)    _PER_FGET(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD)
S
S  #define _CHIP_DEVCFG_FSET(FIELD,field)\
S    _PER_FSET(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD,field)
X  #define _CHIP_DEVCFG_FSET(FIELD,field)    _PER_FSET(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD,field)
S
S  #define _CHIP_DEVCFG_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD,##SYM)
X  #define _CHIP_DEVCFG_FSETS(FIELD,SYM)    _PER_FSETS(_CHIP_DEVCFG_ADDR,CHIP,DEVCFG,##FIELD,##SYM)
S   
N#else
N  
N  #define _CHIP_PERCFG_FGET(FIELD)\
N    _PER_FGET(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD)
X  #define _CHIP_PERCFG_FGET(FIELD)    _PER_FGET(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD)
N
N  #define _CHIP_PERCFG_FSET(FIELD,field)\
N    _PER_FSET(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD,field)
X  #define _CHIP_PERCFG_FSET(FIELD,field)    _PER_FSET(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD,field)
N
N  #define _CHIP_PERCFG_FSETS(FIELD,SYM)\
N    _PER_FSETS(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD,##SYM)
X  #define _CHIP_PERCFG_FSETS(FIELD,SYM)    _PER_FSETS(_CHIP_PERCFG_ADDR,CHIP,PERCFG,##FIELD,##SYM)
N#endif
N
N/*----------------------------------------------------------------------------*/
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  D E V S T A T    |
N* |___________________|
N*
N* DEVSTAT - Device Status Register (1)
N*
N* FIELDS (msb -> lsb)
N* DM642
N* (r) MACEN
N* (r) HPIWIDTH
N* (r) PCIEEAI
N* (r) PCIEN
N* (r) CLKMODE
N* (r) LENDIAN
N* (r) BOOTMODE
N* (r) AECLKINSEL
N*
N* DRI300
N* (r) PLLM
N* (r) OSCEXTRES
N* (r) CLKINSEL
N* (r) CLKMODE3
N* (r) HPIWIDTH
N* (r) HPIENZ
N* (r) CLKMODE2
N* (r) CLKMODE1
N* (r) CLKMODE0
N* (r) LENDIAN
N* (r) BOOTMODE
N* (r) AECLKINSEL
N*
N* 
N\******************************************************************************/
N#if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412)
X#if (0 | 0 | 0 | 0)
S
S  #define   _CHIP_DEVSTAT_ADDR                0x01B3F004u
S  #define   _CHIP_DEVSTAT_OFFSET              0
S
S  #define _CHIP_DEVSTAT_MACEN_MASK            0x00000800u
S  #define _CHIP_DEVSTAT_MACEN_SHIFT           0x0000000Bu
S  #define  CHIP_DEVSTAT_MACEN_DEFAULT         0x00000000u
S  #define  CHIP_DEVSTAT_MACEN_OF(x)           _VALUEOF(x)
S  #define  CHIP_DEVSTAT_MACEN_DISABLE         0x00000000u
S  #define  CHIP_DEVSTAT_MACEN_ENABLE          0x00000001u
S
S #if !(CHIP_DM640)
S  #define _CHIP_DEVSTAT_HPIWIDTH_MASK            0x00000400u
S  #define _CHIP_DEVSTAT_HPIWIDTH_SHIFT           0x0000000Au
S  #define  CHIP_DEVSTAT_HPIWIDTH_DEFAULT         0x00000000u
S  #define  CHIP_DEVSTAT_HPIWIDTH_OF(x)           _VALUEOF(x)
S  #define  CHIP_DEVSTAT_HPIWIDTH_16              0x00000000u
S  #define  CHIP_DEVSTAT_HPIWIDTH_32              0x00000001u
S #endif
S
S #if !(CHIP_DM641 | CHIP_DM640)
S  #define _CHIP_DEVSTAT_PCIEEAI_MASK            0x00000200u
S  #define _CHIP_DEVSTAT_PCIEEAI_SHIFT           0x00000009u
S  #define  CHIP_DEVSTAT_PCIEEAI_DEFAULT         0x00000000u
S  #define  CHIP_DEVSTAT_PCIEEAI_OF(x)           _VALUEOF(x)
S  #define  CHIP_DEVSTAT_PCIEEAI_NONE            0x00000000u
S  #define  CHIP_DEVSTAT_PCIEEAI_INIT            0x00000001u
S
S  #define _CHIP_DEVSTAT_PCIEN_MASK           0x00000100u
S  #define _CHIP_DEVSTAT_PCIEN_SHIFT          0x00000008u
S  #define  CHIP_DEVSTAT_PCIEN_DEFAULT        0x00000000u
S  #define  CHIP_DEVSTAT_PCIEN_OF(x)          _VALUEOF(x)
S  #define  CHIP_DEVSTAT_PCIEN_DISABLE        0x00000000u
S  #define  CHIP_DEVSTAT_PCIEN_ENABLE         0x00000001u
S #endif
S
S  #define _CHIP_DEVSTAT_CLKMODE_MASK         0x00000060u
S  #define _CHIP_DEVSTAT_CLKMODE_SHIFT        0x00000005u
S  #define  CHIP_DEVSTAT_CLKMODE_DEFAULT      0x00000001u
S  #define  CHIP_DEVSTAT_CLKMODE_OF(x)        _VALUEOF(x)
S  #define  CHIP_DEVSTAT_CLKMODE_X1           0x00000000u
S  #define  CHIP_DEVSTAT_CLKMODE_X6           0x00000001u
S  #define  CHIP_DEVSTAT_CLKMODE_X12          0x00000002u
S  #define  CHIP_DEVSTAT_CLKMODE_X20          0x00000003u
S
S  #define _CHIP_DEVSTAT_LENDIAN_MASK         0x00000010u
S  #define _CHIP_DEVSTAT_LENDIAN_SHIFT        0x00000004u
S  #define  CHIP_DEVSTAT_LENDIAN_DEFAULT      0x00000001u
S  #define  CHIP_DEVSTAT_LENDIAN_OF(x)        _VALUEOF(x)
S  #define  CHIP_DEVSTAT_LENDIAN_BIG          0x00000000u
S  #define  CHIP_DEVSTAT_LENDIAN_LITTLE       0x00000001u
S
S  #define _CHIP_DEVSTAT_BOOTMODE_MASK         0x0000000Cu
S  #define _CHIP_DEVSTAT_BOOTMODE_SHIFT        0x00000002u
S  #define  CHIP_DEVSTAT_BOOTMODE_DEFAULT      0x00000000u
S  #define  CHIP_DEVSTAT_BOOTMODE_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVSTAT_BOOTMODE_NONE         0x00000000u
S  #define  CHIP_DEVSTAT_BOOTMODE_HPIPCI       0x00000001u
S  #define  CHIP_DEVSTAT_BOOTMODE_EMIFA        0x00000003u
S
S  #define _CHIP_DEVSTAT_AECLKINSEL_MASK         0x00000003u
S  #define _CHIP_DEVSTAT_AECLKINSEL_SHIFT        0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_DEFAULT      0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVSTAT_AECLKINSEL_ECLKIN       0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_CLKOUT4      0x00000001u
S  #define  CHIP_DEVSTAT_AECLKINSEL_CLKOUT6      0x00000002u
S  #define  CHIP_DEVSTAT_OF(x)              _VALUEOF(x)
S
S  #define  CHIP_DEVSTAT_OF(x)              _VALUEOF(x)
S
S  /* Read only Register */
S
S  #define _CHIP_DEVSTAT_FGET(FIELD)\
S    _PER_FGET(_CHIP_DEVSTAT_ADDR,CHIP,DEVSTAT,##FIELD)
X  #define _CHIP_DEVSTAT_FGET(FIELD)    _PER_FGET(_CHIP_DEVSTAT_ADDR,CHIP,DEVSTAT,##FIELD)
S
S
N#endif /* CHIP_DM642 | CHIP_6412  */
N
N#if (CHIP_6410 || CHIP_6413 || CHIP_6418)
X#if (0 || 0 || 0)
S
S  #define   _CHIP_DEVSTAT_ADDR                 0x01B3F004u
S  #define   _CHIP_DEVSTAT_OFFSET               0
S
S  #define _CHIP_DEVSTAT_PLLM_MASK              0x00F10000u
S  #define _CHIP_DEVSTAT_PLLM_SHIFT             0x00000013u
S  #define  CHIP_DEVSTAT_PLLM_DEFAULT           0x00000000u
S  #define  CHIP_DEVSTAT_PLLM_OF(x)             _VALUEOF(x)
S  #define  CHIP_DEVSTAT_PLLM_BYPASS            0x00000000u
S  #define  CHIP_DEVSTAT_PLLM_5                 0x00000001u
S  #define  CHIP_DEVSTAT_PLLM_6                 0x00000002u
S  #define  CHIP_DEVSTAT_PLLM_7                 0x00000003u
S  #define  CHIP_DEVSTAT_PLLM_8                 0x00000004u
S  #define  CHIP_DEVSTAT_PLLM_9                 0x00000005u
S  #define  CHIP_DEVSTAT_PLLM_10                0x00000006u
S  #define  CHIP_DEVSTAT_PLLM_11                0x00000007u
S  #define  CHIP_DEVSTAT_PLLM_12                0x00000008u
S  #define  CHIP_DEVSTAT_PLLM_16                0x00000009u
S  #define  CHIP_DEVSTAT_PLLM_18                0x0000000Au
S  #define  CHIP_DEVSTAT_PLLM_19                0x0000000Bu
S  #define  CHIP_DEVSTAT_PLLM_20                0x0000000Cu
S  #define  CHIP_DEVSTAT_PLLM_21                0x0000000Du
S  #define  CHIP_DEVSTAT_PLLM_22                0x0000000Eu
S  #define  CHIP_DEVSTAT_PLLM_24                0x0000000Fu
S 
S  #define _CHIP_DEVSTAT_OSCEXTRES_MASK         0x00020000u
S  #define _CHIP_DEVSTAT_OSCEXTRES_SHIFT        0x00000011u
S  #define  CHIP_DEVSTAT_OSCEXTRES_DEFAUL       0x00000001u
S  #define  CHIP_DEVSTAT_OSCEXTRES_OF(x)        _VALUEOF(x)
S  #define  CHIP_DEVSTAT_OSCEXTRES_DISABL       0x00000000u
S  #define  CHIP_DEVSTAT_OSCEXTRES_ENABLE       0x00000001u
S 
S  #define _CHIP_DEVSTAT_CLKINSEL_MASK          0x00010000u
S  #define _CHIP_DEVSTAT_CLKINSEL_SHIFT         0x00000010u
S  #define  CHIP_DEVSTAT_CLKINSEL_DEFAULT       0x00000000u
S  #define  CHIP_DEVSTAT_CLKINSEL_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVSTAT_CLKINSEL_DISABLE       0x00000000u
S  #define  CHIP_DEVSTAT_CLKINSEL_ENABLE        0x00000001u
S 
S  #define _CHIP_DEVSTAT_HPIWIDTH_MASK          0x00000400u
S  #define _CHIP_DEVSTAT_HPIWIDTH_SHIFT         0x0000000Au
S  #define  CHIP_DEVSTAT_HPIWIDTH_DEFAUL        0x00000000u
S  #define  CHIP_DEVSTAT_HPIWIDTH_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVSTAT_HPIWIDTH_16            0x00000000u
S  #define  CHIP_DEVSTAT_HPIWIDTH_32            0x00000001u
S 
S  #define _CHIP_DEVSTAT_HPIENZ_MASK            0x00000100u
S  #define _CHIP_DEVSTAT_HPIENZ_SHIFT           0x00000008u
S  #define  CHIP_DEVSTAT_HPIENZ_DEFAULT         0x00000000u
S  #define  CHIP_DEVSTAT_HPIENZ_OF(x)           _VALUEOF(x)
S  #define  CHIP_DEVSTAT_HPIENZ_ENABLE          0x00000000u
S  #define  CHIP_DEVSTAT_HPIENZ_DISABLE         0x00000001u
S 
S  #define _CHIP_DEVSTAT_CLKMODE_MASK          0x000010E0u
S  #define _CHIP_DEVSTAT_CLKMODE_SHIFT         0x00000005u
S  #define  CHIP_DEVSTAT_CLKMODE_DEFAULT       0x00000001u
S  #define  CHIP_DEVSTAT_CLKMODE_OF(x)         _VALUEOF(x)
S  #define  CHIP_DEVSTAT_CLKMODE_0             0x00000000u
S  #define  CHIP_DEVSTAT_CLKMODE_1             0x00000001u
S  #define  CHIP_DEVSTAT_CLKMODE_2             0x00000002u
S  #define  CHIP_DEVSTAT_CLKMODE_3             0x00000003u
S  #define  CHIP_DEVSTAT_CLKMODE_4             0x00000004u
S  #define  CHIP_DEVSTAT_CLKMODE_5             0x00000005u
S  #define  CHIP_DEVSTAT_CLKMODE_6             0x00000006u
S  #define  CHIP_DEVSTAT_CLKMODE_7             0x00000007u
S  #define  CHIP_DEVSTAT_CLKMODE_8             0x00000080u
S  #define  CHIP_DEVSTAT_CLKMODE_9             0x00000081u
S  #define  CHIP_DEVSTAT_CLKMODE_10            0x00000082u
S  #define  CHIP_DEVSTAT_CLKMODE_11            0x00000083u
S  #define  CHIP_DEVSTAT_CLKMODE_12            0x00000084u
S  #define  CHIP_DEVSTAT_CLKMODE_13            0x00000085u
S  #define  CHIP_DEVSTAT_CLKMODE_14            0x00000086u
S  #define  CHIP_DEVSTAT_CLKMODE_15            0x00000087u
S 
S  #define _CHIP_DEVSTAT_LENDIAN_MASK           0x00000010u
S  #define _CHIP_DEVSTAT_LENDIAN_SHIFT          0x00000004u
S  #define  CHIP_DEVSTAT_LENDIAN_DEFAULT        0x00000001u
S  #define  CHIP_DEVSTAT_LENDIAN_OF(x)          _VALUEOF(x)
S  #define  CHIP_DEVSTAT_LENDIAN_BIG            0x00000000u 
S  #define  CHIP_DEVSTAT_LENDIAN_LITTLE         0x00000001u
S 
S  #define _CHIP_DEVSTAT_BOOTMODE_MASK          0x0000000Cu
S  #define _CHIP_DEVSTAT_BOOTMODE_SHIFT         0x00000002u
S  #define  CHIP_DEVSTAT_BOOTMODE_DEFAULT       0x00000000u
S  #define  CHIP_DEVSTAT_BOOTMODE_OF(x)          _VALUEOF(x)
S  #define  CHIP_DEVSTAT_BOOTMODE_NONE          0x00000000u
S  #define  CHIP_DEVSTAT_BOOTMODE_HPI           0x00000001u
S  #define  CHIP_DEVSTAT_BOOTMODE_EMIFA         0x00000003u
S
S  #define _CHIP_DEVSTAT_AECLKINSEL_MASK        0x00000003u
S  #define _CHIP_DEVSTAT_AECLKINSEL_SHIFT       0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_DEFAULT     0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_OF(x)       _VALUEOF(x)
S  #define  CHIP_DEVSTAT_AECLKINSEL_ECLKIN      0x00000000u
S  #define  CHIP_DEVSTAT_AECLKINSEL_CLKOUT4     0x00000001u
S  #define  CHIP_DEVSTAT_AECLKINSEL_CLKOUT6     0x00000002u
S
S  #define  CHIP_DEVSTAT_OF(x)              _VALUEOF(x)
S
S  /* Read only Register */
S
S  #define _CHIP_DEVSTAT_FGET(FIELD)\
S    _PER_FGET(_CHIP_DEVSTAT_ADDR,CHIP,DEVSTAT,##FIELD)
X  #define _CHIP_DEVSTAT_FGET(FIELD)    _PER_FGET(_CHIP_DEVSTAT_ADDR,CHIP,DEVSTAT,##FIELD)
S
S
N#endif /* CHIP_6410 || CHIP_6413 || CHIP_6418 */
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  J T A G I D      |
N* |___________________|
N*
N* JTAGID - JTAG ID register (1)
N*
N* FIELDS (msb -> lsb)
N* (r) VARIANT
N* (r) PART
N* (r) MANNUFACTURE
N* (r) LSB
N*
N* (1) Only for DM642
N*
N\******************************************************************************/
N
N
N#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412 )
X#if (0 || 0 || 0 || 0 )
S
S  #define   _CHIP_JTAGID_ADDR                0x01B3F008u
S  #define   _CHIP_JTAGID_OFFSET              0
S
S  #define _CHIP_JTAGID_VARIANT_MASK          0xF0000000u
S  #define _CHIP_JTAGID_VARIANT_SHIFT         0x0000001Cu
S  #define  CHIP_JTAGID_VARIANT_DEFAULT       0x00000000u
S  #define  CHIP_JTAGID_VARIANT_OF(x)         _VALUEOF(x)
S
S  #define _CHIP_JTAGID_PART_MASK             0x0FFFF000u
S  #define _CHIP_JTAGID_PART_SHIFT            0x0000000Cu
S  #define  CHIP_JTAGID_PART_DEFAULT          0x00000079u
S  #define  CHIP_JTAGID_PART_OF(x)            _VALUEOF(x)
S
S  #define _CHIP_JTAGID_MANUFACTURE_MASK      0x00000FFEu
S  #define _CHIP_JTAGID_MANUFACTURE_SHIFT     0x00000001u
S  #define  CHIP_JTAGID_MANUFACTURE_DEFAULT   0x00000017u
S  #define  CHIP_JTAGID_MANUFACTURE_OF(x)     _VALUEOF(x)
S
S  #define _CHIP_JTAGID_LSB_MASK              0x00000001u
S  #define _CHIP_JTAGID_LSB_SHIFT             0x00000000u
S  #define  CHIP_JTAGID_LSB_DEFAULT           0x00000001u
S  #define  CHIP_JTAGID_LSB_OF(x)             _VALUEOF(x)
S
S  #define  CHIP_JTAGID_OF(x)              _VALUEOF(x)
S
S  #define _CHIP_JTAGID_FGET(FIELD)\
S    _PER_FGET(_CHIP_JTAGID_ADDR,CHIP,JTAGID,##FIELD)
X  #define _CHIP_JTAGID_FGET(FIELD)    _PER_FGET(_CHIP_JTAGID_ADDR,CHIP,JTAGID,##FIELD)
S
N#endif /* CHIP_DM642 || CHIP_6412  */
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  P C F G L O C K  |
N* |___________________|
N*
N* PCFGLOCK - Peripheral Configuration Lock register (1)
N*
N* FIELDS (msb -> lsb)
N* (r) LOCKSTAT
N* (w) LOCK
N*
N* (1) Only for DM642
N*
N\******************************************************************************/
N
N#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
X#if (0 || 0 || 0 || 0 || 0 || 0 || 0)
S
S  #define   _CHIP_PCFGLOCK_ADDR                 0x01B3F018u
S  #define   _CHIP_PCFGLOCK_OFFSET               0
S
S  #define _CHIP_PCFGLOCK_LOCKSTAT_MASK          0x00000001u
S  #define _CHIP_PCFGLOCK_LOCKSTAT_SHIFT         0x00000000u
S  #define  CHIP_PCFGLOCK_LOCKSTAT_DEFAULT       0x00000001u
S  #define  CHIP_PCFGLOCK_LOCKSTAT_OF(x)         _VALUEOF(x)
S  #define  CHIP_PCFGLOCK_LOCKSTAT_UNLOCK        0x00000000u
S  #define  CHIP_PCFGLOCK_LOCKSTAT_LOCK          0x00000001u
S
S  #define _CHIP_PCFGLOCK_LOCK_MASK              0xFFFFFFFFu
S  #define _CHIP_PCFGLOCK_LOCK_SHIFT             0x00000000u
S  #define  CHIP_PCFGLOCK_LOCK_DEFAULT           0x00000000u
S  #define  CHIP_PCFGLOCK_LOCK_OF(x)             _VALUEOF(x)
S  #define  CHIP_PCFGLOCK_LOCK_UNLOCK            0x10C0010Cu
S  #define  CHIP_PCFGLOCK_LOCK_DISABLE           0x10C0010Cu
S
S  #define  CHIP_PCFGLOCK_OF(x)                  _VALUEOF(x)
S
S  #define _CHIP_PCFGLOCK_FGET(FIELD)\
S    _PER_FGET(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD)
X  #define _CHIP_PCFGLOCK_FGET(FIELD)    _PER_FGET(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD)
S
S  #define _CHIP_PCFGLOCK_FSET(FIELD,field)\
S    _PER_FSET(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD,field)
X  #define _CHIP_PCFGLOCK_FSET(FIELD,field)    _PER_FSET(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD,field)
S
S  #define _CHIP_PCFGLOCK_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD,##SYM)
X  #define _CHIP_PCFGLOCK_FSETS(FIELD,SYM)    _PER_FSETS(_CHIP_PCFGLOCK_ADDR,CHIP,PCFGLOCK,##FIELD,##SYM)
S
S
N#endif /* CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418*/
N
N#ifdef __cplusplus
S}
N#endif
N
N/*----------------------------------------------------------------------------*/
N#endif /* _CSL_CHIPHAL_H_ */
N/******************************************************************************\
N* End of csl_chiphal.h
X
N\******************************************************************************/
N
L 20 "E:/TI_workspase/c6701_t/csl/csl_chip.h" 2
N#include <csl_emifhal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_emifhal.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_emifhal.h
N* DATE CREATED.. 06/12/1999 
N* LAST MODIFIED. 06/17/2003  Added CHIP_6712C
N*                05/28/2003  Added CHIP_6711C
N*                10/03/2000
N*------------------------------------------------------------------------------
N* REGISTERS
N*
N* GBLCTL  - global control register
N* CECTL0  - CE space control register 0
N* CECTL1  - CE space control register 1
N* CECTL2  - CE space control register 2
N* CECTL3  - CE space control register 3
N* SDCTL   - SDRAM control regsiter
N* SDTIM   - SDRAM timing register
N* SDEXT   - SDRAM extension register (1)
N*
N* (1) - only supported on 6211,6711,6712,6713,6711C,6712C
N* 
N\******************************************************************************/
N#ifndef _CSL_EMIFHAL_H_
N#define _CSL_EMIFHAL_H_
N
N#include <csl_stdinc.h>
N#include <csl_chip.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_chip.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2001 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_chip.h
N* DATE CREATED.. 08/19/1999 
N* LAST MODIFIED. 08/02/2004 - Adding support for C6418
N*                06/17/2003   - Added support for 6712C
N*                06/09/2003   - Added support for 6711C
N*                12/03/2001   - CHIP_configArgs
N*                11/08/2001   - CHIP_getSiliconRevId()
N*                             - CHIP_config() / CHIP_getConfig  DM642
N\******************************************************************************/
N#ifndef _CSL_CHIP_H_
S#define _CSL_CHIP_H_
S
S#include <csl_stdinc.h>
S
S#include <csl_chiphal.h>
S#include <csl_emifhal.h>
S
S/******************************************************************************\
S* scope and inline control macros
X
S\******************************************************************************/
S#ifdef __cplusplus
S#define CSLAPI extern "C" far
S#else
S#define CSLAPI extern far
S#endif
S
S#undef  USEDEFS
S#undef  IDECL
S#undef  IDEF
S
S#ifdef  _CHIP_MOD_
S  #define IDECL CSLAPI
S  #define USEDEFS
S  #define IDEF
S#else
S  #ifdef  _INLINE
S    #define IDECL static inline
S    #define USEDEFS
S    #define IDEF  static inline
S  #else
S    #define IDECL CSLAPI
S  #endif
S#endif
S
S
S/******************************************************************************\
S* global macro declarations
X
S\******************************************************************************/
S#define CHIP_ENDIAN_BIG    0
S#define CHIP_ENDIAN_LITTLE 1
S
S#define CHIP_MAP_0         0
S#define CHIP_MAP_1         1
S
S/* Selected devices masks*/
S#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
S#define CHIP_EKSRC_SYSCLK3  0x00000000u
S#define CHIP_EKSRC_ECLKIN   0x00000010u
S#endif
S
S#if (CHIP_6713 || CHIP_DA610)
S#define CHIP_TOUT1          0x00000000u
S#define CHIP_AXR04_AXR111   0x00000008u
S
S#define CHIP_TOUT0          0x00000000u
S#define CHIP_AXR02_AXR113   0x00000004u
S
S#define CHIP_MCASP0         0x00000002u
S#define CHIP_MCBSP0         0x00000000u
S
S#define CHIP_I2C1           0x00000001u
S#define CHIP_MCBSP1         0x00000000u
S/* Full mask value */
S#define _CHIP_DEVCFG_MASK  0x0000001Fu
S#endif
S
S#if CHIP_DM642
S#define CHIP_VP2            0x00000040u
S#define CHIP_VP1            0x00000020u
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000007Fu
S#endif
S
S#if CHIP_DM641
S#define CHIP_VP1            0x00000020u
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000003Fu
S#endif
S
S#if CHIP_DM640
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000001Fu
S#endif
S
S#if CHIP_6412
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define _CHIP_PERCFG_MASK   0x0000000Eu
S#endif
S
S#if (CHIP_6410 || CHIP_6413 || CHIP_6418)
S#define CHIP_AFCMUX         0x00000600u
S#define CHIP_MCASP1         0x00000100u
S#define CHIP_I2C1           0x00000080u
S#define CHIP_I2C0           0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000078Fu
S#endif
S
S/******************************************************************************\
S* global typedef declarations
X
S\******************************************************************************/
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
Stypedef struct{
S   Uint32 devcfg;
S} CHIP_Config;
S#elif (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
Stypedef struct{
S   Uint32 percfg;
S} CHIP_Config; 
S#endif
S/******************************************************************************\
S* global variable declarations
X
S\******************************************************************************/
S
S
S/******************************************************************************\
S* global function declarations
X
S\******************************************************************************/
S
S
S/******************************************************************************\
S* inline function declarations
X
S\******************************************************************************/
SIDECL Uint32 CHIP_getCpuId();
SIDECL Uint32 CHIP_getRevId();
SIDECL Uint32 CHIP_getSiliconRevId();
SIDECL int    CHIP_getEndian();
SIDECL int    CHIP_getMapMode();
S
S#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418) 
SIDECL void   CHIP_config(CHIP_Config *config);
SIDECL void   CHIP_configArgs(Uint32 percfg);
SIDECL void   CHIP_getConfig(CHIP_Config *config); 
S#elif ( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
SIDECL void   CHIP_config(CHIP_Config *config);
SIDECL void   CHIP_configArgs(Uint32 devcfg);
SIDECL void   CHIP_getConfig(CHIP_Config *config); 
S#endif
S
S/******************************************************************************\
S* inline function definitions
X
S\******************************************************************************/
S#ifdef USEDEFS
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getCpuId() {
S  return CHIP_FGET(CSR,CPUID);
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getRevId() {
S  return CHIP_FGET(CSR,REVID);
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getSiliconRevId() {
S  return (( 0x000F0000 & REG32(0x01B00200))>>16);
S}
S/*----------------------------------------------------------------------------*/
SIDEF int CHIP_getEndian() {
S  return CHIP_FGET(CSR,EN);
S}
S/*----------------------------------------------------------------------------*/
SIDEF int CHIP_getMapMode() {
S  int mapmode = 0;
S  #if (!C11_SUPPORT && !C64_SUPPORT)
S    mapmode = EMIF_FGET(GBLCTL,MAP);
S  #endif
S  return mapmode;
S}
S/*----------------------------------------------------------------------------*/
S#if (CHIP_6713 || CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_DA610 || CHIP_6412 || CHIP_6711C || CHIP_6712C || CHIP_6410 || CHIP_6413 || CHIP_6418)
SIDEF void   CHIP_config(CHIP_Config *config) {
S  Uint32 gie = CHIP_FGET(CSR,GIE);
S  
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 ) 
S    volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S    register int x0;
S    CHIP_FSET(CSR,GIE,0);
S    x0 = config->devcfg;
S    base[_CHIP_DEVCFG_OFFSET] = x0;
S    CHIP_FSET(CSR,GIE,gie);
S    
S#else 
S  volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S  register int x0;
S  CHIP_FSET(CSR,GIE,0);
S  CHIP_FSETS(PCFGLOCK,LOCK,UNLOCK);
S  x0 = config->percfg;
S  base[_CHIP_PERCFG_OFFSET] = x0;
S  CHIP_FSET(CSR,GIE,gie);
S  
S#endif
S}      
S#endif
S/*----------------------------------------------------------------------------*/
S#if (CHIP_6713 || CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_DA610 || CHIP_6412 || CHIP_6711C || CHIP_6712C || CHIP_6410 || CHIP_6413 || CHIP_6418)
SIDEF void   CHIP_getConfig(CHIP_Config *config) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 ) 
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S   register int x0;
S
S   CHIP_FSET(CSR,GIE,0);
S
S   x0 = base[_CHIP_DEVCFG_OFFSET]; 
S   config->devcfg=x0;
S
S   CHIP_FSET(CSR,GIE,gie);
S   
S#else
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S   register int x0;
S
S   CHIP_FSET(CSR,GIE,0);
S
S   x0 = base[_CHIP_PERCFG_OFFSET]; 
S   config->percfg=x0;
S
S   CHIP_FSET(CSR,GIE,gie);
S   
S#endif
S}      
S#endif
S/*----------------------------------------------------------------------------*/
S   
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
S   IDEF void   CHIP_configArgs(Uint32 devcfg) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S   CHIP_FSET(CSR,GIE,0);
S   base[_CHIP_DEVCFG_OFFSET]= devcfg; 
S   CHIP_FSET(CSR,GIE,gie);
S}   
S#elif (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
S   IDEF void   CHIP_configArgs(Uint32 percfg) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S
S   CHIP_FSET(CSR,GIE,0);
S
S#if(CHIP_DM642 || CHIP_6412)
S   CHIP_FSETS(PCFGLOCK,LOCK,UNLOCK);
S#endif
S
S   base[_CHIP_PERCFG_OFFSET]= percfg; 
S
S   CHIP_FSET(CSR,GIE,gie);
S}   
S#endif
S
S
S/*----------------------------------------------------------------------------*/
S#endif /* USEDEFS */
S
S
N#endif /* _CSL_CHIP_H_ */
N/******************************************************************************\
N* End of csl_chip.h
X
N\******************************************************************************/
N
L 30 "E:/TI_workspase/c6701_t/csl/csl_emifhal.h" 2
N
N#if (EMIF_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N/******************************************************************************\
N* MISC section
X
N\******************************************************************************/
N#define _EMIF_BASE_GLOBAL   0x01800000u
N
N
N/******************************************************************************\
N* module level register/field access macros
X
N\******************************************************************************/
N
N  /* ----------------- */
N  /* FIELD MAKE MACROS */
N  /* ----------------- */
N
N  #define EMIF_FMK(REG,FIELD,x)\
N    _PER_FMK(EMIF,##REG,##FIELD,x)
X  #define EMIF_FMK(REG,FIELD,x)    _PER_FMK(EMIF,##REG,##FIELD,x)
N
N  #define EMIF_FMKS(REG,FIELD,SYM)\
N    _PER_FMKS(EMIF,##REG,##FIELD,##SYM)
X  #define EMIF_FMKS(REG,FIELD,SYM)    _PER_FMKS(EMIF,##REG,##FIELD,##SYM)
N 
N 
N  /* -------------------------------- */
N  /* RAW REGISTER/FIELD ACCESS MACROS */
N  /* -------------------------------- */
N
N  #define EMIF_ADDR(REG)\
N    _EMIF_##REG##_ADDR
X  #define EMIF_ADDR(REG)    _EMIF_##REG##_ADDR
N
N  #define EMIF_RGET(REG)\
N    _PER_RGET(_EMIF_##REG##_ADDR,EMIF,##REG)
X  #define EMIF_RGET(REG)    _PER_RGET(_EMIF_##REG##_ADDR,EMIF,##REG)
N
N  #define EMIF_RSET(REG,x)\
N    _PER_RSET(_EMIF_##REG##_ADDR,EMIF,##REG,x)
X  #define EMIF_RSET(REG,x)    _PER_RSET(_EMIF_##REG##_ADDR,EMIF,##REG,x)
N
N  #define EMIF_FGET(REG,FIELD)\
N    _EMIF_##REG##_FGET(##FIELD)
X  #define EMIF_FGET(REG,FIELD)    _EMIF_##REG##_FGET(##FIELD)
N
N  #define EMIF_FSET(REG,FIELD,x)\
N    _EMIF_##REG##_FSET(##FIELD,##x)
X  #define EMIF_FSET(REG,FIELD,x)    _EMIF_##REG##_FSET(##FIELD,##x)
N
N  #define EMIF_FSETS(REG,FIELD,SYM)\
N    _EMIF_##REG##_FSETS(##FIELD,##SYM)
X  #define EMIF_FSETS(REG,FIELD,SYM)    _EMIF_##REG##_FSETS(##FIELD,##SYM)
N 
N 
N  /* ------------------------------------------ */
N  /* ADDRESS BASED REGISTER/FIELD ACCESS MACROS */
N  /* ------------------------------------------ */
N
N  #define EMIF_RGETA(addr,REG)\
N    _PER_RGET(addr,EMIF,##REG)
X  #define EMIF_RGETA(addr,REG)    _PER_RGET(addr,EMIF,##REG)
N
N  #define EMIF_RSETA(addr,REG,x)\
N    _PER_RSET(addr,EMIF,##REG,x)
X  #define EMIF_RSETA(addr,REG,x)    _PER_RSET(addr,EMIF,##REG,x)
N
N  #define EMIF_FGETA(addr,REG,FIELD)\
N    _PER_FGET(addr,EMIF,##REG,##FIELD)
X  #define EMIF_FGETA(addr,REG,FIELD)    _PER_FGET(addr,EMIF,##REG,##FIELD)
N
N  #define EMIF_FSETA(addr,REG,FIELD,x)\
N    _PER_FSET(addr,EMIF,##REG,##FIELD,x)
X  #define EMIF_FSETA(addr,REG,FIELD,x)    _PER_FSET(addr,EMIF,##REG,##FIELD,x)
N
N  #define EMIF_FSETSA(addr,REG,FIELD,SYM)\
N    _PER_FSETS(addr,EMIF,##REG,##FIELD,##SYM)
X  #define EMIF_FSETSA(addr,REG,FIELD,SYM)    _PER_FSETS(addr,EMIF,##REG,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  G B L C T L      |
N* |___________________|
N*
N* GBLCTL  - global control register
N*
N* FIELDS (msb -> lsb)
N* (r)  BUSREQ  (1)
N* (r)  ARDY
N* (r)  HOLD
N* (r)  HOLDA
N* (rw) NOHOLD
N* (rw) SDCEN   (2)
N* (rw) SSCEN   (2)
N* (rw) EKEN    (4)
N* (rw) CLK1EN
N* (rw) CLK2EN  (3)
N* (rw) SSCRT   (2)(3)
N* (rw) RBTR8   (2)
N* (r)  MAP     (2)
N*
N* (1) - Field only exists for C11_SUPPORT
N* (2) - Field does not exist for C11_SUPPORT
N* (3) - Field does not exist for 6202/6203/6204/6205 
N* (4) - Field only exixts for C6713, DA610, 6711C, 6712C
N*
N\******************************************************************************/
N  #define _EMIF_GBLCTL_OFFSET          0
N
N  #define _EMIF_GBLCTL_ADDR            0x01800000u
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_GBLCTL_BUSREQ_MASK     0x00000800u
S  #define _EMIF_GBLCTL_BUSREQ_SHIFT    0x0000000Bu
S  #define  EMIF_GBLCTL_BUSREQ_DEFAULT  0x00000000u
S  #define  EMIF_GBLCTL_BUSREQ_OF(x)    _VALUEOF(x)
S  #define  EMIF_GBLCTL_BUSREQ_LOW      0x00000000u
S  #define  EMIF_GBLCTL_BUSREQ_HIGH     0x00000001u
N#endif
N
N  #define _EMIF_GBLCTL_ARDY_MASK       0x00000400u
N  #define _EMIF_GBLCTL_ARDY_SHIFT      0x0000000Au
N  #define  EMIF_GBLCTL_ARDY_DEFAULT    0x00000000u
N  #define  EMIF_GBLCTL_ARDY_OF(x)      _VALUEOF(x)
N  #define  EMIF_GBLCTL_ARDY_LOW        0x00000000u
N  #define  EMIF_GBLCTL_ARDY_HIGH       0x00000001u
N
N  #define _EMIF_GBLCTL_HOLD_MASK       0x00000200u
N  #define _EMIF_GBLCTL_HOLD_SHIFT      0x00000009u
N  #define  EMIF_GBLCTL_HOLD_DEFAULT    0x00000000u
N  #define  EMIF_GBLCTL_HOLD_OF(x)      _VALUEOF(x)
N  #define  EMIF_GBLCTL_HOLD_LOW        0x00000000u
N  #define  EMIF_GBLCTL_HOLD_HIGH       0x00000001u
N
N  #define _EMIF_GBLCTL_HOLDA_MASK      0x00000100u
N  #define _EMIF_GBLCTL_HOLDA_SHIFT     0x00000008u
N  #define  EMIF_GBLCTL_HOLDA_DEFAULT   0x00000000u
N  #define  EMIF_GBLCTL_HOLDA_OF(x)     _VALUEOF(x)
N  #define  EMIF_GBLCTL_HOLDA_LOW       0x00000000u
N  #define  EMIF_GBLCTL_HOLDA_HIGH      0x00000001u
N
N  #define _EMIF_GBLCTL_NOHOLD_MASK     0x00000080u
N  #define _EMIF_GBLCTL_NOHOLD_SHIFT    0x00000007u
N  #define  EMIF_GBLCTL_NOHOLD_DEFAULT  0x00000000u
N  #define  EMIF_GBLCTL_NOHOLD_OF(x)    _VALUEOF(x)
N  #define  EMIF_GBLCTL_NOHOLD_DISABLE  0x00000000u
N  #define  EMIF_GBLCTL_NOHOLD_ENABLE   0x00000001u
N
N#if (CHIP_6201 | CHIP_6202 | CHIP_6203 | CHIP_6204 | CHIP_6205 | CHIP_6701)
X#if (0 | 0 | 0 | 0 | 0 | 1)
N  #define _EMIF_GBLCTL_SDCEN_MASK      0x00000040u
N  #define _EMIF_GBLCTL_SDCEN_SHIFT     0x00000006u
N  #define  EMIF_GBLCTL_SDCEN_DEFAULT   0x00000001u
N  #define  EMIF_GBLCTL_SDCEN_OF(x)     _VALUEOF(x)
N  #define  EMIF_GBLCTL_SDCEN_DISABLE   0x00000000u
N  #define  EMIF_GBLCTL_SDCEN_ENABLE    0x00000001u
N#endif
N
N#if (CHIP_6201 | CHIP_6202 | CHIP_6203 | CHIP_6204 | CHIP_6205 | CHIP_6701)
X#if (0 | 0 | 0 | 0 | 0 | 1)
N  #define _EMIF_GBLCTL_SSCEN_MASK      0x00000020u
N  #define _EMIF_GBLCTL_SSCEN_SHIFT     0x00000005u
N  #define  EMIF_GBLCTL_SSCEN_DEFAULT   0x00000001u
N  #define  EMIF_GBLCTL_SSCEN_OF(x)     _VALUEOF(x)
N  #define  EMIF_GBLCTL_SSCEN_DISABLE   0x00000000u
N  #define  EMIF_GBLCTL_SSCEN_ENABLE    0x00000001u
N#endif
N
N#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
X#if (0 || 0 || 0 || 0)
S  #define _EMIF_GBLCTL_EKEN_MASK       0x00000020u
S  #define _EMIF_GBLCTL_EKEN_SHIFT      0x00000005u
S  #define  EMIF_GBLCTL_EKEN_DEFAULT    0x00000001u
S  #define  EMIF_GBLCTL_EKEN_OF(x)      _VALUEOF(x)
S  #define  EMIF_GBLCTL_EKEN_DISABLE    0x00000000u
S  #define  EMIF_GBLCTL_EKEN_ENABLE     0x00000001u
N#endif
N
N  #define _EMIF_GBLCTL_CLK1EN_MASK     0x00000010u
N  #define _EMIF_GBLCTL_CLK1EN_SHIFT    0x00000004u
N
N#if (CHIP_6713 || CHIP_DA610)
X#if (0 || 0)
S  #define  EMIF_GBLCTL_CLK1EN_DEFAULT  0x00000000u
N#else
N  #define  EMIF_GBLCTL_CLK1EN_DEFAULT  0x00000001u
N#endif
N
N  #define  EMIF_GBLCTL_CLK1EN_OF(x)    _VALUEOF(x)
N  #define  EMIF_GBLCTL_CLK1EN_DISABLE  0x00000000u
N  #define  EMIF_GBLCTL_CLK1EN_ENABLE   0x00000001u
N
N#if (!(CHIP_6202|CHIP_6203|CHIP_6204|CHIP_6205))
X#if (!(0|0|0|0))
N  #define _EMIF_GBLCTL_CLK2EN_MASK     0x00000008u
N  #define _EMIF_GBLCTL_CLK2EN_SHIFT    0x00000003u
N  #define  EMIF_GBLCTL_CLK2EN_DEFAULT  0x00000001u
N  #define  EMIF_GBLCTL_CLK2EN_OF(x)    _VALUEOF(x)
N  #define  EMIF_GBLCTL_CLK2EN_DISABLE  0x00000000u
N  #define  EMIF_GBLCTL_CLK2EN_ENABLE   0x00000001u
N#endif
N
N#if (CHIP_6201|CHIP_6701)
X#if (0|1)
N  #define _EMIF_GBLCTL_SSCRT_MASK      0x00000004u
N  #define _EMIF_GBLCTL_SSCRT_SHIFT     0x00000002u
N  #define  EMIF_GBLCTL_SSCRT_DEFAULT   0x00000000u
N  #define  EMIF_GBLCTL_SSCRT_OF(x)     _VALUEOF(x)
N  #define  EMIF_GBLCTL_SSCRT_CPUOVR2   0x00000000u
N  #define  EMIF_GBLCTL_SSCRT_CPU       0x00000001u
N#endif
N
N#if (!C11_SUPPORT)
X#if (!( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N  #define _EMIF_GBLCTL_RBTR8_MASK      0x00000002u
N  #define _EMIF_GBLCTL_RBTR8_SHIFT     0x00000001u
N  #define  EMIF_GBLCTL_RBTR8_DEFAULT   0x00000000u
N  #define  EMIF_GBLCTL_RBTR8_OF(x)     _VALUEOF(x)
N  #define  EMIF_GBLCTL_RBTR8_HPRI      0x00000000u
N  #define  EMIF_GBLCTL_RBTR8_8ACC      0x00000001u
N#endif
N
N#if (!C11_SUPPORT)
X#if (!( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N  #define _EMIF_GBLCTL_MAP_MASK        0x00000001u
N  #define _EMIF_GBLCTL_MAP_SHIFT       0x00000000u
N  #define  EMIF_GBLCTL_MAP_MAP1        0x00000000u
N  #define  EMIF_GBLCTL_MAP_MAP0        0x00000001u
N  #define  EMIF_GBLCTL_MAP_DEFAULT     0x00000000u
N  #define  EMIF_GBLCTL_MAP_OF(x)       _VALUEOF(x)
N#endif
N
N  #define  EMIF_GBLCTL_OF(x)           _VALUEOF(x)
N
N#if (CHIP_6201|CHIP_6701)
X#if (0|1)
N  #define EMIF_GBLCTL_DEFAULT (Uint32)( \
N    0x00003000\
N    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,SDCEN)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,SSCEN)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,SSCRT)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,RBTR8)\
N    |_PER_FDEFAULT(EMIF,GBLCTL,MAP)\
N  )
X  #define EMIF_GBLCTL_DEFAULT (Uint32)(     0x00003000    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,SDCEN)    |_PER_FDEFAULT(EMIF,GBLCTL,SSCEN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)    |_PER_FDEFAULT(EMIF,GBLCTL,SSCRT)    |_PER_FDEFAULT(EMIF,GBLCTL,RBTR8)    |_PER_FDEFAULT(EMIF,GBLCTL,MAP)  )
N
N  #define EMIF_GBLCTL_RMK(nohold,sdcen,sscen,clk1en,clk2en,sscrt,rbtr8) \
N    (Uint32)( \
N     _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)\
N    |_PER_FMK(EMIF,GBLCTL,SDCEN,sdcen)\
N    |_PER_FMK(EMIF,GBLCTL,SSCEN,sscen)\
N    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)\
N    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)\
N    |_PER_FMK(EMIF,GBLCTL,SSCRT,sscrt)\
N    |_PER_FMK(EMIF,GBLCTL,RBTR8,rbtr8)\
N  )
X  #define EMIF_GBLCTL_RMK(nohold,sdcen,sscen,clk1en,clk2en,sscrt,rbtr8)     (Uint32)(      _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)    |_PER_FMK(EMIF,GBLCTL,SDCEN,sdcen)    |_PER_FMK(EMIF,GBLCTL,SSCEN,sscen)    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)    |_PER_FMK(EMIF,GBLCTL,SSCRT,sscrt)    |_PER_FMK(EMIF,GBLCTL,RBTR8,rbtr8)  )
N#endif
N
N#if (CHIP_6211|CHIP_6711)
X#if (0|0)
S  #define EMIF_GBLCTL_DEFAULT (Uint32)( \
S    0x00003000\
S    |_PER_FDEFAULT(EMIF,GBLCTL,BUSREQ)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)\
S  )
X  #define EMIF_GBLCTL_DEFAULT (Uint32)(     0x00003000    |_PER_FDEFAULT(EMIF,GBLCTL,BUSREQ)    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)  )
S
S  #define EMIF_GBLCTL_RMK(nohold,clk1en,clk2en) (Uint32)( \
S     _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)\
S    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)\
S    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)\
S  )
X  #define EMIF_GBLCTL_RMK(nohold,clk1en,clk2en) (Uint32)(      _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)  )
N#endif
N
N#if (CHIP_6202|CHIP_6203|CHIP_6204|CHIP_6205)
X#if (0|0|0|0)
S  #define EMIF_GBLCTL_DEFAULT (Uint32)( \
S    0x00003008\
S    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,SDCEN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,SSCEN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,RBTR8)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,MAP)\
S  )
X  #define EMIF_GBLCTL_DEFAULT (Uint32)(     0x00003008    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,SDCEN)    |_PER_FDEFAULT(EMIF,GBLCTL,SSCEN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)    |_PER_FDEFAULT(EMIF,GBLCTL,RBTR8)    |_PER_FDEFAULT(EMIF,GBLCTL,MAP)  )
S
S  #define EMIF_GBLCTL_RMK(nohold,sdcen,sscen,clk1en,rbtr8) (Uint32)( \
S     _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)\
S    |_PER_FMK(EMIF,GBLCTL,SDCEN,sdcen)\
S    |_PER_FMK(EMIF,GBLCTL,SSCEN,sscen)\
S    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)\
S    |_PER_FMK(EMIF,GBLCTL,RBTR8,rbtr8)\
S  )
X  #define EMIF_GBLCTL_RMK(nohold,sdcen,sscen,clk1en,rbtr8) (Uint32)(      _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)    |_PER_FMK(EMIF,GBLCTL,SDCEN,sdcen)    |_PER_FMK(EMIF,GBLCTL,SSCEN,sscen)    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)    |_PER_FMK(EMIF,GBLCTL,RBTR8,rbtr8)  )
N#endif
N
N#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
X#if (0 || 0 || 0 || 0)
S  #define EMIF_GBLCTL_DEFAULT (Uint32)( \
S    0x00003000\
S    |_PER_FDEFAULT(EMIF,GBLCTL,BUSREQ)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,EKEN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)\
S    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)\
S  )
X  #define EMIF_GBLCTL_DEFAULT (Uint32)(     0x00003000    |_PER_FDEFAULT(EMIF,GBLCTL,BUSREQ)    |_PER_FDEFAULT(EMIF,GBLCTL,ARDY)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,HOLDA)    |_PER_FDEFAULT(EMIF,GBLCTL,NOHOLD)    |_PER_FDEFAULT(EMIF,GBLCTL,EKEN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK1EN)    |_PER_FDEFAULT(EMIF,GBLCTL,CLK2EN)  )
S
S  #define EMIF_GBLCTL_RMK(nohold,eken,clk1en,clk2en) (Uint32)( \
S     _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)\
S    |_PER_FMK(EMIF,GBLCTL,EKEN,eken)\
S    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)\
S    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)\
S  )
X  #define EMIF_GBLCTL_RMK(nohold,eken,clk1en,clk2en) (Uint32)(      _PER_FMK(EMIF,GBLCTL,NOHOLD,nohold)    |_PER_FMK(EMIF,GBLCTL,EKEN,eken)    |_PER_FMK(EMIF,GBLCTL,CLK1EN,clk1en)    |_PER_FMK(EMIF,GBLCTL,CLK2EN,clk2en)  )
N#endif
N
N  #define _EMIF_GBLCTL_FGET(FIELD)\
N    _PER_FGET(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD)
X  #define _EMIF_GBLCTL_FGET(FIELD)    _PER_FGET(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD)
N
N  #define _EMIF_GBLCTL_FSET(FIELD,field)\
N    _PER_FSET(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD,field)
X  #define _EMIF_GBLCTL_FSET(FIELD,field)    _PER_FSET(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD,field)
N
N  #define _EMIF_GBLCTL_FSETS(FIELD,SYM)\
N    _PER_FSETS(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD,##SYM)
X  #define _EMIF_GBLCTL_FSETS(FIELD,SYM)    _PER_FSETS(_EMIF_GBLCTL_ADDR,EMIF,GBLCTL,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  C E C T L        |
N* |___________________|
N*
N* CECTL0 - CE space control register 0
N* CECTL1 - CE space control register 1
N* CECTL2 - CE space control register 2
N* CECTL3 - CE space control register 3
N*
N* FIELDS (msb -> lsb)
N* (rw) WRSETUP
N* (rw) WRSTRB
N* (rw) WRHLD
N* (rw) RDSETUP
N* (rw) TA       (1) 
N* (rw) RDSTRB
N* (rw) MTYPE
N* (rw) WRHLDMSB 
N* (rw) RDHLD
N*
N* (1) - Field only exists for C11_SUPPORT
N* 
N*
N\******************************************************************************/
N  #define _EMIF_CECTL0_OFFSET          2
N  #define _EMIF_CECTL1_OFFSET          1
N  #define _EMIF_CECTL2_OFFSET          4
N  #define _EMIF_CECTL3_OFFSET          5
N
N  #define _EMIF_CECTL0_ADDR            0x01800008u
N  #define _EMIF_CECTL1_ADDR            0x01800004u
N  #define _EMIF_CECTL2_ADDR            0x01800010u
N  #define _EMIF_CECTL3_ADDR            0x01800014u
N
N  #define _EMIF_CECTL_WRSETUP_MASK     0xF0000000u
N  #define _EMIF_CECTL_WRSETUP_SHIFT    0x0000001Cu
N  #define  EMIF_CECTL_WRSETUP_DEFAULT  0x0000000Fu
N  #define  EMIF_CECTL_WRSETUP_OF(x)    _VALUEOF(x)
N
N  #define _EMIF_CECTL_WRSTRB_MASK      0x0FC00000u
N  #define _EMIF_CECTL_WRSTRB_SHIFT     0x00000016u
N  #define  EMIF_CECTL_WRSTRB_DEFAULT   0x0000003Fu
N  #define  EMIF_CECTL_WRSTRB_OF(x)     _VALUEOF(x)
N
N  #define _EMIF_CECTL_WRHLD_MASK       0x00300000u
N  #define _EMIF_CECTL_WRHLD_SHIFT      0x00000014u
N  #define  EMIF_CECTL_WRHLD_DEFAULT    0x00000003u
N  #define  EMIF_CECTL_WRHLD_OF(x)      _VALUEOF(x)
N  
N  #define _EMIF_CECTL_RDSETUP_MASK     0x000F0000u
N  #define _EMIF_CECTL_RDSETUP_SHIFT    0x00000010u
N  #define  EMIF_CECTL_RDSETUP_DEFAULT  0x0000000Fu
N  #define  EMIF_CECTL_RDSETUP_OF(x)    _VALUEOF(x)
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_CECTL_TA_MASK          0x0000C000u
S  #define _EMIF_CECTL_TA_SHIFT         0x0000000Eu
S  #define  EMIF_CECTL_TA_DEFAULT       0x00000003u
S  #define  EMIF_CECTL_TA_OF(x)         _VALUEOF(x)
N#endif
N
N  #define _EMIF_CECTL_RDSTRB_MASK      0x00003F00u
N  #define _EMIF_CECTL_RDSTRB_SHIFT     0x00000008u
N  #define  EMIF_CECTL_RDSTRB_DEFAULT   0x0000003Fu
N  #define  EMIF_CECTL_RDSTRB_OF(x)     _VALUEOF(x)
N  
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_CECTL_MTYPE_MASK       0x000000F0u
S  #define _EMIF_CECTL_MTYPE_SHIFT      0x00000004u
S  #define  EMIF_CECTL_MTYPE_DEFAULT    0x00000002u
S  #define  EMIF_CECTL_MTYPE_OF(x)      _VALUEOF(x)
S  #if (CHIP_6712 || CHIP_6712C)  /* 16-bit EMIF */ 
S  #define  EMIFB_CECTL_MTYPE_ASYNC8     0x00000000u 
S  #define  EMIFB_CECTL_MTYPE_ASYNC16    0x00000001u 
S  #define  EMIFB_CECTL_MTYPE_SDRAM8     0x00000008u 
S  #define  EMIFB_CECTL_MTYPE_SDRAM16    0x00000009u 
S  #define  EMIFB_CECTL_MTYPE_SYNC8      0x0000000Au 
S  #define  EMIFB_CECTL_MTYPE_SYNC16     0x0000000Bu 
S  #else /* CHIP_6211/C6711/6711C 32-bit EMIF */
S  #define  EMIF_CECTL_MTYPE_ASYNC8     0x00000000u
S  #define  EMIF_CECTL_MTYPE_ASYNC16    0x00000001u
S  #define  EMIF_CECTL_MTYPE_ASYNC32    0x00000002u
S  #define  EMIF_CECTL_MTYPE_SDRAM32    0x00000003u
S  #define  EMIF_CECTL_MTYPE_SBSRAM32   0x00000004u
S  #define  EMIF_CECTL_MTYPE_SDRAM8     0x00000008u
S  #define  EMIF_CECTL_MTYPE_SDRAM16    0x00000009u
S  #define  EMIF_CECTL_MTYPE_SBSRAM8    0x0000000Au
S  #define  EMIF_CECTL_MTYPE_SBSRAM16   0x0000000Bu
S  #endif
S 
N#else
N  #define _EMIF_CECTL_MTYPE_MASK       0x00000070u
N  #define _EMIF_CECTL_MTYPE_SHIFT      0x00000004u
N  #define  EMIF_CECTL_MTYPE_DEFAULT    0x00000002u
N  #define  EMIF_CECTL_MTYPE_OF(x)      _VALUEOF(x)
N  #define  EMIF_CECTL_MTYPE_ASYNC8     0x00000000u
N  #define  EMIF_CECTL_MTYPE_ASYNC16    0x00000001u
N  #define  EMIF_CECTL_MTYPE_ASYNC32    0x00000002u
N  #define  EMIF_CECTL_MTYPE_SDRAM32    0x00000003u
N  #define  EMIF_CECTL_MTYPE_SBSRAM32   0x00000004u
N#endif
N
N
N#if (C11_SUPPORT )
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ) )
S  #define _EMIF_CECTL_RDHLD_MASK       0x00000007u
S  #define _EMIF_CECTL_RDHLD_SHIFT      0x00000000u
S  #define  EMIF_CECTL_RDHLD_DEFAULT    0x00000003u
S  #define  EMIF_CECTL_RDHLD_OF(x)      _VALUEOF(x)
N#else
N  #define _EMIF_CECTL_RDHLD_MASK       0x00000003u
N  #define _EMIF_CECTL_RDHLD_SHIFT      0x00000000u
N  #define  EMIF_CECTL_RDHLD_DEFAULT    0x00000003u
N  #define  EMIF_CECTL_RDHLD_OF(x)      _VALUEOF(x)
N#endif
N
N  #define  EMIF_CECTL_OF(x)            _VALUEOF(x)
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define EMIF_CECTL_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(EMIF,CECTL,WRSETUP)\
S    |_PER_FDEFAULT(EMIF,CECTL,WRSTRB)\
S    |_PER_FDEFAULT(EMIF,CECTL,WRHLD)\
S    |_PER_FDEFAULT(EMIF,CECTL,RDSETUP)\
S    |_PER_FDEFAULT(EMIF,CECTL,TA)\
S    |_PER_FDEFAULT(EMIF,CECTL,RDSTRB)\
S    |_PER_FDEFAULT(EMIF,CECTL,MTYPE)\
S    |_PER_FDEFAULT(EMIF,CECTL,RDHLD)\
S  )
X  #define EMIF_CECTL_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,CECTL,WRSETUP)    |_PER_FDEFAULT(EMIF,CECTL,WRSTRB)    |_PER_FDEFAULT(EMIF,CECTL,WRHLD)    |_PER_FDEFAULT(EMIF,CECTL,RDSETUP)    |_PER_FDEFAULT(EMIF,CECTL,TA)    |_PER_FDEFAULT(EMIF,CECTL,RDSTRB)    |_PER_FDEFAULT(EMIF,CECTL,MTYPE)    |_PER_FDEFAULT(EMIF,CECTL,RDHLD)  )
S
S  #define EMIF_CECTL_RMK(wrsetup,wrstrb,wrhld,rdsetup,ta,rdstrb,mtype,\
S    rdhld) (Uint32)( \
S     _PER_FMK(EMIF,CECTL,WRSETUP,wrsetup)\
S    |_PER_FMK(EMIF,CECTL,WRSTRB,wrstrb)\
S    |_PER_FMK(EMIF,CECTL,WRHLD,wrhld)\
S    |_PER_FMK(EMIF,CECTL,RDSETUP,rdsetup)\
S    |_PER_FMK(EMIF,CECTL,TA,ta)\
S    |_PER_FMK(EMIF,CECTL,RDSTRB,rdstrb)\
S    |_PER_FMK(EMIF,CECTL,MTYPE,mtype)\
S    |_PER_FMK(EMIF,CECTL,RDHLD,rdhld)\
S  )
X  #define EMIF_CECTL_RMK(wrsetup,wrstrb,wrhld,rdsetup,ta,rdstrb,mtype,    rdhld) (Uint32)(      _PER_FMK(EMIF,CECTL,WRSETUP,wrsetup)    |_PER_FMK(EMIF,CECTL,WRSTRB,wrstrb)    |_PER_FMK(EMIF,CECTL,WRHLD,wrhld)    |_PER_FMK(EMIF,CECTL,RDSETUP,rdsetup)    |_PER_FMK(EMIF,CECTL,TA,ta)    |_PER_FMK(EMIF,CECTL,RDSTRB,rdstrb)    |_PER_FMK(EMIF,CECTL,MTYPE,mtype)    |_PER_FMK(EMIF,CECTL,RDHLD,rdhld)  )
N#endif
N
N
N
N#if (!C11_SUPPORT)
X#if (!( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N  #define EMIF_CECTL_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(EMIF,CECTL,WRSETUP)\
N    |_PER_FDEFAULT(EMIF,CECTL,WRSTRB)\
N    |_PER_FDEFAULT(EMIF,CECTL,WRHLD)\
N    |_PER_FDEFAULT(EMIF,CECTL,RDSETUP)\
N    |_PER_FDEFAULT(EMIF,CECTL,RDSTRB)\
N    |_PER_FDEFAULT(EMIF,CECTL,MTYPE)\
N    |_PER_FDEFAULT(EMIF,CECTL,RDHLD)\
N  )
X  #define EMIF_CECTL_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,CECTL,WRSETUP)    |_PER_FDEFAULT(EMIF,CECTL,WRSTRB)    |_PER_FDEFAULT(EMIF,CECTL,WRHLD)    |_PER_FDEFAULT(EMIF,CECTL,RDSETUP)    |_PER_FDEFAULT(EMIF,CECTL,RDSTRB)    |_PER_FDEFAULT(EMIF,CECTL,MTYPE)    |_PER_FDEFAULT(EMIF,CECTL,RDHLD)  )
N
N  #define EMIF_CECTL_RMK(wrsetup,wrstrb,wrhld,rdsetup,rdstrb,mtype,\
N    rdhld) (Uint32)( \
N     _PER_FMK(EMIF,CECTL,WRSETUP,wrsetup)\
N    |_PER_FMK(EMIF,CECTL,WRSTRB,wrstrb)\
N    |_PER_FMK(EMIF,CECTL,WRHLD,wrhld)\
N    |_PER_FMK(EMIF,CECTL,RDSETUP,rdsetup)\
N    |_PER_FMK(EMIF,CECTL,RDSTRB,rdstrb)\
N    |_PER_FMK(EMIF,CECTL,MTYPE,mtype)\
N    |_PER_FMK(EMIF,CECTL,RDHLD,rdhld)\
N  )
X  #define EMIF_CECTL_RMK(wrsetup,wrstrb,wrhld,rdsetup,rdstrb,mtype,    rdhld) (Uint32)(      _PER_FMK(EMIF,CECTL,WRSETUP,wrsetup)    |_PER_FMK(EMIF,CECTL,WRSTRB,wrstrb)    |_PER_FMK(EMIF,CECTL,WRHLD,wrhld)    |_PER_FMK(EMIF,CECTL,RDSETUP,rdsetup)    |_PER_FMK(EMIF,CECTL,RDSTRB,rdstrb)    |_PER_FMK(EMIF,CECTL,MTYPE,mtype)    |_PER_FMK(EMIF,CECTL,RDHLD,rdhld)  )
N#endif
N
N  #define _EMIF_CECTL_FGET(N,FIELD)\
N    _PER_FGET(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD)
X  #define _EMIF_CECTL_FGET(N,FIELD)    _PER_FGET(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD)
N
N  #define _EMIF_CECTL_FSET(N,FIELD,f)\
N    _PER_FSET(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD,f)
X  #define _EMIF_CECTL_FSET(N,FIELD,f)    _PER_FSET(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD,f)
N
N  #define _EMIF_CECTL_FSETS(N,FIELD,SYM)\
N    _PER_FSETS(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD,##SYM)
X  #define _EMIF_CECTL_FSETS(N,FIELD,SYM)    _PER_FSETS(_EMIF_CECTL##N##_ADDR,EMIF,CECTL,##FIELD,##SYM)
N
N  #define _EMIF_CECTL0_FGET(FIELD) _EMIF_CECTL_FGET(0,##FIELD)
N  #define _EMIF_CECTL1_FGET(FIELD) _EMIF_CECTL_FGET(1,##FIELD)
N  #define _EMIF_CECTL2_FGET(FIELD) _EMIF_CECTL_FGET(2,##FIELD)
N  #define _EMIF_CECTL3_FGET(FIELD) _EMIF_CECTL_FGET(3,##FIELD)
N
N  #define _EMIF_CECTL0_FSET(FIELD,f) _EMIF_CECTL_FSET(0,##FIELD,f)
N  #define _EMIF_CECTL1_FSET(FIELD,f) _EMIF_CECTL_FSET(1,##FIELD,f)
N  #define _EMIF_CECTL2_FSET(FIELD,f) _EMIF_CECTL_FSET(2,##FIELD,f)
N  #define _EMIF_CECTL3_FSET(FIELD,f) _EMIF_CECTL_FSET(3,##FIELD,f)
N
N  #define _EMIF_CECTL0_FSETS(FIELD,SYM) _EMIF_CECTL_FSETS(0,##FIELD,##SYM)
N  #define _EMIF_CECTL1_FSETS(FIELD,SYM) _EMIF_CECTL_FSETS(1,##FIELD,##SYM)
N  #define _EMIF_CECTL2_FSETS(FIELD,SYM) _EMIF_CECTL_FSETS(2,##FIELD,##SYM)
N  #define _EMIF_CECTL3_FSETS(FIELD,SYM) _EMIF_CECTL_FSETS(3,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  S D C T L        |
N* |___________________|
N*
N* SDCTL   - SDRAM control regsiter
N*
N* FIELDS (msb -> lsb)
N* (rw) SDBSZ (2)
N* (rw) SDRSZ (2)
N* (rw) SDCSZ (2)
N* (rw) SDWID (1)
N* (rw) RFEN
N* (r)  INIT
N* (rw) TRCD
N* (rw) TRP
N* (rw) TRC
N*
N* (1) - Field only exists for C01_SUPPORT
N* (2) - Field only exists for C11_SUPPORT 
N*
N\******************************************************************************/
N  #define _EMIF_SDCTL_OFFSET           6
N
N  #define _EMIF_SDCTL_ADDR             0x01800018u
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDCTL_SDBSZ_MASK       0x40000000u
S  #define _EMIF_SDCTL_SDBSZ_SHIFT      0x0000001Eu
S  #define  EMIF_SDCTL_SDBSZ_DEFAULT    0x00000000u
S  #define  EMIF_SDCTL_SDBSZ_OF(x)      _VALUEOF(x)
S  #define  EMIF_SDCTL_SDBSZ_2BANKS     0x00000000u
S  #define  EMIF_SDCTL_SDBSZ_4BANKS     0x00000001u
N#endif
N  
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDCTL_SDRSZ_MASK       0x30000000u
S  #define _EMIF_SDCTL_SDRSZ_SHIFT      0x0000001Cu
S  #define  EMIF_SDCTL_SDRSZ_DEFAULT    0x00000000u
S  #define  EMIF_SDCTL_SDRSZ_OF(x)      _VALUEOF(x)
S  #define  EMIF_SDCTL_SDRSZ_11ROW      0x00000000u
S  #define  EMIF_SDCTL_SDRSZ_12ROW      0x00000001u
S  #define  EMIF_SDCTL_SDRSZ_13ROW      0x00000002u
N#endif
N  
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDCTL_SDCSZ_MASK       0x0C000000u
S  #define _EMIF_SDCTL_SDCSZ_SHIFT      0x0000001Au
S  #define  EMIF_SDCTL_SDCSZ_DEFAULT    0x00000000u
S  #define  EMIF_SDCTL_SDCSZ_OF(x)      _VALUEOF(x)
S  #define  EMIF_SDCTL_SDCSZ_9COL       0x00000000u
S  #define  EMIF_SDCTL_SDCSZ_8COL       0x00000001u
S  #define  EMIF_SDCTL_SDCSZ_10COL      0x00000002u
N#endif
N  
N#if !(C11_SUPPORT) 
X#if !(( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) )) 
N  #define _EMIF_SDCTL_SDWID_MASK       0x04000000u
N  #define _EMIF_SDCTL_SDWID_SHIFT      0x0000001Au
N  #define  EMIF_SDCTL_SDWID_DEFAULT    0x00000000u
N  #define  EMIF_SDCTL_SDWID_OF(x)      _VALUEOF(x)
N  #define  EMIF_SDCTL_SDWID_4X8BIT     0x00000000u
N  #define  EMIF_SDCTL_SDWID_2X16BIT    0x00000001u
N#endif
N  
N  #define _EMIF_SDCTL_RFEN_MASK        0x02000000u
N  #define _EMIF_SDCTL_RFEN_SHIFT       0x00000019u
N  #define  EMIF_SDCTL_RFEN_DEFAULT     0x00000001u
N  #define  EMIF_SDCTL_RFEN_OF(x)       _VALUEOF(x)
N  #define  EMIF_SDCTL_RFEN_DISABLE     0x00000000u
N  #define  EMIF_SDCTL_RFEN_ENABLE      0x00000001u
N
N  #define _EMIF_SDCTL_INIT_MASK        0x01000000u
N  #define _EMIF_SDCTL_INIT_SHIFT       0x00000018u
N  #define  EMIF_SDCTL_INIT_DEFAULT     0x00000001u
N  #define  EMIF_SDCTL_INIT_OF(x)       _VALUEOF(x)
N  #define  EMIF_SDCTL_INIT_NO          0x00000000u
N  #define  EMIF_SDCTL_INIT_YES         0x00000001u
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDCTL_TRCD_MASK        0x00F00000u
S  #define _EMIF_SDCTL_TRCD_SHIFT       0x00000014u
S  #define  EMIF_SDCTL_TRCD_DEFAULT     0x00000004u
S  #define  EMIF_SDCTL_TRCD_OF(x)       _VALUEOF(x)
N#else
N  #define _EMIF_SDCTL_TRCD_MASK        0x00F00000u
N  #define _EMIF_SDCTL_TRCD_SHIFT       0x00000014u
N  #define  EMIF_SDCTL_TRCD_DEFAULT     0x00000008u
N  #define  EMIF_SDCTL_TRCD_OF(x)       _VALUEOF(x)
N#endif
N
N  #define _EMIF_SDCTL_TRP_MASK         0x000F0000u
N  #define _EMIF_SDCTL_TRP_SHIFT        0x00000010u
N  #define  EMIF_SDCTL_TRP_DEFAULT      0x00000008u
N  #define  EMIF_SDCTL_TRP_OF(x)        _VALUEOF(x)
N
N  #define _EMIF_SDCTL_TRC_MASK         0x0000F000u
N  #define _EMIF_SDCTL_TRC_SHIFT        0x0000000Cu
N  #define  EMIF_SDCTL_TRC_DEFAULT      0x0000000Fu
N  #define  EMIF_SDCTL_TRC_OF(x)        _VALUEOF(x)
N
N  #define  EMIF_SDCTL_OF(x)            _VALUEOF(x)
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define EMIF_SDCTL_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(EMIF,SDCTL,SDBSZ)\
S    |_PER_FDEFAULT(EMIF,SDCTL,SDRSZ)\
S    |_PER_FDEFAULT(EMIF,SDCTL,SDCSZ)\
S    |_PER_FDEFAULT(EMIF,SDCTL,RFEN)\
S    |_PER_FDEFAULT(EMIF,SDCTL,INIT)\
S    |_PER_FDEFAULT(EMIF,SDCTL,TRCD)\
S    |_PER_FDEFAULT(EMIF,SDCTL,TRP)\
S    |_PER_FDEFAULT(EMIF,SDCTL,TRC)\
S  )
X  #define EMIF_SDCTL_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,SDCTL,SDBSZ)    |_PER_FDEFAULT(EMIF,SDCTL,SDRSZ)    |_PER_FDEFAULT(EMIF,SDCTL,SDCSZ)    |_PER_FDEFAULT(EMIF,SDCTL,RFEN)    |_PER_FDEFAULT(EMIF,SDCTL,INIT)    |_PER_FDEFAULT(EMIF,SDCTL,TRCD)    |_PER_FDEFAULT(EMIF,SDCTL,TRP)    |_PER_FDEFAULT(EMIF,SDCTL,TRC)  )
S
S  #define EMIF_SDCTL_RMK(sdbsz,sdrsz,sdcsz,rfen,init,trcd,trp,trc) (Uint32)(\
S     _PER_FMK(EMIF,SDCTL,SDBSZ,sdbsz)\
S    |_PER_FMK(EMIF,SDCTL,SDRSZ,sdrsz)\
S    |_PER_FMK(EMIF,SDCTL,SDCSZ,sdcsz)\
S    |_PER_FMK(EMIF,SDCTL,RFEN,rfen)\
S    |_PER_FMK(EMIF,SDCTL,INIT,init)\
S    |_PER_FMK(EMIF,SDCTL,TRCD,trcd)\
S    |_PER_FMK(EMIF,SDCTL,TRP,trp)\
S    |_PER_FMK(EMIF,SDCTL,TRC,trc)\
S  )
X  #define EMIF_SDCTL_RMK(sdbsz,sdrsz,sdcsz,rfen,init,trcd,trp,trc) (Uint32)(     _PER_FMK(EMIF,SDCTL,SDBSZ,sdbsz)    |_PER_FMK(EMIF,SDCTL,SDRSZ,sdrsz)    |_PER_FMK(EMIF,SDCTL,SDCSZ,sdcsz)    |_PER_FMK(EMIF,SDCTL,RFEN,rfen)    |_PER_FMK(EMIF,SDCTL,INIT,init)    |_PER_FMK(EMIF,SDCTL,TRCD,trcd)    |_PER_FMK(EMIF,SDCTL,TRP,trp)    |_PER_FMK(EMIF,SDCTL,TRC,trc)  )
N#endif
N
N#if !(C11_SUPPORT)
X#if !(( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N  #define EMIF_SDCTL_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(EMIF,SDCTL,SDWID)\
N    |_PER_FDEFAULT(EMIF,SDCTL,RFEN)\
N    |_PER_FDEFAULT(EMIF,SDCTL,INIT)\
N    |_PER_FDEFAULT(EMIF,SDCTL,TRCD)\
N    |_PER_FDEFAULT(EMIF,SDCTL,TRP)\
N    |_PER_FDEFAULT(EMIF,SDCTL,TRC)\
N  )
X  #define EMIF_SDCTL_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,SDCTL,SDWID)    |_PER_FDEFAULT(EMIF,SDCTL,RFEN)    |_PER_FDEFAULT(EMIF,SDCTL,INIT)    |_PER_FDEFAULT(EMIF,SDCTL,TRCD)    |_PER_FDEFAULT(EMIF,SDCTL,TRP)    |_PER_FDEFAULT(EMIF,SDCTL,TRC)  )
N
N  #define EMIF_SDCTL_RMK(sdwid,rfen,init,trcd,trp,trc) (Uint32)(\
N     _PER_FMK(EMIF,SDCTL,SDWID,sdwid)\
N    |_PER_FMK(EMIF,SDCTL,RFEN,rfen)\
N    |_PER_FMK(EMIF,SDCTL,INIT,init)\
N    |_PER_FMK(EMIF,SDCTL,TRCD,trcd)\
N    |_PER_FMK(EMIF,SDCTL,TRP,trp)\
N    |_PER_FMK(EMIF,SDCTL,TRC,trc)\
N  )
X  #define EMIF_SDCTL_RMK(sdwid,rfen,init,trcd,trp,trc) (Uint32)(     _PER_FMK(EMIF,SDCTL,SDWID,sdwid)    |_PER_FMK(EMIF,SDCTL,RFEN,rfen)    |_PER_FMK(EMIF,SDCTL,INIT,init)    |_PER_FMK(EMIF,SDCTL,TRCD,trcd)    |_PER_FMK(EMIF,SDCTL,TRP,trp)    |_PER_FMK(EMIF,SDCTL,TRC,trc)  )
N#endif
N
N  #define _EMIF_SDCTL_FGET(FIELD)\
N    _PER_FGET(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD)
X  #define _EMIF_SDCTL_FGET(FIELD)    _PER_FGET(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD)
N
N  #define _EMIF_SDCTL_FSET(FIELD,field)\
N    _PER_FSET(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD,field)
X  #define _EMIF_SDCTL_FSET(FIELD,field)    _PER_FSET(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD,field)
N
N  #define _EMIF_SDCTL_FSETS(FIELD,SYM)\
N    _PER_FSETS(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD,##SYM)
X  #define _EMIF_SDCTL_FSETS(FIELD,SYM)    _PER_FSETS(_EMIF_SDCTL_ADDR,EMIF,SDCTL,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  S D T I M        |
N* |___________________|
N*
N* SDTIM   - SDRAM timing register
N*
N* FIELDS (msb -> lsb)
N* (rw) XRFR   (1)
N* (r)  CNTR
N* (rw) PERIOD
N*
N* (1) - Field only exists for C11_SUPPORT
N*
N\******************************************************************************/
N  #define _EMIF_SDTIM_OFFSET           7
N
N  #define _EMIF_SDTIM_ADDR             0x0180001Cu
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDTIM_XRFR_MASK        0x03000000u
S  #define _EMIF_SDTIM_XRFR_SHIFT       0x00000018u
S  #define  EMIF_SDTIM_XRFR_DEFAULT     0x00000000u
S  #define  EMIF_SDTIM_XRFR_OF(x)       _VALUEOF(x)
N#endif
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDTIM_CNTR_MASK        0x00FFF000u
S  #define _EMIF_SDTIM_CNTR_SHIFT       0x0000000Cu
S  #define  EMIF_SDTIM_CNTR_DEFAULT     0x000005DCu
S  #define  EMIF_SDTIM_CNTR_OF(x)       _VALUEOF(x)
N#else
N  #define _EMIF_SDTIM_CNTR_MASK        0x00FFF000u
N  #define _EMIF_SDTIM_CNTR_SHIFT       0x0000000Cu
N  #define  EMIF_SDTIM_CNTR_DEFAULT     0x00000040u
N  #define  EMIF_SDTIM_CNTR_OF(x)       _VALUEOF(x)
N#endif
N  
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDTIM_PERIOD_MASK      0x00000FFFu
S  #define _EMIF_SDTIM_PERIOD_SHIFT     0x00000000u
S  #define  EMIF_SDTIM_PERIOD_DEFAULT   0x000005DCu
S  #define  EMIF_SDTIM_PERIOD_OF(x)     _VALUEOF(x)
N#else
N  #define _EMIF_SDTIM_PERIOD_MASK      0x00000FFFu
N  #define _EMIF_SDTIM_PERIOD_SHIFT     0x00000000u
N  #define  EMIF_SDTIM_PERIOD_DEFAULT   0x00000040u
N  #define  EMIF_SDTIM_PERIOD_OF(x)     _VALUEOF(x)
N#endif
N
N  #define  EMIF_SDTIM_OF(x)            _VALUEOF(x)
N
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define EMIF_SDTIM_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(EMIF,SDTIM,XRFR)\
S    |_PER_FDEFAULT(EMIF,SDTIM,CNTR)\
S    |_PER_FDEFAULT(EMIF,SDTIM,PERIOD)\
S  )
X  #define EMIF_SDTIM_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,SDTIM,XRFR)    |_PER_FDEFAULT(EMIF,SDTIM,CNTR)    |_PER_FDEFAULT(EMIF,SDTIM,PERIOD)  )
S
S  #define EMIF_SDTIM_RMK(xrfr,period) (Uint32)(\
S     _PER_FMK(EMIF,SDTIM,XRFR,xrfr)\
S    |_PER_FMK(EMIF,SDTIM,PERIOD,period)\
S  )
X  #define EMIF_SDTIM_RMK(xrfr,period) (Uint32)(     _PER_FMK(EMIF,SDTIM,XRFR,xrfr)    |_PER_FMK(EMIF,SDTIM,PERIOD,period)  )
N#endif
N
N#if !(C11_SUPPORT)
X#if !(( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
N  #define EMIF_SDTIM_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(EMIF,SDTIM,CNTR)\
N    |_PER_FDEFAULT(EMIF,SDTIM,PERIOD)\
N  )
X  #define EMIF_SDTIM_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,SDTIM,CNTR)    |_PER_FDEFAULT(EMIF,SDTIM,PERIOD)  )
N
N  #define EMIF_SDTIM_RMK(period) (Uint32)(\
N     _PER_FMK(EMIF,SDTIM,PERIOD,period)\
N  )
X  #define EMIF_SDTIM_RMK(period) (Uint32)(     _PER_FMK(EMIF,SDTIM,PERIOD,period)  )
N#endif
N
N
N  #define _EMIF_SDTIM_FGET(FIELD)\
N    _PER_FGET(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD)
X  #define _EMIF_SDTIM_FGET(FIELD)    _PER_FGET(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD)
N
N  #define _EMIF_SDTIM_FSET(FIELD,field)\
N    _PER_FSET(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD,field)
X  #define _EMIF_SDTIM_FSET(FIELD,field)    _PER_FSET(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD,field)
N
N  #define _EMIF_SDTIM_FSETS(FIELD,SYM)\
N    _PER_FSETS(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD,##SYM)
X  #define _EMIF_SDTIM_FSETS(FIELD,SYM)    _PER_FSETS(_EMIF_SDTIM_ADDR,EMIF,SDTIM,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  S D E X T        |
N* |___________________|
N*
N* SDEXT   - SDRAM extension register (1)
N*
N* (1) - only supported on 6211,6711,6713,DA610,6711C,6712C
N*
N* FIELDS (msb -> lsb)
N* (rw) WR2RD
N* (rw) WR2DEAC
N* (rw) WR2WR
N* (rw) R2WDQM
N* (rw) RD2WR
N* (rw) RD2DEAC
N* (rw) RD2RD
N* (rw) THZP
N* (rw) TWR
N* (rw) TRRD
N* (rw) TRAS
N* (rw) TCL
N*
N\******************************************************************************/
N#if (C11_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 0*0)| ( 0*0)| ( 0*0)| ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) ))
S  #define _EMIF_SDEXT_OFFSET           8
S
S  #define _EMIF_SDEXT_ADDR             0x01800020u
S
S  #define _EMIF_SDEXT_WR2RD_MASK       0x00100000u
S  #define _EMIF_SDEXT_WR2RD_SHIFT      0x00000014u
S  #define  EMIF_SDEXT_WR2RD_DEFAULT    0x00000001u
S  #define  EMIF_SDEXT_WR2RD_OF(x)      _VALUEOF(x)
S
S  #define _EMIF_SDEXT_WR2DEAC_MASK     0x000C0000u
S  #define _EMIF_SDEXT_WR2DEAC_SHIFT    0x00000012u
S  #define  EMIF_SDEXT_WR2DEAC_DEFAULT  0x00000001u
S  #define  EMIF_SDEXT_WR2DEAC_OF(x)    _VALUEOF(x)
S
S  #define _EMIF_SDEXT_WR2WR_MASK       0x00020000u
S  #define _EMIF_SDEXT_WR2WR_SHIFT      0x00000011u
S  #define  EMIF_SDEXT_WR2WR_DEFAULT    0x00000001u
S  #define  EMIF_SDEXT_WR2WR_OF(x)      _VALUEOF(x)
S
S  #define _EMIF_SDEXT_R2WDQM_MASK      0x00018000u
S  #define _EMIF_SDEXT_R2WDQM_SHIFT     0x0000000Fu
S  #define  EMIF_SDEXT_R2WDQM_DEFAULT   0x00000003u
S  #define  EMIF_SDEXT_R2WDQM_OF(x)     _VALUEOF(x)
S
S  #define _EMIF_SDEXT_RD2WR_MASK       0x00007000u
S  #define _EMIF_SDEXT_RD2WR_SHIFT      0x0000000Cu
S  #define  EMIF_SDEXT_RD2WR_DEFAULT    0x00000005u
S  #define  EMIF_SDEXT_RD2WR_OF(x)      _VALUEOF(x)
S
S  #define _EMIF_SDEXT_RD2DEAC_MASK     0x00000C00u
S  #define _EMIF_SDEXT_RD2DEAC_SHIFT    0x0000000Au
S  #define  EMIF_SDEXT_RD2DEAC_DEFAULT  0x00000003u
S  #define  EMIF_SDEXT_RD2DEAC_OF(x)    _VALUEOF(x)
S
S  #define _EMIF_SDEXT_RD2RD_MASK       0x00000200u
S  #define _EMIF_SDEXT_RD2RD_SHIFT      0x00000009u
S  #define  EMIF_SDEXT_RD2RD_DEFAULT    0x00000001u
S  #define  EMIF_SDEXT_RD2RD_OF(x)      _VALUEOF(x)
S
S  #define _EMIF_SDEXT_THZP_MASK        0x00000180u
S  #define _EMIF_SDEXT_THZP_SHIFT       0x00000007u
S  #define  EMIF_SDEXT_THZP_DEFAULT     0x00000002u
S  #define  EMIF_SDEXT_THZP_OF(x)       _VALUEOF(x)
S
S  #define _EMIF_SDEXT_TWR_MASK         0x00000060u
S  #define _EMIF_SDEXT_TWR_SHIFT        0x00000005u
S  #define  EMIF_SDEXT_TWR_DEFAULT      0x00000001u
S  #define  EMIF_SDEXT_TWR_OF(x)        _VALUEOF(x)
S
S  #define _EMIF_SDEXT_TRRD_MASK        0x00000010u
S  #define _EMIF_SDEXT_TRRD_SHIFT       0x00000004u
S  #define  EMIF_SDEXT_TRRD_DEFAULT     0x00000001u
S  #define  EMIF_SDEXT_TRRD_OF(x)       _VALUEOF(x)
S
S  #define _EMIF_SDEXT_TRAS_MASK        0x0000000Eu
S  #define _EMIF_SDEXT_TRAS_SHIFT       0x00000001u
S  #define  EMIF_SDEXT_TRAS_DEFAULT     0x00000007u
S  #define  EMIF_SDEXT_TRAS_OF(x)       _VALUEOF(x)
S
S  #define _EMIF_SDEXT_TCL_MASK         0x00000001u
S  #define _EMIF_SDEXT_TCL_SHIFT        0x00000000u
S  #define  EMIF_SDEXT_TCL_DEFAULT      0x00000001u
S  #define  EMIF_SDEXT_TCL_OF(x)        _VALUEOF(x)
S
S  #define  EMIF_SDEXT_OF(x)            _VALUEOF(x)
S
S  #define EMIF_SDEXT_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(EMIF,SDEXT,WR2RD)\
S    |_PER_FDEFAULT(EMIF,SDEXT,WR2DEAC)\
S    |_PER_FDEFAULT(EMIF,SDEXT,WR2WR)\
S    |_PER_FDEFAULT(EMIF,SDEXT,R2WDQM)\
S    |_PER_FDEFAULT(EMIF,SDEXT,RD2WR)\
S    |_PER_FDEFAULT(EMIF,SDEXT,RD2DEAC)\
S    |_PER_FDEFAULT(EMIF,SDEXT,RD2RD)\
S    |_PER_FDEFAULT(EMIF,SDEXT,THZP)\
S    |_PER_FDEFAULT(EMIF,SDEXT,TWR)\
S    |_PER_FDEFAULT(EMIF,SDEXT,TRRD)\
S    |_PER_FDEFAULT(EMIF,SDEXT,TRAS)\
S    |_PER_FDEFAULT(EMIF,SDEXT,TCL)\
S  )
X  #define EMIF_SDEXT_DEFAULT (Uint32)(      _PER_FDEFAULT(EMIF,SDEXT,WR2RD)    |_PER_FDEFAULT(EMIF,SDEXT,WR2DEAC)    |_PER_FDEFAULT(EMIF,SDEXT,WR2WR)    |_PER_FDEFAULT(EMIF,SDEXT,R2WDQM)    |_PER_FDEFAULT(EMIF,SDEXT,RD2WR)    |_PER_FDEFAULT(EMIF,SDEXT,RD2DEAC)    |_PER_FDEFAULT(EMIF,SDEXT,RD2RD)    |_PER_FDEFAULT(EMIF,SDEXT,THZP)    |_PER_FDEFAULT(EMIF,SDEXT,TWR)    |_PER_FDEFAULT(EMIF,SDEXT,TRRD)    |_PER_FDEFAULT(EMIF,SDEXT,TRAS)    |_PER_FDEFAULT(EMIF,SDEXT,TCL)  )
S
S  #define EMIF_SDEXT_RMK(wr2rd,wr2deac,wr2wr,r2wdqm,rd2wr,rd2deac,\
S    rd2rd,thzp,twr,trrd,tras,tcl) (Uint32)( \
S     _PER_FMK(EMIF,SDEXT,WR2RD,wr2rd)\
S    |_PER_FMK(EMIF,SDEXT,WR2DEAC,wr2deac)\
S    |_PER_FMK(EMIF,SDEXT,WR2WR,wr2wr)\
S    |_PER_FMK(EMIF,SDEXT,R2WDQM,r2wdqm)\
S    |_PER_FMK(EMIF,SDEXT,RD2WR,rd2wr)\
S    |_PER_FMK(EMIF,SDEXT,RD2DEAC,rd2deac)\
S    |_PER_FMK(EMIF,SDEXT,RD2RD,rd2rd)\
S    |_PER_FMK(EMIF,SDEXT,THZP,thzp)\
S    |_PER_FMK(EMIF,SDEXT,TWR,twr)\
S    |_PER_FMK(EMIF,SDEXT,TRRD,trrd)\
S    |_PER_FMK(EMIF,SDEXT,TRAS,tras)\
S    |_PER_FMK(EMIF,SDEXT,TCL,tcl)\
S  )
X  #define EMIF_SDEXT_RMK(wr2rd,wr2deac,wr2wr,r2wdqm,rd2wr,rd2deac,    rd2rd,thzp,twr,trrd,tras,tcl) (Uint32)(      _PER_FMK(EMIF,SDEXT,WR2RD,wr2rd)    |_PER_FMK(EMIF,SDEXT,WR2DEAC,wr2deac)    |_PER_FMK(EMIF,SDEXT,WR2WR,wr2wr)    |_PER_FMK(EMIF,SDEXT,R2WDQM,r2wdqm)    |_PER_FMK(EMIF,SDEXT,RD2WR,rd2wr)    |_PER_FMK(EMIF,SDEXT,RD2DEAC,rd2deac)    |_PER_FMK(EMIF,SDEXT,RD2RD,rd2rd)    |_PER_FMK(EMIF,SDEXT,THZP,thzp)    |_PER_FMK(EMIF,SDEXT,TWR,twr)    |_PER_FMK(EMIF,SDEXT,TRRD,trrd)    |_PER_FMK(EMIF,SDEXT,TRAS,tras)    |_PER_FMK(EMIF,SDEXT,TCL,tcl)  )
S
S  #define _EMIF_SDEXT_FGET(FIELD)\
S    _PER_FGET(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD)
X  #define _EMIF_SDEXT_FGET(FIELD)    _PER_FGET(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD)
S
S  #define _EMIF_SDEXT_FSET(FIELD,field)\
S    _PER_FSET(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD,field)
X  #define _EMIF_SDEXT_FSET(FIELD,field)    _PER_FSET(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD,field)
S
S  #define _EMIF_SDEXT_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD,##SYM)
X  #define _EMIF_SDEXT_FSETS(FIELD,SYM)    _PER_FSETS(_EMIF_SDEXT_ADDR,EMIF,SDEXT,##FIELD,##SYM)
S
N#endif
N
N/*----------------------------------------------------------------------------*/
N
N#endif /* EMIF_SUPPORT */
N#endif /* _CSL_EMIFHAL_H_ */
N/******************************************************************************\
N* End of csl_emifhal.h
X
N\******************************************************************************/
N
L 21 "E:/TI_workspase/c6701_t/csl/csl_chip.h" 2
N
N/******************************************************************************\
N* scope and inline control macros
X
N\******************************************************************************/
N#ifdef __cplusplus
S#define CSLAPI extern "C" far
N#else
N#define CSLAPI extern far
N#endif
N
N#undef  USEDEFS
N#undef  IDECL
N#undef  IDEF
N
N#ifdef  _CHIP_MOD_
S  #define IDECL CSLAPI
S  #define USEDEFS
S  #define IDEF
N#else
N  #ifdef  _INLINE
S    #define IDECL static inline
S    #define USEDEFS
S    #define IDEF  static inline
N  #else
N    #define IDECL CSLAPI
N  #endif
N#endif
N
N
N/******************************************************************************\
N* global macro declarations
X
N\******************************************************************************/
N#define CHIP_ENDIAN_BIG    0
N#define CHIP_ENDIAN_LITTLE 1
N
N#define CHIP_MAP_0         0
N#define CHIP_MAP_1         1
N
N/* Selected devices masks*/
N#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
X#if (0 || 0 || 0 || 0)
S#define CHIP_EKSRC_SYSCLK3  0x00000000u
S#define CHIP_EKSRC_ECLKIN   0x00000010u
N#endif
N
N#if (CHIP_6713 || CHIP_DA610)
X#if (0 || 0)
S#define CHIP_TOUT1          0x00000000u
S#define CHIP_AXR04_AXR111   0x00000008u
S
S#define CHIP_TOUT0          0x00000000u
S#define CHIP_AXR02_AXR113   0x00000004u
S
S#define CHIP_MCASP0         0x00000002u
S#define CHIP_MCBSP0         0x00000000u
S
S#define CHIP_I2C1           0x00000001u
S#define CHIP_MCBSP1         0x00000000u
S/* Full mask value */
S#define _CHIP_DEVCFG_MASK  0x0000001Fu
N#endif
N
N#if CHIP_DM642
X#if 0
S#define CHIP_VP2            0x00000040u
S#define CHIP_VP1            0x00000020u
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000007Fu
N#endif
N
N#if CHIP_DM641
X#if 0
S#define CHIP_VP1            0x00000020u
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000003Fu
N#endif
N
N#if CHIP_DM640
X#if 0
S#define CHIP_VP0            0x00000010u
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000001Fu
N#endif
N
N#if CHIP_6412
X#if 0
S#define CHIP_I2C            0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define _CHIP_PERCFG_MASK   0x0000000Eu
N#endif
N
N#if (CHIP_6410 || CHIP_6413 || CHIP_6418)
X#if (0 || 0 || 0)
S#define CHIP_AFCMUX         0x00000600u
S#define CHIP_MCASP1         0x00000100u
S#define CHIP_I2C1           0x00000080u
S#define CHIP_I2C0           0x00000008u
S#define CHIP_MCBSP1         0x00000004u
S#define CHIP_MCBSP0         0x00000002u
S#define CHIP_MCASP0         0x00000001u
S
S#define _CHIP_PERCFG_MASK   0x0000078Fu
N#endif
N
N/******************************************************************************\
N* global typedef declarations
X
N\******************************************************************************/
N#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
X#if( 0 || 0 || 0 || 0 )
Stypedef struct{
S   Uint32 devcfg;
S} CHIP_Config;
S#elif (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
X#elif (0 || 0 || 0  || 0 || 0 || 0 || 0)
Stypedef struct{
S   Uint32 percfg;
S} CHIP_Config; 
N#endif
N/******************************************************************************\
N* global variable declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* global function declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* inline function declarations
X
N\******************************************************************************/
NIDECL Uint32 CHIP_getCpuId();
Xextern far Uint32 CHIP_getCpuId();
NIDECL Uint32 CHIP_getRevId();
Xextern far Uint32 CHIP_getRevId();
NIDECL Uint32 CHIP_getSiliconRevId();
Xextern far Uint32 CHIP_getSiliconRevId();
NIDECL int    CHIP_getEndian();
Xextern far int    CHIP_getEndian();
NIDECL int    CHIP_getMapMode();
Xextern far int    CHIP_getMapMode();
N
N#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418) 
X#if (0 || 0 || 0  || 0 || 0 || 0 || 0) 
SIDECL void   CHIP_config(CHIP_Config *config);
SIDECL void   CHIP_configArgs(Uint32 percfg);
SIDECL void   CHIP_getConfig(CHIP_Config *config); 
S#elif ( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
X#elif ( 0 || 0 || 0 || 0 )
SIDECL void   CHIP_config(CHIP_Config *config);
SIDECL void   CHIP_configArgs(Uint32 devcfg);
SIDECL void   CHIP_getConfig(CHIP_Config *config); 
N#endif
N
N/******************************************************************************\
N* inline function definitions
X
N\******************************************************************************/
N#ifdef USEDEFS
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getCpuId() {
S  return CHIP_FGET(CSR,CPUID);
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getRevId() {
S  return CHIP_FGET(CSR,REVID);
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 CHIP_getSiliconRevId() {
S  return (( 0x000F0000 & REG32(0x01B00200))>>16);
S}
S/*----------------------------------------------------------------------------*/
SIDEF int CHIP_getEndian() {
S  return CHIP_FGET(CSR,EN);
S}
S/*----------------------------------------------------------------------------*/
SIDEF int CHIP_getMapMode() {
S  int mapmode = 0;
S  #if (!C11_SUPPORT && !C64_SUPPORT)
S    mapmode = EMIF_FGET(GBLCTL,MAP);
S  #endif
S  return mapmode;
S}
S/*----------------------------------------------------------------------------*/
S#if (CHIP_6713 || CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_DA610 || CHIP_6412 || CHIP_6711C || CHIP_6712C || CHIP_6410 || CHIP_6413 || CHIP_6418)
SIDEF void   CHIP_config(CHIP_Config *config) {
S  Uint32 gie = CHIP_FGET(CSR,GIE);
S  
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 ) 
S    volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S    register int x0;
S    CHIP_FSET(CSR,GIE,0);
S    x0 = config->devcfg;
S    base[_CHIP_DEVCFG_OFFSET] = x0;
S    CHIP_FSET(CSR,GIE,gie);
S    
S#else 
S  volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S  register int x0;
S  CHIP_FSET(CSR,GIE,0);
S  CHIP_FSETS(PCFGLOCK,LOCK,UNLOCK);
S  x0 = config->percfg;
S  base[_CHIP_PERCFG_OFFSET] = x0;
S  CHIP_FSET(CSR,GIE,gie);
S  
S#endif
S}      
S#endif
S/*----------------------------------------------------------------------------*/
S#if (CHIP_6713 || CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_DA610 || CHIP_6412 || CHIP_6711C || CHIP_6712C || CHIP_6410 || CHIP_6413 || CHIP_6418)
SIDEF void   CHIP_getConfig(CHIP_Config *config) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 ) 
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S   register int x0;
S
S   CHIP_FSET(CSR,GIE,0);
S
S   x0 = base[_CHIP_DEVCFG_OFFSET]; 
S   config->devcfg=x0;
S
S   CHIP_FSET(CSR,GIE,gie);
S   
S#else
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S   register int x0;
S
S   CHIP_FSET(CSR,GIE,0);
S
S   x0 = base[_CHIP_PERCFG_OFFSET]; 
S   config->percfg=x0;
S
S   CHIP_FSET(CSR,GIE,gie);
S   
S#endif
S}      
S#endif
S/*----------------------------------------------------------------------------*/
S   
S#if( CHIP_6711C || CHIP_6712C || CHIP_DA610 || CHIP_6713 )
S   IDEF void   CHIP_configArgs(Uint32 devcfg) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_DEVCFG_ADDR);
S   CHIP_FSET(CSR,GIE,0);
S   base[_CHIP_DEVCFG_OFFSET]= devcfg; 
S   CHIP_FSET(CSR,GIE,gie);
S}   
S#elif (CHIP_DM642 || CHIP_DM641 || CHIP_DM640  || CHIP_6412 || CHIP_6410 || CHIP_6413 || CHIP_6418)
S   IDEF void   CHIP_configArgs(Uint32 percfg) {
S   Uint32 gie = CHIP_FGET(CSR,GIE);
S   volatile Uint32 *base = (volatile Uint32 *)(_CHIP_PERCFG_ADDR);
S
S   CHIP_FSET(CSR,GIE,0);
S
S#if(CHIP_DM642 || CHIP_6412)
S   CHIP_FSETS(PCFGLOCK,LOCK,UNLOCK);
S#endif
S
S   base[_CHIP_PERCFG_OFFSET]= percfg; 
S
S   CHIP_FSET(CSR,GIE,gie);
S}   
S#endif
S
S
S/*----------------------------------------------------------------------------*/
N#endif /* USEDEFS */
N
N
N#endif /* _CSL_CHIP_H_ */
N/******************************************************************************\
N* End of csl_chip.h
X
N\******************************************************************************/
N
L 14 "..\csl\csl_cache.h" 2
N#include <csl_irq.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_irq.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_irq.h
N* DATE CREATED.. 06/09/1999 
N* LAST MODIFIED. 08/02/2004 - Adding support for C6418
N*                07/24/2004 - Re-introducing BIOS components from CSL due to compatibility issues.
N*                02/05/2004 Removed bios related components 
N*                10/03/2001
N*                - CHIP_RSET()/CHIP_RGET() replaced by CHIP_CRSET()/CHIP_CRGET()
N\******************************************************************************/
N#ifndef _CSL_IRQ_H_
N#define _CSL_IRQ_H_
N
N#include <csl_stdinc.h>
N#include <csl_chip.h>
N#include <csl_irqhal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_irqhal.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_irqhal.h
N* DATE CREATED.. 06/20/1999 
N* LAST MODIFIED. 10/03/2000
N*------------------------------------------------------------------------------
N* REGISTERS
N*
N* MUXH   - interrupt multiplexer high register
N* MUXL   - interrupt multiplexer low register
N* EXTPOL - external interrupt polarity register
N*
N\******************************************************************************/
N#ifndef _CSL_IRQHAL_H_
N#define _CSL_IRQHAL_H_
N
N#include <csl_stdinchal.h>
N#include <csl_chip.h>
N
N#if (IRQ_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N/******************************************************************************\
N* MISC section
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* module level register/field access macros
X
N\******************************************************************************/
N
N  /* ----------------- */
N  /* FIELD MAKE MACROS */
N  /* ----------------- */
N
N  #define IRQ_FMK(REG,FIELD,x)\
N    _PER_FMK(IRQ,##REG,##FIELD,x)
X  #define IRQ_FMK(REG,FIELD,x)    _PER_FMK(IRQ,##REG,##FIELD,x)
N
N  #define IRQ_FMKS(REG,FIELD,SYM)\
N    _PER_FMKS(IRQ,##REG,##FIELD,##SYM)
X  #define IRQ_FMKS(REG,FIELD,SYM)    _PER_FMKS(IRQ,##REG,##FIELD,##SYM)
N 
N 
N  /* -------------------------------- */
N  /* RAW REGISTER/FIELD ACCESS MACROS */
N  /* -------------------------------- */
N
N  #define IRQ_ADDR(REG)\
N    _IRQ_##REG##_ADDR
X  #define IRQ_ADDR(REG)    _IRQ_##REG##_ADDR
N
N  #define IRQ_RGET(REG)\
N    _PER_RGET(_IRQ_##REG##_ADDR,IRQ,##REG)
X  #define IRQ_RGET(REG)    _PER_RGET(_IRQ_##REG##_ADDR,IRQ,##REG)
N
N  #define IRQ_RSET(REG,x)\
N    _PER_RSET(_IRQ_##REG##_ADDR,IRQ,##REG,x)
X  #define IRQ_RSET(REG,x)    _PER_RSET(_IRQ_##REG##_ADDR,IRQ,##REG,x)
N
N  #define IRQ_FGET(REG,FIELD)\
N    _IRQ_##REG##_FGET(##FIELD)
X  #define IRQ_FGET(REG,FIELD)    _IRQ_##REG##_FGET(##FIELD)
N
N  #define IRQ_FSET(REG,FIELD,x)\
N    _IRQ_##REG##_FSET(##FIELD,##x)
X  #define IRQ_FSET(REG,FIELD,x)    _IRQ_##REG##_FSET(##FIELD,##x)
N
N  #define IRQ_FSETS(REG,FIELD,SYM)\
N    _IRQ_##REG##_FSETS(##FIELD,##SYM)
X  #define IRQ_FSETS(REG,FIELD,SYM)    _IRQ_##REG##_FSETS(##FIELD,##SYM)
N 
N 
N  /* ------------------------------------------ */
N  /* ADDRESS BASED REGISTER/FIELD ACCESS MACROS */
N  /* ------------------------------------------ */
N
N  #define IRQ_RGETA(addr,REG)\
N    _PER_RGET(addr,IRQ,##REG)
X  #define IRQ_RGETA(addr,REG)    _PER_RGET(addr,IRQ,##REG)
N
N  #define IRQ_RSETA(addr,REG,x)\
N    _PER_RSET(addr,IRQ,##REG,x)
X  #define IRQ_RSETA(addr,REG,x)    _PER_RSET(addr,IRQ,##REG,x)
N
N  #define IRQ_FGETA(addr,REG,FIELD)\
N    _PER_FGET(addr,IRQ,##REG,##FIELD)
X  #define IRQ_FGETA(addr,REG,FIELD)    _PER_FGET(addr,IRQ,##REG,##FIELD)
N
N  #define IRQ_FSETA(addr,REG,FIELD,x)\
N    _PER_FSET(addr,IRQ,##REG,##FIELD,x)
X  #define IRQ_FSETA(addr,REG,FIELD,x)    _PER_FSET(addr,IRQ,##REG,##FIELD,x)
N
N  #define IRQ_FSETSA(addr,REG,FIELD,SYM)\
N    _PER_FSETS(addr,IRQ,##REG,##FIELD,##SYM)
X  #define IRQ_FSETSA(addr,REG,FIELD,SYM)    _PER_FSETS(addr,IRQ,##REG,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  M U X H          |
N* |___________________|
N*
N* MUXH - interrupt multiplexer high register
N*
N* FIELDS (msb -> lsb)
N* (rw) INTSEL15
N* (rw) INTSEL14
N* (rw) INTSEL13
N* (rw) INTSEL12
N* (rw) INTSEL11
N* (rw) INTSEL10
N*
N\******************************************************************************/
N  #define _IRQ_MUXH_ADDR               0x019C0000u
N
N  #define _IRQ_MUXH_INTSEL15_MASK      0x7C000000u
N  #define _IRQ_MUXH_INTSEL15_SHIFT     0x0000001Au
N  #define  IRQ_MUXH_INTSEL15_DEFAULT   0x00000002u
N  #define  IRQ_MUXH_INTSEL15_OF(x)     _VALUEOF(x)
N
N  #define _IRQ_MUXH_INTSEL14_MASK      0x03E00000u
N  #define _IRQ_MUXH_INTSEL14_SHIFT     0x00000015u
N  #define  IRQ_MUXH_INTSEL14_DEFAULT   0x00000001u
N  #define  IRQ_MUXH_INTSEL14_OF(x)     _VALUEOF(x)
N
N  #define _IRQ_MUXH_INTSEL13_MASK      0x001F0000u
N  #define _IRQ_MUXH_INTSEL13_SHIFT     0x00000010u
N  #define  IRQ_MUXH_INTSEL13_DEFAULT   0x00000000u
N  #define  IRQ_MUXH_INTSEL13_OF(x)     _VALUEOF(x)
N
N  #define _IRQ_MUXH_INTSEL12_MASK      0x00007C00u
N  #define _IRQ_MUXH_INTSEL12_SHIFT     0x0000000Au
N  #define  IRQ_MUXH_INTSEL12_DEFAULT   0x0000000Bu
N  #define  IRQ_MUXH_INTSEL12_OF(x)     _VALUEOF(x)
N
N  #define _IRQ_MUXH_INTSEL11_MASK      0x000003E0u
N  #define _IRQ_MUXH_INTSEL11_SHIFT     0x00000005u
N  #define  IRQ_MUXH_INTSEL11_DEFAULT   0x0000000Au
N  #define  IRQ_MUXH_INTSEL11_OF(x)     _VALUEOF(x)
N
N  #define _IRQ_MUXH_INTSEL10_MASK      0x0000001Fu
N  #define _IRQ_MUXH_INTSEL10_SHIFT     0x00000000u
N  #define  IRQ_MUXH_INTSEL10_DEFAULT   0x00000003u
N  #define  IRQ_MUXH_INTSEL10_OF(x)     _VALUEOF(x)
N
N  #define  IRQ_MUXH_OF(x)              _VALUEOF(x)
N
N  #define IRQ_MUXH_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(IRQ,MUXH,INTSEL15) \
N    |_PER_FDEFAULT(IRQ,MUXH,INTSEL14) \
N    |_PER_FDEFAULT(IRQ,MUXH,INTSEL13) \
N    |_PER_FDEFAULT(IRQ,MUXH,INTSEL12) \
N    |_PER_FDEFAULT(IRQ,MUXH,INTSEL11) \
N    |_PER_FDEFAULT(IRQ,MUXH,INTSEL10) \
N  )
X  #define IRQ_MUXH_DEFAULT (Uint32)(      _PER_FDEFAULT(IRQ,MUXH,INTSEL15)     |_PER_FDEFAULT(IRQ,MUXH,INTSEL14)     |_PER_FDEFAULT(IRQ,MUXH,INTSEL13)     |_PER_FDEFAULT(IRQ,MUXH,INTSEL12)     |_PER_FDEFAULT(IRQ,MUXH,INTSEL11)     |_PER_FDEFAULT(IRQ,MUXH,INTSEL10)   )
N
N  #define IRQ_MUXH_RMK(intsel15,intsel14,intsel13,intsel12,intsel11,intsel10) \
N    (Uint32)( \
N     _PER_FMK(IRQ,MUXH,INTSEL15,intsel15) \
N    |_PER_FMK(IRQ,MUXH,INTSEL14,intsel14) \
N    |_PER_FMK(IRQ,MUXH,INTSEL13,intsel13) \
N    |_PER_FMK(IRQ,MUXH,INTSEL12,intsel12) \
N    |_PER_FMK(IRQ,MUXH,INTSEL11,intsel11) \
N    |_PER_FMK(IRQ,MUXH,INTSEL10,intsel10) \
N   )
X  #define IRQ_MUXH_RMK(intsel15,intsel14,intsel13,intsel12,intsel11,intsel10)     (Uint32)(      _PER_FMK(IRQ,MUXH,INTSEL15,intsel15)     |_PER_FMK(IRQ,MUXH,INTSEL14,intsel14)     |_PER_FMK(IRQ,MUXH,INTSEL13,intsel13)     |_PER_FMK(IRQ,MUXH,INTSEL12,intsel12)     |_PER_FMK(IRQ,MUXH,INTSEL11,intsel11)     |_PER_FMK(IRQ,MUXH,INTSEL10,intsel10)    )
N
N  #define _IRQ_MUXH_FGET(FIELD)\
N    _PER_FGET(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD)
X  #define _IRQ_MUXH_FGET(FIELD)    _PER_FGET(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD)
N
N  #define _IRQ_MUXH_FSET(FIELD,field)\
N    _PER_FSET(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD,field)
X  #define _IRQ_MUXH_FSET(FIELD,field)    _PER_FSET(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD,field)
N
N  #define _IRQ_MUXH_FSETS(FIELD,SYM)\
N    _PER_FSETS(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD,##SYM)
X  #define _IRQ_MUXH_FSETS(FIELD,SYM)    _PER_FSETS(_IRQ_MUXH_ADDR,IRQ,MUXH,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  M U X L          |
N* |___________________|
N*
N* MUXL - interrupt multiplexer low register
N*
N* FIELDS (msb -> lsb)
N* (rw) INTSEL9
N* (rw) INTSEL8
N* (rw) INTSEL7
N* (rw) INTSEL6
N* (rw) INTSEL5
N* (rw) INTSEL4
N*
N\******************************************************************************/
N  #define _IRQ_MUXL_ADDR               0x019C0004u
N
N  #define _IRQ_MUXL_INTSEL9_MASK       0x7C000000u
N  #define _IRQ_MUXL_INTSEL9_SHIFT      0x0000001Au
N  #define  IRQ_MUXL_INTSEL9_DEFAULT    0x00000009u
N  #define  IRQ_MUXL_INTSEL9_OF(x)      _VALUEOF(x)
N
N  #define _IRQ_MUXL_INTSEL8_MASK       0x03E00000u
N  #define _IRQ_MUXL_INTSEL8_SHIFT      0x00000015u
N  #define  IRQ_MUXL_INTSEL8_DEFAULT    0x00000008u
N  #define  IRQ_MUXL_INTSEL8_OF(x)      _VALUEOF(x)
N
N  #define _IRQ_MUXL_INTSEL7_MASK       0x001F0000u
N  #define _IRQ_MUXL_INTSEL7_SHIFT      0x00000010u
N  #define  IRQ_MUXL_INTSEL7_DEFAULT    0x00000007u
N  #define  IRQ_MUXL_INTSEL7_OF(x)      _VALUEOF(x)
N
N  #define _IRQ_MUXL_INTSEL6_MASK       0x00007C00u
N  #define _IRQ_MUXL_INTSEL6_SHIFT      0x0000000Au
N  #define  IRQ_MUXL_INTSEL6_DEFAULT    0x00000006u
N  #define  IRQ_MUXL_INTSEL6_OF(x)      _VALUEOF(x)
N
N  #define _IRQ_MUXL_INTSEL5_MASK       0x000003E0u
N  #define _IRQ_MUXL_INTSEL5_SHIFT      0x00000005u
N  #define  IRQ_MUXL_INTSEL5_DEFAULT    0x00000005u
N  #define  IRQ_MUXL_INTSEL5_OF(x)      _VALUEOF(x)
N
N  #define _IRQ_MUXL_INTSEL4_MASK       0x0000001Fu
N  #define _IRQ_MUXL_INTSEL4_SHIFT      0x00000000u
N  #define  IRQ_MUXL_INTSEL4_DEFAULT    0x00000004u
N  #define  IRQ_MUXL_INTSEL4_OF(x)      _VALUEOF(x)
N
N  #define  IRQ_MUXL_OF(x)              _VALUEOF(x)
N
N  #define IRQ_MUXL_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(IRQ,MUXL,INTSEL9) \
N    |_PER_FDEFAULT(IRQ,MUXL,INTSEL8) \
N    |_PER_FDEFAULT(IRQ,MUXL,INTSEL7) \
N    |_PER_FDEFAULT(IRQ,MUXL,INTSEL6) \
N    |_PER_FDEFAULT(IRQ,MUXL,INTSEL5) \
N    |_PER_FDEFAULT(IRQ,MUXL,INTSEL4) \
N  )
X  #define IRQ_MUXL_DEFAULT (Uint32)(      _PER_FDEFAULT(IRQ,MUXL,INTSEL9)     |_PER_FDEFAULT(IRQ,MUXL,INTSEL8)     |_PER_FDEFAULT(IRQ,MUXL,INTSEL7)     |_PER_FDEFAULT(IRQ,MUXL,INTSEL6)     |_PER_FDEFAULT(IRQ,MUXL,INTSEL5)     |_PER_FDEFAULT(IRQ,MUXL,INTSEL4)   )
N
N  #define IRQ_MUXL_RMK(intsel9,intsel8,intsel7,intsel6,intsel5,intsel4) \
N    (Uint32)( \
N     _PER_FMK(IRQ,MUXL,INTSEL9,intsel9) \
N    |_PER_FMK(IRQ,MUXL,INTSEL8,intsel8) \
N    |_PER_FMK(IRQ,MUXL,INTSEL7,intsel7) \
N    |_PER_FMK(IRQ,MUXL,INTSEL6,intsel6) \
N    |_PER_FMK(IRQ,MUXL,INTSEL5,intsel5) \
N    |_PER_FMK(IRQ,MUXL,INTSEL4,intsel4) \
N   )
X  #define IRQ_MUXL_RMK(intsel9,intsel8,intsel7,intsel6,intsel5,intsel4)     (Uint32)(      _PER_FMK(IRQ,MUXL,INTSEL9,intsel9)     |_PER_FMK(IRQ,MUXL,INTSEL8,intsel8)     |_PER_FMK(IRQ,MUXL,INTSEL7,intsel7)     |_PER_FMK(IRQ,MUXL,INTSEL6,intsel6)     |_PER_FMK(IRQ,MUXL,INTSEL5,intsel5)     |_PER_FMK(IRQ,MUXL,INTSEL4,intsel4)    )
N
N  #define _IRQ_MUXL_FGET(FIELD)\
N    _PER_FGET(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD)
X  #define _IRQ_MUXL_FGET(FIELD)    _PER_FGET(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD)
N
N  #define _IRQ_MUXL_FSET(FIELD,field)\
N    _PER_FSET(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD,field)
X  #define _IRQ_MUXL_FSET(FIELD,field)    _PER_FSET(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD,field)
N
N  #define _IRQ_MUXL_FSETS(FIELD,SYM)\
N    _PER_FSETS(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD,##SYM)
X  #define _IRQ_MUXL_FSETS(FIELD,SYM)    _PER_FSETS(_IRQ_MUXL_ADDR,IRQ,MUXL,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  E X T P O L      |
N* |___________________|
N*
N* EXTPOL - external interrupt polarity register
N*
N* FIELDS (msb -> lsb)
N* (rw) XIP
N*
N\******************************************************************************/
N  #define _IRQ_EXTPOL_ADDR            0x019C0008u
N
N  #define _IRQ_EXTPOL_XIP_MASK        0x0000000Fu
N  #define _IRQ_EXTPOL_XIP_SHIFT       0x00000000u
N  #define  IRQ_EXTPOL_XIP_DEFAULT     0x00000000u
N  #define  IRQ_EXTPOL_XIP_OF(x)       _VALUEOF(x)
N
N  #define  IRQ_EXTPOL_OF(x)           _VALUEOF(x)
N
N  #define IRQ_EXTPOL_DEFAULT (Uint32)( \
N     _PER_FDEFAULT(IRQ,EXTPOL,XIP) \
N  )
X  #define IRQ_EXTPOL_DEFAULT (Uint32)(      _PER_FDEFAULT(IRQ,EXTPOL,XIP)   )
N
N  #define IRQ_EXTPOL_RMK(xip) (Uint32)( \
N     _PER_FMK(IRQ,EXTPOL,XIP,xip) \
N  )
X  #define IRQ_EXTPOL_RMK(xip) (Uint32)(      _PER_FMK(IRQ,EXTPOL,XIP,xip)   )
N
N  #define _IRQ_EXTPOL_FGET(FIELD)\
N    _PER_FGET(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD)
X  #define _IRQ_EXTPOL_FGET(FIELD)    _PER_FGET(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD)
N
N  #define _IRQ_EXTPOL_FSET(FIELD,field)\
N    _PER_FSET(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD,field)
X  #define _IRQ_EXTPOL_FSET(FIELD,field)    _PER_FSET(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD,field)
N
N  #define _IRQ_EXTPOL_FSETS(FIELD,SYM)\
N    _PER_FSETS(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD,##SYM)
X  #define _IRQ_EXTPOL_FSETS(FIELD,SYM)    _PER_FSETS(_IRQ_EXTPOL_ADDR,IRQ,EXTPOL,##FIELD,##SYM)
N
N
N/*----------------------------------------------------------------------------*/
N
N#endif /* IRQ_SUPPORT */
N#endif /* _CSL_IRQHAL_H_ */
N/******************************************************************************\
N* End of csl_irqhal.h
X
N\******************************************************************************/
N
L 19 "E:/TI_workspase/c6701_t/csl/csl_irq.h" 2
N
N
N#if (IRQ_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N/******************************************************************************\
N* scope and inline control macros
X
N\******************************************************************************/
N#ifdef __cplusplus
S#define CSLAPI extern "C" far 
N#else
N#define CSLAPI extern far
N#endif
N
N#undef  USEDEFS
N#undef  IDECL
N#undef  IDEF
N
N#ifdef  _IRQ_MOD_
S  #define IDECL CSLAPI
S  #define USEDEFS
S  #define IDEF
N#else
N  #ifdef  _INLINE
S    #define IDECL static inline
S    #define USEDEFS
S    #define IDEF  static inline
N  #else
N    #define IDECL CSLAPI
N  #endif
N#endif
N
N
N/******************************************************************************\
N* global macro declarations
X
N\******************************************************************************/
N
N/* misc global settings */
N#define IRQ_INT_CNT     16
N#define IRQ_EVENT_CNT   32
N
N#define IRQ_EVT_NA         IRQ_EVENT_CNT
N
N/* event id definitions */
N#if (CHIP_6410 | CHIP_6413 | CHIP_6418)
X#if (0 | 0 | 0)
S  
S    #define IRQ_EVT_DSPINT     0
S    #define IRQ_EVT_TINT0      1
S    #define IRQ_EVT_TINT1      2
S    #define IRQ_EVT_SDINTA     3
S    #define IRQ_EVT_EXTINT4    4
S    #define IRQ_EVT_GPINT4     4
S    #define IRQ_EVT_EXTINT5    5
S    #define IRQ_EVT_GPINT5     5
S    #define IRQ_EVT_EXTINT6    6
S    #define IRQ_EVT_GPINT6     6
S    #define IRQ_EVT_EXTINT7    7
S    #define IRQ_EVT_GPINT7     7
S    #define IRQ_EVT_EDMAINT    8
S    #define IRQ_EVT_EMUDTDMA   9
S    #define IRQ_EVT_EMURTDXRX  10
S    #define IRQ_EVT_EMURTDXTX  11
S    #define IRQ_EVT_XINT0      12
S    #define IRQ_EVT_RINT0      13
S    #define IRQ_EVT_XINT1      14
S    #define IRQ_EVT_RINT1      15
S    #define IRQ_EVT_GPINT0     16
S    #define IRQ_EVT_TINT2      19
S    #define IRQ_EVT_I2CINT0    22
S    #define IRQ_EVT_I2CINT1    23
S    #define IRQ_EVT_AXINT1     24
S    #define IRQ_EVT_ARINT1     25
S    #define IRQ_EVT_AXINT0     28
S    #define IRQ_EVT_ARINT0     29
S
S  #if (CHIP_6418)
S    #define IRQ_EVT_VCPINT     30
S  #endif
S
N#else 
N
N  #if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412)
X  #if (0 | 0 | 0 | 0)
S
S    #define IRQ_EVT_DSPINT     0
S    #define IRQ_EVT_TINT0      1
S    #define IRQ_EVT_TINT1      2
S    #define IRQ_EVT_SDINTA     3
S    #define IRQ_EVT_EXTINT4    4
S    #define IRQ_EVT_GPINT4     4
S    #define IRQ_EVT_EXTINT5    5
S    #define IRQ_EVT_GPINT5     5
S    #define IRQ_EVT_EXTINT6    6
S    #define IRQ_EVT_GPINT6     6
S    #define IRQ_EVT_EXTINT7    7
S    #define IRQ_EVT_GPINT7     7
S    #define IRQ_EVT_EDMAINT    8
S    #define IRQ_EVT_EMUDTDMA   9
S    #define IRQ_EVT_EMURTDXRX  10
S    #define IRQ_EVT_EMURTDXTX  11
S    #define IRQ_EVT_XINT0      12
S    #define IRQ_EVT_RINT0      13
S    #define IRQ_EVT_XINT1      14
S    #define IRQ_EVT_RINT1      15
S    #define IRQ_EVT_GPINT0     16
S    #define IRQ_EVT_TINT2      19
S    #define IRQ_EVT_I2CINT0    22
S    #define IRQ_EVT_MACINT     24
S
S   #if !(CHIP_6412)
S    #define IRQ_EVT_VINT0      25
S    #define IRQ_EVT_VINT1      26
S   #endif
S
S   #if (CHIP_DM642)
S    #define IRQ_EVT_VINT2      27
S   #endif
S
S   #if !(CHIP_6412)
S    #define IRQ_EVT_AXINT0     28
S    #define IRQ_EVT_ARINT0     29
S   #endif
S
N  #else
N
N    #define IRQ_EVT_DSPINT     0
N    #define IRQ_EVT_TINT0      1
N    #define IRQ_EVT_TINT1      2
N    #define IRQ_EVT_SDINT      3
N    #define IRQ_EVT_SDINTA     3
N    #define IRQ_EVT_EXTINT4    4
N    #define IRQ_EVT_GPINT4     4
N    #define IRQ_EVT_EXTINT5    5
N    #define IRQ_EVT_GPINT5     5
N    #define IRQ_EVT_EXTINT6    6
N    #define IRQ_EVT_GPINT6     6
N    #define IRQ_EVT_EXTINT7    7
N    #define IRQ_EVT_GPINT7     7
N    #define IRQ_EVT_EDMAINT    8
N    #define IRQ_EVT_DMAINT0    8
N    #define IRQ_EVT_EMUDTDMA   9
N    #define IRQ_EVT_DMAINT1    9
N    #define IRQ_EVT_EMURTDXRX  10
N    #define IRQ_EVT_DMAINT2    10
N    #define IRQ_EVT_EMURTDXTX  11
N    #define IRQ_EVT_DMAINT3    11
N    #define IRQ_EVT_XINT0      12
N    #define IRQ_EVT_RINT0      13
N    #define IRQ_EVT_XINT1      14
N    #define IRQ_EVT_RINT1      15
N    #define IRQ_EVT_GPINT0     16
N    #define IRQ_EVT_XINT2      17
N    #define IRQ_EVT_RINT2      18
N    #define IRQ_EVT_TINT2      19
N    #define IRQ_EVT_SDINTB     20
N    #define IRQ_EVT_PCIWAKE    21
N    #define IRQ_EVT_MDIO       21
N    #define IRQ_EVT_QDMAERR    22
N    #define IRQ_EVT_I2CINT0    22 /* 6713 */
N    #define IRQ_EVT_UINT       23
N    #define IRQ_EVT_I2CINT1    23 /* 6713 */
N    #define IRQ_EVT_I2CINT2    23 
N    #define IRQ_EVT_I2CINT3    22 
N    #define IRQ_EVT_I2CINT4    23 
N
N    #define IRQ_EVT_MACINT     24 /* DM642 */
N    #define IRQ_EVT_VINT0      25 /* DM642 */
N    #define IRQ_EVT_VINT1      26 /* DM642 */
N    #define IRQ_EVT_VINT2      27 /* DM642 */
N    #define IRQ_EVT_AXINT0     28 /* 6713 / DM642 */
N    #define IRQ_EVT_ARINT0     29 /* 6713 / DM642 */
N    #define IRQ_EVT_AXINT3     IRQ_EVT_EXTINT4 /* 6713 */
N    #define IRQ_EVT_ARINT3     IRQ_EVT_EXTINT5 /* 6713 */
N    #define IRQ_EVT_VCPINT     30
N    #define IRQ_EVT_AXINT1     30 /* 6713 */
N
N    #define IRQ_EVT_AXINT2     30 /* 6713 */
N    #define IRQ_EVT_AXINT4     IRQ_EVT_EXTINT6
N    #define IRQ_EVT_TCPINT     31
N    #define IRQ_EVT_ARINT1     31 /* 6713 */
N
N    #define IRQ_EVT_ARINT2     31 /* 6713 */
N    #define IRQ_EVT_ARINT4     IRQ_EVT_EXTINT7
N
N  #endif
N
N#endif
N/* interrupt masks */
N#define IRQ_MASK_NA 0x00000000
N#define IRQ_MASK_00 0x00000001
N#define IRQ_MASK_01 0x00000002
N#define IRQ_MASK_02 0x00000004
N#define IRQ_MASK_03 0x00000008
N#define IRQ_MASK_04 0x00000010
N#define IRQ_MASK_05 0x00000020
N#define IRQ_MASK_06 0x00000040
N#define IRQ_MASK_07 0x00000080
N#define IRQ_MASK_08 0x00000100
N#define IRQ_MASK_09 0x00000200
N#define IRQ_MASK_10 0x00000400
N#define IRQ_MASK_11 0x00000800
N#define IRQ_MASK_12 0x00001000
N#define IRQ_MASK_13 0x00002000
N#define IRQ_MASK_14 0x00004000
N#define IRQ_MASK_15 0x00008000
N
N/* defines used with the dispatcher functions */
N#define IRQ_CCMASK_NONE        0x00000001u
N#define IRQ_CCMASK_PCC_MAPPED  0x00000000u
N#define IRQ_CCMASK_PCC_ENABLE  0x00000040u
N#define IRQ_CCMASK_PCC_FREEZE  0x00000060u
N#define IRQ_CCMASK_PCC_BYPASS  0x00000080u
N#define IRQ_CCMASK_DCC_MAPPED  0x00000000u
N#define IRQ_CCMASK_DCC_ENABLE  0x00000008u
N#define IRQ_CCMASK_DCC_FREEZE  0x0000000Cu
N#define IRQ_CCMASK_DCC_BYPASS  0x00000010u
N#define IRQ_CCMASK_DEFAULT     IRQ_CCMASK_NONE
N#define IRQ_IEMASK_SELF        0x80000000u
N#define IRQ_IEMASK_ALL         0x0000FFFFu
N#define IRQ_IEMASK_DEFAULT     IRQ_IEMASK_SELF
N
N
N/* private stuff */
N#define _IRQ_DISPATCHTABLE_CNT   (IRQ_INT_CNT)
N#define _IRQ_EVENT2INTTABLE_CNT  (IRQ_EVENT_CNT+1)
N#define _IRQ_INT2EVENTTABLE_CNT  (IRQ_INT_CNT)
N
N
N/******************************************************************************\
N* global typedef declarations
X
N\******************************************************************************/
Ntypedef struct {
N  void   *funcAddr;
N  Uint32 ieMask;
N  Uint32 ccMask;
N  Uint32 funcArg;
N} _IRQ_Dispatch;
N
Ntypedef struct {
N  void   *funcAddr;
N  Uint32 funcArg;
N  Uint32 ccMask;
N  Uint32 ieMask;
N} IRQ_Config;
N
N
N/******************************************************************************\
N* global variable declarations
X
N\******************************************************************************/
N
N/* private vars */
Nextern far Uint32 _IRQ_eventTable[IRQ_EVENT_CNT+1];
Xextern far Uint32 _IRQ_eventTable[32+1];
Nextern far Uint32 _IRQ_intTable[IRQ_INT_CNT];
Xextern far Uint32 _IRQ_intTable[16];
Nextern far _IRQ_Dispatch _IRQ_internalDispatchTable[_IRQ_DISPATCHTABLE_CNT];
Xextern far _IRQ_Dispatch _IRQ_internalDispatchTable[(16)];
Nextern far _IRQ_Dispatch *_IRQ_dispatchTable;
N
N/******************************************************************************\
N* global function declarations
X
N\******************************************************************************/
N
N/* private functions */
NCSLAPI void _IRQ_init(Uint32 biosPresent, _IRQ_Dispatch *dispatchTable);
Xextern far void _IRQ_init(Uint32 biosPresent, _IRQ_Dispatch *dispatchTable);
N
N/* API functions */
NCSLAPI void IRQ_map(Uint32 eventId, Uint32 intNumber);
Xextern far void IRQ_map(Uint32 eventId, Uint32 intNumber);
NCSLAPI void *IRQ_setVecs(void *vecs);
Xextern far void *IRQ_setVecs(void *vecs);
NCSLAPI Uint32 IRQ_biosPresent();
Xextern far Uint32 IRQ_biosPresent();
NCSLAPI void IRQ_hook(int intNum, void *func);
Xextern far void IRQ_hook(int intNum, void *func);
N
N/* These functions only work with the DSP/BIOS HWI dispatcher */
NCSLAPI void IRQ_config(Uint32 eventId, IRQ_Config *config);
Xextern far void IRQ_config(Uint32 eventId, IRQ_Config *config);
NCSLAPI void IRQ_configArgs(Uint32 eventId, void *funcAddr, Uint32 funcArg,
Xextern far void IRQ_configArgs(Uint32 eventId, void *funcAddr, Uint32 funcArg,
N  Uint32 ccMask, Uint32 ieMask);
NCSLAPI void IRQ_getConfig(Uint32 eventId, IRQ_Config *config);
Xextern far void IRQ_getConfig(Uint32 eventId, IRQ_Config *config);
NCSLAPI Uint32 IRQ_getArg(Uint32 eventId);
Xextern far Uint32 IRQ_getArg(Uint32 eventId);
NCSLAPI void IRQ_setArg(Uint32 eventId, Uint32 arg);
Xextern far void IRQ_setArg(Uint32 eventId, Uint32 arg);
N
N
N/******************************************************************************\
N* inline function declarations
X
N\******************************************************************************/
NIDECL void IRQ_enable(Uint32 eventId);
Xextern far void IRQ_enable(Uint32 eventId);
NIDECL Uint32 IRQ_disable(Uint32 eventId);
Xextern far Uint32 IRQ_disable(Uint32 eventId);
NIDECL void IRQ_restore(Uint32 eventId, Uint32 ie);
Xextern far void IRQ_restore(Uint32 eventId, Uint32 ie);
NIDECL void IRQ_set(Uint32 eventId);
Xextern far void IRQ_set(Uint32 eventId);
NIDECL void IRQ_clear(Uint32 eventId);
Xextern far void IRQ_clear(Uint32 eventId);
NIDECL Uint32 IRQ_test(Uint32 eventId);
Xextern far Uint32 IRQ_test(Uint32 eventId);
NIDECL void IRQ_reset(Uint32 eventId);
Xextern far void IRQ_reset(Uint32 eventId);
NIDECL void IRQ_resetAll();
Xextern far void IRQ_resetAll();
N
NIDECL void IRQ_globalEnable();
Xextern far void IRQ_globalEnable();
NIDECL Uint32 IRQ_globalDisable();
Xextern far Uint32 IRQ_globalDisable();
NIDECL void IRQ_globalRestore(Uint32 gie);
Xextern far void IRQ_globalRestore(Uint32 gie);
N
NIDECL void IRQ_nmiEnable();
Xextern far void IRQ_nmiEnable();
NIDECL void IRQ_nmiDisable();
Xextern far void IRQ_nmiDisable();
N
N
N/******************************************************************************\
N* inline function definitions
X
N\******************************************************************************/
N#ifdef USEDEFS
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_enable(Uint32 eventId) {
S  IER |= _IRQ_eventTable[eventId];
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 IRQ_disable(Uint32 eventId) {
S  Uint32 ie = IER & _IRQ_eventTable[eventId];
S  IER &= ~_IRQ_eventTable[eventId];
S  return ie;
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_restore(Uint32 eventId, Uint32 ie) {
S  if (ie) {
S    IER |= _IRQ_eventTable[eventId];
S  } else {
S    IER &= ~_IRQ_eventTable[eventId];
S  }
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_set(Uint32 eventId) {
S  ISR = _IRQ_eventTable[eventId];
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_clear(Uint32 eventId) {
S  ICR = _IRQ_eventTable[eventId];
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 IRQ_test(Uint32 eventId) {
S  return (Uint32)((IFR & _IRQ_eventTable[eventId]) ? 1 : 0);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_globalEnable() {
S  CHIP_FSET(CSR,GIE,1);
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 IRQ_globalDisable() {
S  Uint32 gie = CHIP_FGET(CSR,GIE);
S  CHIP_FSET(CSR,GIE,0);
S  return gie;
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_globalRestore(Uint32 gie) {
S  CHIP_FSET(CSR,GIE,gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_reset(Uint32 eventId) {
S  IER &= ~_IRQ_eventTable[eventId];
S  ICR = _IRQ_eventTable[eventId];
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_resetAll() {
S  CHIP_FSET(CSR,GIE,0);
S  CHIP_CRSET(IER,0x00000000);
S  CHIP_CRSET(ICR,0xFFFFFFFF);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_nmiEnable() {
S  IER |= 0x00000002;
S}
S/*----------------------------------------------------------------------------*/
SIDEF void IRQ_nmiDisable() {
S  IER &= ~0x00000002;
S}
S/*----------------------------------------------------------------------------*/
N#endif /* USEDEFS */
N
N
N#endif /* IRQ_SUPPORT */
N#endif /* _CSL_IRQ_H_ */
N/******************************************************************************\
N* End of csl_irq.h
X
N\******************************************************************************/
N
L 15 "..\csl\csl_cache.h" 2
N#include <csl_cachehal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_cachehal.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_cachehal.h
N* DATE CREATED.. 06/12/1999 
N* LAST MODIFIED. 04/13/2001
N*------------------------------------------------------------------------------
N* REGISTERS
N*
N* CCFG     - cache configuration register
N* L2FBAR   - L2 flush base address register
N* L2FWC    - L2 flush word count register
N* L2CBAR   - L2 clean base register
N* L2CWC    - L2 clean word count register
N* L1PFBAR  - L1P flush base address register
N* L1PFWC   - L1P flush word count register
N* L1DFBAR  - L1D flush base address register
N* L1DFWC   - L1D flush word count register
N* L2FLUSH  - L2 flush register
N* L2CLEAN  - L2 clean register
N*
N*
N*
N*
N*
N* New Register Names based on SPRU609 for C621x/C671x
N* and
N* New Register Names based on SPRU610 for C64x
N*
N*
N* L2WBAR   - L2 writeback base address register
N* L2WWC    - L2 writeback word count register
N* L2WIBAR  - L2 writeback-invalidate base address register
N* L2WIWC   - L2 writeback-invalidate word count register
N* L2IBAR   - L2 invalidate base address register(2)
N* L2IWC    - L2 invalidate word count register(2)
N* L1PIBAR  - L1P invalidate base address register
N* L1PIWC   - L1P invalidate word count register
N* L1DWIBAR - L1D writeback-invalidate base address register
N* L1DWIWC  - L1D writeback-invalidate word count register
N* L1DIBAR  - L1D invalidate base address register(2)
N* L1DIWC   - L1D invalidate word count register(2)
N* L2WB     - L2 writeback all register
N* L2WBINV  - L2 writeback-invalidate all register
N*
N* MAR0     - memory attribute register 0
N* MAR1     - memory attribute register 1
N* ...      - ...
N* MARn     - memory attribute register n (1)
N* L2ALLOC0 - L2 Allocation register 0    (2)
N* L2ALLOC1 - L2 Allocation register 1    (2)
N* L2ALLOC2 - L2 Allocation register 2    (2)
N* L2ALLOC3 - L2 Allocation register 3    (2)
N*
N* (1) n is different between C6x1x and C64x
N* (2) C64x devices only
N*
N\******************************************************************************/
N#ifndef _CSL_CACHEHAL_H_
N#define _CSL_CACHEHAL_H_
N
N#include <csl_stdinc.h>
N#include <csl_chip.h>
N
N#if (CACHE_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N/******************************************************************************\
N* MISC section
X
N\******************************************************************************/
N#define _CACHE_BASE_GLOBAL  0x01840000u
N#define CACHE_L2_SUPPORT L2CACHE_SUPPORT 
N
N
N/******************************************************************************\
N* module level register/field access macros
X
N\******************************************************************************/
N
N  /* ----------------- */
N  /* FIELD MAKE MACROS */
N  /* ----------------- */
N
N  #define CACHE_FMK(REG,FIELD,x)\
N    _PER_FMK(CACHE,##REG,##FIELD,x)
X  #define CACHE_FMK(REG,FIELD,x)    _PER_FMK(CACHE,##REG,##FIELD,x)
N
N  #define CACHE_FMKS(REG,FIELD,SYM)\
N    _PER_FMKS(CACHE,##REG,##FIELD,##SYM)
X  #define CACHE_FMKS(REG,FIELD,SYM)    _PER_FMKS(CACHE,##REG,##FIELD,##SYM)
N 
N 
N  /* -------------------------------- */
N  /* RAW REGISTER/FIELD ACCESS MACROS */
N  /* -------------------------------- */
N
N  #define CACHE_ADDR(REG)\
N    _CACHE_##REG##_ADDR
X  #define CACHE_ADDR(REG)    _CACHE_##REG##_ADDR
N
N  #define CACHE_RGET(REG)\
N    _PER_RGET(_CACHE_##REG##_ADDR,CACHE,##REG)
X  #define CACHE_RGET(REG)    _PER_RGET(_CACHE_##REG##_ADDR,CACHE,##REG)
N
N  #define CACHE_RSET(REG,x)\
N    _PER_RSET(_CACHE_##REG##_ADDR,CACHE,##REG,x)
X  #define CACHE_RSET(REG,x)    _PER_RSET(_CACHE_##REG##_ADDR,CACHE,##REG,x)
N
N  #define CACHE_FGET(REG,FIELD)\
N    _CACHE_##REG##_FGET(##FIELD)
X  #define CACHE_FGET(REG,FIELD)    _CACHE_##REG##_FGET(##FIELD)
N
N  #define CACHE_FSET(REG,FIELD,x)\
N    _CACHE_##REG##_FSET(##FIELD,x)
X  #define CACHE_FSET(REG,FIELD,x)    _CACHE_##REG##_FSET(##FIELD,x)
N
N  #define CACHE_FSETS(REG,FIELD,SYM)\
N    _CACHE_##REG##_FSETS(##FIELD,##SYM)
X  #define CACHE_FSETS(REG,FIELD,SYM)    _CACHE_##REG##_FSETS(##FIELD,##SYM)
N 
N 
N  /* ------------------------------------------ */
N  /* ADDRESS BASED REGISTER/FIELD ACCESS MACROS */
N  /* ------------------------------------------ */
N
N  #define CACHE_RGETA(addr,REG)\
N    _PER_RGET(addr,CACHE,##REG)
X  #define CACHE_RGETA(addr,REG)    _PER_RGET(addr,CACHE,##REG)
N
N  #define CACHE_RSETA(addr,REG,x)\
N    _PER_RSET(addr,CACHE,##REG,x)
X  #define CACHE_RSETA(addr,REG,x)    _PER_RSET(addr,CACHE,##REG,x)
N
N  #define CACHE_FGETA(addr,REG,FIELD)\
N    _PER_FGET(addr,CACHE,##REG,##FIELD)
X  #define CACHE_FGETA(addr,REG,FIELD)    _PER_FGET(addr,CACHE,##REG,##FIELD)
N
N  #define CACHE_FSETA(addr,REG,FIELD,x)\
N    _PER_FSET(addr,CACHE,##REG,##FIELD,x)
X  #define CACHE_FSETA(addr,REG,FIELD,x)    _PER_FSET(addr,CACHE,##REG,##FIELD,x)
N
N  #define CACHE_FSETSA(addr,REG,FIELD,SYM)\
N    _PER_FSETS(addr,CACHE,##REG,##FIELD,##SYM)
X  #define CACHE_FSETSA(addr,REG,FIELD,SYM)    _PER_FSETS(addr,CACHE,##REG,##FIELD,##SYM)
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  C C F G          |
N* |___________________|
N*
N* CCFG - cache configuration register
N*
N* FIELDS (msb -> lsb)
N* (rw) P (1)
N* (w)  IP
N* (w)  ID
N* (rw) L2MODE
N*
N* (1) only supported for C64x devices
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_CCFG_ADDR             0x01840000u
S
S#if (C64_SUPPORT)
S  #define _CACHE_CCFG_P_MASK           0xE0000000u
S  #define _CACHE_CCFG_P_SHIFT          0x0000001Du
S  #define  CACHE_CCFG_P_DEFAULT        0x00000000u
S  #define  CACHE_CCFG_P_OF(x)          _VALUEOF(x)
S  #define  CACHE_CCFG_P_URGENT         0x00000000u
S  #define  CACHE_CCFG_P_HIGH           0x00000001u
S  #define  CACHE_CCFG_P_MEDIUM         0x00000002u
S  #define  CACHE_CCFG_P_LOW            0x00000003u
S#endif
S
S  #define _CACHE_CCFG_IP_MASK          0x00000200u
S  #define _CACHE_CCFG_IP_SHIFT         0x00000009u
S  #define  CACHE_CCFG_IP_DEFAULT       0x00000000u
S  #define  CACHE_CCFG_IP_OF(x)         _VALUEOF(x)
S  #define  CACHE_CCFG_IP_NORMAL        0x00000000u
S  #define  CACHE_CCFG_IP_INVALIDATE    0x00000001u
S
S  #define _CACHE_CCFG_ID_MASK          0x00000100u
S  #define _CACHE_CCFG_ID_SHIFT         0x00000008u
S  #define  CACHE_CCFG_ID_DEFAULT       0x00000000u
S  #define  CACHE_CCFG_ID_OF(x)         _VALUEOF(x)
S  #define  CACHE_CCFG_ID_NORMAL        0x00000000u
S  #define  CACHE_CCFG_ID_INVALIDATE    0x00000001u
S
S#if (!C64_SUPPORT)
S  #define _CACHE_CCFG_L2MODE_MASK      0x00000007u
S  #define _CACHE_CCFG_L2MODE_SHIFT     0x00000000u
S  #define  CACHE_CCFG_L2MODE_DEFAULT   0x00000000u
S  #define  CACHE_CCFG_L2MODE_OF(x)     _VALUEOF(x)
S  #define  CACHE_CCFG_L2MODE_0KC       0x00000000u
S  #define  CACHE_CCFG_L2MODE_16KC      0x00000001u
S  #define  CACHE_CCFG_L2MODE_32KC      0x00000002u
S  #define  CACHE_CCFG_L2MODE_48KC      0x00000003u
S  #define  CACHE_CCFG_L2MODE_64KC      0x00000007u
S#else 
S  #define _CACHE_CCFG_L2MODE_MASK      0x00000007u
S  #define _CACHE_CCFG_L2MODE_SHIFT     0x00000000u
S  #define  CACHE_CCFG_L2MODE_DEFAULT   0x00000000u
S  #define  CACHE_CCFG_L2MODE_OF(x)     _VALUEOF(x)
S  #define  CACHE_CCFG_L2MODE_0KC       0x00000000u
S  #define  CACHE_CCFG_L2MODE_32KC      0x00000001u
S  #define  CACHE_CCFG_L2MODE_64KC      0x00000002u
S  #define  CACHE_CCFG_L2MODE_128KC     0x00000003u
S
S  #if (!(CHIP_6410 | CHIP_DM641 | CHIP_DM640))
S      #define  CACHE_CCFG_L2MODE_256KC     0x00000007u /* This is an invalid mode for C6410,DM641,DM640 */
S  #endif
S
S#endif
S
S  #define  CACHE_CCFG_OF(x)            _VALUEOF(x)
S
S#if (!C64_SUPPORT)
S  #define CACHE_CCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,CCFG,IP) \
S    |_PER_FDEFAULT(CACHE,CCFG,ID) \
S    |_PER_FDEFAULT(CACHE,CCFG,L2MODE) \
S  )
X  #define CACHE_CCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,CCFG,IP)     |_PER_FDEFAULT(CACHE,CCFG,ID)     |_PER_FDEFAULT(CACHE,CCFG,L2MODE)   )
S
S  #define CACHE_CCFG_RMK(ip,id,l2mode) (Uint32)( \
S     _PER_FMK(CACHE,CCFG,IP,ip) \
S    |_PER_FMK(CACHE,CCFG,ID,id) \
S    |_PER_FMK(CACHE,CCFG,L2MODE,l2mode) \
S  )
X  #define CACHE_CCFG_RMK(ip,id,l2mode) (Uint32)(      _PER_FMK(CACHE,CCFG,IP,ip)     |_PER_FMK(CACHE,CCFG,ID,id)     |_PER_FMK(CACHE,CCFG,L2MODE,l2mode)   )
S#else 
S  #define CACHE_CCFG_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,CCFG,P) \
S    |_PER_FDEFAULT(CACHE,CCFG,IP) \
S    |_PER_FDEFAULT(CACHE,CCFG,ID) \
S    |_PER_FDEFAULT(CACHE,CCFG,L2MODE) \
S  )
X  #define CACHE_CCFG_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,CCFG,P)     |_PER_FDEFAULT(CACHE,CCFG,IP)     |_PER_FDEFAULT(CACHE,CCFG,ID)     |_PER_FDEFAULT(CACHE,CCFG,L2MODE)   )
S
S  #define CACHE_CCFG_RMK(p,ip,id,l2mode) (Uint32)( \
S     _PER_FMK(CACHE,CCFG,P,p) \
S    |_PER_FMK(CACHE,CCFG,IP,ip) \
S    |_PER_FMK(CACHE,CCFG,ID,id) \
S    |_PER_FMK(CACHE,CCFG,L2MODE,l2mode) \
S  )
X  #define CACHE_CCFG_RMK(p,ip,id,l2mode) (Uint32)(      _PER_FMK(CACHE,CCFG,P,p)     |_PER_FMK(CACHE,CCFG,IP,ip)     |_PER_FMK(CACHE,CCFG,ID,id)     |_PER_FMK(CACHE,CCFG,L2MODE,l2mode)   )
S#endif
S
S  #define _CACHE_CCFG_FGET(FIELD)\
S    _PER_FGET(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD)
X  #define _CACHE_CCFG_FGET(FIELD)    _PER_FGET(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD)
S
S  #define _CACHE_CCFG_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD,field)
X  #define _CACHE_CCFG_FSET(FIELD,field)    _PER_FSET(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD,field)
S
S  #define _CACHE_CCFG_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD,##SYM)
X  #define _CACHE_CCFG_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_CCFG_ADDR,CACHE,CCFG,##FIELD,##SYM)
S
N#endif /* L2CACHE_SUPPORT */
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 F B A R      |
N* |___________________|
N*
N* L2FBAR - L2 flush base address register
N*
N* Fields:
N* (rw) L2FBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2FBAR_ADDR           0x01844000u
S
S  #define _CACHE_L2FBAR_L2FBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L2FBAR_L2FBAR_SHIFT   0x00000000u
S  #define  CACHE_L2FBAR_L2FBAR_DEFAULT 0x00000000u
S  #define  CACHE_L2FBAR_L2FBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L2FBAR_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2FBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2FBAR,L2FBAR) \
S  )
X  #define CACHE_L2FBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2FBAR,L2FBAR)   )
S
S  #define CACHE_L2FBAR_RMK(l2fbar) (Uint32)( \
S     _PER_FMK(CACHE,L2FBAR,L2FBAR,l2fbar) \
S  )
X  #define CACHE_L2FBAR_RMK(l2fbar) (Uint32)(      _PER_FMK(CACHE,L2FBAR,L2FBAR,l2fbar)   )
S
S  #define _CACHE_L2FBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD)
X  #define _CACHE_L2FBAR_FGET(FIELD)    _PER_FGET(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD)
S
S  #define _CACHE_L2FBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD,field)
X  #define _CACHE_L2FBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD,field)
S
S  #define _CACHE_L2FBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD,##SYM)
X  #define _CACHE_L2FBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2FBAR_ADDR,CACHE,L2FBAR,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 F W C        |
N* |___________________|
N*
N* L2FWC - L2 flush word count register
N*
N* Fields:
N* (rw) L2FWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2FWC_ADDR           0x01844004u
S
S  #define _CACHE_L2FWC_L2FWC_MASK     0x0000FFFFu
S  #define _CACHE_L2FWC_L2FWC_SHIFT    0x00000000u
S  #define  CACHE_L2FWC_L2FWC_DEFAULT  0x00000000u
S  #define  CACHE_L2FWC_L2FWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L2FWC_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2FWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2FWC,L2FWC) \
S  )
X  #define CACHE_L2FWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2FWC,L2FWC)   )
S
S  #define CACHE_L2FWC_RMK(l2fwc) (Uint32)( \
S     _PER_FMK(CACHE,L2FWC,L2FWC,l2fwc) \
S  )
X  #define CACHE_L2FWC_RMK(l2fwc) (Uint32)(      _PER_FMK(CACHE,L2FWC,L2FWC,l2fwc)   )
S
S  #define _CACHE_L2FWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD)
X  #define _CACHE_L2FWC_FGET(FIELD)    _PER_FGET(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD)
S
S  #define _CACHE_L2FWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD,field)
X  #define _CACHE_L2FWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD,field)
S
S  #define _CACHE_L2FWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD,##SYM)
X  #define _CACHE_L2FWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2FWC_ADDR,CACHE,L2FWC,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 C B A R      |
N* |___________________|
N*
N* L2CBAR - L2 clean base address register
N*
N* Fields:
N* (rw) L2CBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2CBAR_ADDR           0x01844010u
S
S  #define _CACHE_L2CBAR_L2CBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L2CBAR_L2CBAR_SHIFT   0x00000000u
S  #define  CACHE_L2CBAR_L2CBAR_DEFAULT 0x00000000u
S  #define  CACHE_L2CBAR_L2CBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L2CBAR_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2CBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2CBAR,L2CBAR) \
S  )
X  #define CACHE_L2CBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2CBAR,L2CBAR)   )
S
S  #define CACHE_L2CBAR_RMK(l2cbar) (Uint32)( \
S     _PER_FMK(CACHE,L2CBAR,L2CBAR,l2cbar) \
S  )
X  #define CACHE_L2CBAR_RMK(l2cbar) (Uint32)(      _PER_FMK(CACHE,L2CBAR,L2CBAR,l2cbar)   )
S
S  #define _CACHE_L2CBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD)
X  #define _CACHE_L2CBAR_FGET(FIELD)    _PER_FGET(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD)
S
S  #define _CACHE_L2CBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD,field)
X  #define _CACHE_L2CBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD,field)
S
S  #define _CACHE_L2CBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD,##SYM)
X  #define _CACHE_L2CBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2CBAR_ADDR,CACHE,L2CBAR,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 C W C        |
N* |___________________|
N*
N* L2CWC - L2 clean word count register
N*
N* Fields:
N* (rw) L2CWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2CWC_ADDR           0x01844014u
S
S  #define _CACHE_L2CWC_L2CWC_MASK     0x0000FFFFu
S  #define _CACHE_L2CWC_L2CWC_SHIFT    0x00000000u
S  #define  CACHE_L2CWC_L2CWC_DEFAULT  0x00000000u
S  #define  CACHE_L2CWC_L2CWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L2CWC_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2CWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2CWC,L2CWC) \
S  )
X  #define CACHE_L2CWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2CWC,L2CWC)   )
S
S  #define CACHE_L2CWC_RMK(l2cwc) (Uint32)( \
S     _PER_FMK(CACHE,L2CWC,L2CWC,l2cwc) \
S  )
X  #define CACHE_L2CWC_RMK(l2cwc) (Uint32)(      _PER_FMK(CACHE,L2CWC,L2CWC,l2cwc)   )
S
S  #define _CACHE_L2CWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD)
X  #define _CACHE_L2CWC_FGET(FIELD)    _PER_FGET(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD)
S
S  #define _CACHE_L2CWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD,field)
X  #define _CACHE_L2CWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD,field)
S
S  #define _CACHE_L2CWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD,##SYM)
X  #define _CACHE_L2CWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2CWC_ADDR,CACHE,L2CWC,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 P F B A R    |
N* |___________________|
N*
N* L1PFBAR - L1P flush base address register
N*
N* Fields:
N* (rw) L1PFBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S   #define _CACHE_L1PFBAR_ADDR            0x01844020u
S
S  #define _CACHE_L1PFBAR_L1PFBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L1PFBAR_L1PFBAR_SHIFT   0x00000000u
S  #define  CACHE_L1PFBAR_L1PFBAR_DEFAULT 0x00000000u
S  #define  CACHE_L1PFBAR_L1PFBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L1PFBAR_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1PFBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1PFBAR,L1PFBAR) \
S  )
X  #define CACHE_L1PFBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1PFBAR,L1PFBAR)   )
S
S  #define _CACHE_L1PFBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD)
X  #define _CACHE_L1PFBAR_FGET(FIELD)    _PER_FGET(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD)
S
S  #define _CACHE_L1PFBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD,field)
X  #define _CACHE_L1PFBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD,field)
S
S  #define _CACHE_L1PFBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD,##SYM)
X  #define _CACHE_L1PFBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1PFBAR_ADDR,CACHE,L1PFBAR,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 P F W C      |
N* |___________________|
N*
N* L1PFWC - L1P flush word count register
N*
N* Fields:
N* (rw) L1PFWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1PFWC_ADDR            0x01844024u
S
S  #define _CACHE_L1PFWC_L1PFWC_MASK     0x0000FFFFu
S  #define _CACHE_L1PFWC_L1PFWC_SHIFT    0x00000000u
S  #define  CACHE_L1PFWC_L1PFWC_DEFAULT  0x00000000u
S  #define  CACHE_L1PFWC_L1PFWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L1PFWC_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1PFWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1PFWC,L1PFWC) \
S  )
X  #define CACHE_L1PFWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1PFWC,L1PFWC)   )
S
S  #define CACHE_L1PFWC_RMK(l1pfwc) (Uint32)( \
S     _PER_FMK(CACHE,L1PFWC,L1PFWC,l1pfwc) \
S  )
X  #define CACHE_L1PFWC_RMK(l1pfwc) (Uint32)(      _PER_FMK(CACHE,L1PFWC,L1PFWC,l1pfwc)   )
S
S  #define _CACHE_L1PFWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD)
X  #define _CACHE_L1PFWC_FGET(FIELD)    _PER_FGET(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD)
S
S  #define _CACHE_L1PFWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD,field)
X  #define _CACHE_L1PFWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD,field)
S
S  #define _CACHE_L1PFWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD,##SYM)
X  #define _CACHE_L1PFWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1PFWC_ADDR,CACHE,L1PFWC,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 D F B A R    |
N* |___________________|
N*
N* L1DFBAR - L1D flush base address register
N*
N* Fields:
N* (rw) L1DFBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DFBAR_ADDR            0x01844030u
S
S  #define _CACHE_L1DFBAR_L1DFBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L1DFBAR_L1DFBAR_SHIFT   0x00000000u
S  #define  CACHE_L1DFBAR_L1DFBAR_DEFAULT 0x00000000u
S  #define  CACHE_L1DFBAR_L1DFBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L1DFBAR_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DFBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DFBAR,L1DFBAR) \
S  )
X  #define CACHE_L1DFBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DFBAR,L1DFBAR)   )
S
S  #define CACHE_L1DFBAR_RMK(l1dfbar) (Uint32)( \
S     _PER_FMK(CACHE,L1DFBAR,L1DFBAR,l1dfbar) \
S  )
X  #define CACHE_L1DFBAR_RMK(l1dfbar) (Uint32)(      _PER_FMK(CACHE,L1DFBAR,L1DFBAR,l1dfbar)   )
S
S  #define _CACHE_L1DFBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD)
X  #define _CACHE_L1DFBAR_FGET(FIELD)    _PER_FGET(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD)
S
S  #define _CACHE_L1DFBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD,field)
X  #define _CACHE_L1DFBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD,field)
S
S  #define _CACHE_L1DFBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD,##SYM)
X  #define _CACHE_L1DFBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DFBAR_ADDR,CACHE,L1DFBAR,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 D F W C      |
N* |___________________|
N*
N* L1DFWC - L1D flush word count register
N*
N* Fields:
N* (rw) L1DFWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DFWC_ADDR            0x01844034u
S
S  #define _CACHE_L1DFWC_L1DFWC_MASK     0x0000FFFFu
S  #define _CACHE_L1DFWC_L1DFWC_SHIFT    0x00000000u
S  #define  CACHE_L1DFWC_L1DFWC_DEFAULT  0x00000000u
S  #define  CACHE_L1DFWC_L1DFWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L1DFWC_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DFWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DFWC,L1DFWC) \
S  )
X  #define CACHE_L1DFWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DFWC,L1DFWC)   )
S
S  #define CACHE_L1DFWC_RMK(l1dfwc) (Uint32)( \
S     _PER_FMK(CACHE,L1DFWC,L1DFWC,l1dfwc) \
S  )
X  #define CACHE_L1DFWC_RMK(l1dfwc) (Uint32)(      _PER_FMK(CACHE,L1DFWC,L1DFWC,l1dfwc)   )
S
S  #define _CACHE_L1DFWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD)
X  #define _CACHE_L1DFWC_FGET(FIELD)    _PER_FGET(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD)
S
S  #define _CACHE_L1DFWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD,field)
X  #define _CACHE_L1DFWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD,field)
S
S  #define _CACHE_L1DFWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD,##SYM)
X  #define _CACHE_L1DFWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DFWC_ADDR,CACHE,L1DFWC,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 F L U S H    |
N* |___________________|
N*
N* L2FLUSH - L2 flush register
N*
N* Fields:
N* (rw) F
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2FLUSH_ADDR      0x01845000u
S
S  #define _CACHE_L2FLUSH_F_MASK    0x00000001u
S  #define _CACHE_L2FLUSH_F_SHIFT   0x00000000u
S  #define  CACHE_L2FLUSH_F_DEFAULT 0x00000000u
S  #define  CACHE_L2FLUSH_F_OF(x)   _VALUEOF(x)
S  #define  CACHE_L2FLUSH_F_NORMAL  0x00000000u
S  #define  CACHE_L2FLUSH_F_FLUSH   0x00000001u
S
S  #define  CACHE_L2FLUSH_OF(x)     _VALUEOF(x)
S
S  #define CACHE_L2FLUSH_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2FLUSH,F) \
S  )
X  #define CACHE_L2FLUSH_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2FLUSH,F)   )
S
S  #define CACHE_L2FLUSH_RMK(f) (Uint32)( \
S     _PER_FMK(CACHE,L2FLUSH,F,f) \
S  )
X  #define CACHE_L2FLUSH_RMK(f) (Uint32)(      _PER_FMK(CACHE,L2FLUSH,F,f)   )
S
S  #define _CACHE_L2FLUSH_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD)
X  #define _CACHE_L2FLUSH_FGET(FIELD)    _PER_FGET(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD)
S
S  #define _CACHE_L2FLUSH_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD,field)
X  #define _CACHE_L2FLUSH_FSET(FIELD,field)    _PER_FSET(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD,field)
S
S  #define _CACHE_L2FLUSH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD,##SYM)
X  #define _CACHE_L2FLUSH_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2FLUSH_ADDR,CACHE,L2FLUSH,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 C L E A N    |
N* |___________________|
N*
N* L2CLEAN - L2 clean register
N*
N* Fields:
N* (rw) C
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2CLEAN_ADDR       0x01845004u
S
S  #define _CACHE_L2CLEAN_C_MASK     0x00000001u
S  #define _CACHE_L2CLEAN_C_SHIFT    0x00000000u
S  #define  CACHE_L2CLEAN_C_DEFAULT  0x00000000u
S  #define  CACHE_L2CLEAN_C_OF(x)    _VALUEOF(x)
S  #define  CACHE_L2CLEAN_C_NORMAL   0x00000000u
S  #define  CACHE_L2CLEAN_C_CLEAN    0x00000001u
S
S  #define  CACHE_L2CLEAN_OF(x)      _VALUEOF(x)
S
S  #define CACHE_L2CLEAN_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2CLEAN,C) \
S  )
X  #define CACHE_L2CLEAN_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2CLEAN,C)   )
S
S  #define CACHE_L2CLEAN_RMK(c) (Uint32)( \
S     _PER_FMK(CACHE,L2CLEAN,C,c) \
S  )
X  #define CACHE_L2CLEAN_RMK(c) (Uint32)(      _PER_FMK(CACHE,L2CLEAN,C,c)   )
S
S  #define _CACHE_L2CLEAN_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD)
X  #define _CACHE_L2CLEAN_FGET(FIELD)    _PER_FGET(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD)
S
S  #define _CACHE_L2CLEAN_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD,field)
X  #define _CACHE_L2CLEAN_FSET(FIELD,field)    _PER_FSET(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD,field)
S
S  #define _CACHE_L2CLEAN_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD,##SYM)
X  #define _CACHE_L2CLEAN_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2CLEAN_ADDR,CACHE,L2CLEAN,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 A L L O C 0  |
N* |___________________|
N*
N* L2ALLOC0 - L2 allocation register 0 (1)
N*
N* Fields:
N* (rw) Q0CNT (2) (3)
N*
N* (1) only supported for C6400
N* (2) default value is different from L2ALLOC1, L2ALLOC2, L2ALLOC3
N* (3) Rename bit filed based on spru610
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2ALLOC0_ADDR               0x01842000u
S
S  #define _CACHE_L2ALLOC0_Q0CNT_MASK       0x00000007u
S  #define _CACHE_L2ALLOC0_Q0CNT_SHIFT      0x00000000u
S  #define  CACHE_L2ALLOC0_Q0CNT_DEFAULT    0x00000006u
S  #define  CACHE_L2ALLOC0_Q0CNT_OF(x)      _VALUEOF(x)
S
S  #define _CACHE_L2ALLOC0_L2ALLOC_MASK       _CACHE_L2ALLOC0_Q0CNT_MASK   
S  #define _CACHE_L2ALLOC0_L2ALLOC_SHIFT      _CACHE_L2ALLOC0_Q0CNT_SHIFT  
S  #define  CACHE_L2ALLOC0_L2ALLOC_DEFAULT     CACHE_L2ALLOC0_Q0CNT_DEFAULT
S  #define  CACHE_L2ALLOC0_L2ALLOC_OF(x)       CACHE_L2ALLOC0_Q0CNT_OF(x)  
S
S  #define  CACHE_L2ALLOC0_OF(x)            _VALUEOF(x)
S
S  #define CACHE_L2ALLOC0_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2ALLOC0,Q0CNT) \
S  )
X  #define CACHE_L2ALLOC0_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2ALLOC0,Q0CNT)   )
S
S  #define CACHE_L2ALLOC0_RMK(q0cnt)(Uint32)( \
S     _PER_FMK(CACHE,L2ALLOC0,Q0CNT,q0cnt) \
S  )
X  #define CACHE_L2ALLOC0_RMK(q0cnt)(Uint32)(      _PER_FMK(CACHE,L2ALLOC0,Q0CNT,q0cnt)   )
S
S  #define _CACHE_L2ALLOC0_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD)
X  #define _CACHE_L2ALLOC0_FGET(FIELD)    _PER_FGET(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD)
S
S  #define _CACHE_L2ALLOC0_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD,field)
X  #define _CACHE_L2ALLOC0_FSET(FIELD,field)    _PER_FSET(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD,field)
S
S  #define _CACHE_L2ALLOC0_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD,##SYM)
X  #define _CACHE_L2ALLOC0_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2ALLOC0_ADDR,CACHE,L2ALLOC0,##FIELD,##SYM)
S
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 A L L O C 1  |
N* |___________________|
N*
N* L2ALLOC1 - L2 allocation register 1 (1)
N*
N* Fields:
N* (rw) Q1CNT (2)
N*
N* (1) only supported for C6400
N* (2) Rename bit filed based on spru610
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2ALLOC1_ADDR             0x01842004u
S
S  #define _CACHE_L2ALLOC1_Q1CNT_MASK       0x00000007u
S  #define _CACHE_L2ALLOC1_Q1CNT_SHIFT      0x00000000u
S  #define  CACHE_L2ALLOC1_Q1CNT_DEFAULT    0x00000002u
S  #define  CACHE_L2ALLOC1_Q1CNT_OF(x)      _VALUEOF(x)
S
S  #define _CACHE_L2ALLOC1_L2ALLOC_MASK     _CACHE_L2ALLOC1_Q1CNT_MASK   
S  #define _CACHE_L2ALLOC1_L2ALLOC_SHIFT    _CACHE_L2ALLOC1_Q1CNT_SHIFT  
S  #define  CACHE_L2ALLOC1_L2ALLOC_DEFAULT   CACHE_L2ALLOC1_Q1CNT_DEFAULT
S  #define  CACHE_L2ALLOC1_L2ALLOC_OF(x)     CACHE_L2ALLOC1_Q1CNT_OF(x)  
S
S  #define  CACHE_L2ALLOC1_OF(x)            _VALUEOF(x)
S
S  #define CACHE_L2ALLOC1_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2ALLOC1,Q1CNT) \
S  )
X  #define CACHE_L2ALLOC1_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2ALLOC1,Q1CNT)   )
S
S  #define CACHE_L2ALLOC1_RMK(q1cnt) (Uint32)( \
S     _PER_FMK(CACHE,L2ALLOC1,Q1CNT,q1cnt) \
S  )
X  #define CACHE_L2ALLOC1_RMK(q1cnt) (Uint32)(      _PER_FMK(CACHE,L2ALLOC1,Q1CNT,q1cnt)   )
S
S  #define _CACHE_L2ALLOC1_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD)
X  #define _CACHE_L2ALLOC1_FGET(FIELD)    _PER_FGET(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD)
S
S  #define _CACHE_L2ALLOC1_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD,field)
X  #define _CACHE_L2ALLOC1_FSET(FIELD,field)    _PER_FSET(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD,field)
S
S  #define _CACHE_L2ALLOC1_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD,##SYM)
X  #define _CACHE_L2ALLOC1_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2ALLOC1_ADDR,CACHE,L2ALLOC1,##FIELD,##SYM)
S
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 A L L O C 2  |
N* |___________________|
N*
N* L2ALLOC2 - L2 allocation register 2 (1)
N*
N* Fields:
N* (rw) Q2CNT (2)
N*
N* (1) only supported for C6400
N* (2) Rename bit filed based on spru610
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2ALLOC2_ADDR               0x01842008u
S
S  #define _CACHE_L2ALLOC2_Q2CNT_MASK       0x00000007u
S  #define _CACHE_L2ALLOC2_Q2CNT_SHIFT      0x00000000u
S  #define  CACHE_L2ALLOC2_Q2CNT_DEFAULT    0x00000002u
S  #define  CACHE_L2ALLOC2_Q2CNT_OF(x)      _VALUEOF(x)
S
S  #define _CACHE_L2ALLOC2_L2ALLOC_MASK       _CACHE_L2ALLOC2_Q2CNT_MASK   
S  #define _CACHE_L2ALLOC2_L2ALLOC_SHIFT      _CACHE_L2ALLOC2_Q2CNT_SHIFT  
S  #define  CACHE_L2ALLOC2_L2ALLOC_DEFAULT     CACHE_L2ALLOC2_Q2CNT_DEFAULT
S  #define  CACHE_L2ALLOC2_L2ALLOC_OF(x)       CACHE_L2ALLOC2_Q2CNT_OF(x)  
S
S  #define  CACHE_L2ALLOC2_OF(x)            _VALUEOF(x)
S
S  #define CACHE_L2ALLOC2_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2ALLOC2,Q2CNT) \
S  )
X  #define CACHE_L2ALLOC2_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2ALLOC2,Q2CNT)   )
S
S  #define CACHE_L2ALLOC2_RMK(q2cnt) (Uint32)( \
S     _PER_FMK(CACHE,L2ALLOC2,Q2CNT,q2cnt) \
S  )
X  #define CACHE_L2ALLOC2_RMK(q2cnt) (Uint32)(      _PER_FMK(CACHE,L2ALLOC2,Q2CNT,q2cnt)   )
S
S  #define _CACHE_L2ALLOC2_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD)
X  #define _CACHE_L2ALLOC2_FGET(FIELD)    _PER_FGET(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD)
S
S  #define _CACHE_L2ALLOC2_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD,field)
X  #define _CACHE_L2ALLOC2_FSET(FIELD,field)    _PER_FSET(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD,field)
S
S  #define _CACHE_L2ALLOC2_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD,##SYM)
X  #define _CACHE_L2ALLOC2_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2ALLOC2_ADDR,CACHE,L2ALLOC2,##FIELD,##SYM)
S
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 A L L O C 3  |
N* |___________________|
N*
N* L2ALLOC3 - L2 allocation register 3 (1)
N*
N* Fields:
N* (rw) Q2CNT (2)
N*
N* (1) only supported for C6400
N* (2) Rename bit filed based on spru610
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2ALLOC3_ADDR               0x0184200Cu
S
S  #define _CACHE_L2ALLOC3_Q3CNT_MASK       0x00000007u
S  #define _CACHE_L2ALLOC3_Q3CNT_SHIFT      0x00000000u
S  #define  CACHE_L2ALLOC3_Q3CNT_DEFAULT    0x00000002u
S  #define  CACHE_L2ALLOC3_Q3CNT_OF(x)      _VALUEOF(x)
S
S  #define _CACHE_L2ALLOC3_L2ALLOC_MASK       _CACHE_L2ALLOC3_Q3CNT_MASK    
S  #define _CACHE_L2ALLOC3_L2ALLOC_SHIFT      _CACHE_L2ALLOC3_Q3CNT_SHIFT   
S  #define  CACHE_L2ALLOC3_L2ALLOC_DEFAULT     CACHE_L2ALLOC3_Q3CNT_DEFAULT 
S  #define  CACHE_L2ALLOC3_L2ALLOC_OF(x)       CACHE_L2ALLOC3_Q3CNT_OF(x)   
S
S  #define  CACHE_L2ALLOC3_OF(x)            _VALUEOF(x)
S
S  #define CACHE_L2ALLOC3_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2ALLOC3,Q3CNT) \
S  )
X  #define CACHE_L2ALLOC3_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2ALLOC3,Q3CNT)   )
S
S  #define CACHE_L2ALLOC3_RMK(q3cnt) (Uint32)( \
S     _PER_FMK(CACHE,L2ALLOC3,Q3CNT,q3cnt) \
S  )
X  #define CACHE_L2ALLOC3_RMK(q3cnt) (Uint32)(      _PER_FMK(CACHE,L2ALLOC3,Q3CNT,q3cnt)   )
S
S  #define _CACHE_L2ALLOC3_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD)
X  #define _CACHE_L2ALLOC3_FGET(FIELD)    _PER_FGET(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD)
S
S  #define _CACHE_L2ALLOC3_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD,field)
X  #define _CACHE_L2ALLOC3_FSET(FIELD,field)    _PER_FSET(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD,field)
S
S  #define _CACHE_L2ALLOC3_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD,##SYM)
X  #define _CACHE_L2ALLOC3_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2ALLOC3_ADDR,CACHE,L2ALLOC3,##FIELD,##SYM)
S
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  M A R            |
N* |___________________|
N*
N* MAR0   - memory attribute register 0 (1)
N* MAR1   - memory attribute register 1 (1)
N* MAR2   - memory attribute register 2 (1)
N* ...    - ...
N* MARn   - memory attribute register n (2)
N*
N* Fields:
N* (rw) NR (3)
N* (rw) NW (3)
N* (rw) PE (3)
N* (rw) CE (4)
N*
N* (1) register values are read only for C6400 
N* (2) n = 15 for C6211/C6711, n = 255 for C6400
N* (3) only supported for C6400
N* (4) only supported for C6211/C6711
N* (5) MARn range for C6211/C6711 are from MAR0 ~ MAR15,
N*     MARn range for C6400 are:
N*     MAR0   ~ MAR2   : register values read only
N*     MAR48  ~ MAR51  : McBSP0 
N*     MAR52  ~ MAR55  : McBSP1 
N*     MAR56  ~ MAR59  : McBSP2 
N*     MAR60  ~ MAR63  : UTOPIAII 
N*     MAR64  ~ MAR79  : HPI/PCI 
N*     MAR96  ~ MAR111 : EMIFB (1)
N*     MAR128 ~ MAR191 : EMIFA 
N*
N* (1) only in C6414, C6415, C6416 variants
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && !C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && !( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_MAR0_ADDR             _CACHE_MAR128_ADDR 
S  #define _CACHE_MAR1_ADDR             _CACHE_MAR129_ADDR 
S  #define _CACHE_MAR2_ADDR             _CACHE_MAR130_ADDR 
S  #define _CACHE_MAR3_ADDR             _CACHE_MAR131_ADDR 
S  #define _CACHE_MAR4_ADDR             _CACHE_MAR144_ADDR 
S  #define _CACHE_MAR5_ADDR             _CACHE_MAR145_ADDR 
S  #define _CACHE_MAR6_ADDR             _CACHE_MAR146_ADDR 
S  #define _CACHE_MAR7_ADDR             _CACHE_MAR147_ADDR 
S  #define _CACHE_MAR8_ADDR             _CACHE_MAR160_ADDR 
S  #define _CACHE_MAR9_ADDR             _CACHE_MAR161_ADDR 
S  #define _CACHE_MAR10_ADDR            _CACHE_MAR162_ADDR
S  #define _CACHE_MAR11_ADDR            _CACHE_MAR163_ADDR
S  #define _CACHE_MAR12_ADDR            _CACHE_MAR176_ADDR
S  #define _CACHE_MAR13_ADDR            _CACHE_MAR177_ADDR
S  #define _CACHE_MAR14_ADDR            _CACHE_MAR178_ADDR
S  #define _CACHE_MAR15_ADDR            _CACHE_MAR179_ADDR
S
S  #define _CACHE_MAR128_ADDR            0x01848200u
S  #define _CACHE_MAR129_ADDR            0x01848204u
S  #define _CACHE_MAR130_ADDR            0x01848208u
S  #define _CACHE_MAR131_ADDR            0x0184820Cu
S  #define _CACHE_MAR144_ADDR            0x01848240u
S  #define _CACHE_MAR145_ADDR            0x01848244u
S  #define _CACHE_MAR146_ADDR            0x01848248u
S  #define _CACHE_MAR147_ADDR            0x0184824Cu
S  #define _CACHE_MAR160_ADDR            0x01848280u
S  #define _CACHE_MAR161_ADDR            0x01848284u
S  #define _CACHE_MAR162_ADDR            0x01848288u
S  #define _CACHE_MAR163_ADDR            0x0184828Cu
S  #define _CACHE_MAR176_ADDR            0x018482C0u
S  #define _CACHE_MAR177_ADDR            0x018482C4u
S  #define _CACHE_MAR178_ADDR            0x018482C8u
S  #define _CACHE_MAR179_ADDR            0x018482CCu
S
S  #define _CACHE_MAR_CE_MASK           0x00000001u
S  #define _CACHE_MAR_CE_SHIFT          0x00000000u
S  #define  CACHE_MAR_CE_DEFAULT        0x00000000u
S  #define  CACHE_MAR_CE_OF(x)          _VALUEOF(x)
S  #define  CACHE_MAR_CE_DISABLE        0x00000000u
S  #define  CACHE_MAR_CE_ENABLE         0x00000001u
S
S  #define  CACHE_MAR_OF(x)             _VALUEOF(x)
S
S  #define CACHE_MAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,MAR,CE) \
S  )
X  #define CACHE_MAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,MAR,CE)   )
S
S  #define CACHE_MAR_RMK(ce) (Uint32)( \
S     _PER_FMK(CACHE,MAR,CE,ce) \
S  )
X  #define CACHE_MAR_RMK(ce) (Uint32)(      _PER_FMK(CACHE,MAR,CE,ce)   )
S
S  #define _CACHE_MAR_FGET(N,FIELD)\
S    _PER_FGET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD)
X  #define _CACHE_MAR_FGET(N,FIELD)    _PER_FGET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD)
S
S  #define _CACHE_MAR_FSET(N,FIELD,field)\
S    _PER_FSET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,field)
X  #define _CACHE_MAR_FSET(N,FIELD,field)    _PER_FSET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,field)
S
S  #define _CACHE_MAR_FSETS(N,FIELD,SYM)\
S    _PER_FSETS(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,##SYM)
X  #define _CACHE_MAR_FSETS(N,FIELD,SYM)    _PER_FSETS(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,##SYM)
S
S  #define _CACHE_MAR0_FGET(FIELD)  _CACHE_MAR_FGET(128,##FIELD)
S  #define _CACHE_MAR1_FGET(FIELD)  _CACHE_MAR_FGET(129,##FIELD)
S  #define _CACHE_MAR2_FGET(FIELD)  _CACHE_MAR_FGET(130,##FIELD)
S  #define _CACHE_MAR3_FGET(FIELD)  _CACHE_MAR_FGET(131,##FIELD)
S  #define _CACHE_MAR4_FGET(FIELD)  _CACHE_MAR_FGET(144,##FIELD)
S  #define _CACHE_MAR5_FGET(FIELD)  _CACHE_MAR_FGET(145,##FIELD)
S  #define _CACHE_MAR6_FGET(FIELD)  _CACHE_MAR_FGET(146,##FIELD)
S  #define _CACHE_MAR7_FGET(FIELD)  _CACHE_MAR_FGET(147,##FIELD)
S  #define _CACHE_MAR8_FGET(FIELD)  _CACHE_MAR_FGET(160,##FIELD)
S  #define _CACHE_MAR9_FGET(FIELD)  _CACHE_MAR_FGET(161,##FIELD)
S  #define _CACHE_MAR10_FGET(FIELD) _CACHE_MAR_FGET(162,##FIELD)
S  #define _CACHE_MAR11_FGET(FIELD) _CACHE_MAR_FGET(163,##FIELD)
S  #define _CACHE_MAR12_FGET(FIELD) _CACHE_MAR_FGET(176,##FIELD)
S  #define _CACHE_MAR13_FGET(FIELD) _CACHE_MAR_FGET(177,##FIELD)
S  #define _CACHE_MAR14_FGET(FIELD) _CACHE_MAR_FGET(178,##FIELD)
S  #define _CACHE_MAR15_FGET(FIELD) _CACHE_MAR_FGET(179,##FIELD)
S
S  #define _CACHE_MAR0_FSET(FIELD,f)  _CACHE_MAR_FSET(128,##FIELD,f)
S  #define _CACHE_MAR1_FSET(FIELD,f)  _CACHE_MAR_FSET(129,##FIELD,f)
S  #define _CACHE_MAR2_FSET(FIELD,f)  _CACHE_MAR_FSET(130,##FIELD,f)
S  #define _CACHE_MAR3_FSET(FIELD,f)  _CACHE_MAR_FSET(131,##FIELD,f)
S  #define _CACHE_MAR4_FSET(FIELD,f)  _CACHE_MAR_FSET(144,##FIELD,f)
S  #define _CACHE_MAR5_FSET(FIELD,f)  _CACHE_MAR_FSET(145,##FIELD,f)
S  #define _CACHE_MAR6_FSET(FIELD,f)  _CACHE_MAR_FSET(146,##FIELD,f)
S  #define _CACHE_MAR7_FSET(FIELD,f)  _CACHE_MAR_FSET(147,##FIELD,f)
S  #define _CACHE_MAR8_FSET(FIELD,f)  _CACHE_MAR_FSET(160,##FIELD,f)
S  #define _CACHE_MAR9_FSET(FIELD,f)  _CACHE_MAR_FSET(161,##FIELD,f)
S  #define _CACHE_MAR10_FSET(FIELD,f) _CACHE_MAR_FSET(162,##FIELD,f)
S  #define _CACHE_MAR11_FSET(FIELD,f) _CACHE_MAR_FSET(163,##FIELD,f)
S  #define _CACHE_MAR12_FSET(FIELD,f) _CACHE_MAR_FSET(176,##FIELD,f)
S  #define _CACHE_MAR13_FSET(FIELD,f) _CACHE_MAR_FSET(177,##FIELD,f)
S  #define _CACHE_MAR14_FSET(FIELD,f) _CACHE_MAR_FSET(178,##FIELD,f)
S  #define _CACHE_MAR15_FSET(FIELD,f) _CACHE_MAR_FSET(179,##FIELD,f)
S
S  #define _CACHE_MAR0_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(128,##FIELD,##SYM)
S  #define _CACHE_MAR1_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(129,##FIELD,##SYM)
S  #define _CACHE_MAR2_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(130,##FIELD,##SYM)
S  #define _CACHE_MAR3_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(131,##FIELD,##SYM)
S  #define _CACHE_MAR4_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(144,##FIELD,##SYM)
S  #define _CACHE_MAR5_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(145,##FIELD,##SYM)
S  #define _CACHE_MAR6_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(146,##FIELD,##SYM)
S  #define _CACHE_MAR7_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(147,##FIELD,##SYM)
S  #define _CACHE_MAR8_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(160,##FIELD,##SYM)
S  #define _CACHE_MAR9_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(161,##FIELD,##SYM)
S  #define _CACHE_MAR10_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(162,##FIELD,##SYM)
S  #define _CACHE_MAR11_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(163,##FIELD,##SYM)
S  #define _CACHE_MAR12_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(176,##FIELD,##SYM)
S  #define _CACHE_MAR13_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(177,##FIELD,##SYM)
S  #define _CACHE_MAR14_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(178,##FIELD,##SYM)
S  #define _CACHE_MAR15_FSETS(FIELD,SYM) _CACHE_MAR_FSETS(179,##FIELD,##SYM)
S
S  #define _CACHE_MAR128_FGET(FIELD)  _CACHE_MAR_FGET(128,##FIELD)
S  #define _CACHE_MAR129_FGET(FIELD)  _CACHE_MAR_FGET(129,##FIELD)
S  #define _CACHE_MAR130_FGET(FIELD)  _CACHE_MAR_FGET(130,##FIELD)
S  #define _CACHE_MAR131_FGET(FIELD)  _CACHE_MAR_FGET(131,##FIELD)
S  #define _CACHE_MAR144_FGET(FIELD)  _CACHE_MAR_FGET(144,##FIELD)
S  #define _CACHE_MAR145_FGET(FIELD)  _CACHE_MAR_FGET(145,##FIELD)
S  #define _CACHE_MAR146_FGET(FIELD)  _CACHE_MAR_FGET(146,##FIELD)
S  #define _CACHE_MAR147_FGET(FIELD)  _CACHE_MAR_FGET(147,##FIELD)
S  #define _CACHE_MAR160_FGET(FIELD)  _CACHE_MAR_FGET(160,##FIELD)
S  #define _CACHE_MAR161_FGET(FIELD)  _CACHE_MAR_FGET(161,##FIELD)
S  #define _CACHE_MAR162_FGET(FIELD)  _CACHE_MAR_FGET(162,##FIELD)
S  #define _CACHE_MAR163_FGET(FIELD)  _CACHE_MAR_FGET(163,##FIELD)
S  #define _CACHE_MAR176_FGET(FIELD)  _CACHE_MAR_FGET(176,##FIELD)
S  #define _CACHE_MAR177_FGET(FIELD)  _CACHE_MAR_FGET(177,##FIELD)
S  #define _CACHE_MAR178_FGET(FIELD)  _CACHE_MAR_FGET(178,##FIELD)
S  #define _CACHE_MAR179_FGET(FIELD)  _CACHE_MAR_FGET(179,##FIELD)
S
S  #define _CACHE_MAR128_FSET(FIELD,f)  _CACHE_MAR_FSET(128,##FIELD,f)
S  #define _CACHE_MAR129_FSET(FIELD,f)  _CACHE_MAR_FSET(129,##FIELD,f)
S  #define _CACHE_MAR130_FSET(FIELD,f)  _CACHE_MAR_FSET(130,##FIELD,f)
S  #define _CACHE_MAR131_FSET(FIELD,f)  _CACHE_MAR_FSET(131,##FIELD,f)
S  #define _CACHE_MAR144_FSET(FIELD,f)  _CACHE_MAR_FSET(144,##FIELD,f)
S  #define _CACHE_MAR145_FSET(FIELD,f)  _CACHE_MAR_FSET(145,##FIELD,f)
S  #define _CACHE_MAR146_FSET(FIELD,f)  _CACHE_MAR_FSET(146,##FIELD,f)
S  #define _CACHE_MAR147_FSET(FIELD,f)  _CACHE_MAR_FSET(147,##FIELD,f)
S  #define _CACHE_MAR160_FSET(FIELD,f)  _CACHE_MAR_FSET(160,##FIELD,f)
S  #define _CACHE_MAR161_FSET(FIELD,f)  _CACHE_MAR_FSET(161,##FIELD,f)
S  #define _CACHE_MAR162_FSET(FIELD,f)  _CACHE_MAR_FSET(162,##FIELD,f)
S  #define _CACHE_MAR163_FSET(FIELD,f)  _CACHE_MAR_FSET(163,##FIELD,f)
S  #define _CACHE_MAR176_FSET(FIELD,f)  _CACHE_MAR_FSET(176,##FIELD,f)
S  #define _CACHE_MAR177_FSET(FIELD,f)  _CACHE_MAR_FSET(177,##FIELD,f)
S  #define _CACHE_MAR178_FSET(FIELD,f)  _CACHE_MAR_FSET(178,##FIELD,f)
S  #define _CACHE_MAR179_FSET(FIELD,f)  _CACHE_MAR_FSET(179,##FIELD,f)
S
S  #define _CACHE_MAR128_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(128,##FIELD,##SYM)
S  #define _CACHE_MAR129_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(129,##FIELD,##SYM)
S  #define _CACHE_MAR130_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(130,##FIELD,##SYM)
S  #define _CACHE_MAR131_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(131,##FIELD,##SYM)
S  #define _CACHE_MAR144_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(144,##FIELD,##SYM)
S  #define _CACHE_MAR145_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(145,##FIELD,##SYM)
S  #define _CACHE_MAR146_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(146,##FIELD,##SYM)
S  #define _CACHE_MAR147_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(147,##FIELD,##SYM)
S  #define _CACHE_MAR160_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(160,##FIELD,##SYM)
S  #define _CACHE_MAR161_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(161,##FIELD,##SYM)
S  #define _CACHE_MAR162_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(162,##FIELD,##SYM)
S  #define _CACHE_MAR163_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(163,##FIELD,##SYM)
S  #define _CACHE_MAR176_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(176,##FIELD,##SYM)
S  #define _CACHE_MAR177_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(177,##FIELD,##SYM)
S  #define _CACHE_MAR178_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(178,##FIELD,##SYM)
S  #define _CACHE_MAR179_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(179,##FIELD,##SYM)
S
N#else 
N
N  /* MAR96 to MAR111 are associated with EMIFB memory space. This is supported only in  
N   * the C6414, C6415 and C6416 chips                                                  */
N    
N  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X  #if (0 | 0 | 0)
S    #define _CACHE_MAR96_ADDR            0x01848180u
S    #define _CACHE_MAR97_ADDR            0x01848184u
S    #define _CACHE_MAR98_ADDR            0x01848188u
S    #define _CACHE_MAR99_ADDR            0x0184818Cu
S    #define _CACHE_MAR100_ADDR           0x01848190u
S    #define _CACHE_MAR101_ADDR           0x01848194u
S    #define _CACHE_MAR102_ADDR           0x01848198u
S    #define _CACHE_MAR103_ADDR           0x0184819Cu
S    #define _CACHE_MAR104_ADDR           0x018481A0u
S    #define _CACHE_MAR105_ADDR           0x018481A4u
S    #define _CACHE_MAR106_ADDR           0x018481A8u
S    #define _CACHE_MAR107_ADDR           0x018481ACu
S    #define _CACHE_MAR108_ADDR           0x018481B0u
S    #define _CACHE_MAR109_ADDR           0x018481B4u
S    #define _CACHE_MAR110_ADDR           0x018481B8u
S    #define _CACHE_MAR111_ADDR           0x018481BCu
N  #endif
N
N  #define _CACHE_MAR128_ADDR           0x01848200u
N  #define _CACHE_MAR129_ADDR           0x01848204u
N  #define _CACHE_MAR130_ADDR           0x01848208u
N  #define _CACHE_MAR131_ADDR           0x0184820Cu
N  #define _CACHE_MAR132_ADDR           0x01848210u
N  #define _CACHE_MAR133_ADDR           0x01848214u
N  #define _CACHE_MAR134_ADDR           0x01848218u
N  #define _CACHE_MAR135_ADDR           0x0184821Cu
N  #define _CACHE_MAR136_ADDR           0x01848220u
N  #define _CACHE_MAR137_ADDR           0x01848224u
N  #define _CACHE_MAR138_ADDR           0x01848228u
N  #define _CACHE_MAR139_ADDR           0x0184822Cu
N  #define _CACHE_MAR140_ADDR           0x01848230u
N  #define _CACHE_MAR141_ADDR           0x01848234u
N  #define _CACHE_MAR142_ADDR           0x01848238u
N  #define _CACHE_MAR143_ADDR           0x0184823Cu
N  #define _CACHE_MAR144_ADDR           0x01848240u
N  #define _CACHE_MAR145_ADDR           0x01848244u
N  #define _CACHE_MAR146_ADDR           0x01848248u
N  #define _CACHE_MAR147_ADDR           0x0184824Cu
N  #define _CACHE_MAR148_ADDR           0x01848250u
N  #define _CACHE_MAR149_ADDR           0x01848254u
N  #define _CACHE_MAR150_ADDR           0x01848258u
N  #define _CACHE_MAR151_ADDR           0x0184825Cu
N  #define _CACHE_MAR152_ADDR           0x01848260u
N  #define _CACHE_MAR153_ADDR           0x01848264u
N  #define _CACHE_MAR154_ADDR           0x01848268u
N  #define _CACHE_MAR155_ADDR           0x0184826Cu
N  #define _CACHE_MAR156_ADDR           0x01848270u
N  #define _CACHE_MAR157_ADDR           0x01848274u
N  #define _CACHE_MAR158_ADDR           0x01848278u
N  #define _CACHE_MAR159_ADDR           0x0184827Cu
N  #define _CACHE_MAR160_ADDR           0x01848280u
N  #define _CACHE_MAR161_ADDR           0x01848284u
N  #define _CACHE_MAR162_ADDR           0x01848288u
N  #define _CACHE_MAR163_ADDR           0x0184828Cu
N  #define _CACHE_MAR164_ADDR           0x01848290u
N  #define _CACHE_MAR165_ADDR           0x01848294u
N  #define _CACHE_MAR166_ADDR           0x01848298u
N  #define _CACHE_MAR167_ADDR           0x0184829Cu
N  #define _CACHE_MAR168_ADDR           0x018482A0u
N  #define _CACHE_MAR169_ADDR           0x018482A4u
N  #define _CACHE_MAR170_ADDR           0x018482A8u
N  #define _CACHE_MAR171_ADDR           0x018482ACu
N  #define _CACHE_MAR172_ADDR           0x018482B0u
N  #define _CACHE_MAR173_ADDR           0x018482B4u
N  #define _CACHE_MAR174_ADDR           0x018482B8u
N  #define _CACHE_MAR175_ADDR           0x018482BCu
N  #define _CACHE_MAR176_ADDR           0x018482C0u
N  #define _CACHE_MAR177_ADDR           0x018482C4u
N  #define _CACHE_MAR178_ADDR           0x018482C8u
N  #define _CACHE_MAR179_ADDR           0x018482CCu
N  #define _CACHE_MAR180_ADDR           0x018482D0u
N  #define _CACHE_MAR181_ADDR           0x018482D4u
N  #define _CACHE_MAR182_ADDR           0x018482D8u
N  #define _CACHE_MAR183_ADDR           0x018482DCu
N  #define _CACHE_MAR184_ADDR           0x018482E0u
N  #define _CACHE_MAR185_ADDR           0x018482E4u
N  #define _CACHE_MAR186_ADDR           0x018482E8u
N  #define _CACHE_MAR187_ADDR           0x018482ECu
N  #define _CACHE_MAR188_ADDR           0x018482F0u
N  #define _CACHE_MAR189_ADDR           0x018482F4u
N  #define _CACHE_MAR190_ADDR           0x018482F8u
N  #define _CACHE_MAR191_ADDR           0x018482FCu
N
N  #define _CACHE_MAR_CE_MASK           0x00000001u
N  #define _CACHE_MAR_CE_SHIFT          0x00000000u
N  #define  CACHE_MAR_CE_DEFAULT        0x00000000u
N  #define  CACHE_MAR_CE_OF(x)          _VALUEOF(x)
N  #define  CACHE_MAR_CE_DISABLE        0x00000000u
N  #define  CACHE_MAR_CE_ENABLE         0x00000001u
N
N  #define  CACHE_MAR_OF(x)             _VALUEOF(x)
N
N  #define CACHE_MAR_DEFAULT (Uint32)( \
N    _PER_FDEFAULT(CACHE,MAR,CE) \
N  )
X  #define CACHE_MAR_DEFAULT (Uint32)(     _PER_FDEFAULT(CACHE,MAR,CE)   )
N
N  #define CACHE_MAR_RMK(ce) (Uint32)( \
N    _PER_FMK(CACHE,MAR,CE,ce) \
N  )
X  #define CACHE_MAR_RMK(ce) (Uint32)(     _PER_FMK(CACHE,MAR,CE,ce)   )
N
N  #define _CACHE_MAR_FGET(N,FIELD)\
N    _PER_FGET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD)
X  #define _CACHE_MAR_FGET(N,FIELD)    _PER_FGET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD)
N
N  #define _CACHE_MAR_FSET(N,FIELD,field)\
N    _PER_FSET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,field)
X  #define _CACHE_MAR_FSET(N,FIELD,field)    _PER_FSET(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,field)
N
N  #define _CACHE_MAR_FSETS(N,FIELD,SYM)\
N    _PER_FSETS(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,##SYM)
X  #define _CACHE_MAR_FSETS(N,FIELD,SYM)    _PER_FSETS(_CACHE_MAR##N##_ADDR,CACHE,MAR,##FIELD,##SYM)
N
N  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X  #if (0 | 0 | 0)
S    #define _CACHE_MAR96_FGET(FIELD)   _CACHE_MAR_FGET(96,##FIELD)
S    #define _CACHE_MAR97_FGET(FIELD)   _CACHE_MAR_FGET(97,##FIELD)
S    #define _CACHE_MAR98_FGET(FIELD)   _CACHE_MAR_FGET(98,##FIELD)
S    #define _CACHE_MAR99_FGET(FIELD)   _CACHE_MAR_FGET(99,##FIELD)
S    #define _CACHE_MAR100_FGET(FIELD)  _CACHE_MAR_FGET(100,##FIELD)
S    #define _CACHE_MAR101_FGET(FIELD)  _CACHE_MAR_FGET(101,##FIELD)
S    #define _CACHE_MAR102_FGET(FIELD)  _CACHE_MAR_FGET(102,##FIELD)
S    #define _CACHE_MAR103_FGET(FIELD)  _CACHE_MAR_FGET(103,##FIELD)
S    #define _CACHE_MAR104_FGET(FIELD)  _CACHE_MAR_FGET(104,##FIELD)
S    #define _CACHE_MAR105_FGET(FIELD)  _CACHE_MAR_FGET(105,##FIELD)
S    #define _CACHE_MAR106_FGET(FIELD)  _CACHE_MAR_FGET(106,##FIELD)
S    #define _CACHE_MAR107_FGET(FIELD)  _CACHE_MAR_FGET(107,##FIELD)
S    #define _CACHE_MAR108_FGET(FIELD)  _CACHE_MAR_FGET(108,##FIELD)
S    #define _CACHE_MAR109_FGET(FIELD)  _CACHE_MAR_FGET(109,##FIELD)
S    #define _CACHE_MAR110_FGET(FIELD)  _CACHE_MAR_FGET(110,##FIELD)
S    #define _CACHE_MAR111_FGET(FIELD)  _CACHE_MAR_FGET(111,##FIELD)
N  #endif
N
N  #define _CACHE_MAR128_FGET(FIELD)  _CACHE_MAR_FGET(128,##FIELD)
N  #define _CACHE_MAR129_FGET(FIELD)  _CACHE_MAR_FGET(129,##FIELD)
N  #define _CACHE_MAR130_FGET(FIELD)  _CACHE_MAR_FGET(130,##FIELD)
N  #define _CACHE_MAR131_FGET(FIELD)  _CACHE_MAR_FGET(131,##FIELD)
N  #define _CACHE_MAR132_FGET(FIELD)  _CACHE_MAR_FGET(132,##FIELD)
N  #define _CACHE_MAR133_FGET(FIELD)  _CACHE_MAR_FGET(133,##FIELD)
N  #define _CACHE_MAR134_FGET(FIELD)  _CACHE_MAR_FGET(134,##FIELD)
N  #define _CACHE_MAR135_FGET(FIELD)  _CACHE_MAR_FGET(135,##FIELD)
N  #define _CACHE_MAR136_FGET(FIELD)  _CACHE_MAR_FGET(136,##FIELD)
N  #define _CACHE_MAR137_FGET(FIELD)  _CACHE_MAR_FGET(137,##FIELD)
N  #define _CACHE_MAR138_FGET(FIELD)  _CACHE_MAR_FGET(138,##FIELD)
N  #define _CACHE_MAR139_FGET(FIELD)  _CACHE_MAR_FGET(139,##FIELD)
N  #define _CACHE_MAR140_FGET(FIELD)  _CACHE_MAR_FGET(140,##FIELD)
N  #define _CACHE_MAR141_FGET(FIELD)  _CACHE_MAR_FGET(141,##FIELD)
N  #define _CACHE_MAR142_FGET(FIELD)  _CACHE_MAR_FGET(142,##FIELD)
N  #define _CACHE_MAR143_FGET(FIELD)  _CACHE_MAR_FGET(143,##FIELD)
N  #define _CACHE_MAR144_FGET(FIELD)  _CACHE_MAR_FGET(144,##FIELD)
N  #define _CACHE_MAR145_FGET(FIELD)  _CACHE_MAR_FGET(145,##FIELD)
N  #define _CACHE_MAR146_FGET(FIELD)  _CACHE_MAR_FGET(146,##FIELD)
N  #define _CACHE_MAR147_FGET(FIELD)  _CACHE_MAR_FGET(147,##FIELD)
N  #define _CACHE_MAR148_FGET(FIELD)  _CACHE_MAR_FGET(148,##FIELD)
N  #define _CACHE_MAR149_FGET(FIELD)  _CACHE_MAR_FGET(149,##FIELD)
N  #define _CACHE_MAR150_FGET(FIELD)  _CACHE_MAR_FGET(150,##FIELD)
N  #define _CACHE_MAR151_FGET(FIELD)  _CACHE_MAR_FGET(151,##FIELD)
N  #define _CACHE_MAR152_FGET(FIELD)  _CACHE_MAR_FGET(152,##FIELD)
N  #define _CACHE_MAR153_FGET(FIELD)  _CACHE_MAR_FGET(153,##FIELD)
N  #define _CACHE_MAR154_FGET(FIELD)  _CACHE_MAR_FGET(154,##FIELD)
N  #define _CACHE_MAR155_FGET(FIELD)  _CACHE_MAR_FGET(155,##FIELD)
N  #define _CACHE_MAR156_FGET(FIELD)  _CACHE_MAR_FGET(156,##FIELD)
N  #define _CACHE_MAR157_FGET(FIELD)  _CACHE_MAR_FGET(157,##FIELD)
N  #define _CACHE_MAR158_FGET(FIELD)  _CACHE_MAR_FGET(158,##FIELD)
N  #define _CACHE_MAR159_FGET(FIELD)  _CACHE_MAR_FGET(159,##FIELD)
N  #define _CACHE_MAR160_FGET(FIELD)  _CACHE_MAR_FGET(160,##FIELD)
N  #define _CACHE_MAR161_FGET(FIELD)  _CACHE_MAR_FGET(161,##FIELD)
N  #define _CACHE_MAR162_FGET(FIELD)  _CACHE_MAR_FGET(162,##FIELD)
N  #define _CACHE_MAR163_FGET(FIELD)  _CACHE_MAR_FGET(163,##FIELD)
N  #define _CACHE_MAR164_FGET(FIELD)  _CACHE_MAR_FGET(164,##FIELD)
N  #define _CACHE_MAR165_FGET(FIELD)  _CACHE_MAR_FGET(165,##FIELD)
N  #define _CACHE_MAR166_FGET(FIELD)  _CACHE_MAR_FGET(166,##FIELD)
N  #define _CACHE_MAR167_FGET(FIELD)  _CACHE_MAR_FGET(167,##FIELD)
N  #define _CACHE_MAR168_FGET(FIELD)  _CACHE_MAR_FGET(168,##FIELD)
N  #define _CACHE_MAR169_FGET(FIELD)  _CACHE_MAR_FGET(169,##FIELD)
N  #define _CACHE_MAR170_FGET(FIELD)  _CACHE_MAR_FGET(170,##FIELD)
N  #define _CACHE_MAR171_FGET(FIELD)  _CACHE_MAR_FGET(171,##FIELD)
N  #define _CACHE_MAR172_FGET(FIELD)  _CACHE_MAR_FGET(172,##FIELD)
N  #define _CACHE_MAR173_FGET(FIELD)  _CACHE_MAR_FGET(173,##FIELD)
N  #define _CACHE_MAR174_FGET(FIELD)  _CACHE_MAR_FGET(174,##FIELD)
N  #define _CACHE_MAR175_FGET(FIELD)  _CACHE_MAR_FGET(175,##FIELD)
N  #define _CACHE_MAR176_FGET(FIELD)  _CACHE_MAR_FGET(176,##FIELD)
N  #define _CACHE_MAR177_FGET(FIELD)  _CACHE_MAR_FGET(177,##FIELD)
N  #define _CACHE_MAR178_FGET(FIELD)  _CACHE_MAR_FGET(178,##FIELD)
N  #define _CACHE_MAR179_FGET(FIELD)  _CACHE_MAR_FGET(179,##FIELD)
N  #define _CACHE_MAR180_FGET(FIELD)  _CACHE_MAR_FGET(180,##FIELD)
N  #define _CACHE_MAR181_FGET(FIELD)  _CACHE_MAR_FGET(181,##FIELD)
N  #define _CACHE_MAR182_FGET(FIELD)  _CACHE_MAR_FGET(182,##FIELD)
N  #define _CACHE_MAR183_FGET(FIELD)  _CACHE_MAR_FGET(183,##FIELD)
N  #define _CACHE_MAR184_FGET(FIELD)  _CACHE_MAR_FGET(184,##FIELD)
N  #define _CACHE_MAR185_FGET(FIELD)  _CACHE_MAR_FGET(185,##FIELD)
N  #define _CACHE_MAR186_FGET(FIELD)  _CACHE_MAR_FGET(186,##FIELD)
N  #define _CACHE_MAR187_FGET(FIELD)  _CACHE_MAR_FGET(187,##FIELD)
N  #define _CACHE_MAR188_FGET(FIELD)  _CACHE_MAR_FGET(188,##FIELD)
N  #define _CACHE_MAR189_FGET(FIELD)  _CACHE_MAR_FGET(189,##FIELD)
N  #define _CACHE_MAR190_FGET(FIELD)  _CACHE_MAR_FGET(190,##FIELD)
N  #define _CACHE_MAR191_FGET(FIELD)  _CACHE_MAR_FGET(191,##FIELD)
N
N  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X  #if (0 | 0 | 0)
S    #define _CACHE_MAR96_FSET(FIELD,f)   _CACHE_MAR_FSET(96,##FIELD,f)
S    #define _CACHE_MAR97_FSET(FIELD,f)   _CACHE_MAR_FSET(97,##FIELD,f)
S    #define _CACHE_MAR98_FSET(FIELD,f)   _CACHE_MAR_FSET(98,##FIELD,f)
S    #define _CACHE_MAR99_FSET(FIELD,f)   _CACHE_MAR_FSET(99,##FIELD,f)
S    #define _CACHE_MAR100_FSET(FIELD,f)  _CACHE_MAR_FSET(100,##FIELD,f)
S    #define _CACHE_MAR101_FSET(FIELD,f)  _CACHE_MAR_FSET(101,##FIELD,f)
S    #define _CACHE_MAR102_FSET(FIELD,f)  _CACHE_MAR_FSET(102,##FIELD,f)
S    #define _CACHE_MAR103_FSET(FIELD,f)  _CACHE_MAR_FSET(103,##FIELD,f)
S    #define _CACHE_MAR104_FSET(FIELD,f)  _CACHE_MAR_FSET(104,##FIELD,f)
S    #define _CACHE_MAR105_FSET(FIELD,f)  _CACHE_MAR_FSET(105,##FIELD,f)
S    #define _CACHE_MAR106_FSET(FIELD,f)  _CACHE_MAR_FSET(106,##FIELD,f)
S    #define _CACHE_MAR107_FSET(FIELD,f)  _CACHE_MAR_FSET(107,##FIELD,f)
S    #define _CACHE_MAR108_FSET(FIELD,f)  _CACHE_MAR_FSET(108,##FIELD,f)
S    #define _CACHE_MAR109_FSET(FIELD,f)  _CACHE_MAR_FSET(109,##FIELD,f)
S    #define _CACHE_MAR110_FSET(FIELD,f)  _CACHE_MAR_FSET(110,##FIELD,f)
S    #define _CACHE_MAR111_FSET(FIELD,f)  _CACHE_MAR_FSET(111,##FIELD,f)
N  #endif
N
N  #define _CACHE_MAR128_FSET(FIELD,f)  _CACHE_MAR_FSET(128,##FIELD,f)
N  #define _CACHE_MAR129_FSET(FIELD,f)  _CACHE_MAR_FSET(129,##FIELD,f)
N  #define _CACHE_MAR130_FSET(FIELD,f)  _CACHE_MAR_FSET(130,##FIELD,f)
N  #define _CACHE_MAR131_FSET(FIELD,f)  _CACHE_MAR_FSET(131,##FIELD,f)
N  #define _CACHE_MAR132_FSET(FIELD,f)  _CACHE_MAR_FSET(132,##FIELD,f)
N  #define _CACHE_MAR133_FSET(FIELD,f)  _CACHE_MAR_FSET(133,##FIELD,f)
N  #define _CACHE_MAR134_FSET(FIELD,f)  _CACHE_MAR_FSET(134,##FIELD,f)
N  #define _CACHE_MAR135_FSET(FIELD,f)  _CACHE_MAR_FSET(135,##FIELD,f)
N  #define _CACHE_MAR136_FSET(FIELD,f)  _CACHE_MAR_FSET(136,##FIELD,f)
N  #define _CACHE_MAR137_FSET(FIELD,f)  _CACHE_MAR_FSET(137,##FIELD,f)
N  #define _CACHE_MAR138_FSET(FIELD,f)  _CACHE_MAR_FSET(138,##FIELD,f)
N  #define _CACHE_MAR139_FSET(FIELD,f)  _CACHE_MAR_FSET(139,##FIELD,f)
N  #define _CACHE_MAR140_FSET(FIELD,f)  _CACHE_MAR_FSET(140,##FIELD,f)
N  #define _CACHE_MAR141_FSET(FIELD,f)  _CACHE_MAR_FSET(141,##FIELD,f)
N  #define _CACHE_MAR142_FSET(FIELD,f)  _CACHE_MAR_FSET(142,##FIELD,f)
N  #define _CACHE_MAR143_FSET(FIELD,f)  _CACHE_MAR_FSET(143,##FIELD,f)
N  #define _CACHE_MAR144_FSET(FIELD,f)  _CACHE_MAR_FSET(144,##FIELD,f)
N  #define _CACHE_MAR145_FSET(FIELD,f)  _CACHE_MAR_FSET(145,##FIELD,f)
N  #define _CACHE_MAR146_FSET(FIELD,f)  _CACHE_MAR_FSET(146,##FIELD,f)
N  #define _CACHE_MAR147_FSET(FIELD,f)  _CACHE_MAR_FSET(147,##FIELD,f)
N  #define _CACHE_MAR148_FSET(FIELD,f)  _CACHE_MAR_FSET(148,##FIELD,f)
N  #define _CACHE_MAR149_FSET(FIELD,f)  _CACHE_MAR_FSET(149,##FIELD,f)
N  #define _CACHE_MAR150_FSET(FIELD,f)  _CACHE_MAR_FSET(150,##FIELD,f)
N  #define _CACHE_MAR151_FSET(FIELD,f)  _CACHE_MAR_FSET(151,##FIELD,f)
N  #define _CACHE_MAR152_FSET(FIELD,f)  _CACHE_MAR_FSET(152,##FIELD,f)
N  #define _CACHE_MAR153_FSET(FIELD,f)  _CACHE_MAR_FSET(153,##FIELD,f)
N  #define _CACHE_MAR154_FSET(FIELD,f)  _CACHE_MAR_FSET(154,##FIELD,f)
N  #define _CACHE_MAR155_FSET(FIELD,f)  _CACHE_MAR_FSET(155,##FIELD,f)
N  #define _CACHE_MAR156_FSET(FIELD,f)  _CACHE_MAR_FSET(156,##FIELD,f)
N  #define _CACHE_MAR157_FSET(FIELD,f)  _CACHE_MAR_FSET(157,##FIELD,f)
N  #define _CACHE_MAR158_FSET(FIELD,f)  _CACHE_MAR_FSET(158,##FIELD,f)
N  #define _CACHE_MAR159_FSET(FIELD,f)  _CACHE_MAR_FSET(159,##FIELD,f)
N  #define _CACHE_MAR160_FSET(FIELD,f)  _CACHE_MAR_FSET(160,##FIELD,f)
N  #define _CACHE_MAR161_FSET(FIELD,f)  _CACHE_MAR_FSET(161,##FIELD,f)
N  #define _CACHE_MAR162_FSET(FIELD,f)  _CACHE_MAR_FSET(162,##FIELD,f)
N  #define _CACHE_MAR163_FSET(FIELD,f)  _CACHE_MAR_FSET(163,##FIELD,f)
N  #define _CACHE_MAR164_FSET(FIELD,f)  _CACHE_MAR_FSET(164,##FIELD,f)
N  #define _CACHE_MAR165_FSET(FIELD,f)  _CACHE_MAR_FSET(165,##FIELD,f)
N  #define _CACHE_MAR166_FSET(FIELD,f)  _CACHE_MAR_FSET(166,##FIELD,f)
N  #define _CACHE_MAR167_FSET(FIELD,f)  _CACHE_MAR_FSET(167,##FIELD,f)
N  #define _CACHE_MAR168_FSET(FIELD,f)  _CACHE_MAR_FSET(168,##FIELD,f)
N  #define _CACHE_MAR169_FSET(FIELD,f)  _CACHE_MAR_FSET(169,##FIELD,f)
N  #define _CACHE_MAR170_FSET(FIELD,f)  _CACHE_MAR_FSET(170,##FIELD,f)
N  #define _CACHE_MAR171_FSET(FIELD,f)  _CACHE_MAR_FSET(171,##FIELD,f)
N  #define _CACHE_MAR172_FSET(FIELD,f)  _CACHE_MAR_FSET(172,##FIELD,f)
N  #define _CACHE_MAR173_FSET(FIELD,f)  _CACHE_MAR_FSET(173,##FIELD,f)
N  #define _CACHE_MAR174_FSET(FIELD,f)  _CACHE_MAR_FSET(174,##FIELD,f)
N  #define _CACHE_MAR175_FSET(FIELD,f)  _CACHE_MAR_FSET(175,##FIELD,f)
N  #define _CACHE_MAR176_FSET(FIELD,f)  _CACHE_MAR_FSET(176,##FIELD,f)
N  #define _CACHE_MAR177_FSET(FIELD,f)  _CACHE_MAR_FSET(177,##FIELD,f)
N  #define _CACHE_MAR178_FSET(FIELD,f)  _CACHE_MAR_FSET(178,##FIELD,f)
N  #define _CACHE_MAR179_FSET(FIELD,f)  _CACHE_MAR_FSET(179,##FIELD,f)
N  #define _CACHE_MAR180_FSET(FIELD,f)  _CACHE_MAR_FSET(180,##FIELD,f)
N  #define _CACHE_MAR181_FSET(FIELD,f)  _CACHE_MAR_FSET(181,##FIELD,f)
N  #define _CACHE_MAR182_FSET(FIELD,f)  _CACHE_MAR_FSET(182,##FIELD,f)
N  #define _CACHE_MAR183_FSET(FIELD,f)  _CACHE_MAR_FSET(183,##FIELD,f)
N  #define _CACHE_MAR184_FSET(FIELD,f)  _CACHE_MAR_FSET(184,##FIELD,f)
N  #define _CACHE_MAR185_FSET(FIELD,f)  _CACHE_MAR_FSET(185,##FIELD,f)
N  #define _CACHE_MAR186_FSET(FIELD,f)  _CACHE_MAR_FSET(186,##FIELD,f)
N  #define _CACHE_MAR187_FSET(FIELD,f)  _CACHE_MAR_FSET(187,##FIELD,f)
N  #define _CACHE_MAR188_FSET(FIELD,f)  _CACHE_MAR_FSET(188,##FIELD,f)
N  #define _CACHE_MAR189_FSET(FIELD,f)  _CACHE_MAR_FSET(189,##FIELD,f)
N  #define _CACHE_MAR190_FSET(FIELD,f)  _CACHE_MAR_FSET(190,##FIELD,f)
N  #define _CACHE_MAR191_FSET(FIELD,f)  _CACHE_MAR_FSET(191,##FIELD,f)
N
N  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X  #if (0 | 0 | 0)
S    #define _CACHE_MAR96_FSETS(FIELD,SYM)   _CACHE_MAR_FSETS(96,##FIELD,##SYM)
S    #define _CACHE_MAR97_FSETS(FIELD,SYM)   _CACHE_MAR_FSETS(97,##FIELD,##SYM)
S    #define _CACHE_MAR98_FSETS(FIELD,SYM)   _CACHE_MAR_FSETS(98,##FIELD,##SYM)
S    #define _CACHE_MAR99_FSETS(FIELD,SYM)   _CACHE_MAR_FSETS(99,##FIELD,##SYM)
S    #define _CACHE_MAR100_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(100,##FIELD,##SYM)
S    #define _CACHE_MAR101_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(101,##FIELD,##SYM)
S    #define _CACHE_MAR102_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(102,##FIELD,##SYM)
S    #define _CACHE_MAR103_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(103,##FIELD,##SYM)
S    #define _CACHE_MAR104_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(104,##FIELD,##SYM)
S    #define _CACHE_MAR105_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(105,##FIELD,##SYM)
S    #define _CACHE_MAR106_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(106,##FIELD,##SYM)
S    #define _CACHE_MAR107_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(107,##FIELD,##SYM)
S    #define _CACHE_MAR108_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(108,##FIELD,##SYM)
S    #define _CACHE_MAR109_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(109,##FIELD,##SYM)
S    #define _CACHE_MAR110_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(110,##FIELD,##SYM)
S    #define _CACHE_MAR111_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(111,##FIELD,##SYM)
N  #endif
N
N  #define _CACHE_MAR128_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(128,##FIELD,##SYM)
N  #define _CACHE_MAR129_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(129,##FIELD,##SYM)
N  #define _CACHE_MAR130_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(130,##FIELD,##SYM)
N  #define _CACHE_MAR131_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(131,##FIELD,##SYM)
N  #define _CACHE_MAR132_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(132,##FIELD,##SYM)
N  #define _CACHE_MAR133_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(133,##FIELD,##SYM)
N  #define _CACHE_MAR134_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(134,##FIELD,##SYM)
N  #define _CACHE_MAR135_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(135,##FIELD,##SYM)
N  #define _CACHE_MAR136_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(136,##FIELD,##SYM)
N  #define _CACHE_MAR137_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(137,##FIELD,##SYM)
N  #define _CACHE_MAR138_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(138,##FIELD,##SYM)
N  #define _CACHE_MAR139_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(139,##FIELD,##SYM)
N  #define _CACHE_MAR140_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(140,##FIELD,##SYM)
N  #define _CACHE_MAR141_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(141,##FIELD,##SYM)
N  #define _CACHE_MAR142_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(142,##FIELD,##SYM)
N  #define _CACHE_MAR143_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(143,##FIELD,##SYM)
N  #define _CACHE_MAR144_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(144,##FIELD,##SYM)
N  #define _CACHE_MAR145_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(145,##FIELD,##SYM)
N  #define _CACHE_MAR146_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(146,##FIELD,##SYM)
N  #define _CACHE_MAR147_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(147,##FIELD,##SYM)
N  #define _CACHE_MAR148_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(148,##FIELD,##SYM)
N  #define _CACHE_MAR149_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(149,##FIELD,##SYM)
N  #define _CACHE_MAR150_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(150,##FIELD,##SYM)
N  #define _CACHE_MAR151_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(151,##FIELD,##SYM)
N  #define _CACHE_MAR152_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(152,##FIELD,##SYM)
N  #define _CACHE_MAR153_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(153,##FIELD,##SYM)
N  #define _CACHE_MAR154_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(154,##FIELD,##SYM)
N  #define _CACHE_MAR155_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(155,##FIELD,##SYM)
N  #define _CACHE_MAR156_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(156,##FIELD,##SYM)
N  #define _CACHE_MAR157_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(157,##FIELD,##SYM)
N  #define _CACHE_MAR158_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(158,##FIELD,##SYM)
N  #define _CACHE_MAR159_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(159,##FIELD,##SYM)
N  #define _CACHE_MAR160_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(160,##FIELD,##SYM)
N  #define _CACHE_MAR161_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(161,##FIELD,##SYM)
N  #define _CACHE_MAR162_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(162,##FIELD,##SYM)
N  #define _CACHE_MAR163_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(163,##FIELD,##SYM)
N  #define _CACHE_MAR164_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(164,##FIELD,##SYM)
N  #define _CACHE_MAR165_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(165,##FIELD,##SYM)
N  #define _CACHE_MAR166_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(166,##FIELD,##SYM)
N  #define _CACHE_MAR167_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(167,##FIELD,##SYM)
N  #define _CACHE_MAR168_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(168,##FIELD,##SYM)
N  #define _CACHE_MAR169_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(169,##FIELD,##SYM)
N  #define _CACHE_MAR170_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(170,##FIELD,##SYM)
N  #define _CACHE_MAR171_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(171,##FIELD,##SYM)
N  #define _CACHE_MAR172_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(172,##FIELD,##SYM)
N  #define _CACHE_MAR173_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(173,##FIELD,##SYM)
N  #define _CACHE_MAR174_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(174,##FIELD,##SYM)
N  #define _CACHE_MAR175_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(175,##FIELD,##SYM)
N  #define _CACHE_MAR176_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(176,##FIELD,##SYM)
N  #define _CACHE_MAR177_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(177,##FIELD,##SYM)
N  #define _CACHE_MAR178_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(178,##FIELD,##SYM)
N  #define _CACHE_MAR179_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(179,##FIELD,##SYM)
N  #define _CACHE_MAR180_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(180,##FIELD,##SYM)
N  #define _CACHE_MAR181_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(181,##FIELD,##SYM)
N  #define _CACHE_MAR182_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(182,##FIELD,##SYM)
N  #define _CACHE_MAR183_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(183,##FIELD,##SYM)
N  #define _CACHE_MAR184_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(184,##FIELD,##SYM)
N  #define _CACHE_MAR185_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(185,##FIELD,##SYM)
N  #define _CACHE_MAR186_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(186,##FIELD,##SYM)
N  #define _CACHE_MAR187_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(187,##FIELD,##SYM)
N  #define _CACHE_MAR188_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(188,##FIELD,##SYM)
N  #define _CACHE_MAR189_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(189,##FIELD,##SYM)
N  #define _CACHE_MAR190_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(190,##FIELD,##SYM)
N  #define _CACHE_MAR191_FSETS(FIELD,SYM)  _CACHE_MAR_FSETS(191,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 W B A R      |
N* |___________________|
N*
N* L2WBAR   - L2 writeback base address register
N*
N* Fields:
N* (rw) L2WBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WBAR_ADDR           0x01844000u
S
S  #define _CACHE_L2WBAR_L2WBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L2WBAR_L2WBAR_SHIFT   0x00000000u
S  #define  CACHE_L2WBAR_L2WBAR_DEFAULT 0x00000000u
S  #define  CACHE_L2WBAR_L2WBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L2WBAR_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2WBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WBAR,L2WBAR) \
S  )
X  #define CACHE_L2WBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WBAR,L2WBAR)   )
S
S  #define CACHE_L2WBAR_RMK(l2wbar) (Uint32)( \
S     _PER_FMK(CACHE,L2WBAR,L2WBAR,l2wbar) \
S  )
X  #define CACHE_L2WBAR_RMK(l2wbar) (Uint32)(      _PER_FMK(CACHE,L2WBAR,L2WBAR,l2wbar)   )
S
S  #define _CACHE_L2WBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD)
X  #define _CACHE_L2WBAR_FGET(FIELD)    _PER_FGET(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD)
S
S  #define _CACHE_L2WBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD,field)
X  #define _CACHE_L2WBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD,field)
S
S  #define _CACHE_L2WBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD,##SYM)
X  #define _CACHE_L2WBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WBAR_ADDR,CACHE,L2WBAR,##FIELD,##SYM)
N#endif
N
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 W W C        |
N* |___________________|
N*
N* L2WWC    - L2 writeback word count register
N*
N* Fields:
N* (rw) L2WWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WWC_ADDR           0x01844004u
S
S  #define _CACHE_L2WWC_L2WWC_MASK     0x0000FFFFu
S  #define _CACHE_L2WWC_L2WWC_SHIFT    0x00000000u
S  #define  CACHE_L2WWC_L2WWC_DEFAULT  0x00000000u
S  #define  CACHE_L2WWC_L2WWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L2WWC_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2WWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WWC,L2WWC) \
S  )
X  #define CACHE_L2WWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WWC,L2WWC)   )
S
S  #define CACHE_L2WWC_RMK(l2wwc) (Uint32)( \
S     _PER_FMK(CACHE,L2WWC,L2WWC,l2wwc) \
S  )
X  #define CACHE_L2WWC_RMK(l2wwc) (Uint32)(      _PER_FMK(CACHE,L2WWC,L2WWC,l2wwc)   )
S
S  #define _CACHE_L2WWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD)
X  #define _CACHE_L2WWC_FGET(FIELD)    _PER_FGET(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD)
S
S  #define _CACHE_L2WWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD,field)
X  #define _CACHE_L2WWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD,field)
S
S  #define _CACHE_L2WWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD,##SYM)
X  #define _CACHE_L2WWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WWC_ADDR,CACHE,L2WWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 W I B A R    |
N* |___________________|
N*
N* L2WIBAR  - L2 writeback-invalidate base address register
N*
N* Fields:
N* (rw) L2WIBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WIBAR_ADDR           0x01844010u
S
S  #define _CACHE_L2WIBAR_L2WIBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L2WIBAR_L2WIBAR_SHIFT   0x00000000u
S  #define  CACHE_L2WIBAR_L2WIBAR_DEFAULT 0x00000000u
S  #define  CACHE_L2WIBAR_L2WIBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L2WIBAR_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2WIBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WIBAR,L2WIBAR) \
S  )
X  #define CACHE_L2WIBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WIBAR,L2WIBAR)   )
S
S  #define CACHE_L2WIBAR_RMK(l2wibar) (Uint32)( \
S     _PER_FMK(CACHE,L2WIBAR,L2WIBAR,l2wibar) \
S  )
X  #define CACHE_L2WIBAR_RMK(l2wibar) (Uint32)(      _PER_FMK(CACHE,L2WIBAR,L2WIBAR,l2wibar)   )
S
S  #define _CACHE_L2WIBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD)
X  #define _CACHE_L2WIBAR_FGET(FIELD)    _PER_FGET(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD)
S
S  #define _CACHE_L2WIBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD,field)
X  #define _CACHE_L2WIBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD,field)
S
S  #define _CACHE_L2WIBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD,##SYM)
X  #define _CACHE_L2WIBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WIBAR_ADDR,CACHE,L2WIBAR,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 W I W C      |
N* |___________________|
N*
N* L2WIWC   - L2 writeback-invalidate word count register
N*
N* Fields:
N* (rw) L2WIWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WIWC_ADDR           0x01844014u
S
S  #define _CACHE_L2WIWC_L2WIWC_MASK     0x0000FFFFu
S  #define _CACHE_L2WIWC_L2WIWC_SHIFT    0x00000000u
S  #define  CACHE_L2WIWC_L2WIWC_DEFAULT  0x00000000u
S  #define  CACHE_L2WIWC_L2WIWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L2WIWC_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2WIWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WIWC,L2WIWC) \
S  )
X  #define CACHE_L2WIWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WIWC,L2WIWC)   )
S
S  #define CACHE_L2WIWC_RMK(l2wiwc) (Uint32)( \
S     _PER_FMK(CACHE,L2WIWC,L2WIWC,l2wiwc) \
S  )
X  #define CACHE_L2WIWC_RMK(l2wiwc) (Uint32)(      _PER_FMK(CACHE,L2WIWC,L2WIWC,l2wiwc)   )
S
S  #define _CACHE_L2WIWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD)
X  #define _CACHE_L2WIWC_FGET(FIELD)    _PER_FGET(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD)
S
S  #define _CACHE_L2WIWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD,field)
X  #define _CACHE_L2WIWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD,field)
S
S  #define _CACHE_L2WIWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD,##SYM)
X  #define _CACHE_L2WIWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WIWC_ADDR,CACHE,L2WIWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* ___________________
X
N* |                 |
N* |  L 2 I B A R    |
N* |_________________|
N*
N* L2IBAR  - L2 invalidate base address register
N*
N* Fields:
N* (rw) L2IBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2IBAR_ADDR           0x01844018u
S
S  #define _CACHE_L2IBAR_L2IBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L2IBAR_L2IBAR_SHIFT   0x00000000u
S  #define  CACHE_L2IBAR_L2IBAR_DEFAULT 0x00000000u
S  #define  CACHE_L2IBAR_L2IBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L2IBAR_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2IBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2IBAR,L2IBAR) \
S  )
X  #define CACHE_L2IBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2IBAR,L2IBAR)   )
S
S  #define CACHE_L2IBAR_RMK(l2ibar) (Uint32)( \
S     _PER_FMK(CACHE,L2IBAR,L2IBAR,l2ibar) \
S  )
X  #define CACHE_L2IBAR_RMK(l2ibar) (Uint32)(      _PER_FMK(CACHE,L2IBAR,L2IBAR,l2ibar)   )
S
S  #define _CACHE_L2IBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD)
X  #define _CACHE_L2IBAR_FGET(FIELD)    _PER_FGET(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD)
S
S  #define _CACHE_L2IBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD,field)
X  #define _CACHE_L2IBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD,field)
S
S  #define _CACHE_L2IBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD,##SYM)
X  #define _CACHE_L2IBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2IBAR_ADDR,CACHE,L2IBAR,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* ___________________
X
N* |                 |
N* |  L 2 I W C      |
N* |_________________|
N*
N* L2IWC   - L2 invalidate word count register
N*
N* Fields:
N* (rw) L2IWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2IWC_ADDR           0x0184401Cu
S
S  #define _CACHE_L2IWC_L2IWC_MASK     0x0000FFFFu
S  #define _CACHE_L2IWC_L2IWC_SHIFT    0x00000000u
S  #define  CACHE_L2IWC_L2IWC_DEFAULT  0x00000000u
S  #define  CACHE_L2IWC_L2IWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L2IWC_OF(x)          _VALUEOF(x)
S
S  #define CACHE_L2IWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2IWC,L2IWC) \
S  )
X  #define CACHE_L2IWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2IWC,L2IWC)   )
S
S  #define CACHE_L2IWC_RMK(l2iwc) (Uint32)( \
S     _PER_FMK(CACHE,L2IWC,L2IWC,l2iwc) \
S  )
X  #define CACHE_L2IWC_RMK(l2iwc) (Uint32)(      _PER_FMK(CACHE,L2IWC,L2IWC,l2iwc)   )
S
S  #define _CACHE_L2IWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD)
X  #define _CACHE_L2IWC_FGET(FIELD)    _PER_FGET(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD)
S
S  #define _CACHE_L2IWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD,field)
X  #define _CACHE_L2IWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD,field)
S
S  #define _CACHE_L2IWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD,##SYM)
X  #define _CACHE_L2IWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2IWC_ADDR,CACHE,L2IWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 P I B A R    |
N* |___________________|
N*
N* L1PIBAR  - L1P invalidate base address register
N*
N* Fields:
N* (rw) L1PIBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1PIBAR_ADDR            0x01844020u
S
S  #define _CACHE_L1PIBAR_L1PIBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L1PIBAR_L1PIBAR_SHIFT   0x00000000u
S  #define  CACHE_L1PIBAR_L1PIBAR_DEFAULT 0x00000000u
S  #define  CACHE_L1PIBAR_L1PIBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L1PIBAR_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1PIBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1PIBAR,L1PIBAR) \
S  )
X  #define CACHE_L1PIBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1PIBAR,L1PIBAR)   )
S
S  #define CACHE_L1PIBAR_RMK(l1pibar) (Uint32)( \
S     _PER_FMK(CACHE,L1PIBAR,L1PIBAR,l1pibar) \
S  )
X  #define CACHE_L1PIBAR_RMK(l1pibar) (Uint32)(      _PER_FMK(CACHE,L1PIBAR,L1PIBAR,l1pibar)   )
S
S  #define _CACHE_L1PIBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD)
X  #define _CACHE_L1PIBAR_FGET(FIELD)    _PER_FGET(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD)
S
S  #define _CACHE_L1PIBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD,field)
X  #define _CACHE_L1PIBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD,field)
S
S  #define _CACHE_L1PIBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD,##SYM)
X  #define _CACHE_L1PIBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1PIBAR_ADDR,CACHE,L1PIBAR,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 P I W C      |
N* |___________________|
N*
N* L1PIWC   - L1P invalidate word count register
N*
N* Fields:
N* (rw) L1PFWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1PIWC_ADDR            0x01844024u
S
S  #define _CACHE_L1PIWC_L1PIWC_MASK     0x0000FFFFu
S  #define _CACHE_L1PIWC_L1PIWC_SHIFT    0x00000000u
S  #define  CACHE_L1PIWC_L1PIWC_DEFAULT  0x00000000u
S  #define  CACHE_L1PIWC_L1PIWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L1PIWC_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1PIWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1PIWC,L1PIWC) \
S  )
X  #define CACHE_L1PIWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1PIWC,L1PIWC)   )
S
S  #define CACHE_L1PIWC_RMK(l1piwc) (Uint32)( \
S     _PER_FMK(CACHE,L1PIWC,L1PIWC,l1piwc) \
S  )
X  #define CACHE_L1PIWC_RMK(l1piwc) (Uint32)(      _PER_FMK(CACHE,L1PIWC,L1PIWC,l1piwc)   )
S
S  #define _CACHE_L1PIWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD)
X  #define _CACHE_L1PIWC_FGET(FIELD)    _PER_FGET(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD)
S
S  #define _CACHE_L1PIWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD,field)
X  #define _CACHE_L1PIWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD,field)
S
S  #define _CACHE_L1PIWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD,##SYM)
X  #define _CACHE_L1PIWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1PIWC_ADDR,CACHE,L1PIWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 D W I B A R  |
N* |___________________|
N*
N* L1DWIBAR - L1D writeback-invalidate base address register
N*
N* Fields:
N* (rw) L1DWIBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DWIBAR_ADDR            0x01844030u
S
S  #define _CACHE_L1DWIBAR_L1DWIBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L1DWIBAR_L1DWIBAR_SHIFT   0x00000000u
S  #define  CACHE_L1DWIBAR_L1DWIBAR_DEFAULT 0x00000000u
S  #define  CACHE_L1DWIBAR_L1DWIBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L1DWIBAR_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DWIBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DWIBAR,L1DWIBAR) \
S  )
X  #define CACHE_L1DWIBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DWIBAR,L1DWIBAR)   )
S
S  #define CACHE_L1DWIBAR_RMK(l1dwibar) (Uint32)( \
S     _PER_FMK(CACHE,L1DWIBAR,L1DWIBAR,l1dwibar) \
S  )
X  #define CACHE_L1DWIBAR_RMK(l1dwibar) (Uint32)(      _PER_FMK(CACHE,L1DWIBAR,L1DWIBAR,l1dwibar)   )
S
S  #define _CACHE_L1DWIBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD)
X  #define _CACHE_L1DWIBAR_FGET(FIELD)    _PER_FGET(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD)
S
S  #define _CACHE_L1DWIBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD,field)
X  #define _CACHE_L1DWIBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD,field)
S
S  #define _CACHE_L1DWIBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD,##SYM)
X  #define _CACHE_L1DWIBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DWIBAR_ADDR,CACHE,L1DWIBAR,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 1 D W I W C    |
N* |___________________|
N*
N* L1DWIWC  - L1D writeback-invalidate word count register
N*
N* Fields:
N* (rw) L1DWIWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DWIWC_ADDR            0x01844034u
S
S  #define _CACHE_L1DWIWC_L1DWIWC_MASK     0x0000FFFFu
S  #define _CACHE_L1DWIWC_L1DWIWC_SHIFT    0x00000000u
S  #define  CACHE_L1DWIWC_L1DWIWC_DEFAULT  0x00000000u
S  #define  CACHE_L1DWIWC_L1DWIWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L1DWIWC_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DWIWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DWIWC,L1DWIWC) \
S  )
X  #define CACHE_L1DWIWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DWIWC,L1DWIWC)   )
S
S  #define CACHE_L1DWIWC_RMK(l1dwiwc) (Uint32)( \
S     _PER_FMK(CACHE,L1DWIWC,L1DWIWC,l1dwiwc) \
S  )
X  #define CACHE_L1DWIWC_RMK(l1dwiwc) (Uint32)(      _PER_FMK(CACHE,L1DWIWC,L1DWIWC,l1dwiwc)   )
S
S  #define _CACHE_L1DWIWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD)
X  #define _CACHE_L1DWIWC_FGET(FIELD)    _PER_FGET(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD)
S
S  #define _CACHE_L1DWIWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD,field)
X  #define _CACHE_L1DWIWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD,field)
S
S  #define _CACHE_L1DWIWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD,##SYM)
X  #define _CACHE_L1DWIWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DWIWC_ADDR,CACHE,L1DWIWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* ___________________
X
N* |                 |
N* |  L 1 D I B A R  |
N* |_________________|
N*
N* L1DIBAR - L1D invalidate base address register
N*
N* Fields:
N* (rw) L1DIBAR
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DIBAR_ADDR            0x01844048u
S
S  #define _CACHE_L1DIBAR_L1DIBAR_MASK    0xFFFFFFFFu
S  #define _CACHE_L1DIBAR_L1DIBAR_SHIFT   0x00000000u
S  #define  CACHE_L1DIBAR_L1DIBAR_DEFAULT 0x00000000u
S  #define  CACHE_L1DIBAR_L1DIBAR_OF(x)   _VALUEOF(x)
S
S  #define  CACHE_L1DIBAR_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DIBAR_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DIBAR,L1DIBAR) \
S  )
X  #define CACHE_L1DIBAR_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DIBAR,L1DIBAR)   )
S
S  #define CACHE_L1DIBAR_RMK(l1dibar) (Uint32)( \
S     _PER_FMK(CACHE,L1DIBAR,L1DIBAR,l1dibar) \
S  )
X  #define CACHE_L1DIBAR_RMK(l1dibar) (Uint32)(      _PER_FMK(CACHE,L1DIBAR,L1DIBAR,l1dibar)   )
S
S  #define _CACHE_L1DIBAR_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD)
X  #define _CACHE_L1DIBAR_FGET(FIELD)    _PER_FGET(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD)
S
S  #define _CACHE_L1DIBAR_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD,field)
X  #define _CACHE_L1DIBAR_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD,field)
S
S  #define _CACHE_L1DIBAR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD,##SYM)
X  #define _CACHE_L1DIBAR_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DIBAR_ADDR,CACHE,L1DIBAR,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* ___________________
X
N* |                 |
N* |  L 1 D I W C    |
N* |_________________|
N*
N* L1DIWC  - L1D invalidate word count register
N*
N* Fields:
N* (rw) L1DIWC
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT && C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L1DIWC_ADDR            0x0184404Cu
S
S  #define _CACHE_L1DIWC_L1DIWC_MASK     0x0000FFFFu
S  #define _CACHE_L1DIWC_L1DIWC_SHIFT    0x00000000u
S  #define  CACHE_L1DIWC_L1DIWC_DEFAULT  0x00000000u
S  #define  CACHE_L1DIWC_L1DIWC_OF(x)    _VALUEOF(x)
S
S  #define  CACHE_L1DIWC_OF(x)           _VALUEOF(x)
S
S  #define CACHE_L1DIWC_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L1DIWC,L1DIWC) \
S  )
X  #define CACHE_L1DIWC_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L1DIWC,L1DIWC)   )
S
S  #define CACHE_L1DIWC_RMK(l1diwc) (Uint32)( \
S     _PER_FMK(CACHE,L1DIWC,L1DIWC,l1diwc) \
S  )
X  #define CACHE_L1DIWC_RMK(l1diwc) (Uint32)(      _PER_FMK(CACHE,L1DIWC,L1DIWC,l1diwc)   )
S
S  #define _CACHE_L1DIWC_FGET(FIELD)\
S    _PER_FGET(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD)
X  #define _CACHE_L1DIWC_FGET(FIELD)    _PER_FGET(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD)
S
S  #define _CACHE_L1DIWC_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD,field)
X  #define _CACHE_L1DIWC_FSET(FIELD,field)    _PER_FSET(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD,field)
S
S  #define _CACHE_L1DIWC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD,##SYM)
X  #define _CACHE_L1DIWC_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L1DIWC_ADDR,CACHE,L1DIWC,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _______________
X
N* |             |
N* |  L 2 W B    |
N* |_____________|
N*
N* L2WB     - L2 writeback all register
N*
N* Fields:
N* (rw) L2WB
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WB_ADDR      0x01845000u
S
S  #define _CACHE_L2WB_C_MASK    0x00000001u
S  #define _CACHE_L2WB_C_SHIFT   0x00000000u
S  #define  CACHE_L2WB_C_DEFAULT 0x00000000u
S  #define  CACHE_L2WB_C_OF(x)   _VALUEOF(x)
S  #define  CACHE_L2WB_C_NORMAL  0x00000000u
S  #define  CACHE_L2WB_C_FLUSH   0x00000001u
S
S  #define  CACHE_L2WB_OF(x)     _VALUEOF(x)
S
S  #define CACHE_L2WB_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WB,C) \
S  )
X  #define CACHE_L2WB_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WB,C)   )
S
S  #define CACHE_L2WB_RMK(c) (Uint32)( \
S     _PER_FMK(CACHE,L2WB,C,c) \
S  )
X  #define CACHE_L2WB_RMK(c) (Uint32)(      _PER_FMK(CACHE,L2WB,C,c)   )
S
S  #define _CACHE_L2WB_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD)
X  #define _CACHE_L2WB_FGET(FIELD)    _PER_FGET(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD)
S
S  #define _CACHE_L2WB_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD,field)
X  #define _CACHE_L2WB_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD,field)
S
S  #define _CACHE_L2WB_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD,##SYM)
X  #define _CACHE_L2WB_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WB_ADDR,CACHE,L2WB,##FIELD,##SYM)
N#endif
N
N/******************************************************************************\
N* _____________________
X
N* |                   |
N* |  L 2 W B I N V    |
N* |___________________|
N*
N* L2WBINV  - L2 writeback-invalidate all register
N*
N* Fields:
N* (rw) C
N*
N\******************************************************************************/
N#if (L2CACHE_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  #define _CACHE_L2WBINV_ADDR       0x01845004u
S
S  #define _CACHE_L2WBINV_C_MASK     0x00000001u
S  #define _CACHE_L2WBINV_C_SHIFT    0x00000000u
S  #define  CACHE_L2WBINV_C_DEFAULT  0x00000000u
S  #define  CACHE_L2WBINV_C_OF(x)    _VALUEOF(x)
S  #define  CACHE_L2WBINV_C_NORMAL   0x00000000u
S  #define  CACHE_L2WBINV_C_CLEAN    0x00000001u
S
S  #define  CACHE_L2WBINV_OF(x)      _VALUEOF(x)
S
S  #define CACHE_L2WBINV_DEFAULT (Uint32)( \
S     _PER_FDEFAULT(CACHE,L2WBINV,C) \
S  )
X  #define CACHE_L2WBINV_DEFAULT (Uint32)(      _PER_FDEFAULT(CACHE,L2WBINV,C)   )
S
S  #define CACHE_L2WBINV_RMK(c) (Uint32)( \
S     _PER_FMK(CACHE,L2WBINV,C,c) \
S  )
X  #define CACHE_L2WBINV_RMK(c) (Uint32)(      _PER_FMK(CACHE,L2WBINV,C,c)   )
S
S  #define _CACHE_L2WBINV_FGET(FIELD)\
S    _PER_FGET(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD)
X  #define _CACHE_L2WBINV_FGET(FIELD)    _PER_FGET(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD)
S
S  #define _CACHE_L2WBINV_FSET(FIELD,field)\
S    _PER_FSET(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD,field)
X  #define _CACHE_L2WBINV_FSET(FIELD,field)    _PER_FSET(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD,field)
S
S  #define _CACHE_L2WBINV_FSETS(FIELD,SYM)\
S    _PER_FSETS(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD,##SYM)
X  #define _CACHE_L2WBINV_FSETS(FIELD,SYM)    _PER_FSETS(_CACHE_L2WBINV_ADDR,CACHE,L2WBINV,##FIELD,##SYM)
N#endif
N
N/*----------------------------------------------------------------------------*/
N
N#endif /* CACHE_SUPPORT */
N#endif /* _CSL_CACHEHAL_H_ */
N/******************************************************************************\
N* End of csl_cachehal.h
X
N\******************************************************************************/
N
L 16 "..\csl\csl_cache.h" 2
N
N
N#if (CACHE_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N/******************************************************************************\
N* scope and inline control macros
X
N\******************************************************************************/
N#ifdef __cplusplus
S#define CSLAPI extern "C" far
N#else
N#define CSLAPI extern far
N#endif
N
N#undef  USEDEFS
N#undef  IDECL
N#undef  IDEF
N
N#ifdef  _CACHE_MOD_
N  #define IDECL CSLAPI
N  #define USEDEFS
N  #define IDEF
N#else
S  #ifdef  _INLINE
S    #define IDECL static inline
S    #define USEDEFS
S    #define IDEF  static inline
S  #else
S    #define IDECL CSLAPI
S  #endif
N#endif
N
N
N/******************************************************************************\
N* global macro declarations
X
N\******************************************************************************/
N#if (!C64_SUPPORT)
X#if (!( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N  #define CACHE_CE00   CACHE_ADDR(MAR0)
N  #define CACHE_CE01   CACHE_ADDR(MAR1)
N  #define CACHE_CE02   CACHE_ADDR(MAR2)
N  #define CACHE_CE03   CACHE_ADDR(MAR3)
N  #define CACHE_CE10   CACHE_ADDR(MAR4)
N  #define CACHE_CE11   CACHE_ADDR(MAR5)
N  #define CACHE_CE12   CACHE_ADDR(MAR6)
N  #define CACHE_CE13   CACHE_ADDR(MAR7)
N  #define CACHE_CE20   CACHE_ADDR(MAR8)
N  #define CACHE_CE21   CACHE_ADDR(MAR9)
N  #define CACHE_CE22   CACHE_ADDR(MAR10)
N  #define CACHE_CE23   CACHE_ADDR(MAR11)
N  #define CACHE_CE30   CACHE_ADDR(MAR12)
N  #define CACHE_CE31   CACHE_ADDR(MAR13)
N  #define CACHE_CE32   CACHE_ADDR(MAR14)
N  #define CACHE_CE33   CACHE_ADDR(MAR15)
N#else
S
S  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
S    
S     #define CACHE_EMIFB_CE00   CACHE_ADDR(MAR96)
S     #define CACHE_EMIFB_CE01   CACHE_ADDR(MAR97)
S     #define CACHE_EMIFB_CE02   CACHE_ADDR(MAR98)
S     #define CACHE_EMIFB_CE03   CACHE_ADDR(MAR99)
S     #define CACHE_EMIFB_CE10   CACHE_ADDR(MAR100)
S     #define CACHE_EMIFB_CE11   CACHE_ADDR(MAR101)
S     #define CACHE_EMIFB_CE12   CACHE_ADDR(MAR102)
S     #define CACHE_EMIFB_CE13   CACHE_ADDR(MAR103)
S     #define CACHE_EMIFB_CE20   CACHE_ADDR(MAR104)
S     #define CACHE_EMIFB_CE21   CACHE_ADDR(MAR105)
S     #define CACHE_EMIFB_CE22   CACHE_ADDR(MAR106)
S     #define CACHE_EMIFB_CE23   CACHE_ADDR(MAR107)
S     #define CACHE_EMIFB_CE30   CACHE_ADDR(MAR108)
S     #define CACHE_EMIFB_CE31   CACHE_ADDR(MAR109)
S     #define CACHE_EMIFB_CE32   CACHE_ADDR(MAR110)
S     #define CACHE_EMIFB_CE33   CACHE_ADDR(MAR111)
S
S  #endif
S
S  #define CACHE_EMIFA_CE00    CACHE_ADDR(MAR128)
S  #define CACHE_EMIFA_CE01    CACHE_ADDR(MAR129)
S  #define CACHE_EMIFA_CE02    CACHE_ADDR(MAR130)
S  #define CACHE_EMIFA_CE03    CACHE_ADDR(MAR131)
S  #define CACHE_EMIFA_CE04    CACHE_ADDR(MAR132)
S  #define CACHE_EMIFA_CE05    CACHE_ADDR(MAR133)
S  #define CACHE_EMIFA_CE06    CACHE_ADDR(MAR134)
S  #define CACHE_EMIFA_CE07    CACHE_ADDR(MAR135)
S  #define CACHE_EMIFA_CE08    CACHE_ADDR(MAR136)
S  #define CACHE_EMIFA_CE09    CACHE_ADDR(MAR137)
S  #define CACHE_EMIFA_CE010   CACHE_ADDR(MAR138)
S  #define CACHE_EMIFA_CE011   CACHE_ADDR(MAR139)
S  #define CACHE_EMIFA_CE012   CACHE_ADDR(MAR140)
S  #define CACHE_EMIFA_CE013   CACHE_ADDR(MAR141)
S  #define CACHE_EMIFA_CE014   CACHE_ADDR(MAR142)
S  #define CACHE_EMIFA_CE015   CACHE_ADDR(MAR143)
S
S  #define CACHE_EMIFA_CE10    CACHE_ADDR(MAR144)
S  #define CACHE_EMIFA_CE11    CACHE_ADDR(MAR145)
S  #define CACHE_EMIFA_CE12    CACHE_ADDR(MAR146)
S  #define CACHE_EMIFA_CE13    CACHE_ADDR(MAR147)
S  #define CACHE_EMIFA_CE14    CACHE_ADDR(MAR148)
S  #define CACHE_EMIFA_CE15    CACHE_ADDR(MAR149)
S  #define CACHE_EMIFA_CE16    CACHE_ADDR(MAR150)
S  #define CACHE_EMIFA_CE17    CACHE_ADDR(MAR151)
S  #define CACHE_EMIFA_CE18    CACHE_ADDR(MAR152)
S  #define CACHE_EMIFA_CE19    CACHE_ADDR(MAR153)
S  #define CACHE_EMIFA_CE110   CACHE_ADDR(MAR154)
S  #define CACHE_EMIFA_CE111   CACHE_ADDR(MAR155)
S  #define CACHE_EMIFA_CE112   CACHE_ADDR(MAR156)
S  #define CACHE_EMIFA_CE113   CACHE_ADDR(MAR157)
S  #define CACHE_EMIFA_CE114   CACHE_ADDR(MAR158)
S  #define CACHE_EMIFA_CE115   CACHE_ADDR(MAR159)
S
S  #define CACHE_EMIFA_CE20    CACHE_ADDR(MAR160)
S  #define CACHE_EMIFA_CE21    CACHE_ADDR(MAR161)
S  #define CACHE_EMIFA_CE22    CACHE_ADDR(MAR162)
S  #define CACHE_EMIFA_CE23    CACHE_ADDR(MAR163)
S  #define CACHE_EMIFA_CE24    CACHE_ADDR(MAR164)
S  #define CACHE_EMIFA_CE25    CACHE_ADDR(MAR165)
S  #define CACHE_EMIFA_CE26    CACHE_ADDR(MAR166)
S  #define CACHE_EMIFA_CE27    CACHE_ADDR(MAR167)
S  #define CACHE_EMIFA_CE28    CACHE_ADDR(MAR168)
S  #define CACHE_EMIFA_CE29    CACHE_ADDR(MAR169)
S  #define CACHE_EMIFA_CE210   CACHE_ADDR(MAR170)
S  #define CACHE_EMIFA_CE211   CACHE_ADDR(MAR171)
S  #define CACHE_EMIFA_CE212   CACHE_ADDR(MAR172)
S  #define CACHE_EMIFA_CE213   CACHE_ADDR(MAR173)
S  #define CACHE_EMIFA_CE214   CACHE_ADDR(MAR174)
S  #define CACHE_EMIFA_CE215   CACHE_ADDR(MAR175)
S
S  #define CACHE_EMIFA_CE30    CACHE_ADDR(MAR176)
S  #define CACHE_EMIFA_CE31    CACHE_ADDR(MAR177)
S  #define CACHE_EMIFA_CE32    CACHE_ADDR(MAR178)
S  #define CACHE_EMIFA_CE33    CACHE_ADDR(MAR179)
S  #define CACHE_EMIFA_CE34    CACHE_ADDR(MAR180)
S  #define CACHE_EMIFA_CE35    CACHE_ADDR(MAR181)
S  #define CACHE_EMIFA_CE36    CACHE_ADDR(MAR182)
S  #define CACHE_EMIFA_CE37    CACHE_ADDR(MAR183)
S  #define CACHE_EMIFA_CE38    CACHE_ADDR(MAR184)
S  #define CACHE_EMIFA_CE39    CACHE_ADDR(MAR185)
S  #define CACHE_EMIFA_CE310   CACHE_ADDR(MAR186)
S  #define CACHE_EMIFA_CE311   CACHE_ADDR(MAR187)
S  #define CACHE_EMIFA_CE312   CACHE_ADDR(MAR188)
S  #define CACHE_EMIFA_CE313   CACHE_ADDR(MAR189)
S  #define CACHE_EMIFA_CE314   CACHE_ADDR(MAR190)
S  #define CACHE_EMIFA_CE315   CACHE_ADDR(MAR191)
S
N#endif
N
N
N#if (C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S    #define CACHE_L2_LINESIZE    128
S    #define CACHE_L1D_LINESIZE    64
S    #define CACHE_L1P_LINESIZE    32
N#else
N    #define CACHE_L2_LINESIZE    128
N    #define CACHE_L1D_LINESIZE    32
N    #define CACHE_L1P_LINESIZE    64
N#endif /* C64_SUPPORT */
N
N//#define CACHE_ROUND_TO_LINESIZE(CACHE,ELCNT,ELSIZE)          \
N//        ((CACHE_##CACHE##_LINESIZE *                           \
N//         (( (((ELCNT)*(ELSIZE)) -1)/CACHE_##CACHE##_LINESIZE ) + 1))/ \
N//           (ELSIZE))
X
N
N#define CACHE_ROUND_TO_LINESIZE(CACHE,ELCNT,ELSIZE)\
N( ( ( ( (ELCNT) * (ELSIZE)\
N        + CACHE_##CACHE##_LINESIZE - 1\
N      ) / CACHE_##CACHE##_LINESIZE\
N      * CACHE_##CACHE##_LINESIZE\
N    ) + (ELSIZE) - 1\
N  ) / (ELSIZE)\
N)
X#define CACHE_ROUND_TO_LINESIZE(CACHE,ELCNT,ELSIZE)( ( ( ( (ELCNT) * (ELSIZE)        + CACHE_##CACHE##_LINESIZE - 1      ) / CACHE_##CACHE##_LINESIZE      * CACHE_##CACHE##_LINESIZE    ) + (ELSIZE) - 1  ) / (ELSIZE))
N
N/******************************************************************************\
N* global typedef declarations
X
N\******************************************************************************/
N#if(!C64_SUPPORT && !CHIP_6713 && !CHIP_DA610)
X#if(!( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && !0 && !0)
Ntypedef enum {
N  CACHE_64KSRAM  = 0,
N  CACHE_0KCACHE  = 0,
N  CACHE_48KSRAM  = 1,
N  CACHE_16KCACHE = 1,
N  CACHE_32KSRAM  = 2,
N  CACHE_32KCACHE = 2,
N  CACHE_16KSRAM  = 3,
N  CACHE_48KCACHE = 3,
N  CACHE_0KSRAM   = 7,
N  CACHE_64KCACHE = 7
N} CACHE_L2Mode;
N#endif
N
N#if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X#if (0 | 0 | 0)
Stypedef enum {
S  CACHE_1024KSRAM = 0,
S  CACHE_0KCACHE   = 0,
S  CACHE_992KSRAM  = 1,
S  CACHE_32KCACHE  = 1,
S  CACHE_960KSRAM  = 2,
S  CACHE_64KCACHE  = 2,
S  CACHE_896KSRAM  = 3,
S  CACHE_128KCACHE = 3,
S  CACHE_768KSRAM  = 7,
S  CACHE_256KCACHE = 7   /* 4-way cache 128Kbytes max */
S} CACHE_L2Mode;
N  #endif
N
N#if (CHIP_6713 || CHIP_DA610)
X#if (0 || 0)
S typedef enum {
S  CACHE_256KSRAM = 0,
S  CACHE_0KCACHE  = 0,
S  CACHE_240KSRAM = 1,
S  CACHE_16KCACHE = 1,
S  CACHE_224KSRAM = 2,
S  CACHE_32KCACHE = 2,
S  CACHE_208KSRAM = 3,
S  CACHE_48KCACHE = 3,
S  CACHE_192KSRAM = 7,
S  CACHE_64KCACHE = 7
S} CACHE_L2Mode;
N #endif
N
N#if (CHIP_DM642 | CHIP_6412 | CHIP_6411)
X#if (0 | 0 | 0)
S  typedef enum {
S  CACHE_256KSRAM  = 0,
S  CACHE_0KCACHE   = 0,
S  CACHE_224KSRAM  = 1,
S  CACHE_32KCACHE  = 1,
S  CACHE_192KSRAM  = 2,
S  CACHE_64KCACHE  = 2,
S  CACHE_128KSRAM  = 3,
S  CACHE_128KCACHE = 3,
S  CACHE_0KSRAM    = 7,
S  CACHE_256KCACHE = 7
S} CACHE_L2Mode;
N#endif
N
N/* Cache sizes for the DRI300 variants
N - 6418 : 512 K 
N - 6413 : 256 K
N - 6410 : 128 K
N*/
N
N#if (CHIP_6418)
X#if (0)
S  typedef enum {
S  CACHE_512KSRAM  = 0,
S  CACHE_0KCACHE   = 0,
S  CACHE_480KSRAM  = 1,
S  CACHE_32KCACHE  = 1,
S  CACHE_448KSRAM  = 2,
S  CACHE_64KCACHE  = 2,
S  CACHE_384KSRAM  = 3,
S  CACHE_128KCACHE = 3,
S  CACHE_256KSRAM  = 7,
S  CACHE_256KCACHE = 7
S} CACHE_L2Mode; 
N#endif
N
N#if (CHIP_6413)
X#if (0)
S  typedef enum {
S  CACHE_256KSRAM  = 0,
S  CACHE_0KCACHE   = 0,
S  CACHE_224KSRAM  = 1,
S  CACHE_32KCACHE  = 1,
S  CACHE_192KSRAM  = 2,
S  CACHE_64KCACHE  = 2,
S  CACHE_128KSRAM  = 3,
S  CACHE_128KCACHE = 3,
S  CACHE_256KCACHE = 7
S} CACHE_L2Mode; 
N#endif
N
N#if (CHIP_6410 | CHIP_DM641 | CHIP_DM640)
X#if (0 | 0 | 0)
S  typedef enum {
S  CACHE_128KSRAM  = 0,
S  CACHE_0KCACHE   = 0,
S  CACHE_96KSRAM   = 1,
S  CACHE_32KCACHE  = 1,
S  CACHE_64KSRAM   = 2,
S  CACHE_64KCACHE  = 2,
S  CACHE_128KCACHE = 3 /* All other modes are invalid */
S} CACHE_L2Mode; 
N#endif
N
Ntypedef enum {
N  CACHE_L2,
N  CACHE_L2ALL,
N  CACHE_L1P,
N  CACHE_L1PALL,
N  CACHE_L1D,
N  CACHE_L1DALL
N} CACHE_Region;
N
Ntypedef enum {
N  CACHE_PCC_MAPPED = 0,
N  CACHE_PCC_ENABLE = 2,
N  CACHE_PCC_FREEZE = 3,
N  CACHE_PCC_BYPASS = 4
N} CACHE_Pcc;
N
N/* Define macros for L2 priority Level */
N#define CACHE_L2PRIURG           0
N#define CACHE_L2PRIHIGH          1
N#define CACHE_L2PRIMED           2
N#define CACHE_L2PRILOW           3
N
N/* Define macros for L2 Queues  */
N#define CACHE_L2Q0              0
N#define CACHE_L2Q1              1
N#define CACHE_L2Q2              2
N#define CACHE_L2Q3              3
N
N/* Define CACHE wait flag */
Ntypedef enum {
N  CACHE_NOWAIT = 0,
N  CACHE_WAIT   = 1
N} CACHE_Wait;
N
N#define CACHE_WAIT_L2WB         0x00000001
N#define CACHE_WAIT_L2INV        0x00000002
N#define CACHE_WAIT_L2WBINV      0x00000004
N#define CACHE_WAIT_L2WBALL      0x00000008
N#define CACHE_WAIT_L2WBINVALL   0x00000010
N#define CACHE_WAIT_L1DINV       0x00000020
N#define CACHE_WAIT_L1DWBINV     0x00000040
N#define CACHE_WAIT_L1PINV       0x00000080
N
N/*  Renaming Function  */
N
N#define CACHE_resetEMIFA  CACHE_resetEmifa
N
N#if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X#if (0 | 0 | 0)
S  #define CACHE_resetEMIFB  CACHE_resetEmifb
N#endif
N
N/******************************************************************************\
N* global variable declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* global function declarations
X
N\******************************************************************************/
NCSLAPI void CACHE_reset();
Xextern far void CACHE_reset();
NCSLAPI void CACHE_resetEmifa();
Xextern far void CACHE_resetEmifa();
N
N#if (CHIP_6414 | CHIP_6415 | CHIP_6416)
X#if (0 | 0 | 0)
S  CSLAPI void CACHE_resetEmifb();
N#endif
N
NCSLAPI CACHE_L2Mode CACHE_setL2Mode(CACHE_L2Mode newMode);
Xextern far CACHE_L2Mode CACHE_setL2Mode(CACHE_L2Mode newMode);
NCSLAPI CACHE_L2Mode CACHE_getL2Mode();
Xextern far CACHE_L2Mode CACHE_getL2Mode();
NCSLAPI CACHE_Pcc CACHE_setPccMode(CACHE_Pcc newMode);
Xextern far CACHE_Pcc CACHE_setPccMode(CACHE_Pcc newMode);
NCSLAPI void CACHE_flush(CACHE_Region region,void *addr,Uint32 wordCnt);
Xextern far void CACHE_flush(CACHE_Region region,void *addr,Uint32 wordCnt);
NCSLAPI void CACHE_clean(CACHE_Region region,void *addr,Uint32 wordCnt);
Xextern far void CACHE_clean(CACHE_Region region,void *addr,Uint32 wordCnt);
NCSLAPI void CACHE_invalidate(CACHE_Region region,void *addr,Uint32 wordCnt);
Xextern far void CACHE_invalidate(CACHE_Region region,void *addr,Uint32 wordCnt);
NCSLAPI Uint32 CACHE_getL2SramSize();
Xextern far Uint32 CACHE_getL2SramSize();
N
N/* New API base on SPRU609 and SPRU610 */
NCSLAPI void CACHE_wbL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
Xextern far void CACHE_wbL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
NCSLAPI void CACHE_wbInvL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
Xextern far void CACHE_wbInvL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
NCSLAPI void CACHE_wbAllL2(CACHE_Wait wait);
Xextern far void CACHE_wbAllL2(CACHE_Wait wait);
NCSLAPI void CACHE_wbInvAllL2(CACHE_Wait wait);
Xextern far void CACHE_wbInvAllL2(CACHE_Wait wait);
N//CSLAPI void CACHE_invAllL1d();
NCSLAPI void CACHE_wbInvL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
Xextern far void CACHE_wbInvL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
NCSLAPI void CACHE_invL1p(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
Xextern far void CACHE_invL1p(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
NCSLAPI void CACHE_invAllL1p();
Xextern far void CACHE_invAllL1p();
NCSLAPI void CACHE_wait();
Xextern far void CACHE_wait();
N
N#if (C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
SCSLAPI void CACHE_invL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
SCSLAPI void CACHE_invL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait);
N#endif
N
N/* new APIs with  user-determined blocksize option */
NCSLAPI void CACHE_wbInvL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize);
Xextern far void CACHE_wbInvL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize);
NCSLAPI void CACHE_wbL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize);
Xextern far void CACHE_wbL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize);
N
N
N/*#if (C64_SUPPORT)
NCSLAPI void CACHE_freezeL1P();
NCSLAPI void CACHE_freezeL1D();
N#endif */
N
N/******************************************************************************\
N* inline function declarations
X
N\******************************************************************************/
NIDECL void CACHE_enableCaching(Uint32 block);
Xextern far void CACHE_enableCaching(Uint32 block);
N#if  (CHIP_6414 | CHIP_6415 | CHIP_6416| CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412 | CHIP_6411 | CHIP_6410 | CHIP_6413 | CHIP_6418)
X#if  (0 | 0 | 0| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0)
SIDECL void CACHE_setPriL2Req(Uint32 priority);
SIDECL void CACHE_setL2Queue(Uint32 queueNum,Uint32 length);
SIDECL void CACHE_resetL2Queue(Uint32 queueNum);
N#endif
N
N/******************************************************************************\
N* inline function definitions
X
N\******************************************************************************/
N#ifdef USEDEFS
N/*----------------------------------------------------------------------------*/
NIDEF void CACHE_enableCaching(Uint32 block) {
X void CACHE_enableCaching(Uint32 block) {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S    CACHE_FSETA(block,MAR,CE,1);
S    while (!CACHE_FGETA(block,MAR,CE))
S                   ;
N  #else
N    UNREFERENCED_PARAMETER(block);
X    ((block)=(block));
N  #endif
N}
N
N/*----------------------------------------------------------------------------*/
N#if (CHIP_6414 | CHIP_6415 | CHIP_6416| CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412 | CHIP_6411 | CHIP_6410 | CHIP_6413 | CHIP_6418)
X#if (0 | 0 | 0| 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0)
SIDEF void CACHE_setPriL2Req(Uint32 priority) {
S   CACHE_FSET(CCFG,P,priority);
S }
S/*----------------------------------------------------------------------------*/
SIDEF void CACHE_setL2Queue(Uint32 queueNum,Uint32 length) {
S  if (queueNum ==  0 ){
S     CACHE_FSET(L2ALLOC0,Q0CNT,length);
S  } else {
S     if (queueNum ==  1 ){
S        CACHE_FSET(L2ALLOC1,Q1CNT,length);
S     } else {
S        if (queueNum ==  2 ){
S           CACHE_FSET(L2ALLOC2,Q2CNT,length);
S        } else {
S           if (queueNum ==  3 ){
S              CACHE_FSET(L2ALLOC3,Q3CNT,length);
S           }
S        }
S     }
S  }
S}
S/*----------------------------------------------------------------------------*/
SIDEF void CACHE_resetL2Queue(Uint32 queueNum) {
S  if (queueNum ==  0 ) {
S    CACHE_FSET(L2ALLOC0,Q0CNT,6);
S  } else {
S     if (queueNum ==  1 ) { 
S        CACHE_FSET(L2ALLOC1,Q1CNT,2);
S     } else {
S        if (queueNum ==  2 ) {
S           CACHE_FSET(L2ALLOC2,Q2CNT,2);
S        } else {
S           if (queueNum ==  3 ) {
S              CACHE_FSET(L2ALLOC3,Q3CNT,2);
S           }
S        }
S     }
S  }
S}
N#endif
N/*----------------------------------------------------------------------------*/
N
N#endif /* USEDEFS */
N
N#endif /* CACHE_SUPPORT */
N#endif /* _CSL_CACHE_H_ */
N/******************************************************************************\
N* End of csl_cache.h
X
N\******************************************************************************/
L 50 "../csl/csl_cache.c" 2
N#include "csl_edma.h"
L 1 "..\csl\csl_edma.h" 1
N/******************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_edma.h
N* DATE CREATED.. 06/11/1999
N* LAST MODIFIED. 14/9/2005  Modified EDMA_getChannel and EDMA_setEvtPolarity to return 
N*							proper values and can set polarities properly respectively. 
N*				 08/13/2004 Modified tccAllocTable type from static to global fn.
N*                08/02/2004 - Adding support for C6418
N*                04/16/2004 Modified tccAllocTable[0] entry to 0.
N*                02/09/2004 Removed volatile variable type from EDMA_chain inline fn.
N*                06/17/2003 added support for 6712C
N*                05/28/2003 added support for 6711C
N*                02/15/2002 added EDMA channel events 6713/DM642 - EDMA_map()
N*                04/16/2001
N\******************************************************************************/
N#ifndef _CSL_EDMA_H_
N#define _CSL_EDMA_H_
N
N#include <csl_chip.h>
N#include <csl_irq.h>
N#include <csl_edmahal.h>
L 1 "E:/TI_workspase/c6701_t/csl/csl_edmahal.h" 1
N/*****************************************************************************\
N*           Copyright (C) 1999-2000 Texas Instruments Incorporated.
X
N*                           All Rights Reserved
N*------------------------------------------------------------------------------
N* FILENAME...... csl_edmahal.h
N* DATE CREATED.. 12 Jun 1999
N* LAST MODIFIED. 02 Aug 2004  Adding support for C6418
N*                17 Jun 2003  6712C
N*                28 May 2003  6711C
N*                22 Feb 2002  DM642
N*------------------------------------------------------------------------------
N* REGISTERS/PARAMETERS
N*
N* OPT    - options parameter
N* SRC    - source address parameter
N* CNT    - transfer count parameter
N* DST    - destination address parameter
N* IDX    - index parameter
N* RLD    - count reload + link parameter
N* QOPT   - QDMA options register
N* QSRC   - QDMA source address register
N* QCNT   - QDMA transfer count register
N* QDST   - QDMA destination address register
N* QIDX   - QDMA index register
N* QSOPT  - QDMA options pseudo register
N* QSSRC  - QDMA source address pseudo register
N* QSCNT  - QDMA transfer count pseudo register
N* QSDST  - QDMA destination address pseudo register
N* QSIDX  - QDMA index pseudo register
N* PQSR   - priority queue status register
N* PQAR0  - priority queue allocation register 0
N* PQAR1  - priority queue allocation register 1
N* PQAR2  - priority queue allocation register 2
N* PQAR3  - priority queue allocation register 3
N* CIPR   - channel interrupt pending register
N* CIPRL  - channel interrupt pending register, low half (1)
N* CIPRH  - channel interrupt pending register, high half (1)
N* CIER   - channel interrupt enable register
N* CIERL  - channel interrupt enable register, low half (1)
N* CIERH  - channel interrupt enable register, high half (1)
N* CCER   - channel chain enable register
N* CCERL  - channel chain enable register, low half (1)
N* CCERH  - channel chain enable register, high half (1)
N* ER     - event register
N* ERL    - event register, low half (1)
N* ERH    - event register, high half (1)
N* EER    - event enable register
N* EERL   - event enable register, low half (1)
N* EERH   - event enable register, high half (1)
N* EPRL   - event polarity register, low half (1)
N* EPRH   - event polarity register, high half (1)
N* ECR    - event clear register
N* ECRL   - event clear register, low half (1)
N* ECRH   - event clear register, high half (1)
N* ESR    - event set register
N* ESRL   - event set register, low half (1)
N* ESRH   - event set register, high half (1)
N*
N*
N* CHIP_6713, CHIP_DA610, CHIP_6711C and CHIP_6712C
N* ESEL0  - event selection register 0 (2)
N* ESEL1  - event selection register 1 (2)
N* ESEL2  - event selection register 2 (2) (3)
N* ESEL3  - event selection register 3 (2)
N*
N* (1) - only supported on C64x devices
N* (2) - only supported on C6713, DA610, 6711C and 6712C
N* (3) - the whole register is reserved
N\******************************************************************************/
N#ifndef _CSL_EDMAHAL_H
N#define _CSL_EDMAHAL_H_
N
N#include <csl_stdinc.h>
N#include <csl_chip.h>
N
N#if (EDMA_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S/******************************************************************************\
S* MISC section
X
S\******************************************************************************/
S
S#if (CHIP_6414 | CHIP_6415 | CHIP_6416 | CHIP_6411 )
S  #define _EDMA_CHA_CNT         64
S  #define _EDMA_BASE_PRAM       0x01A00000u
S  #define _EDMA_PRAM_START      _EDMA_BASE_PRAM
S  #define _EDMA_PRAM_SIZE       0x00000800u
S  #define _EDMA_PRAM_ERASE       0x00000600u
S#endif
S
S#if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640 | CHIP_6412 | CHIP_6410 | CHIP_6413 | CHIP_6418)
S  #define _EDMA_CHA_CNT         64
S  #define _EDMA_BASE_PRAM       0x01A00000u
S  #define _EDMA_PRAM_START      _EDMA_BASE_PRAM
S  #define _EDMA_PRAM_SIZE       0x00001400u
S  #define _EDMA_PRAM_ERASE      0x00000600u
S#endif
S
S#if (CHIP_6211 | CHIP_6711 | CHIP_6712 | CHIP_6713 | CHIP_DA610 | CHIP_6711C | CHIP_6712C)
S  #define _EDMA_CHA_CNT         16
S  #define _EDMA_BASE_PRAM       0x01A00000u
S  #define _EDMA_PRAM_START      _EDMA_BASE_PRAM
S  #define _EDMA_PRAM_SIZE       0x00000800u
S  #define _EDMA_PRAM_ERASE      0x00000180u
S#endif
S
S  #define _EDMA_ENTRY_SIZE      0x00000018u
S  #define _EDMA_NULL_PARAM      (_EDMA_PRAM_START+_EDMA_ENTRY_SIZE*_EDMA_CHA_CNT)
S  #define _EDMA_RSVD_PARAM      (_EDMA_NULL_PARAM+_EDMA_ENTRY_SIZE)
S  #define _EDMA_LINK_START      (_EDMA_RSVD_PARAM+_EDMA_ENTRY_SIZE)
S  #define _EDMA_LINK_CNT        ((_EDMA_PRAM_SIZE/_EDMA_ENTRY_SIZE)-(_EDMA_CHA_CNT+2))
S  #define _EDMA_SCRATCH_START   (_EDMA_LINK_START+_EDMA_LINK_CNT*_EDMA_ENTRY_SIZE)
S  #define _EDMA_SCRATCH_SIZE    (_EDMA_PRAM_START+_EDMA_PRAM_SIZE-_EDMA_SCRATCH_START)
S
S/******************************************************************************\
S* module level register/field access macros
X
S\******************************************************************************/
S
S  /* ----------------- */
S  /* FIELD MAKE MACROS */
S  /* ----------------- */
S
S  #define EDMA_FMK(REG,FIELD,x)\
S    _PER_FMK(EDMA,##REG,##FIELD,x)
X  #define EDMA_FMK(REG,FIELD,x)    _PER_FMK(EDMA,##REG,##FIELD,x)
S
S  #define EDMA_FMKS(REG,FIELD,SYM)\
S    _PER_FMKS(EDMA,##REG,##FIELD,##SYM)
X  #define EDMA_FMKS(REG,FIELD,SYM)    _PER_FMKS(EDMA,##REG,##FIELD,##SYM)
S
S
S  /* -------------------------------- */
S  /* RAW REGISTER/FIELD ACCESS MACROS */
S  /* -------------------------------- */
S
S  #define EDMA_REG(REG)  (*(volatile Uint32*)(_EDMA_##REG##_ADDR))
S
S  #define EDMA_ADDR(REG)\
S    _EDMA_##REG##_ADDR
X  #define EDMA_ADDR(REG)    _EDMA_##REG##_ADDR
S
S  #define EDMA_RGET(REG)\
S    _PER_RGET(_EDMA_##REG##_ADDR,EDMA,##REG)
X  #define EDMA_RGET(REG)    _PER_RGET(_EDMA_##REG##_ADDR,EDMA,##REG)
S
S  #define EDMA_RSET(REG,x)\
S    _PER_RSET(_EDMA_##REG##_ADDR,EDMA,##REG,x)
X  #define EDMA_RSET(REG,x)    _PER_RSET(_EDMA_##REG##_ADDR,EDMA,##REG,x)
S
S  #define EDMA_FGET(REG,FIELD)\
S    _EDMA_##REG##_FGET(##FIELD)
X  #define EDMA_FGET(REG,FIELD)    _EDMA_##REG##_FGET(##FIELD)
S
S  #define EDMA_FSET(REG,FIELD,x)\
S    _EDMA_##REG##_FSET(##FIELD,##x)
X  #define EDMA_FSET(REG,FIELD,x)    _EDMA_##REG##_FSET(##FIELD,##x)
S
S  #define EDMA_FSETS(REG,FIELD,SYM)\
S    _EDMA_##REG##_FSETS(##FIELD,##SYM)
X  #define EDMA_FSETS(REG,FIELD,SYM)    _EDMA_##REG##_FSETS(##FIELD,##SYM)
S
S
S  /* ------------------------------------------ */
S  /* ADDRESS BASED REGISTER/FIELD ACCESS MACROS */
S  /* ------------------------------------------ */
S
S  #define EDMA_RGETA(addr,REG)\
S    _PER_RGET(addr,EDMA,##REG)
X  #define EDMA_RGETA(addr,REG)    _PER_RGET(addr,EDMA,##REG)
S
S  #define EDMA_RSETA(addr,REG,x)\
S    _PER_RSET(addr,EDMA,##REG,x)
X  #define EDMA_RSETA(addr,REG,x)    _PER_RSET(addr,EDMA,##REG,x)
S
S  #define EDMA_FGETA(addr,REG,FIELD)\
S    _PER_FGET(addr,EDMA,##REG,##FIELD)
X  #define EDMA_FGETA(addr,REG,FIELD)    _PER_FGET(addr,EDMA,##REG,##FIELD)
S
S  #define EDMA_FSETA(addr,REG,FIELD,x)\
S    _PER_FSET(addr,EDMA,##REG,##FIELD,x)
X  #define EDMA_FSETA(addr,REG,FIELD,x)    _PER_FSET(addr,EDMA,##REG,##FIELD,x)
S
S  #define EDMA_FSETSA(addr,REG,FIELD,SYM)\
S    _PER_FSETS(addr,EDMA,##REG,##FIELD,##SYM)
X  #define EDMA_FSETSA(addr,REG,FIELD,SYM)    _PER_FSETS(addr,EDMA,##REG,##FIELD,##SYM)
S
S
S  /* ----------------------------------------- */
S  /* HANDLE BASED REGISTER/FIELD ACCESS MACROS */
S  /* ----------------------------------------- */
S
S  #define EDMA_ADDRH(h,REG)\
S    ((((Uint32)(h))&0x0000FFFF)+_EDMA_PRAM_START+(_EDMA_##REG##_OFFSET<<2))
X  #define EDMA_ADDRH(h,REG)    ((((Uint32)(h))&0x0000FFFF)+_EDMA_PRAM_START+(_EDMA_##REG##_OFFSET<<2))
S
S  #define EDMA_RGETH(h,REG)\
S    EDMA_RGETA(EDMA_ADDRH(h,##REG),##REG)
X  #define EDMA_RGETH(h,REG)    EDMA_RGETA(EDMA_ADDRH(h,##REG),##REG)
S
S
S  #define EDMA_RSETH(h,REG,x)\
S    EDMA_RSETA(EDMA_ADDRH(h,##REG),##REG,x)
X  #define EDMA_RSETH(h,REG,x)    EDMA_RSETA(EDMA_ADDRH(h,##REG),##REG,x)
S
S
S  #define EDMA_FGETH(h,REG,FIELD)\
S    EDMA_FGETA(EDMA_ADDRH(h,##REG),##REG,##FIELD)
X  #define EDMA_FGETH(h,REG,FIELD)    EDMA_FGETA(EDMA_ADDRH(h,##REG),##REG,##FIELD)
S
S
S  #define EDMA_FSETH(h,REG,FIELD,x)\
S    EDMA_FSETA(EDMA_ADDRH(h,##REG),##REG,##FIELD,x)
X  #define EDMA_FSETH(h,REG,FIELD,x)    EDMA_FSETA(EDMA_ADDRH(h,##REG),##REG,##FIELD,x)
S
S
S  #define EDMA_FSETSH(h,REG,FIELD,SYM)\
S    EDMA_FSETSA(EDMA_ADDRH(h,##REG),##REG,##FIELD,##SYM)
X  #define EDMA_FSETSH(h,REG,FIELD,SYM)    EDMA_FSETSA(EDMA_ADDRH(h,##REG),##REG,##FIELD,##SYM)
S
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  O P T            |
S* |  Q O P T          |
S* |  Q S O P T        |
S* |___________________|
S*
S* OPT    - options parameter
S* QOPT   - QDMA options register
S* QSOPT  - QDMA options pseudo register
S*
S* FIELDS (msb -> lsb)
S* (rw) PRI
S* (rw) ESIZE
S* (rw) 2DS
S* (rw) SUM
S* (rw) 2DD
S* (rw) DUM
S* (rw) TCINT
S* (rw) TCC
S* (rw) TCCM (1)
S* (rw) ATCINT (1)
S* (rw) ATCC (1)
S* (rw) PDTS (1)
S* (rw) PDTD (1)
S* (rw) LINK
S* (rw) FS
S*
S* (1) - only supported on C64x devices
S*
S\******************************************************************************/
S  #define _EDMA_OPT_OFFSET             0
S  #define _EDMA_QOPT_OFFSET            0
S  #define _EDMA_QSOPT_OFFSET           8
S
S  #define _EDMA_QOPT_ADDR              0x02000000u
S  #define _EDMA_QSOPT_ADDR             0x02000020u
S
S  #define  EDMA_QOPT                   EDMA_REG(QOPT)
S  #define  EDMA_QSOPT                  EDMA_REG(QSOPT)
S
S  #define _EDMA_OPT_PRI_MASK           0xE0000000u
S  #define _EDMA_OPT_PRI_SHIFT          0x0000001Du
S  #define  EDMA_OPT_PRI_DEFAULT        0x00000000u
S  #define  EDMA_OPT_PRI_OF(x)          _VALUEOF(x)
S  #if (C64_SUPPORT)
S    #define  EDMA_OPT_PRI_URGENT       0x00000000u
S    #define  EDMA_OPT_PRI_HIGH         0x00000001u
S    #define  EDMA_OPT_PRI_MEDIUM       0x00000002u
S    #define  EDMA_OPT_PRI_LOW          0x00000003u
S  #else
S    #define  EDMA_OPT_PRI_HIGH         0x00000001u
S    #define  EDMA_OPT_PRI_LOW          0x00000002u
S  #endif
S
S  #define _EDMA_OPT_ESIZE_MASK         0x18000000u
S  #define _EDMA_OPT_ESIZE_SHIFT        0x0000001Bu
S  #define  EDMA_OPT_ESIZE_DEFAULT      0x00000000u
S  #define  EDMA_OPT_ESIZE_OF(x)        _VALUEOF(x)
S  #define  EDMA_OPT_ESIZE_32BIT        0x00000000u
S  #define  EDMA_OPT_ESIZE_16BIT        0x00000001u
S  #define  EDMA_OPT_ESIZE_8BIT         0x00000002u
S
S  #define _EDMA_OPT_2DS_MASK           0x04000000u
S  #define _EDMA_OPT_2DS_SHIFT          0x0000001Au
S  #define  EDMA_OPT_2DS_DEFAULT        0x00000000u
S  #define  EDMA_OPT_2DS_OF(x)          _VALUEOF(x)
S  #define  EDMA_OPT_2DS_NO             0x00000000u
S  #define  EDMA_OPT_2DS_YES            0x00000001u
S
S  #define _EDMA_OPT_SUM_MASK           0x03000000u
S  #define _EDMA_OPT_SUM_SHIFT          0x00000018u
S  #define  EDMA_OPT_SUM_DEFAULT        0x00000000u
S  #define  EDMA_OPT_SUM_OF(x)          _VALUEOF(x)
S  #define  EDMA_OPT_SUM_NONE           0x00000000u
S  #define  EDMA_OPT_SUM_INC            0x00000001u
S  #define  EDMA_OPT_SUM_DEC            0x00000002u
S  #define  EDMA_OPT_SUM_IDX            0x00000003u
S
S  #define _EDMA_OPT_2DD_MASK           0x00800000u
S  #define _EDMA_OPT_2DD_SHIFT          0x00000017u
S  #define  EDMA_OPT_2DD_DEFAULT        0x00000000u
S  #define  EDMA_OPT_2DD_OF(x)          _VALUEOF(x)
S  #define  EDMA_OPT_2DD_NO             0x00000000u
S  #define  EDMA_OPT_2DD_YES            0x00000001u
S
S  #define _EDMA_OPT_DUM_MASK           0x00600000u
S  #define _EDMA_OPT_DUM_SHIFT          0x00000015u
S  #define  EDMA_OPT_DUM_DEFAULT        0x00000000u
S  #define  EDMA_OPT_DUM_OF(x)          _VALUEOF(x)
S  #define  EDMA_OPT_DUM_NONE           0x00000000u
S  #define  EDMA_OPT_DUM_INC            0x00000001u
S  #define  EDMA_OPT_DUM_DEC            0x00000002u
S  #define  EDMA_OPT_DUM_IDX            0x00000003u
S
S  #define _EDMA_OPT_TCINT_MASK         0x00100000u
S  #define _EDMA_OPT_TCINT_SHIFT        0x00000014u
S  #define  EDMA_OPT_TCINT_DEFAULT      0x00000000u
S  #define  EDMA_OPT_TCINT_OF(x)        _VALUEOF(x)
S  #define  EDMA_OPT_TCINT_NO           0x00000000u
S  #define  EDMA_OPT_TCINT_YES          0x00000001u
S
S  #define _EDMA_OPT_TCC_MASK           0x000F0000u
S  #define _EDMA_OPT_TCC_SHIFT          0x00000010u
S  #define  EDMA_OPT_TCC_DEFAULT        0x00000000u
S  #define  EDMA_OPT_TCC_OF(x)          _VALUEOF(x)
S
S#if (C64_SUPPORT)
S  #define _EDMA_OPT_TCCM_MASK          0x00006000u
S  #define _EDMA_OPT_TCCM_SHIFT         0x0000000Du
S  #define  EDMA_OPT_TCCM_DEFAULT       0x00000000u
S  #define  EDMA_OPT_TCCM_OF(x)         _VALUEOF(x)
S
S  #define _EDMA_OPT_ATCINT_MASK        0x00001000u
S  #define _EDMA_OPT_ATCINT_SHIFT       0x0000000Cu
S  #define  EDMA_OPT_ATCINT_DEFAULT     0x00000000u
S  #define  EDMA_OPT_ATCINT_OF(x)       _VALUEOF(x)
S  #define  EDMA_OPT_ATCINT_NO          0x00000000u
S  #define  EDMA_OPT_ATCINT_YES         0x00000001u
S
S  #define _EDMA_OPT_ATCC_MASK          0x000007E0u
S  #define _EDMA_OPT_ATCC_SHIFT         0x00000005u
S  #define  EDMA_OPT_ATCC_DEFAULT       0x00000000u
S  #define  EDMA_OPT_ATCC_OF(x)         _VALUEOF(x)
S
S  #define _EDMA_OPT_PDTS_MASK          0x00000008u
S  #define _EDMA_OPT_PDTS_SHIFT         0x00000003u
S  #define  EDMA_OPT_PDTS_DEFAULT       0x00000000u
S  #define  EDMA_OPT_PDTS_OF(x)         _VALUEOF(x)
S  #define  EDMA_OPT_PDTS_DISABLE       0x00000000u
S  #define  EDMA_OPT_PDTS_ENABLE        0x00000001u
S
S  #define _EDMA_OPT_PDTD_MASK          0x00000004u
S  #define _EDMA_OPT_PDTD_SHIFT         0x00000002u
S  #define  EDMA_OPT_PDTD_DEFAULT       0x00000000u
S  #define  EDMA_OPT_PDTD_OF(x)         _VALUEOF(x)
S  #define  EDMA_OPT_PDTD_DISABLE       0x00000000u
S  #define  EDMA_OPT_PDTD_ENABLE        0x00000001u
S#endif
S
S  #define _EDMA_OPT_LINK_MASK          0x00000002u
S  #define _EDMA_OPT_LINK_SHIFT         0x00000001u
S  #define  EDMA_OPT_LINK_DEFAULT       0x00000000u
S  #define  EDMA_OPT_LINK_OF(x)         _VALUEOF(x)
S  #define  EDMA_OPT_LINK_NA            0x00000000u
S  #define  EDMA_OPT_LINK_NO            0x00000000u
S  #define  EDMA_OPT_LINK_YES           0x00000001u
S
S  #define _EDMA_OPT_FS_MASK            0x00000001u
S  #define _EDMA_OPT_FS_SHIFT           0x00000000u
S  #define  EDMA_OPT_FS_DEFAULT         0x00000000u
S  #define  EDMA_OPT_FS_OF(x)           _VALUEOF(x)
S  #define  EDMA_OPT_FS_NO              0x00000000u
S  #define  EDMA_OPT_FS_YES             0x00000001u
S
S  #define  EDMA_OPT_OF(x)              _VALUEOF(x)
S
S#if (C64_SUPPORT)
S  #define EDMA_OPT_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,OPT,PRI)\
S    |_PER_FDEFAULT(EDMA,OPT,ESIZE)\
S    |_PER_FDEFAULT(EDMA,OPT,2DS)\
S    |_PER_FDEFAULT(EDMA,OPT,SUM)\
S    |_PER_FDEFAULT(EDMA,OPT,2DD)\
S    |_PER_FDEFAULT(EDMA,OPT,DUM)\
S    |_PER_FDEFAULT(EDMA,OPT,TCINT)\
S    |_PER_FDEFAULT(EDMA,OPT,TCC)\
S    |_PER_FDEFAULT(EDMA,OPT,TCCM)\
S    |_PER_FDEFAULT(EDMA,OPT,ATCINT) \
S    |_PER_FDEFAULT(EDMA,OPT,ATCC) \
S    |_PER_FDEFAULT(EDMA,OPT,PDTS) \
S    |_PER_FDEFAULT(EDMA,OPT,PDTD) \
S    |_PER_FDEFAULT(EDMA,OPT,LINK)\
S    |_PER_FDEFAULT(EDMA,OPT,FS)\
S  )
X  #define EDMA_OPT_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,OPT,PRI)    |_PER_FDEFAULT(EDMA,OPT,ESIZE)    |_PER_FDEFAULT(EDMA,OPT,2DS)    |_PER_FDEFAULT(EDMA,OPT,SUM)    |_PER_FDEFAULT(EDMA,OPT,2DD)    |_PER_FDEFAULT(EDMA,OPT,DUM)    |_PER_FDEFAULT(EDMA,OPT,TCINT)    |_PER_FDEFAULT(EDMA,OPT,TCC)    |_PER_FDEFAULT(EDMA,OPT,TCCM)    |_PER_FDEFAULT(EDMA,OPT,ATCINT)     |_PER_FDEFAULT(EDMA,OPT,ATCC)     |_PER_FDEFAULT(EDMA,OPT,PDTS)     |_PER_FDEFAULT(EDMA,OPT,PDTD)     |_PER_FDEFAULT(EDMA,OPT,LINK)    |_PER_FDEFAULT(EDMA,OPT,FS)  )
S
S  #define EDMA_OPT_RMK(pri,esize,ds2,sum,dd2,dum,tcint,tcc,tccm,atcint,atcc,\
S    pdts,pdtd,link,fs) (Uint32)(\
S     _PER_FMK(EDMA,OPT,PRI,pri) \
S    |_PER_FMK(EDMA,OPT,ESIZE,esize) \
S    |_PER_FMK(EDMA,OPT,2DS,ds2) \
S    |_PER_FMK(EDMA,OPT,SUM,sum) \
S    |_PER_FMK(EDMA,OPT,2DD,dd2) \
S    |_PER_FMK(EDMA,OPT,DUM,dum) \
S    |_PER_FMK(EDMA,OPT,TCINT,tcint) \
S    |_PER_FMK(EDMA,OPT,TCC,tcc) \
S    |_PER_FMK(EDMA,OPT,TCCM,tccm) \
S    |_PER_FMK(EDMA,OPT,ATCINT,atcint) \
S    |_PER_FMK(EDMA,OPT,ATCC,atcc) \
S    |_PER_FMK(EDMA,OPT,PDTS,pdts) \
S    |_PER_FMK(EDMA,OPT,PDTD,pdtd) \
S    |_PER_FMK(EDMA,OPT,LINK,link) \
S    |_PER_FMK(EDMA,OPT,FS,fs) \
S  )
X  #define EDMA_OPT_RMK(pri,esize,ds2,sum,dd2,dum,tcint,tcc,tccm,atcint,atcc,    pdts,pdtd,link,fs) (Uint32)(     _PER_FMK(EDMA,OPT,PRI,pri)     |_PER_FMK(EDMA,OPT,ESIZE,esize)     |_PER_FMK(EDMA,OPT,2DS,ds2)     |_PER_FMK(EDMA,OPT,SUM,sum)     |_PER_FMK(EDMA,OPT,2DD,dd2)     |_PER_FMK(EDMA,OPT,DUM,dum)     |_PER_FMK(EDMA,OPT,TCINT,tcint)     |_PER_FMK(EDMA,OPT,TCC,tcc)     |_PER_FMK(EDMA,OPT,TCCM,tccm)     |_PER_FMK(EDMA,OPT,ATCINT,atcint)     |_PER_FMK(EDMA,OPT,ATCC,atcc)     |_PER_FMK(EDMA,OPT,PDTS,pdts)     |_PER_FMK(EDMA,OPT,PDTD,pdtd)     |_PER_FMK(EDMA,OPT,LINK,link)     |_PER_FMK(EDMA,OPT,FS,fs)   )
S#endif
S
S#if (!C64_SUPPORT)
S  #define EDMA_OPT_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,OPT,PRI)\
S    |_PER_FDEFAULT(EDMA,OPT,ESIZE)\
S    |_PER_FDEFAULT(EDMA,OPT,2DS)\
S    |_PER_FDEFAULT(EDMA,OPT,SUM)\
S    |_PER_FDEFAULT(EDMA,OPT,2DD)\
S    |_PER_FDEFAULT(EDMA,OPT,DUM)\
S    |_PER_FDEFAULT(EDMA,OPT,TCINT)\
S    |_PER_FDEFAULT(EDMA,OPT,TCC)\
S    |_PER_FDEFAULT(EDMA,OPT,LINK)\
S    |_PER_FDEFAULT(EDMA,OPT,FS)\
S  )
X  #define EDMA_OPT_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,OPT,PRI)    |_PER_FDEFAULT(EDMA,OPT,ESIZE)    |_PER_FDEFAULT(EDMA,OPT,2DS)    |_PER_FDEFAULT(EDMA,OPT,SUM)    |_PER_FDEFAULT(EDMA,OPT,2DD)    |_PER_FDEFAULT(EDMA,OPT,DUM)    |_PER_FDEFAULT(EDMA,OPT,TCINT)    |_PER_FDEFAULT(EDMA,OPT,TCC)    |_PER_FDEFAULT(EDMA,OPT,LINK)    |_PER_FDEFAULT(EDMA,OPT,FS)  )
S
S  #define EDMA_OPT_RMK(pri,esize,ds2,sum,dd2,dum,tcint,tcc,link,fs) (Uint32)(\
S     _PER_FMK(EDMA,OPT,PRI,pri) \
S    |_PER_FMK(EDMA,OPT,ESIZE,esize) \
S    |_PER_FMK(EDMA,OPT,2DS,ds2) \
S    |_PER_FMK(EDMA,OPT,SUM,sum) \
S    |_PER_FMK(EDMA,OPT,2DD,dd2) \
S    |_PER_FMK(EDMA,OPT,DUM,dum) \
S    |_PER_FMK(EDMA,OPT,TCINT,tcint) \
S    |_PER_FMK(EDMA,OPT,TCC,tcc) \
S    |_PER_FMK(EDMA,OPT,LINK,link) \
S    |_PER_FMK(EDMA,OPT,FS,fs) \
S  )
X  #define EDMA_OPT_RMK(pri,esize,ds2,sum,dd2,dum,tcint,tcc,link,fs) (Uint32)(     _PER_FMK(EDMA,OPT,PRI,pri)     |_PER_FMK(EDMA,OPT,ESIZE,esize)     |_PER_FMK(EDMA,OPT,2DS,ds2)     |_PER_FMK(EDMA,OPT,SUM,sum)     |_PER_FMK(EDMA,OPT,2DD,dd2)     |_PER_FMK(EDMA,OPT,DUM,dum)     |_PER_FMK(EDMA,OPT,TCINT,tcint)     |_PER_FMK(EDMA,OPT,TCC,tcc)     |_PER_FMK(EDMA,OPT,LINK,link)     |_PER_FMK(EDMA,OPT,FS,fs)   )
S#endif
S
S  #define _EDMA_QOPT_FGET(FIELD)\
S    _PER_FGET(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD)
X  #define _EDMA_QOPT_FGET(FIELD)    _PER_FGET(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD)
S
S  #define _EDMA_QOPT_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD,field)
X  #define _EDMA_QOPT_FSET(FIELD,field)    _PER_FSET(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD,field)
S
S  #define _EDMA_QOPT_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD,##SYM)
X  #define _EDMA_QOPT_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QOPT_ADDR,EDMA,OPT,##FIELD,##SYM)
S
S  #define _EDMA_QSOPT_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD)
X  #define _EDMA_QSOPT_FGET(FIELD)    _PER_FGET(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD)
S
S  #define _EDMA_QSOPT_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD,field)
X  #define _EDMA_QSOPT_FSET(FIELD,field)    _PER_FSET(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD,field)
S
S  #define _EDMA_QSOPT_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD,##SYM)
X  #define _EDMA_QSOPT_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSOPT_ADDR,EDMA,OPT,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  S R C            |
S* |  Q S R C          |
S* |  Q S S R C        |
S* |___________________|
S*
S* SRC    - source address parameter
S* QSRC   - QDMA source address register
S* QSSRC  - QDMA source address pseudo register
S*
S* FIELDS (msb -> lsb)
S* (rw) SRC
S*
S\******************************************************************************/
S  #define _EDMA_SRC_OFFSET             1
S  #define _EDMA_QSRC_OFFSET            1
S  #define _EDMA_QSSRC_OFFSET           9
S
S  #define _EDMA_QSRC_ADDR              0x02000004u
S  #define _EDMA_QSSRC_ADDR             0x02000024u
S
S  #define  EDMA_QSRC                   EDMA_REG(QSRC)
S  #define  EDMA_QSSRC                  EDMA_REG(QSSRC)
S
S  #define _EDMA_SRC_SRC_MASK           0xFFFFFFFFu
S  #define _EDMA_SRC_SRC_SHIFT          0x00000000u
S  #define  EDMA_SRC_SRC_DEFAULT        0x00000000u
S  #define  EDMA_SRC_SRC_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_SRC_OF(x)              _VALUEOF(x)
S
S  #define EDMA_SRC_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,SRC,SRC)\
S  )
X  #define EDMA_SRC_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,SRC,SRC)  )
S
S  #define EDMA_SRC_RMK(src) (Uint32)(\
S     _PER_FMK(EDMA,SRC,SRC,src) \
S  )
X  #define EDMA_SRC_RMK(src) (Uint32)(     _PER_FMK(EDMA,SRC,SRC,src)   )
S
S  #define _EDMA_QSRC_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD)
X  #define _EDMA_QSRC_FGET(FIELD)    _PER_FGET(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD)
S
S  #define _EDMA_QSRC_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD,field)
X  #define _EDMA_QSRC_FSET(FIELD,field)    _PER_FSET(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD,field)
S
S  #define _EDMA_QSRC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD,##SYM)
X  #define _EDMA_QSRC_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSRC_ADDR,EDMA,SRC,##FIELD,##SYM)
S
S  #define _EDMA_QSSRC_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD)
X  #define _EDMA_QSSRC_FGET(FIELD)    _PER_FGET(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD)
S
S  #define _EDMA_QSSRC_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD,field)
X  #define _EDMA_QSSRC_FSET(FIELD,field)    _PER_FSET(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD,field)
S
S  #define _EDMA_QSSRC_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD,##SYM)
X  #define _EDMA_QSSRC_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSSRC_ADDR,EDMA,SRC,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C N T            |
S* |  Q C N T          |
S* |  Q S C N T        |
S* |___________________|
S*
S* CNT    - transfer count parameter
S* QCNT   - QDMA transfer count register
S* QSCNT  - QDMA transfer count pseudo register
S*
S* FIELDS (msb -> lsb)
S* (rw) FRMCNT
S* (rw) ELECNT
S*
S\******************************************************************************/
S  #define _EDMA_CNT_OFFSET             2
S  #define _EDMA_QCNT_OFFSET            2
S  #define _EDMA_QSCNT_OFFSET           10
S
S  #define _EDMA_QCNT_ADDR              0x02000008u
S  #define _EDMA_QSCNT_ADDR             0x02000028u
S
S  #define  EDMA_QCNT                   EDMA_REG(QCNT)
S  #define  EDMA_QSCNT                  EDMA_REG(QSCNT)
S
S  #define _EDMA_CNT_FRMCNT_MASK        0xFFFF0000u
S  #define _EDMA_CNT_FRMCNT_SHIFT       0x00000010u
S  #define  EDMA_CNT_FRMCNT_DEFAULT     0x00000000u
S  #define  EDMA_CNT_FRMCNT_OF(x)       _VALUEOF(x)
S
S  #define _EDMA_CNT_ELECNT_MASK        0x0000FFFFu
S  #define _EDMA_CNT_ELECNT_SHIFT       0x00000000u
S  #define  EDMA_CNT_ELECNT_DEFAULT     0x00000000u
S  #define  EDMA_CNT_ELECNT_OF(x)       _VALUEOF(x)
S
S  #define  EDMA_CNT_OF(x)              _VALUEOF(x)
S
S  #define EDMA_CNT_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CNT,FRMCNT)\
S    |_PER_FDEFAULT(EDMA,CNT,ELECNT)\
S  )
X  #define EDMA_CNT_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CNT,FRMCNT)    |_PER_FDEFAULT(EDMA,CNT,ELECNT)  )
S
S  #define EDMA_CNT_RMK(frmcnt,elecnt) (Uint32)(\
S     _PER_FMK(EDMA,CNT,FRMCNT,frmcnt) \
S    |_PER_FMK(EDMA,CNT,ELECNT,elecnt) \
S  )
X  #define EDMA_CNT_RMK(frmcnt,elecnt) (Uint32)(     _PER_FMK(EDMA,CNT,FRMCNT,frmcnt)     |_PER_FMK(EDMA,CNT,ELECNT,elecnt)   )
S
S  #define _EDMA_QCNT_FGET(FIELD)\
S    _PER_FGET(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD)
X  #define _EDMA_QCNT_FGET(FIELD)    _PER_FGET(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD)
S
S  #define _EDMA_QCNT_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD,field)
X  #define _EDMA_QCNT_FSET(FIELD,field)    _PER_FSET(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD,field)
S
S  #define _EDMA_QCNT_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD,##SYM)
X  #define _EDMA_QCNT_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QCNT_ADDR,EDMA,CNT,##FIELD,##SYM)
S
S  #define _EDMA_QSCNT_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD)
X  #define _EDMA_QSCNT_FGET(FIELD)    _PER_FGET(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD)
S
S  #define _EDMA_QSCNT_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD,field)
X  #define _EDMA_QSCNT_FSET(FIELD,field)    _PER_FSET(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD,field)
S
S  #define _EDMA_QSCNT_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD,##SYM)
X  #define _EDMA_QSCNT_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSCNT_ADDR,EDMA,CNT,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  D S T            |
S* |  Q D S T          |
S* |  Q S D S T        |
S* |___________________|
S*
S* DST    - destination address parameter
S* QDST   - QDMA destination address register
S* QSDST  - QDMA destination address pseudo register
S*
S* FIELDS (msb -> lsb)
S* (rw) DST
S*
S\******************************************************************************/
S  #define _EDMA_DST_OFFSET             3
S  #define _EDMA_QDST_OFFSET            3
S  #define _EDMA_QSDST_OFFSET           11
S
S  #define _EDMA_QDST_ADDR              0x0200000Cu
S  #define _EDMA_QSDST_ADDR             0x0200002Cu
S
S  #define  EDMA_QDST                   EDMA_REG(QDST)
S  #define  EDMA_QSDST                  EDMA_REG(QSDST)
S
S  #define _EDMA_DST_DST_MASK           0xFFFFFFFFu
S  #define _EDMA_DST_DST_SHIFT          0x00000000u
S  #define  EDMA_DST_DST_DEFAULT        0x00000000u
S  #define  EDMA_DST_DST_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_DST_OF(x)              _VALUEOF(x)
S
S  #define EDMA_DST_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,DST,DST)\
S  )
X  #define EDMA_DST_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,DST,DST)  )
S
S  #define EDMA_DST_RMK(dst) (Uint32)(\
S     _PER_FMK(EDMA,DST,DST,dst) \
S  )
X  #define EDMA_DST_RMK(dst) (Uint32)(     _PER_FMK(EDMA,DST,DST,dst)   )
S
S  #define _EDMA_QDST_FGET(FIELD)\
S    _PER_FGET(_EDMA_QDST_ADDR,EDMA,DST,##FIELD)
X  #define _EDMA_QDST_FGET(FIELD)    _PER_FGET(_EDMA_QDST_ADDR,EDMA,DST,##FIELD)
S
S  #define _EDMA_QDST_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QDST_ADDR,EDMA,DST,##FIELD,field)
X  #define _EDMA_QDST_FSET(FIELD,field)    _PER_FSET(_EDMA_QDST_ADDR,EDMA,DST,##FIELD,field)
S
S  #define _EDMA_QDST_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QDST_ADDR,EDMA,DST,##FIELD,##SYM)
X  #define _EDMA_QDST_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QDST_ADDR,EDMA,DST,##FIELD,##SYM)
S
S  #define _EDMA_QSDST_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD)
X  #define _EDMA_QSDST_FGET(FIELD)    _PER_FGET(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD)
S
S  #define _EDMA_QSDST_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD,field)
X  #define _EDMA_QSDST_FSET(FIELD,field)    _PER_FSET(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD,field)
S
S  #define _EDMA_QSDST_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD,##SYM)
X  #define _EDMA_QSDST_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSDST_ADDR,EDMA,DST,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  I D X            |
S* |  Q I D X          |
S* |  Q S I D X        |
S* |___________________|
S*
S* IDX    - index parameter
S* QIDX   - QDMA index register
S* QSIDX  - QDMA index pseudo register
S*
S* FIELDS (msb -> lsb)
S* (rw) FRMIDX
S* (rw) ELEIDX
S*
S\******************************************************************************/
S  #define _EDMA_IDX_OFFSET             4
S  #define _EDMA_QIDX_OFFSET            4
S  #define _EDMA_QSIDX_OFFSET           12
S
S  #define _EDMA_QIDX_ADDR              0x02000010u
S  #define _EDMA_QSIDX_ADDR             0x02000030u
S
S  #define  EDMA_QIDX                   EDMA_REG(QIDX)
S  #define  EDMA_QSIDX                  EDMA_REG(QSIDX)
S
S  #define _EDMA_IDX_FRMIDX_MASK        0xFFFF0000u
S  #define _EDMA_IDX_FRMIDX_SHIFT       0x00000010u
S  #define  EDMA_IDX_FRMIDX_DEFAULT     0x00000000u
S  #define  EDMA_IDX_FRMIDX_OF(x)       _VALUEOF(x)
S
S  #define _EDMA_IDX_ELEIDX_MASK        0x0000FFFFu
S  #define _EDMA_IDX_ELEIDX_SHIFT       0x00000000u
S  #define  EDMA_IDX_ELEIDX_DEFAULT     0x00000000u
S  #define  EDMA_IDX_ELEIDX_OF(x)       _VALUEOF(x)
S
S  #define  EDMA_IDX_OF(x)              _VALUEOF(x)
S
S  #define EDMA_IDX_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,IDX,FRMIDX)\
S    |_PER_FDEFAULT(EDMA,IDX,ELEIDX)\
S  )
X  #define EDMA_IDX_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,IDX,FRMIDX)    |_PER_FDEFAULT(EDMA,IDX,ELEIDX)  )
S
S  #define EDMA_IDX_RMK(frmidx,eleidx) (Uint32)(\
S     _PER_FMK(EDMA,IDX,FRMIDX,frmidx) \
S    |_PER_FMK(EDMA,IDX,ELEIDX,eleidx) \
S  )
X  #define EDMA_IDX_RMK(frmidx,eleidx) (Uint32)(     _PER_FMK(EDMA,IDX,FRMIDX,frmidx)     |_PER_FMK(EDMA,IDX,ELEIDX,eleidx)   )
S
S  #define _EDMA_QIDX_FGET(FIELD)\
S    _PER_FGET(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD)
X  #define _EDMA_QIDX_FGET(FIELD)    _PER_FGET(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD)
S
S  #define _EDMA_QIDX_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD,field)
X  #define _EDMA_QIDX_FSET(FIELD,field)    _PER_FSET(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD,field)
S
S  #define _EDMA_QIDX_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD,##SYM)
X  #define _EDMA_QIDX_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QIDX_ADDR,EDMA,IDX,##FIELD,##SYM)
S
S  #define _EDMA_QSIDX_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD)
X  #define _EDMA_QSIDX_FGET(FIELD)    _PER_FGET(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD)
S
S  #define _EDMA_QSIDX_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD,field)
X  #define _EDMA_QSIDX_FSET(FIELD,field)    _PER_FSET(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD,field)
S
S  #define _EDMA_QSIDX_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD,##SYM)
X  #define _EDMA_QSIDX_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSIDX_ADDR,EDMA,IDX,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  R L D            |
S* |  Q R L D          |
S* |  Q S R L D        |
S* |___________________|
S*
S* RLD - count reload/link parameter
S*
S* FIELDS (msb -> lsb)
S* (rw) ELECNT
S* (rw) LINK
S*
S\******************************************************************************/
S  #define _EDMA_RLD_OFFSET             5
S  #define _EDMA_QRLD_OFFSET            5
S  #define _EDMA_QSRLD_OFFSET           13
S
S  #define _EDMA_QRLD_ADDR              0x02000014u
S  #define _EDMA_QSRLD_ADDR             0x02000034u
S
S  #define  EDMA_QRLD                   EDMA_REG(QRLD)
S  #define  EDMA_QSRLD                  EDMA_REG(QSRLD)
S
S  #define _EDMA_RLD_ELERLD_MASK        0xFFFF0000u
S  #define _EDMA_RLD_ELERLD_SHIFT       0x00000010u
S  #define  EDMA_RLD_ELERLD_DEFAULT     0x00000000u
S  #define  EDMA_RLD_ELERLD_OF(x)       _VALUEOF(x)
S
S  #define _EDMA_RLD_LINK_MASK          0x0000FFFFu
S  #define _EDMA_RLD_LINK_SHIFT         0x00000000u
S  #define  EDMA_RLD_LINK_DEFAULT       0x00000000u
S  #define  EDMA_RLD_LINK_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_RLD_OF(x)              _VALUEOF(x)
S
S  #define EDMA_RLD_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,RLD,ELERLD)\
S    |_PER_FDEFAULT(EDMA,RLD,LINK)\
S  )
X  #define EDMA_RLD_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,RLD,ELERLD)    |_PER_FDEFAULT(EDMA,RLD,LINK)  )
S
S  #define EDMA_RLD_RMK(elerld,link) (Uint32)(\
S     _PER_FMK(EDMA,RLD,ELERLD,elerld) \
S    |_PER_FMK(EDMA,RLD,LINK,link) \
S  )
X  #define EDMA_RLD_RMK(elerld,link) (Uint32)(     _PER_FMK(EDMA,RLD,ELERLD,elerld)     |_PER_FMK(EDMA,RLD,LINK,link)   )
S
S  #define _EDMA_QRLD_FGET(FIELD)\
S    _PER_FGET(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD)
X  #define _EDMA_QRLD_FGET(FIELD)    _PER_FGET(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD)
S
S  #define _EDMA_QRLD_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD,field)
X  #define _EDMA_QRLD_FSET(FIELD,field)    _PER_FSET(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD,field)
S
S  #define _EDMA_QRLD_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD,##SYM)
X  #define _EDMA_QRLD_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QRLD_ADDR,EDMA,RLD,##FIELD,##SYM)
S
S  #define _EDMA_QSRLD_FGET(FIELD)\
S    _PER_FGET(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD)
X  #define _EDMA_QSRLD_FGET(FIELD)    _PER_FGET(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD)
S
S  #define _EDMA_QSRLD_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD,field)
X  #define _EDMA_QSRLD_FSET(FIELD,field)    _PER_FSET(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD,field)
S
S  #define _EDMA_QSRLD_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD,##SYM)
X  #define _EDMA_QSRLD_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_QSRLD_ADDR,EDMA,RLD,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  P Q S R          |
S* |___________________|
S*
S* PQSR - priority queue status register
S*
S* FIELDS (msb -> lsb)
S* (r) PQ
S*
S\******************************************************************************/
S  #define _EDMA_PQSR_ADDR              0x01A0FFE0u
S
S  #define  EDMA_PQSR                   EDMA_REG(PQSR)
S
S#if (C64_SUPPORT)
S  #define _EDMA_PQSR_PQ_MASK           0x0000000Fu
S  #define _EDMA_PQSR_PQ_SHIFT          0x00000000u
S  #define  EDMA_PQSR_PQ_DEFAULT        0x0000000Fu
S  #define  EDMA_PQSR_PQ_OF(x)          _VALUEOF(x)
S#else
S  #define _EDMA_PQSR_PQ_MASK           0x00000007u
S  #define _EDMA_PQSR_PQ_SHIFT          0x00000000u
S  #define  EDMA_PQSR_PQ_DEFAULT        0x00000007u
S  #define  EDMA_PQSR_PQ_OF(x)          _VALUEOF(x)
S#endif
S
S  #define  EDMA_PQSR_OF(x)             _VALUEOF(x)
S
S  #define EDMA_PQSR_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,PQSR,PQ)\
S  )
X  #define EDMA_PQSR_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,PQSR,PQ)  )
S #if (!(CHIP_6410 | CHIP_6413 | CHIP_6418))
S  #define EDMA_PQSR_RMK(pq) (Uint32)(\
S     _PER_FMK(EDMA,PQSR,PQ,pq)\
S  )
X  #define EDMA_PQSR_RMK(pq) (Uint32)(     _PER_FMK(EDMA,PQSR,PQ,pq)  )
S #endif
S  
S  #define _EDMA_PQSR_FGET(FIELD)\
S    _PER_FGET(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD)
X  #define _EDMA_PQSR_FGET(FIELD)    _PER_FGET(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD)
S
S  #define _EDMA_PQSR_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD,field)
X  #define _EDMA_PQSR_FSET(FIELD,field)    _PER_FSET(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD,field)
S
S  #define _EDMA_PQSR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD,##SYM)
X  #define _EDMA_PQSR_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_PQSR_ADDR,EDMA,PQSR,##FIELD,##SYM)
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  P Q A R 0        |
S* |___________________|
S*
S* PQAR0 - priority queue allocation register 0
S*
S* FIELDS (msb -> lsb)
S* (r) PQA
S*
S\******************************************************************************/
S  #if (C64_SUPPORT)
S  #define _EDMA_PQAR0_ADDR              0x01A0FFC0u
S
S  #define  EDMA_PQAR0                   EDMA_REG(PQAR0)
S
S  #define _EDMA_PQAR0_PQA_MASK           0x00000007u
S  #define _EDMA_PQAR0_PQA_SHIFT          0x00000000u
S  #define  EDMA_PQAR0_PQA_DEFAULT        0x00000002u
S  #define  EDMA_PQAR0_PQA_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_PQAR0_OF(x)             _VALUEOF(x)
S
S  #define EDMA_PQAR0_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,PQAR0,PQA)\
S  )
X  #define EDMA_PQAR0_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,PQAR0,PQA)  )
S
S  #if (!(CHIP_6413 | CHIP_6418 | CHIP_6410))
S    #define EDMA_PQAR0_RMK(pqa) (Uint32)(\
S       _PER_FMK(EDMA,PQAR0,PQA,pqa)\
S    )  
X    #define EDMA_PQAR0_RMK(pqa) (Uint32)(       _PER_FMK(EDMA,PQAR0,PQA,pqa)    )  
S  #endif
S
S  #define _EDMA_PQAR0_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD,field)
X  #define _EDMA_PQAR0_FSET(FIELD,field)    _PER_FSET(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD,field)
S
S  #define _EDMA_PQAR0_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD,##SYM)
X  #define _EDMA_PQAR0_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD,##SYM)
S
S  #define _EDMA_PQAR0_FGET(FIELD)\
S    _PER_FGET(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD)
X  #define _EDMA_PQAR0_FGET(FIELD)    _PER_FGET(_EDMA_PQAR0_ADDR,EDMA,PQAR0,FIELD)
S
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  P Q A R 1        |
S* |___________________|
S*
S* PQAR1 - priority queue allocation register 1
S*
S* FIELDS (msb -> lsb)
S* (r) PQA
S*
S\******************************************************************************/
S  #if (C64_SUPPORT)
S  #define _EDMA_PQAR1_ADDR              0x01A0FFC4u
S
S  #define  EDMA_PQAR1                   EDMA_REG(PQAR1)
S
S  #define _EDMA_PQAR1_PQA_MASK           0x00000007u
S  #define _EDMA_PQAR1_PQA_SHIFT          0x00000000u
S  #define  EDMA_PQAR1_PQA_DEFAULT        0x00000006u
S  #define  EDMA_PQAR1_PQA_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_PQAR1_OF(x)             _VALUEOF(x)
S
S  #if (!(CHIP_6413 | CHIP_6418 | CHIP_6410))
S     #define EDMA_PQAR1_RMK(pqa) (Uint32)(\
S        _PER_FMK(EDMA,PQAR1,PQA,pqa)\
S     )
X     #define EDMA_PQAR1_RMK(pqa) (Uint32)(        _PER_FMK(EDMA,PQAR1,PQA,pqa)     )
S  #endif
S   
S  #define EDMA_PQAR1_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,PQAR1,PQA)\
S  )
X  #define EDMA_PQAR1_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,PQAR1,PQA)  )
S
S  #define _EDMA_PQAR1_FGET(FIELD)\
S    _PER_FGET(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD)
X  #define _EDMA_PQAR1_FGET(FIELD)    _PER_FGET(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD)
S
S  #define _EDMA_PQAR1_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD,field)
X  #define _EDMA_PQAR1_FSET(FIELD,field)    _PER_FSET(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD,field)
S                                                    
S  #define _EDMA_PQAR1_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD,##SYM)
X  #define _EDMA_PQAR1_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_PQAR1_ADDR,EDMA,PQAR1,FIELD,##SYM)
S   
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  P Q A R 2        |
S* |___________________|
S*
S* PQAR2 - priority queue allocation register 2
S*
S* FIELDS (msb -> lsb)
S* (r) PQA
S*
S\******************************************************************************/
S  #if (C64_SUPPORT)
S  #define _EDMA_PQAR2_ADDR              0x01A0FFC8u
S
S  #define  EDMA_PQAR2                   EDMA_REG(PQAR2)
S
S  #define _EDMA_PQAR2_PQA_MASK           0x00000007u
S  #define _EDMA_PQAR2_PQA_SHIFT          0x00000000u
S  #define  EDMA_PQAR2_PQA_DEFAULT        0x00000002u
S  #define  EDMA_PQAR2_PQA_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_PQAR2_OF(x)             _VALUEOF(x)
S
S  #define EDMA_PQAR2_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,PQAR2,PQA)\
S  )
X  #define EDMA_PQAR2_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,PQAR2,PQA)  )
S
S  #if (!(CHIP_6413 | CHIP_6418 | CHIP_6410))
S      #define EDMA_PQAR2_RMK(pqa) (Uint32)(\
S         _PER_FMK(EDMA,PQAR2,PQA,pqa)\
S      )
X      #define EDMA_PQAR2_RMK(pqa) (Uint32)(         _PER_FMK(EDMA,PQAR2,PQA,pqa)      )
S  #endif
S
S  #define _EDMA_PQAR2_FGET(FIELD)\
S    _PER_FGET(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD)
X  #define _EDMA_PQAR2_FGET(FIELD)    _PER_FGET(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD)
S
S  #define _EDMA_PQAR2_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD,field)
X  #define _EDMA_PQAR2_FSET(FIELD,field)    _PER_FSET(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD,field)
S                                                    
S  #define _EDMA_PQAR2_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD,##SYM)
X  #define _EDMA_PQAR2_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_PQAR2_ADDR,EDMA,PQAR2,FIELD,##SYM)
S    
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  P Q A R 3        |
S* |___________________|
S*
S* PQAR3 - priority queue allocation register 3
S*
S* FIELDS (msb -> lsb)
S* (r) PQA
S*
S\******************************************************************************/
S  #if (C64_SUPPORT)
S  #define _EDMA_PQAR3_ADDR              0x01A0FFCCu
S
S  #define  EDMA_PQAR3                   EDMA_REG(PQAR3)
S
S  #define _EDMA_PQAR3_PQA_MASK           0x00000007u
S  #define _EDMA_PQAR3_PQA_SHIFT          0x00000000u
S  #define  EDMA_PQAR3_PQA_DEFAULT        0x00000006u
S  #define  EDMA_PQAR3_PQA_OF(x)          _VALUEOF(x)
S
S  #define  EDMA_PQAR3_OF(x)             _VALUEOF(x)
S
S  #define EDMA_PQAR3_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,PQAR3,PQA)\
S  )
X  #define EDMA_PQAR3_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,PQAR3,PQA)  )
S
S  #if (!(CHIP_6413 | CHIP_6418 | CHIP_6410))
S      #define EDMA_PQAR3_RMK(pqa) (Uint32)(\
S         _PER_FMK(EDMA,PQAR3,PQA,pqa)\
S      )
X      #define EDMA_PQAR3_RMK(pqa) (Uint32)(         _PER_FMK(EDMA,PQAR3,PQA,pqa)      )
S  #endif
S
S  #define _EDMA_PQAR3_FGET(FIELD)\
S    _PER_FGET(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD)
X  #define _EDMA_PQAR3_FGET(FIELD)    _PER_FGET(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD)
S
S  #define _EDMA_PQAR3_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD,field)
X  #define _EDMA_PQAR3_FSET(FIELD,field)    _PER_FSET(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD,field)
S                                                     
S  #define _EDMA_PQAR3_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD,##SYM)
X  #define _EDMA_PQAR3_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_PQAR3_ADDR,EDMA,PQAR3,FIELD,##SYM)
S
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I P R          |
S* |___________________|
S*
S* CIPR  - channel interrupt pending register
S*
S* FIELDS (msb -> lsb)
S* (rw) CIP
S*
S\******************************************************************************/
S #define _EDMA_CIPR_ADDR              0x01A0FFE4u
S
S  #define  EDMA_CIPR                   EDMA_REG(CIPR)
S
S#if (C64_SUPPORT)
S  #define _EDMA_CIPR_CIP_MASK          0xFFFFFFFFu
S  #define _EDMA_CIPR_CIP_SHIFT         0x00000000u
S  #define  EDMA_CIPR_CIP_DEFAULT       0x00000000u
S  #define  EDMA_CIPR_CIP_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_CIPR_CIP_MASK          0x0000FFFFu
S  #define _EDMA_CIPR_CIP_SHIFT         0x00000000u
S  #define  EDMA_CIPR_CIP_DEFAULT       0x00000000u
S  #define  EDMA_CIPR_CIP_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_CIPR_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIPR_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIPR,CIP)\
S  )
X  #define EDMA_CIPR_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIPR,CIP)  )
S
S  #define EDMA_CIPR_RMK(cip) (Uint32)(\
S     _PER_FMK(EDMA,CIPR,CIP,cip)\
S  )
X  #define EDMA_CIPR_RMK(cip) (Uint32)(     _PER_FMK(EDMA,CIPR,CIP,cip)  )
S
S  #define _EDMA_CIPR_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD)
X  #define _EDMA_CIPR_FGET(FIELD)    _PER_FGET(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD)
S
S  #define _EDMA_CIPR_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD,field)
X  #define _EDMA_CIPR_FSET(FIELD,field)    _PER_FSET(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD,field)
S
S  #define _EDMA_CIPR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD,##SYM)
X  #define _EDMA_CIPR_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIPR_ADDR,EDMA,CIPR,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I P R L        |
S* |___________________|
S*
S* CIPRL - channel interrupt pending register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CIP
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CIPRL_ADDR              0x01A0FFE4u
S
S  #define  EDMA_CIPRL                   EDMA_REG(CIPRL)
S
S  #define _EDMA_CIPRL_CIP_MASK          0xFFFFFFFFu
S  #define _EDMA_CIPRL_CIP_SHIFT         0x00000000u
S  #define  EDMA_CIPRL_CIP_DEFAULT       0x00000000u
S  #define  EDMA_CIPRL_CIP_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CIPRL_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIPRL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIPRL,CIP)\
S  )
X  #define EDMA_CIPRL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIPRL,CIP)  )
S
S  #define EDMA_CIPRL_RMK(cip) (Uint32)(\
S     _PER_FMK(EDMA,CIPRL,CIP,cip)\
S  )
X  #define EDMA_CIPRL_RMK(cip) (Uint32)(     _PER_FMK(EDMA,CIPRL,CIP,cip)  )
S
S  #define _EDMA_CIPRL_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD)
X  #define _EDMA_CIPRL_FGET(FIELD)    _PER_FGET(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD)
S
S  #define _EDMA_CIPRL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD,field)
X  #define _EDMA_CIPRL_FSET(FIELD,field)    _PER_FSET(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD,field)
S
S  #define _EDMA_CIPRL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD,##SYM)
X  #define _EDMA_CIPRL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIPRL_ADDR,EDMA,CIPRL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I P R H        |
S* |___________________|
S*
S* CIPRH - channel interrupt pending register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CIP
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CIPRH_ADDR              0x01A0FFA4u
S
S  #define  EDMA_CIPRH                   EDMA_REG(CIPRH)
S
S  #define _EDMA_CIPRH_CIP_MASK          0xFFFFFFFFu
S  #define _EDMA_CIPRH_CIP_SHIFT         0x00000000u
S  #define  EDMA_CIPRH_CIP_DEFAULT       0x00000000u
S  #define  EDMA_CIPRH_CIP_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CIPRH_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIPRH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIPRH,CIP)\
S  )
X  #define EDMA_CIPRH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIPRH,CIP)  )
S
S  #define EDMA_CIPRH_RMK(cip) (Uint32)(\
S     _PER_FMK(EDMA,CIPRH,CIP,cip)\
S  )
X  #define EDMA_CIPRH_RMK(cip) (Uint32)(     _PER_FMK(EDMA,CIPRH,CIP,cip)  )
S
S  #define _EDMA_CIPRH_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD)
X  #define _EDMA_CIPRH_FGET(FIELD)    _PER_FGET(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD)
S
S  #define _EDMA_CIPRH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD,field)
X  #define _EDMA_CIPRH_FSET(FIELD,field)    _PER_FSET(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD,field)
S
S  #define _EDMA_CIPRH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD,##SYM)
X  #define _EDMA_CIPRH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIPRH_ADDR,EDMA,CIPRH,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I E R          |
S* |___________________|
S*
S* CIER - channel interrupt enable register
S*
S* FIELDS (msb -> lsb)
S* (rw) CIE
S*
S\******************************************************************************/
S  #define _EDMA_CIER_ADDR              0x01A0FFE8u
S
S  #define  EDMA_CIER                   EDMA_REG(CIER)
S
S#if (C64_SUPPORT)
S  #define _EDMA_CIER_CIE_MASK          0xFFFFFFFFu
S  #define _EDMA_CIER_CIE_SHIFT         0x00000000u
S  #define  EDMA_CIER_CIE_DEFAULT       0x00000000u
S  #define  EDMA_CIER_CIE_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_CIER_CIE_MASK          0x0000FFFFu
S  #define _EDMA_CIER_CIE_SHIFT         0x00000000u
S  #define  EDMA_CIER_CIE_DEFAULT       0x00000000u
S  #define  EDMA_CIER_CIE_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_CIER_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIER_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIER,CIE)\
S  )
X  #define EDMA_CIER_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIER,CIE)  )
S
S  #define EDMA_CIER_RMK(cie) (Uint32)(\
S     _PER_FMK(EDMA,CIER,CIE,cie)\
S  )
X  #define EDMA_CIER_RMK(cie) (Uint32)(     _PER_FMK(EDMA,CIER,CIE,cie)  )
S
S  #define _EDMA_CIER_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD)
X  #define _EDMA_CIER_FGET(FIELD)    _PER_FGET(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD)
S
S  #define _EDMA_CIER_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD,field)
X  #define _EDMA_CIER_FSET(FIELD,field)    _PER_FSET(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD,field)
S
S  #define _EDMA_CIER_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD,##SYM)
X  #define _EDMA_CIER_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIER_ADDR,EDMA,CIER,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I E R L        |
S* |___________________|
S*
S* CIERL - channel interrupt enable register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CIE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CIERL_ADDR              0x01A0FFE8u
S
S  #define  EDMA_CIERL                   EDMA_REG(CIERL)
S
S  #define _EDMA_CIERL_CIE_MASK          0xFFFFFFFFu
S  #define _EDMA_CIERL_CIE_SHIFT         0x00000000u
S  #define  EDMA_CIERL_CIE_DEFAULT       0x00000000u
S  #define  EDMA_CIERL_CIE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CIERL_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIERL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIERL,CIE)\
S  )
X  #define EDMA_CIERL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIERL,CIE)  )
S
S  #define EDMA_CIERL_RMK(cie) (Uint32)(\
S     _PER_FMK(EDMA,CIERL,CIE,cie)\
S  )
X  #define EDMA_CIERL_RMK(cie) (Uint32)(     _PER_FMK(EDMA,CIERL,CIE,cie)  )
S
S  #define _EDMA_CIERL_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD)
X  #define _EDMA_CIERL_FGET(FIELD)    _PER_FGET(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD)
S
S  #define _EDMA_CIERL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD,field)
X  #define _EDMA_CIERL_FSET(FIELD,field)    _PER_FSET(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD,field)
S
S  #define _EDMA_CIERL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD,##SYM)
X  #define _EDMA_CIERL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIERL_ADDR,EDMA,CIERL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C I E R H        |
S* |___________________|
S*
S* CIERL - channel interrupt enable register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CIE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CIERH_ADDR              0x01A0FFA8u
S
S  #define  EDMA_CIERH                   EDMA_REG(CIERH)
S
S  #define _EDMA_CIERH_CIE_MASK          0xFFFFFFFFu
S  #define _EDMA_CIERH_CIE_SHIFT         0x00000000u
S  #define  EDMA_CIERH_CIE_DEFAULT       0x00000000u
S  #define  EDMA_CIERH_CIE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CIERH_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CIERH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CIERH,CIE)\
S  )
X  #define EDMA_CIERH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CIERH,CIE)  )
S
S  #define EDMA_CIERH_RMK(cie) (Uint32)(\
S     _PER_FMK(EDMA,CIERH,CIE,cie)\
S  )
X  #define EDMA_CIERH_RMK(cie) (Uint32)(     _PER_FMK(EDMA,CIERH,CIE,cie)  )
S
S  #define _EDMA_CIERH_FGET(FIELD)\
S    _PER_FGET(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD)
X  #define _EDMA_CIERH_FGET(FIELD)    _PER_FGET(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD)
S
S  #define _EDMA_CIERH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD,field)
X  #define _EDMA_CIERH_FSET(FIELD,field)    _PER_FSET(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD,field)
S
S  #define _EDMA_CIERH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD,##SYM)
X  #define _EDMA_CIERH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CIERH_ADDR,EDMA,CIERH,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C C E R          |
S* |___________________|
S*
S* CCER - channel chain enable register
S*
S* FIELDS (msb -> lsb)
S* (rw) CCE
S*
S\******************************************************************************/
S  #define _EDMA_CCER_ADDR              0x01A0FFECu
S
S  #define  EDMA_CCER                   EDMA_REG(CCER)
S
S#if (C64_SUPPORT)
S  #define _EDMA_CCER_CCE_MASK          0xFFFFFFFFu
S  #define _EDMA_CCER_CCE_SHIFT         0x00000000u
S  #define  EDMA_CCER_CCE_DEFAULT       0x00000000u
S  #define  EDMA_CCER_CCE_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_CCER_CCE_MASK          0x00000F00u
S  #define _EDMA_CCER_CCE_SHIFT         0x00000008u
S  #define  EDMA_CCER_CCE_DEFAULT       0x00000000u
S  #define  EDMA_CCER_CCE_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_CCER_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CCER_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CCER,CCE)\
S  )
X  #define EDMA_CCER_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CCER,CCE)  )
S
S  #define EDMA_CCER_RMK(cce) (Uint32)(\
S     _PER_FMK(EDMA,CCER,CCE,cce)\
S  )
X  #define EDMA_CCER_RMK(cce) (Uint32)(     _PER_FMK(EDMA,CCER,CCE,cce)  )
S
S  #define _EDMA_CCER_FGET(FIELD)\
S    _PER_FGET(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD)
X  #define _EDMA_CCER_FGET(FIELD)    _PER_FGET(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD)
S
S  #define _EDMA_CCER_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD,field)
X  #define _EDMA_CCER_FSET(FIELD,field)    _PER_FSET(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD,field)
S
S  #define _EDMA_CCER_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD,##SYM)
X  #define _EDMA_CCER_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CCER_ADDR,EDMA,CCER,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C C E R L        |
S* |___________________|
S*
S* CCERL - channel chain enable register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CCE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CCERL_ADDR              0x01A0FFECu
S
S  #define  EDMA_CCERL                   EDMA_REG(CCERL)
S
S  #define _EDMA_CCERL_CCE_MASK          0xFFFFFFFFu
S  #define _EDMA_CCERL_CCE_SHIFT         0x00000000u
S  #define  EDMA_CCERL_CCE_DEFAULT       0x00000000u
S  #define  EDMA_CCERL_CCE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CCERL_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CCERL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CCERL,CCE)\
S  )
X  #define EDMA_CCERL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CCERL,CCE)  )
S
S  #define EDMA_CCERL_RMK(cce) (Uint32)(\
S     _PER_FMK(EDMA,CCERL,CCE,cce)\
S  )
X  #define EDMA_CCERL_RMK(cce) (Uint32)(     _PER_FMK(EDMA,CCERL,CCE,cce)  )
S
S  #define _EDMA_CCERL_FGET(FIELD)\
S    _PER_FGET(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD)
X  #define _EDMA_CCERL_FGET(FIELD)    _PER_FGET(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD)
S
S  #define _EDMA_CCERL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD,field)
X  #define _EDMA_CCERL_FSET(FIELD,field)    _PER_FSET(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD,field)
S
S  #define _EDMA_CCERL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD,##SYM)
X  #define _EDMA_CCERL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CCERL_ADDR,EDMA,CCERL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  C C E R H        |
S* |___________________|
S*
S* CCERH - channel chain enable register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) CCE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_CCERH_ADDR              0x01A0FFACu
S
S  #define  EDMA_CCERH                   EDMA_REG(CCERH)
S
S  #define _EDMA_CCERH_CCE_MASK          0xFFFFFFFFu
S  #define _EDMA_CCERH_CCE_SHIFT         0x00000000u
S  #define  EDMA_CCERH_CCE_DEFAULT       0x00000000u
S  #define  EDMA_CCERH_CCE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_CCERH_OF(x)             _VALUEOF(x)
S
S  #define EDMA_CCERH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,CCERH,CCE)\
S  )
X  #define EDMA_CCERH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,CCERH,CCE)  )
S
S  #define EDMA_CCERH_RMK(cce) (Uint32)(\
S     _PER_FMK(EDMA,CCERH,CCE,cce)\
S  )
X  #define EDMA_CCERH_RMK(cce) (Uint32)(     _PER_FMK(EDMA,CCERH,CCE,cce)  )
S
S  #define _EDMA_CCERH_FGET(FIELD)\
S    _PER_FGET(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD)
X  #define _EDMA_CCERH_FGET(FIELD)    _PER_FGET(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD)
S
S  #define _EDMA_CCERH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD,field)
X  #define _EDMA_CCERH_FSET(FIELD,field)    _PER_FSET(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD,field)
S
S  #define _EDMA_CCERH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD,##SYM)
X  #define _EDMA_CCERH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_CCERH_ADDR,EDMA,CCERH,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E R              |
S* |___________________|
S*
S* ER - event register
S*
S* FIELDS (msb -> lsb)
S* (r) EVT
S*
S\******************************************************************************/
S  #define _EDMA_ER_ADDR              0x01A0FFF0u
S
S  #define  EDMA_ER                   EDMA_REG(ER)
S
S#if (C64_SUPPORT)
S  #define _EDMA_ER_EVT_MASK          0xFFFFFFFFu
S  #define _EDMA_ER_EVT_SHIFT         0x00000000u
S  #define  EDMA_ER_EVT_DEFAULT       0x00000000u
S  #define  EDMA_ER_EVT_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_ER_EVT_MASK          0x0000FFFFu
S  #define _EDMA_ER_EVT_SHIFT         0x00000000u
S  #define  EDMA_ER_EVT_DEFAULT       0x00000000u
S  #define  EDMA_ER_EVT_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_ER_OF(x)             _VALUEOF(x)
S
S  #define EDMA_ER_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ER,EVT)\
S  )
X  #define EDMA_ER_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ER,EVT)  )
S
S  #define _EDMA_ER_FGET(FIELD)\
S    _PER_FGET(_EDMA_ER_ADDR,EDMA,ER,##FIELD)
X  #define _EDMA_ER_FGET(FIELD)    _PER_FGET(_EDMA_ER_ADDR,EDMA,ER,##FIELD)
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E R L            |
S* |___________________|
S*
S* ERL - event register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (r) EVT
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ERL_ADDR              0x01A0FFF0u
S
S  #define  EDMA_ERL                   EDMA_REG(ERL)
S
S  #define _EDMA_ERL_EVT_MASK          0xFFFFFFFFu
S  #define _EDMA_ERL_EVT_SHIFT         0x00000000u
S  #define  EDMA_ERL_EVT_DEFAULT       0x00000000u
S  #define  EDMA_ERL_EVT_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_ERL_OF(x)             _VALUEOF(x)
S
S  #define EDMA_ERL_DEFAULT (Uint32)(\
S     _PERL_FDEFAULT(EDMA,ERL,EVT)\
S  )
X  #define EDMA_ERL_DEFAULT (Uint32)(     _PERL_FDEFAULT(EDMA,ERL,EVT)  )
S
S  #define _EDMA_ERL_FGET(FIELD)\
S    _PERL_FGET(_EDMA_ERL_ADDR,EDMA,ERL,FIELD)
X  #define _EDMA_ERL_FGET(FIELD)    _PERL_FGET(_EDMA_ERL_ADDR,EDMA,ERL,FIELD)
S
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E R H            |
S* |___________________|
S*
S* ERH - event register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (r) EVT
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ERH_ADDR              0x01A0FFB0u
S
S  #define  EDMA_ERH                   EDMA_REG(ERH)
S
S  #define _EDMA_ERH_EVT_MASK          0xFFFFFFFFu
S  #define _EDMA_ERH_EVT_SHIFT         0x00000000u
S  #define  EDMA_ERH_EVT_DEFAULT       0x00000000u
S  #define  EDMA_ERH_EVT_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_ERH_OF(x)             _VALUEOF(x)
S
S  #define EDMA_ERH_DEFAULT (Uint32)(\
S     _PERH_FDEFAULT(EDMA,ERH,EVT)\
S  )
X  #define EDMA_ERH_DEFAULT (Uint32)(     _PERH_FDEFAULT(EDMA,ERH,EVT)  )
S
S  #define _EDMA_ERH_FGET(FIELD)\
S    _PERH_FGET(_EDMA_ERH_ADDR,EDMA,ERH,FIELD)
X  #define _EDMA_ERH_FGET(FIELD)    _PERH_FGET(_EDMA_ERH_ADDR,EDMA,ERH,FIELD)
S
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E E R            |
S* |___________________|
S*
S* EER - event enable register
S*
S* FIELDS (msb -> lsb)
S* (rw) EE
S*
S\******************************************************************************/
S  #define _EDMA_EER_ADDR             0x01A0FFF4u
S
S  #define  EDMA_EER                  EDMA_REG(EER)
S
S#if (C64_SUPPORT)
S  #define _EDMA_EER_EE_MASK          0xFFFFFFFFu
S  #define _EDMA_EER_EE_SHIFT         0x00000000u
S  #define  EDMA_EER_EE_DEFAULT       0x00000000u
S  #define  EDMA_EER_EE_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_EER_EE_MASK          0x0000FFFFu
S  #define _EDMA_EER_EE_SHIFT         0x00000000u
S  #define  EDMA_EER_EE_DEFAULT       0x00000000u
S  #define  EDMA_EER_EE_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_EER_OF(x)            _VALUEOF(x)
S
S  #define EDMA_EER_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,EER,EE)\
S  )
X  #define EDMA_EER_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,EER,EE)  )
S
S  #define EDMA_EER_RMK(ee) (Uint32)(\
S     _PER_FMK(EDMA,EER,EE,ee)\
S  )
X  #define EDMA_EER_RMK(ee) (Uint32)(     _PER_FMK(EDMA,EER,EE,ee)  )
S
S  #define _EDMA_EER_FGET(FIELD)\
S    _PER_FGET(_EDMA_EER_ADDR,EDMA,EER,##FIELD)
X  #define _EDMA_EER_FGET(FIELD)    _PER_FGET(_EDMA_EER_ADDR,EDMA,EER,##FIELD)
S
S  #define _EDMA_EER_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_EER_ADDR,EDMA,EER,##FIELD,field)
X  #define _EDMA_EER_FSET(FIELD,field)    _PER_FSET(_EDMA_EER_ADDR,EDMA,EER,##FIELD,field)
S
S  #define _EDMA_EER_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_EER_ADDR,EDMA,EER,##FIELD,##SYM)
X  #define _EDMA_EER_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_EER_ADDR,EDMA,EER,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E E R L          |
S* |___________________|
S*
S* EERL - event enable register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_EERL_ADDR             0x01A0FFF4u
S
S  #define  EDMA_EERL                  EDMA_REG(EERL)
S
S  #define _EDMA_EERL_EE_MASK          0xFFFFFFFFu
S  #define _EDMA_EERL_EE_SHIFT         0x00000000u
S  #define  EDMA_EERL_EE_DEFAULT       0x00000000u
S  #define  EDMA_EERL_EE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EERL_OF(x)            _VALUEOF(x)
S
S  #define EDMA_EERL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,EERL,EE)\
S  )
X  #define EDMA_EERL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,EERL,EE)  )
S
S  #define EDMA_EERL_RMK(ee) (Uint32)(\
S     _PER_FMK(EDMA,EERL,EE,ee)\
S  )
X  #define EDMA_EERL_RMK(ee) (Uint32)(     _PER_FMK(EDMA,EERL,EE,ee)  )
S
S  #define _EDMA_EERL_FGET(FIELD)\
S    _PER_FGET(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD)
X  #define _EDMA_EERL_FGET(FIELD)    _PER_FGET(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD)
S
S  #define _EDMA_EERL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD,field)
X  #define _EDMA_EERL_FSET(FIELD,field)    _PER_FSET(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD,field)
S
S  #define _EDMA_EERL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD,##SYM)
X  #define _EDMA_EERL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_EERL_ADDR,EDMA,EERL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E E R H          |
S* |___________________|
S*
S* EERH - event enable register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EE
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_EERH_ADDR             0x01A0FFB4u
S
S  #define  EDMA_EERH                  EDMA_REG(EERH)
S
S  #define _EDMA_EERH_EE_MASK          0xFFFFFFFFu
S  #define _EDMA_EERH_EE_SHIFT         0x00000000u
S  #define  EDMA_EERH_EE_DEFAULT       0x00000000u
S  #define  EDMA_EERH_EE_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EERH_OF(x)            _VALUEOF(x)
S
S  #define EDMA_EERH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,EERH,EE)\
S  )
X  #define EDMA_EERH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,EERH,EE)  )
S
S  #define EDMA_EERH_RMK(ee) (Uint32)(\
S     _PER_FMK(EDMA,EERH,EE,ee)\
S  )
X  #define EDMA_EERH_RMK(ee) (Uint32)(     _PER_FMK(EDMA,EERH,EE,ee)  )
S
S  #define _EDMA_EERH_FGET(FIELD)\
S    _PER_FGET(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD)
X  #define _EDMA_EERH_FGET(FIELD)    _PER_FGET(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD)
S
S  #define _EDMA_EERH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD,field)
X  #define _EDMA_EERH_FSET(FIELD,field)    _PER_FSET(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD,field)
S
S  #define _EDMA_EERH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD,##SYM)
X  #define _EDMA_EERH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_EERH_ADDR,EDMA,EERH,##FIELD,##SYM)
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E P R L          |
S* |___________________|
S*
S* EPRL - event polarity register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EP
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_EPRL_ADDR             0x01A0FFDCu
S
S  #define  EDMA_EPRL                  EDMA_REG(EPRL)
S
S  #define _EDMA_EPRL_EP_MASK          0xFFFFFFFFu
S  #define _EDMA_EPRL_EP_SHIFT         0x00000000u
S  #define  EDMA_EPRL_EP_DEFAULT       0x00000000u
S  #define  EDMA_EPRL_EP_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EPRL_OF(x)            _VALUEOF(x)
S
S  #define EDMA_EPRL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,EPRL,EP)\
S  )
X  #define EDMA_EPRL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,EPRL,EP)  )
S
S  #define EDMA_EPRL_RMK(ep) (Uint32)(\
S     _PER_FMK(EDMA,EPRL,EP,ep)\
S  )
X  #define EDMA_EPRL_RMK(ep) (Uint32)(     _PER_FMK(EDMA,EPRL,EP,ep)  )
S
S  #define _EDMA_EPRL_FGET(FIELD)\
S    _PER_FGET(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD)
X  #define _EDMA_EPRL_FGET(FIELD)    _PER_FGET(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD)
S
S  #define _EDMA_EPRL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD,field)
X  #define _EDMA_EPRL_FSET(FIELD,field)    _PER_FSET(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD,field)
S
S  #define _EDMA_EPRL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD,##SYM)
X  #define _EDMA_EPRL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_EPRL_ADDR,EDMA,EPRL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E E R H          |
S* |___________________|
S*
S* EPRH - event enable register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EP
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_EPRH_ADDR             0x01A0FF9Cu
S
S  #define  EDMA_EPRH                  EDMA_REG(EPRH)
S
S  #define _EDMA_EPRH_EP_MASK          0xFFFFFFFFu
S  #define _EDMA_EPRH_EP_SHIFT         0x00000000u
S  #define  EDMA_EPRH_EP_DEFAULT       0x00000000u
S  #define  EDMA_EPRH_EP_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EPRH_OF(x)            _VALUEOF(x)
S
S  #define EDMA_EPRH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,EPRH,EP)\
S  )
X  #define EDMA_EPRH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,EPRH,EP)  )
S
S  #define EDMA_EPRH_RMK(ep) (Uint32)(\
S     _PER_FMK(EDMA,EPRH,EP,ep)\
S  )
X  #define EDMA_EPRH_RMK(ep) (Uint32)(     _PER_FMK(EDMA,EPRH,EP,ep)  )
S
S  #define _EDMA_EPRH_FGET(FIELD)\
S    _PER_FGET(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD)
X  #define _EDMA_EPRH_FGET(FIELD)    _PER_FGET(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD)
S
S  #define _EDMA_EPRH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD,field)
X  #define _EDMA_EPRH_FSET(FIELD,field)    _PER_FSET(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD,field)
S
S  #define _EDMA_EPRH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD,##SYM)
X  #define _EDMA_EPRH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_EPRH_ADDR,EDMA,EPRH,##FIELD,##SYM)
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E C R            |
S* |___________________|
S*
S* ECR - event clear register
S*
S* FIELDS (msb -> lsb)
S* (rw) EC
S*
S\******************************************************************************/
S  #define _EDMA_ECR_ADDR             0x01A0FFF8u
S
S  #define  EDMA_ECR                  EDMA_REG(ECR)
S
S#if (C64_SUPPORT)
S  #define _EDMA_ECR_EC_MASK          0xFFFFFFFFu
S  #define _EDMA_ECR_EC_SHIFT         0x00000000u
S  #define  EDMA_ECR_EC_DEFAULT       0x00000000u
S  #define  EDMA_ECR_EC_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_ECR_EC_MASK          0x0000FFFFu
S  #define _EDMA_ECR_EC_SHIFT         0x00000000u
S  #define  EDMA_ECR_EC_DEFAULT       0x00000000u
S  #define  EDMA_ECR_EC_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_EPR_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ECR_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ECR,EC)\
S  )
X  #define EDMA_ECR_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ECR,EC)  )
S
S  #define EDMA_ECR_RMK(ec) (Uint32)(\
S     _PER_FMK(EDMA,ECR,EC,ec)\
S  )
X  #define EDMA_ECR_RMK(ec) (Uint32)(     _PER_FMK(EDMA,ECR,EC,ec)  )
S
S  #define _EDMA_ECR_FGET(FIELD)\
S    _PER_FGET(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD)
X  #define _EDMA_ECR_FGET(FIELD)    _PER_FGET(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD)
S
S  #define _EDMA_ECR_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD,field)
X  #define _EDMA_ECR_FSET(FIELD,field)    _PER_FSET(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD,field)
S
S  #define _EDMA_ECR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD,##SYM)
X  #define _EDMA_ECR_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ECR_ADDR,EDMA,ECR,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E C R L          |
S* |___________________|
S*
S* ECRL - event clear register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EC
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ECRL_ADDR             0x01A0FFF8u
S
S  #define  EDMA_ECRL                  EDMA_REG(ECRL)
S
S  #define _EDMA_ECRL_EC_MASK          0xFFFFFFFFu
S  #define _EDMA_ECRL_EC_SHIFT         0x00000000u
S  #define  EDMA_ECRL_EC_DEFAULT       0x00000000u
S  #define  EDMA_ECRL_EC_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EER_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ECRL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ECRL,EC)\
S  )
X  #define EDMA_ECRL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ECRL,EC)  )
S
S  #define EDMA_ECRL_RMK(ec) (Uint32)(\
S     _PER_FMK(EDMA,ECRL,EC,ec)\
S  )
X  #define EDMA_ECRL_RMK(ec) (Uint32)(     _PER_FMK(EDMA,ECRL,EC,ec)  )
S
S  #define _EDMA_ECRL_FGET(FIELD)\
S    _PER_FGET(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD)
X  #define _EDMA_ECRL_FGET(FIELD)    _PER_FGET(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD)
S
S  #define _EDMA_ECRL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD,field)
X  #define _EDMA_ECRL_FSET(FIELD,field)    _PER_FSET(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD,field)
S
S  #define _EDMA_ECRL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD,##SYM)
X  #define _EDMA_ECRL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ECRL_ADDR,EDMA,ECRL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E C R H          |
S* |___________________|
S*
S* ECRH - event clear register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) EC
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ECRH_ADDR             0x01A0FFB8u
S
S  #define  EDMA_ECRH                  EDMA_REG(ECRH)
S
S  #define _EDMA_ECRH_EC_MASK          0xFFFFFFFFu
S  #define _EDMA_ECRH_EC_SHIFT         0x00000000u
S  #define  EDMA_ECRH_EC_DEFAULT       0x00000000u
S  #define  EDMA_ECRH_EC_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_EER_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ECRH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ECRH,EC)\
S  )
X  #define EDMA_ECRH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ECRH,EC)  )
S
S  #define EDMA_ECRH_RMK(ec) (Uint32)(\
S     _PER_FMK(EDMA,ECRH,EC,ec)\
S  )
X  #define EDMA_ECRH_RMK(ec) (Uint32)(     _PER_FMK(EDMA,ECRH,EC,ec)  )
S
S  #define _EDMA_ECRH_FGET(FIELD)\
S    _PER_FGET(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD)
X  #define _EDMA_ECRH_FGET(FIELD)    _PER_FGET(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD)
S
S  #define _EDMA_ECRH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD,field)
X  #define _EDMA_ECRH_FSET(FIELD,field)    _PER_FSET(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD,field)
S
S  #define _EDMA_ECRH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD,##SYM)
X  #define _EDMA_ECRH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ECRH_ADDR,EDMA,ECRH,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S R            |
S* |___________________|
S*
S* ESR - event set register
S*
S* FIELDS (msb -> lsb)
S* (rw) ES
S*
S\******************************************************************************/
S  #define _EDMA_ESR_ADDR             0x01A0FFFCu
S
S  #define  EDMA_ESR                  EDMA_REG(ESR)
S
S#if (C64_SUPPORT)
S  #define _EDMA_ESR_ES_MASK          0xFFFFFFFFu
S  #define _EDMA_ESR_ES_SHIFT         0x00000000u
S  #define  EDMA_ESR_ES_DEFAULT       0x00000000u
S  #define  EDMA_ESR_ES_OF(x)         _VALUEOF(x)
S#else
S  #define _EDMA_ESR_ES_MASK          0x0000FFFFu
S  #define _EDMA_ESR_ES_SHIFT         0x00000000u
S  #define  EDMA_ESR_ES_DEFAULT       0x00000000u
S  #define  EDMA_ESR_ES_OF(x)         _VALUEOF(x)
S#endif
S
S  #define  EDMA_ESR_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESR_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESR,ES)\
S  )
X  #define EDMA_ESR_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESR,ES)  )
S
S  #define EDMA_ESR_RMK(es) (Uint32)(\
S     _PER_FMK(EDMA,ESR,ES,es)\
S  )
X  #define EDMA_ESR_RMK(es) (Uint32)(     _PER_FMK(EDMA,ESR,ES,es)  )
S
S  #define _EDMA_ESR_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD)
X  #define _EDMA_ESR_FGET(FIELD)    _PER_FGET(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD)
S
S  #define _EDMA_ESR_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD,field)
X  #define _EDMA_ESR_FSET(FIELD,field)    _PER_FSET(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD,field)
S
S  #define _EDMA_ESR_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD,##SYM)
X  #define _EDMA_ESR_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESR_ADDR,EDMA,ESR,##FIELD,##SYM)
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S R L          |
S* |___________________|
S*
S* ESRL - event set register, low half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) ES
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ESRL_ADDR             0x01A0FFFCu
S
S  #define  EDMA_ESRL                  EDMA_REG(ESRL)
S
S  #define _EDMA_ESRL_ES_MASK          0xFFFFFFFFu
S  #define _EDMA_ESRL_ES_SHIFT         0x00000000u
S  #define  EDMA_ESRL_ES_DEFAULT       0x00000000u
S  #define  EDMA_ESRL_ES_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_ESRL_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESRL_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESRL,ES)\
S  )
X  #define EDMA_ESRL_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESRL,ES)  )
S
S  #define EDMA_ESRL_RMK(es) (Uint32)(\
S     _PER_FMK(EDMA,ESRL,ES,es)\
S  )
X  #define EDMA_ESRL_RMK(es) (Uint32)(     _PER_FMK(EDMA,ESRL,ES,es)  )
S
S  #define _EDMA_ESRL_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD)
X  #define _EDMA_ESRL_FGET(FIELD)    _PER_FGET(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD)
S
S  #define _EDMA_ESRL_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD,field)
X  #define _EDMA_ESRL_FSET(FIELD,field)    _PER_FSET(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD,field)
S
S  #define _EDMA_ESRL_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD,##SYM)
X  #define _EDMA_ESRL_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESRL_ADDR,EDMA,ESRL,##FIELD,##SYM)
S#endif
S
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S R H          |
S* |___________________|
S*
S* ESRH - event set register, high half (1)
S*
S* (1) - C64x devices only
S*
S* FIELDS (msb -> lsb)
S* (rw) ES
S*
S\******************************************************************************/
S#if (C64_SUPPORT)
S  #define _EDMA_ESRH_ADDR             0x01A0FFBCu
S
S  #define  EDMA_ESRH                  EDMA_REG(ESRH)
S
S  #define _EDMA_ESRH_ES_MASK          0xFFFFFFFFu
S  #define _EDMA_ESRH_ES_SHIFT         0x00000000u
S  #define  EDMA_ESRH_ES_DEFAULT       0x00000000u
S  #define  EDMA_ESRH_ES_OF(x)         _VALUEOF(x)
S
S  #define  EDMA_ESRH_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESRH_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESRH,ES)\
S  )
X  #define EDMA_ESRH_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESRH,ES)  )
S
S  #define EDMA_ESRH_RMK(es) (Uint32)(\
S     _PER_FMK(EDMA,ESRH,ES,es)\
S  )
X  #define EDMA_ESRH_RMK(es) (Uint32)(     _PER_FMK(EDMA,ESRH,ES,es)  )
S
S  #define _EDMA_ESRH_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD)
X  #define _EDMA_ESRH_FGET(FIELD)    _PER_FGET(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD)
S
S  #define _EDMA_ESRH_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD,field)
X  #define _EDMA_ESRH_FSET(FIELD,field)    _PER_FSET(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD,field)
S
S  #define _EDMA_ESRH_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD,##SYM)
X  #define _EDMA_ESRH_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESRH_ADDR,EDMA,ESRH,##FIELD,##SYM)
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S E L 0        |
S* |___________________|
S*
S* ESEL0 - event selection register 0
S*
S* FIELDS (msb -> lsb)
S* (rw) EVTSEL3
S* (rw) EVTSEL2
S* (rw) EVTSEL1
S* (rw) EVTSEL0
S*
S\******************************************************************************/
S#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
S  #define _EDMA_ESEL0_ADDR             0x01A0FF00u
S
S  #define  EDMA_ESEL0                  EDMA_REG(ESEL0)
S
S  #define _EDMA_ESEL0_EVTSEL0_MASK     0x0000003Fu
S  #define _EDMA_ESEL0_EVTSEL0_SHIFT    0x00000000u
S  #define  EDMA_ESEL0_EVTSEL0_DEFAULT  0x00000000u
S  #define  EDMA_ESEL0_EVTSEL0_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL0_EVTSEL1_MASK     0x00003F00u
S  #define _EDMA_ESEL0_EVTSEL1_SHIFT    0x00000008u
S  #define  EDMA_ESEL0_EVTSEL1_DEFAULT  0x00000001u
S  #define  EDMA_ESEL0_EVTSEL1_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL0_EVTSEL2_MASK     0x003F0000u
S  #define _EDMA_ESEL0_EVTSEL2_SHIFT    0x00000010u
S  #define  EDMA_ESEL0_EVTSEL2_DEFAULT  0x00000002u
S  #define  EDMA_ESEL0_EVTSEL2_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL0_EVTSEL3_MASK     0x3F000000u
S  #define _EDMA_ESEL0_EVTSEL3_SHIFT    0x00000018u
S  #define  EDMA_ESEL0_EVTSEL3_DEFAULT  0x00000003u
S  #define  EDMA_ESEL0_EVTSEL3_OF(x)    _VALUEOF(x)
S
S  #define  EDMA_ESEL0_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESEL0_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESEL0,EVTSEL0)\
S    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL1)\
S    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL2)\
S    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL3)\
S  )
X  #define EDMA_ESEL0_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESEL0,EVTSEL0)    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL1)    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL2)    |_PER_FDEFAULT(EDMA,ESEL0,EVTSEL3)  )
S
S  #define EDMA_ESEL0_RMK(evtsel0, evtsel1, evtsel2, evtsel3) (Uint32)(\
S     _PER_FMK(EDMA,ESEL0,EVTSEL0,evtsel0)\
S    |_PER_FMK(EDMA,ESEL0,EVTSEL1,evtsel1)\
S    |_PER_FMK(EDMA,ESEL0,EVTSEL2,evtsel2)\
S    |_PER_FMK(EDMA,ESEL0,EVTSEL3,evtsel3)\
S  )
X  #define EDMA_ESEL0_RMK(evtsel0, evtsel1, evtsel2, evtsel3) (Uint32)(     _PER_FMK(EDMA,ESEL0,EVTSEL0,evtsel0)    |_PER_FMK(EDMA,ESEL0,EVTSEL1,evtsel1)    |_PER_FMK(EDMA,ESEL0,EVTSEL2,evtsel2)    |_PER_FMK(EDMA,ESEL0,EVTSEL3,evtsel3)  )
S
S  #define _EDMA_ESEL0_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD)
X  #define _EDMA_ESEL0_FGET(FIELD)    _PER_FGET(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD)
S
S  #define _EDMA_ESEL0_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD,field)
X  #define _EDMA_ESEL0_FSET(FIELD,field)    _PER_FSET(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD,field)
S
S  #define _EDMA_ESEL0_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD,##SYM)
X  #define _EDMA_ESEL0_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESEL0_ADDR,EDMA,ESEL0,##FIELD,##SYM)
S
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S E L 1        |
S* |___________________|
S*
S* ESEL1 - event selection register 1
S*
S* FIELDS (msb -> lsb)
S* (rw) EVTSEL7
S* (rw) EVTSEL6
S* (rw) EVTSEL5
S* (rw) EVTSEL4
S*
S\******************************************************************************/
S#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
S  #define _EDMA_ESEL1_ADDR             0x01A0FF04u
S
S  #define  EDMA_ESEL1                  EDMA_REG(ESEL1)
S
S  #define _EDMA_ESEL1_EVTSEL4_MASK     0x0000003Fu
S  #define _EDMA_ESEL1_EVTSEL4_SHIFT    0x00000000u
S  #define  EDMA_ESEL1_EVTSEL4_DEFAULT  0x00000004u
S  #define  EDMA_ESEL1_EVTSEL4_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL1_EVTSEL5_MASK     0x00003F00u
S  #define _EDMA_ESEL1_EVTSEL5_SHIFT    0x00000008u
S  #define  EDMA_ESEL1_EVTSEL5_DEFAULT  0x00000005u
S  #define  EDMA_ESEL1_EVTSEL5_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL1_EVTSEL6_MASK     0x003F0000u
S  #define _EDMA_ESEL1_EVTSEL6_SHIFT    0x00000010u
S  #define  EDMA_ESEL1_EVTSEL6_DEFAULT  0x00000006u
S  #define  EDMA_ESEL1_EVTSEL6_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL1_EVTSEL7_MASK     0x3F000000u
S  #define _EDMA_ESEL1_EVTSEL7_SHIFT    0x00000018u
S  #define  EDMA_ESEL1_EVTSEL7_DEFAULT  0x00000007u
S  #define  EDMA_ESEL1_EVTSEL7_OF(x)    _VALUEOF(x)
S
S  #define  EDMA_ESEL1_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESEL1_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESEL1,EVTSEL4)\
S    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL5)\
S    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL6)\
S    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL7)\
S  )
X  #define EDMA_ESEL1_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESEL1,EVTSEL4)    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL5)    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL6)    |_PER_FDEFAULT(EDMA,ESEL1,EVTSEL7)  )
S
S  #define EDMA_ESEL1_RMK(evtsel4, evtsel5, evtsel6, evtsel7) (Uint32)(\
S     _PER_FMK(EDMA,ESEL1,EVTSEL4,evtsel4)\
S    |_PER_FMK(EDMA,ESEL1,EVTSEL5,evtsel5)\
S    |_PER_FMK(EDMA,ESEL1,EVTSEL6,evtsel6)\
S    |_PER_FMK(EDMA,ESEL1,EVTSEL7,evtsel7)\
S  )
X  #define EDMA_ESEL1_RMK(evtsel4, evtsel5, evtsel6, evtsel7) (Uint32)(     _PER_FMK(EDMA,ESEL1,EVTSEL4,evtsel4)    |_PER_FMK(EDMA,ESEL1,EVTSEL5,evtsel5)    |_PER_FMK(EDMA,ESEL1,EVTSEL6,evtsel6)    |_PER_FMK(EDMA,ESEL1,EVTSEL7,evtsel7)  )
S
S  #define _EDMA_ESEL1_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD)
X  #define _EDMA_ESEL1_FGET(FIELD)    _PER_FGET(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD)
S
S  #define _EDMA_ESEL1_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD,field)
X  #define _EDMA_ESEL1_FSET(FIELD,field)    _PER_FSET(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD,field)
S
S  #define _EDMA_ESEL1_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD,##SYM)
X  #define _EDMA_ESEL1_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESEL1_ADDR,EDMA,ESEL1,##FIELD,##SYM)
S
S#endif
S
S/******************************************************************************\
S* _____________________
X
S* |                   |
S* |  E S E L 3        |
S* |___________________|
S*
S* ESEL3 - event selection register 3
S*
S* FIELDS (msb -> lsb)
S* (rw) EVTSEL15
S* (rw) EVTSEL14
S* (rw) EVTSEL13
S* (rw) EVTSEL12
S*
S\******************************************************************************/
S#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
S  #define _EDMA_ESEL3_ADDR             0x01A0FF0Cu
S
S  #define  EDMA_ESEL3                  EDMA_REG(ESEL3)
S
S  #define _EDMA_ESEL3_EVTSEL12_MASK     0x0000003Fu
S  #define _EDMA_ESEL3_EVTSEL12_SHIFT    0x00000000u
S  #define  EDMA_ESEL3_EVTSEL12_DEFAULT  0x0000000Cu
S  #define  EDMA_ESEL3_EVTSEL12_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL3_EVTSEL13_MASK     0x00003F00u
S  #define _EDMA_ESEL3_EVTSEL13_SHIFT    0x00000008u
S  #define  EDMA_ESEL3_EVTSEL13_DEFAULT  0x0000000Du
S  #define  EDMA_ESEL3_EVTSEL13_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL3_EVTSEL14_MASK     0x003F0000u
S  #define _EDMA_ESEL3_EVTSEL14_SHIFT    0x00000010u
S  #define  EDMA_ESEL3_EVTSEL14_DEFAULT  0x0000000Eu
S  #define  EDMA_ESEL3_EVTSEL14_OF(x)    _VALUEOF(x)
S
S  #define _EDMA_ESEL3_EVTSEL15_MASK     0x3F000000u
S  #define _EDMA_ESEL3_EVTSEL15_SHIFT    0x00000018u
S  #define  EDMA_ESEL3_EVTSEL15_DEFAULT  0x0000000Fu
S  #define  EDMA_ESEL3_EVTSEL15_OF(x)    _VALUEOF(x)
S
S  #define  EDMA_ESEL3_OF(x)            _VALUEOF(x)
S
S  #define EDMA_ESEL3_DEFAULT (Uint32)(\
S     _PER_FDEFAULT(EDMA,ESEL3,EVTSEL12)\
S    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL13)\
S    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL14)\
S    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL15)\
S  )
X  #define EDMA_ESEL3_DEFAULT (Uint32)(     _PER_FDEFAULT(EDMA,ESEL3,EVTSEL12)    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL13)    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL14)    |_PER_FDEFAULT(EDMA,ESEL3,EVTSEL15)  )
S
S  #define EDMA_ESEL3_RMK(evtsel12, evtsel13, evtsel14, evtsel15) (Uint32)(\
S     _PER_FMK(EDMA,ESEL3,EVTSEL12,evtsel12)\
S    |_PER_FMK(EDMA,ESEL3,EVTSEL13,evtsel13)\
S    |_PER_FMK(EDMA,ESEL3,EVTSEL14,evtsel14)\
S    |_PER_FMK(EDMA,ESEL3,EVTSEL15,evtsel15)\
S  )
X  #define EDMA_ESEL3_RMK(evtsel12, evtsel13, evtsel14, evtsel15) (Uint32)(     _PER_FMK(EDMA,ESEL3,EVTSEL12,evtsel12)    |_PER_FMK(EDMA,ESEL3,EVTSEL13,evtsel13)    |_PER_FMK(EDMA,ESEL3,EVTSEL14,evtsel14)    |_PER_FMK(EDMA,ESEL3,EVTSEL15,evtsel15)  )
S
S  #define _EDMA_ESEL3_FGET(FIELD)\
S    _PER_FGET(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD)
X  #define _EDMA_ESEL3_FGET(FIELD)    _PER_FGET(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD)
S
S  #define _EDMA_ESEL3_FSET(FIELD,field)\
S    _PER_FSET(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD,field)
X  #define _EDMA_ESEL3_FSET(FIELD,field)    _PER_FSET(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD,field)
S
S  #define _EDMA_ESEL3_FSETS(FIELD,SYM)\
S    _PER_FSETS(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD,##SYM)
X  #define _EDMA_ESEL3_FSETS(FIELD,SYM)    _PER_FSETS(_EDMA_ESEL3_ADDR,EDMA,ESEL3,##FIELD,##SYM)
S
S#endif
S
S/*----------------------------------------------------------------------------*/
S
N#endif /* (EDMA_SUPPORT) */
N#endif /* _CSL_EDMAHAL_H_ */
N/******************************************************************************\
N* End of csl_edmahal.h
X
N\******************************************************************************/
L 24 "..\csl\csl_edma.h" 2
N
N
N#if (EDMA_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S/******************************************************************************\
S* scope and inline control macros
X
S\******************************************************************************/
S#ifdef __cplusplus
S#define CSLAPI extern "C" far
S#else
S#define CSLAPI extern far
S#endif
S
S#undef  USEDEFS
S#undef  IDECL
S#undef  IDEF
S
S#ifdef  _EDMA_MOD_
S  #define IDECL CSLAPI
S  #define USEDEFS
S  #define IDEF
S#else
S  #ifdef  _INLINE
S    #define IDECL static inline
S    #define USEDEFS
S    #define IDEF  static inline
S  #else
S    #define IDECL CSLAPI
S  #endif
S#endif
S
S
S/******************************************************************************\
S* global macro declarations
X
S\******************************************************************************/
S#define EDMA_CHA_CNT              (_EDMA_CHA_CNT)
S#define EDMA_TABLE_CNT            (_EDMA_LINK_CNT)
S#define EDMA_ALLOC_ANY            (-1)
S
S#define EDMA_OPEN_RESET           (0x00000001)
S#define EDMA_OPEN_ENABLE          (0x00000002)
S
S
S#if(CHIP_6201|CHIP_6202|CHIP_6203|CHIP_6204|CHIP_6205|CHIP_6701|C11_SUPPORT)
S  #define EDMA_CHA_ANY      -1
S  #define EDMA_CHA_DSPINT   0
S  #define EDMA_CHA_TINT0    1
S  #define EDMA_CHA_TINT1    2
S  #define EDMA_CHA_SDINT    3
S  #define EDMA_CHA_EXTINT4  4
S  #define EDMA_CHA_EXTINT5  5
S  #define EDMA_CHA_EXTINT6  6
S  #define EDMA_CHA_EXTINT7  7
S  #define EDMA_CHA_TCC8     8
S  #define EDMA_CHA_TCC9     9
S  #define EDMA_CHA_TCC10    10
S  #define EDMA_CHA_TCC11    11
S  #define EDMA_CHA_XEVT0    12
S  #define EDMA_CHA_REVT0    13
S  #define EDMA_CHA_XEVT1    14
S  #define EDMA_CHA_REVT1    15
S#endif
S#if (CHIP_6711C || CHIP_6712C)
S  #define EDMA_CHA_GPINT4    4
S  #define EDMA_CHA_GPINT5    5
S  #define EDMA_CHA_GPINT6    6
S  #define EDMA_CHA_GPINT7    7
S  #define EDMA_CHA_GPINT2   10
S#endif
S
S#if (CHIP_6713 | CHIP_DA610 | CHIP_6414 | CHIP_6415 | CHIP_6416 | \
S     CHIP_6411 | CHIP_6410 | CHIP_6413 | CHIP_6418)
X#if (CHIP_6713 | CHIP_DA610 | CHIP_6414 | CHIP_6415 | CHIP_6416 |      CHIP_6411 | CHIP_6410 | CHIP_6413 | CHIP_6418)
S  #define EDMA_CHA_ANY      -1
S  #define EDMA_CHA_DSPINT    0
S  #define EDMA_CHA_TINT0     1
S  #define EDMA_CHA_TINT1     2
S  #define EDMA_CHA_SDINT     3
S  #define EDMA_CHA_EXTINT4   4
S  #define EDMA_CHA_GPINT4    4
S  #define EDMA_CHA_EXTINT5   5
S  #define EDMA_CHA_GPINT5    5
S  #define EDMA_CHA_EXTINT6   6
S  #define EDMA_CHA_GPINT6    6
S  #define EDMA_CHA_EXTINT7   7
S  #define EDMA_CHA_GPINT7    7
S  #define EDMA_CHA_TCC8      8
S  #define EDMA_CHA_GPINT0    8
S  #define EDMA_CHA_TCC9      9
S  #define EDMA_CHA_GPINT1    9
S  #define EDMA_CHA_TCC10    10
S  #define EDMA_CHA_GPINT2   10
S  #define EDMA_CHA_TCC11    11
S  #define EDMA_CHA_GPINT3   11
S  #define EDMA_CHA_XEVT0    12
S  #define EDMA_CHA_REVT0    13
S  #define EDMA_CHA_XEVT1    14
S  #define EDMA_CHA_REVT1    15
S
S/* Individual mappings for next few events are specified under #ifdefs below. */
S
S  #define EDMA_CHA_GPINT8   48
S  #define EDMA_CHA_GPINT9   49
S  #define EDMA_CHA_GPINT10  50
S  #define EDMA_CHA_GPINT11  51
S  #define EDMA_CHA_GPINT12  52
S  #define EDMA_CHA_GPINT13  53
S  #define EDMA_CHA_GPINT14  54
S  #define EDMA_CHA_GPINT15  55
S#endif
S
S
S/* 3rd MCBSP/ PCI / UTOPIA / VCP / TCP channel numbers for EDMA_open() */
S#if (CHIP_6414 || CHIP_6415 || CHIP_6416 )
S  #define EDMA_CHA_XEVT2    17
S  #define EDMA_CHA_REVT2    18
S  #define EDMA_CHA_TINT2    19
S  #define EDMA_CHA_SDINTB   20
S  #define EDMA_CHA_PCI      21
S  #define EDMA_CHA_VCPREVT  28
S  #define EDMA_CHA_VCPXEVT  29
S  #define EDMA_CHA_TCPREVT  30
S  #define EDMA_CHA_TCPXEVT  31
S  #define EDMA_CHA_UREVT    32
S  #define EDMA_CHA_UREVT0   32
S  #define EDMA_CHA_UREVT1   33
S  #define EDMA_CHA_UREVT2   34
S  #define EDMA_CHA_UREVT3   35
S  #define EDMA_CHA_UREVT4   36
S  #define EDMA_CHA_UREVT5   37
S  #define EDMA_CHA_UREVT6   38
S  #define EDMA_CHA_UREVT7   39
S  #define EDMA_CHA_UXEVT    40
S  #define EDMA_CHA_UXEVT0   40
S  #define EDMA_CHA_UXEVT1   41
S  #define EDMA_CHA_UXEVT2   42
S  #define EDMA_CHA_UXEVT3   43
S  #define EDMA_CHA_UXEVT4   44
S  #define EDMA_CHA_UXEVT5   45
S  #define EDMA_CHA_UXEVT6   46
S  #define EDMA_CHA_UXEVT7   47
S#endif
S
S/* 2 MCASPs - 2 I2Cs channels */
S#if (CHIP_6713 || CHIP_DA610)
S  #define EDMA_CHA_AXEVTE0  32
S  #define EDMA_CHA_AXEVTO0  33
S  #define EDMA_CHA_AXEVT0   34
S  #define EDMA_CHA_AREVTE0  35
S  #define EDMA_CHA_AREVTO0  36
S  #define EDMA_CHA_AREVT0   37
S  #define EDMA_CHA_AXEVTE1  38
S  #define EDMA_CHA_AXEVTO1  39
S  #define EDMA_CHA_AXEVT1   40
S  #define EDMA_CHA_AREVTE1  41
S  #define EDMA_CHA_AREVTO1  42
S  #define EDMA_CHA_AREVT1   43
S  #define EDMA_CHA_ICREVT0  44
S  #define EDMA_CHA_ICXEVT0  45
S  #define EDMA_CHA_ICREVT1  46
S  #define EDMA_CHA_ICXEVT1  47
S#endif
S
S#if (CHIP_6410 || CHIP_6413| CHIP_6418)
S  #define EDMA_CHA_TINT2    19
S
S #if (CHIP_6418)
S  #define EDMA_CHA_VCPREVT0 28  
S  #define EDMA_CHA_VCPXEVT0 29  
S #endif
S
S  #define EDMA_CHA_AXEVTE0  32  
S  #define EDMA_CHA_AXEVTO0  33  
S  #define EDMA_CHA_AXEVT0   34  
S  #define EDMA_CHA_AREVTE0  35  
S  #define EDMA_CHA_AREVTO0  36  
S  #define EDMA_CHA_AREVT0   37  
S  #define EDMA_CHA_AXEVTE1  38  
S  #define EDMA_CHA_AXEVTO1  39  
S  #define EDMA_CHA_AXEVT1   40  
S  #define EDMA_CHA_AREVTE1  41  
S  #define EDMA_CHA_AREVTO1  42  
S  #define EDMA_CHA_AREVT1   43  
S  #define EDMA_CHA_ICREVT0  44  
S  #define EDMA_CHA_ICXEVT0  45  
S  #define EDMA_CHA_ICREVT1  46  
S  #define EDMA_CHA_ICXEVT1  47  
S#endif
S
S#if (CHIP_DM642 || CHIP_DM641 || CHIP_DM640 || CHIP_6412)
S  #define EDMA_CHA_ANY      -1
S
S #if !(CHIP_DM640)
S  #define EDMA_CHA_DSPINT   0
S #endif
S
S  #define EDMA_CHA_TINT0    1
S  #define EDMA_CHA_TINT1    2
S  #define EDMA_CHA_SDINT    3
S  #define EDMA_CHA_EXTINT4  4
S  #define EDMA_CHA_GPINT4   4
S  #define EDMA_CHA_EXTINT5  5
S  #define EDMA_CHA_GPINT5   5
S  #define EDMA_CHA_EXTINT6  6
S  #define EDMA_CHA_GPINT6   6
S  #define EDMA_CHA_EXTINT7  7
S  #define EDMA_CHA_GPINT7   7
S  #define EDMA_CHA_GPINT0   8
S  #define EDMA_CHA_TCC8     8
S  #define EDMA_CHA_GPINT1   9
S  #define EDMA_CHA_TCC9     9
S  #define EDMA_CHA_GPINT2   10
S  #define EDMA_CHA_TCC10    10
S  #define EDMA_CHA_GPINT3   11
S  #define EDMA_CHA_TCC11    11
S  #define EDMA_CHA_XEVT0    12
S  #define EDMA_CHA_REVT0    13
S  #define EDMA_CHA_XEVT1    14
S  #define EDMA_CHA_REVT1    15
S
S/* Note: EDMA_CHA_TCC8, EDMA_CHA_TCC9, EDMA_CHA_TCC10 and EDMA_CHA_TCC11 are 
S   NOT defined here, but they are included here for C64x consistency purposes */
S
S #if (CHIP_DM642 | CHIP_DM641 | CHIP_DM640)
S  #define EDMA_CHA_VP0EVTYA 16
S  #define EDMA_CHA_VP0EVTUA 17
S  #define EDMA_CHA_VP0EVTVA 18
S #endif
S
S  #define EDMA_CHA_TINT2    19
S
S #if (CHIP_DM642)
S  #define EDMA_CHA_VP0EVTYB 24
S  #define EDMA_CHA_VP0EVTUB 25
S  #define EDMA_CHA_VP0EVTVB 26
S #endif
S
S #if !(CHIP_6412)
S  #define EDMA_CHA_AXEVTE0  32
S  #define EDMA_CHA_AXEVTO0  33
S  #define EDMA_CHA_AXEVT0   34
S  #define EDMA_CHA_AREVTE0  35
S  #define EDMA_CHA_AREVTO0  36
S  #define EDMA_CHA_AREVT0   37
S #endif
S
S #if (CHIP_DM642)
S  #define EDMA_CHA_VP1EVTYB 38
S  #define EDMA_CHA_VP1EVTUB 39
S  #define EDMA_CHA_VP1EVTVB 40
S  #define EDMA_CHA_VP2EVTYB 41
S  #define EDMA_CHA_VP2EVTUB 42
S  #define EDMA_CHA_VP2EVTVB 43
S #endif
S 
S  #define EDMA_CHA_ICREVT0  44
S  #define EDMA_CHA_ICXEVT0  45
S
S  #define EDMA_CHA_GPINT8   48
S  #define EDMA_CHA_GPINT9   49
S  #define EDMA_CHA_GPINT10  50
S  #define EDMA_CHA_GPINT11  51
S  #define EDMA_CHA_GPINT12  52
S  #define EDMA_CHA_GPINT13  53
S  #define EDMA_CHA_GPINT14  54
S  #define EDMA_CHA_GPINT15  55
S
S
S #if (CHIP_DM642 | CHIP_DM641)
S  #define EDMA_CHA_VP1EVTYA 56
S  #define EDMA_CHA_VP1EVTUA 57
S  #define EDMA_CHA_VP1EVTVA 58
S #endif
S
S #if (CHIP_DM642)
S  #define EDMA_CHA_VP2EVTYA 59
S  #define EDMA_CHA_VP2EVTUA 60
S  #define EDMA_CHA_VP2EVTVA 61
S #endif
S
S#endif
S
S#define _EDMA_TYPE_C   (0x80000000)
S#define _EDMA_TYPE_T   (0x40000000)
S#define _EDMA_TYPE_Q   (0x20000000)
S#define _EDMA_TYPE_S   (0x10000000)
S
S
S#define _EDMA_MK_HANDLE(base,index,flags) (EDMA_Handle)(\
S  ((base)&0x0000FFFF)|(((index)<<16)&0x00FF0000)|((flags)&0xFF000000)\
S)
X#define _EDMA_MK_HANDLE(base,index,flags) (EDMA_Handle)(  ((base)&0x0000FFFF)|(((index)<<16)&0x00FF0000)|((flags)&0xFF000000))
S
S#define EDMA_HQDMA    _EDMA_MK_HANDLE(0x00000000,0,_EDMA_TYPE_Q)
S#define EDMA_HQSDMA   _EDMA_MK_HANDLE(0x00000000,0,_EDMA_TYPE_S)
S#define EDMA_HINV     _EDMA_MK_HANDLE(0x00000000,0,0)
S
S#define EDMA_HNULL    _EDMA_MK_HANDLE(_EDMA_NULL_PARAM,0,_EDMA_TYPE_T)
S#define NULL_FUNC     0
S
S/* Priority Queues */
S#define EDMA_Q0       0
S#define EDMA_Q1       1
S#define EDMA_Q2       2
S#define EDMA_Q3       3
S
S/* Event Polarity */
S#define EDMA_EVT_LOWHIGH   0
S#define EDMA_EVT_HIGHLOW   1
S
S/* Chaining Flag */
S#define EDMA_TCC_SET       1
S#define EDMA_ATCC_SET      1
S
S/******************************************************************************\
S* global typedef declarations
X
S\******************************************************************************/
Stypedef Uint32 EDMA_Handle;
S
Stypedef struct {
S  Uint32 opt;
S  Uint32 src;
S  Uint32 cnt;
S  Uint32 dst;
S  Uint32 idx;
S  Uint32 rld;
S} EDMA_Config;
S
Stypedef void (*EDMA_IntHandler)(int tccNum);
S
S/******************************************************************************\
S* global variable declarations
X
S\******************************************************************************/
S#define EDMA_hNull EDMA_HNULL
S
Sextern far Uint8 tccAllocTable[64];
S
S/******************************************************************************\
S* global function declarations
X
S\******************************************************************************/
SCSLAPI void EDMA_reset(EDMA_Handle hEdma);
SCSLAPI void EDMA_resetAll();
S
SCSLAPI EDMA_Handle EDMA_open(int chaNum, Uint32 flags);
SCSLAPI void EDMA_close(EDMA_Handle hEdma);
SCSLAPI EDMA_Handle EDMA_allocTable(int tableNum);
SCSLAPI void EDMA_freeTable(EDMA_Handle hEdma);
S
SCSLAPI int EDMA_allocTableEx(int cnt, EDMA_Handle *array);
SCSLAPI void EDMA_freeTableEx(int cnt, EDMA_Handle *array);
S
SCSLAPI void EDMA_clearPram(Uint32 val);
S
SCSLAPI int EDMA_intAlloc(int tcc);
SCSLAPI void EDMA_intFree(int tcc);
S/* 6713? */
S
S#if (CHIP_6713 || CHIP_DA610 || CHIP_6711C || CHIP_6712C)
SCSLAPI int EDMA_map(int eventNum,int chaNum);
S#endif
S
SCSLAPI EDMA_IntHandler EDMA_intHook(int tccNum, EDMA_IntHandler funcAddr);
SCSLAPI void EDMA_intDefaultHandler(int tccNum);
SCSLAPI void EDMA_intDispatcher();
S
S/******************************************************************************\
S* inline function declarations
X
S\******************************************************************************/
SIDECL Uint32 EDMA_getScratchAddr();
SIDECL Uint32 EDMA_getScratchSize();
SIDECL Uint32 EDMA_getPriQStatus();
S#if (C64_SUPPORT)
SIDECL void   EDMA_setPriQLength(Uint32 priNum, Uint32 length);
SIDECL void   EDMA_resetPriQLength(Uint32 priNum);
S#endif
S
SIDECL void   EDMA_enableChannel(EDMA_Handle hEdma);
SIDECL void   EDMA_disableChannel(EDMA_Handle hEdma);
SIDECL void   EDMA_setChannel(EDMA_Handle hEdma);
SIDECL Uint32 EDMA_getChannel(EDMA_Handle hEdma);
SIDECL void   EDMA_clearChannel(EDMA_Handle hEdma);
S#if (C64_SUPPORT)
SIDECL void   EDMA_setEvtPolarity(EDMA_Handle hEdma,int polarity);
S#endif
SIDECL Uint32 EDMA_getTableAddress(EDMA_Handle hEdma);
S
SIDECL void   EDMA_intEnable(Uint32 tccIntNum);
SIDECL void   EDMA_intDisable(Uint32 tccIntNum);
SIDECL void   EDMA_intClear(Uint32 tccIntNum);
SIDECL Uint32 EDMA_intTest(Uint32 tccIntNum);
SIDECL void   EDMA_intReset(Uint32 tccIntNum);
SIDECL void   EDMA_intResetAll();
S
SIDECL void   EDMA_link(EDMA_Handle parent, EDMA_Handle child);
S
SIDECL void EDMA_config(EDMA_Handle hEdma, EDMA_Config *config);
SIDECL void EDMA_configArgs(EDMA_Handle hEdma, Uint32 opt, Uint32 src,
S  Uint32 cnt, Uint32 dst, Uint32 idx, Uint32 rld);
SIDECL void EDMA_getConfig(EDMA_Handle hEdma, EDMA_Config *config);
S
SIDECL void EDMA_qdmaConfig(EDMA_Config *config);
SIDECL void EDMA_qdmaConfigArgs(Uint32 opt, Uint32 src, Uint32 cnt, Uint32 dst,
S  Uint32 idx);
SIDECL void EDMA_qdmaGetConfig(EDMA_Config *config);
S
SIDECL void EDMA_enableChaining(EDMA_Handle hEdma);
SIDECL void EDMA_disableChaining(EDMA_Handle hEdma);
SIDECL  void EDMA_chain(EDMA_Handle parent, EDMA_Handle nextChannel,int tccflag, int atccflag);
S
S/******************************************************************************\
S* inline function definitions
X
S\******************************************************************************/
S#ifdef USEDEFS
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_getScratchAddr() {
S  return (Uint32)_EDMA_SCRATCH_START;
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_getScratchSize() {
S  return (Uint32)_EDMA_SCRATCH_SIZE;
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_getPriQStatus() {
S  return (Uint32)EDMA_RGET(PQSR);
S}
S/*----------------------------------------------------------------------------*/
S#if (C64_SUPPORT)
SIDEF void EDMA_setPriQLength(Uint32 priNum, Uint32 length) {
S  if (priNum == 0x0) {
S   EDMA_RSET(PQAR0,(0x00000007 & length));
S   } else {
S          if (priNum == 0x1) {
S           EDMA_RSET(PQAR1,(0x00000007 & length));
S          }else {
S                if (priNum == 0x2) {
S                EDMA_RSET(PQAR2,(0x00000007 & length));
S                } else {
S                       if (priNum == 0x3) {
S                       EDMA_RSET(PQAR3,(0x00000007 & length));
S                       }
S                  }
S           }
S    }
S}
S/*----------------------------------------------------------------------------*/
S
SIDEF void EDMA_resetPriQLength(Uint32 priNum) {
S if (priNum == 0x0) {
S   EDMA_FSET(PQAR0,PQA,EDMA_PQAR0_PQA_DEFAULT);
S   } else {
S          if (priNum == 0x1) {
S          EDMA_FSET(PQAR1,PQA,EDMA_PQAR1_PQA_DEFAULT);
S          } else {
S                 if (priNum == 0x2) {
S                 EDMA_FSET(PQAR2,PQA,EDMA_PQAR2_PQA_DEFAULT);
S                 }else {
S                      if (priNum == 0x3) {
S                      EDMA_FSET(PQAR3,PQA,EDMA_PQAR3_PQA_DEFAULT);
S                       }
S                  }
S           }
S   }
S}
S#endif
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_enableChannel(EDMA_Handle hEdma) {
S
S  int gie;
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  gie = IRQ_globalDisable();
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(EERL,EDMA_RGET(EERL) | (1<<chaNum));
S    else EDMA_RSET(EERH,EDMA_RGET(EERH) | (1<<(chaNum-32)));
S  #else
S    EDMA_RSET(EER,EDMA_RGET(EER) | (1<<chaNum));
S  #endif
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_enableChaining(EDMA_Handle hEdma) {
S
S  int gie;
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  gie = IRQ_globalDisable();
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(CCERL,EDMA_RGET(CCERL) | (1<<chaNum));
S    else EDMA_RSET(CCERH,EDMA_RGET(CCERH) | (1<<(chaNum-32)));
S  #else
S
S    EDMA_RSET(CCER,EDMA_RGET(CCER) | (1<<chaNum));
S  #endif
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_chain(EDMA_Handle parent, EDMA_Handle nextChannel,int tccflag, int atccflag) {
S  #if (C64_SUPPORT)
S  Uint32 tccm,tcc;
S  #endif
S  int TccNum = (nextChannel & 0x00FF0000)>>16;
S  int gie;
S
S  gie = IRQ_globalDisable();
S
S if (tccflag) {
S  #if (C64_SUPPORT) /* SET TCCM and TCC fields */
S   if (TccNum < 16) {  tccm = 0 ; tcc = TccNum;}
S   else {
S        if ( TccNum  > 15 && TccNum < 32 ) { tccm = 1 ; tcc = TccNum-16;}
S         else  {
S              if ( TccNum  > 31 && TccNum < 48 ) { tccm = 2 ; tcc = TccNum-32;}
S              else {   /* channel > 47 */
S                     tccm = 3 ; tcc = TccNum-48;
S                   }
S         }
S    }
S      EDMA_FSETH(parent,OPT,TCCM,tccm);
S      EDMA_FSETH(parent,OPT,TCC,tcc);
S      EDMA_FSETH(parent,OPT,TCINT,1);
S  #else
S    EDMA_FSETH(parent,OPT,TCC,TccNum);
S    EDMA_FSETH(parent,OPT,TCINT,1);
S  #endif
S}
S/* ATCC */
Sif (atccflag) {
S    #if C64_SUPPORT
S    EDMA_FSETH(parent,OPT,ATCC,TccNum);
S    EDMA_FSETH(parent,OPT,ATCINT,1);
S    #endif
S}
S  tccAllocTable[TccNum] = 1;
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_disableChannel(EDMA_Handle hEdma) {
S
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S  int gie;
S
S  gie = IRQ_globalDisable();
S
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(EERL,EDMA_RGET(EERL) & ~(1<<chaNum));
S    else EDMA_RSET(EERH,EDMA_RGET(EERH) & ~(1<<(chaNum-32)));
S  #else
S    EDMA_RSET(EER,EDMA_RGET(EER) & ~(1<<chaNum));
S  #endif
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_disableChaining(EDMA_Handle hEdma) {
S
S int chaNum = (hEdma & 0x00FF0000)>>16;
S int gie;
S
S  gie = IRQ_globalDisable();
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(CCERL,EDMA_RGET(CCERL) & ~(1<<chaNum));
S    else EDMA_RSET(CCERH,EDMA_RGET(CCERH) & ~(1<<(chaNum-32)));
S  #else
S    EDMA_RSET(CCER,EDMA_RGET(CCER) & ~(1<<chaNum));
S  #endif
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_setChannel(EDMA_Handle hEdma) {
S
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(ESRL,1<<chaNum);
S    else EDMA_RSET(ESRH,1<<(chaNum-32));
S  #else
S    EDMA_RSET(ESR,1<<chaNum);
S  #endif
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_getChannel(EDMA_Handle hEdma) {
S
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  #if (C64_SUPPORT)
S    if (chaNum < 32) return (Uint32)(((EDMA_RGET(ERL) >> chaNum) & 1)==1);
S    else return (Uint32)(((EDMA_RGET(ERH) >> (chaNum-32)) & 1)==1);
S    
S  #else
S    return (Uint32)(((EDMA_RGET(ER) >>chaNum) & 1)==1);
S  #endif
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_clearChannel(EDMA_Handle hEdma) {
S
S  int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(ECRL,1<<chaNum);
S    else EDMA_RSET(ECRH,1<<(chaNum-32));
S  #else
S    EDMA_RSET(ECR,1<<chaNum);
S  #endif
S}
S/*----------------------------------------------------------------------------*/
S#if (C64_SUPPORT)
SIDEF void   EDMA_setEvtPolarity(EDMA_Handle hEdma,int polarity) {
S
S    int chaNum = (hEdma & 0x00FF0000)>>16;
S
S  #if (C64_SUPPORT)
S    if (chaNum < 32) EDMA_RSET(EPRL,(polarity<<chaNum) | EDMA_RGET(EPRL));
S    else EDMA_RSET(EPRH,(polarity<<(chaNum-32)) | EDMA_RGET(EPRH));
S  #endif
S}
S#endif
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_getTableAddress(EDMA_Handle hEdma) {
S
S  return (hEdma&0x0000FFFF)+_EDMA_PRAM_START;
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_intEnable(Uint32 tccIntNum) {
S  int gie;
S
S  gie = IRQ_globalDisable();
S
S  #if (C64_SUPPORT)
S    if (tccIntNum < 32) EDMA_RSET(CIERL,EDMA_RGET(CIERL) | (1<<tccIntNum));
S    else EDMA_RSET(CIERH,EDMA_RGET(CIERH) | (1<<(tccIntNum-32)));
S  #else
S    EDMA_RSET(CIER,EDMA_RGET(CIER) | (1<<tccIntNum));
S  #endif
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_intDisable(Uint32 tccIntNum) {
S  int gie;
S
S  gie = IRQ_globalDisable();
S
S  #if (C64_SUPPORT)
S    if (tccIntNum < 32) EDMA_RSET(CIERL,EDMA_RGET(CIERL) & ~(1<<tccIntNum));
S    else EDMA_RSET(CIERH,EDMA_RGET(CIERH) & ~(1<<(tccIntNum-32)));
S  #else
S    EDMA_RSET(CIER,EDMA_RGET(CIER) & ~(1<<tccIntNum));
S  #endif
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_intClear(Uint32 tccIntNum) {
S
S  #if (C64_SUPPORT)
S    if (tccIntNum < 32) EDMA_RSET(CIPRL,1<<tccIntNum);
S    else EDMA_RSET(CIPRH,1<<(tccIntNum-32));
S  #else
S    EDMA_RSET(CIPR,1<<tccIntNum);
S  #endif
S}
S/*----------------------------------------------------------------------------*/
SIDEF Uint32 EDMA_intTest(Uint32 tccIntNum) {
S
S  #if (C64_SUPPORT)
S    if (tccIntNum < 32) return ((EDMA_RGET(CIPRL) & (1<<tccIntNum)) ? 1 : 0);
S    else return ((EDMA_RGET(CIPRH) & (1<<(tccIntNum-32))) ? 1 : 0);
S  #else
S    return ((EDMA_RGET(CIPR) & (1<<tccIntNum)) ? 1 : 0);
S  #endif
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_intReset(Uint32 tccIntNum) {
S  int gie;
S
S  gie = IRQ_globalDisable();
S
S  /* disable then clear interrupt */
S  #if (C64_SUPPORT)
S    if (tccIntNum < 32) {
S      EDMA_RSET(CIERL,EDMA_RGET(CIERL) & ~(1<<tccIntNum));
S      EDMA_RSET(CIPRL,1<<tccIntNum);
S    } else {
S      EDMA_RSET(CIERH,EDMA_RGET(CIERH) & ~(1<<(tccIntNum-32)));
S      EDMA_RSET(CIPRH,1<<(tccIntNum-32));
S    }
S  #else
S    EDMA_RSET(CIER,EDMA_RGET(CIER) & ~(1<<tccIntNum));
S    EDMA_RSET(CIPR,1<<tccIntNum);
S  #endif
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_intResetAll() {
S
S  int gie = IRQ_globalDisable();
S  /* disable then clear all interrupts */
S
S  #if (C64_SUPPORT)
S    EDMA_RSET(CIERL, 0x00000000);
S    EDMA_RSET(CIERH, 0x00000000);
S    EDMA_RSET(CIPRL, 0xFFFFFFFF);
S    EDMA_RSET(CIPRH, 0xFFFFFFFF);
S  #else
S    EDMA_RSET(CIER, 0x00000000);
S    EDMA_RSET(CIPR, 0xFFFFFFFF);
S  #endif
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_link(EDMA_Handle parent, EDMA_Handle child) {
S  EDMA_FSETH(parent,RLD,LINK,child);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_config(EDMA_Handle hEdma, EDMA_Config *config) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S  register Uint32 x0,x1,x2,x3,x4,x5;
S
S/* Test if QDMA handle was passed with CCS 1.2 */
S if ( hEdma == (0x20000000) || hEdma == (0x10000000)) {
S        EDMA_qdmaConfig(config);
S }else{
S
S  gie = IRQ_globalDisable();
S
S  x0 = config->opt;
S  x1 = config->src;
S  x2 = config->cnt;
S  x3 = config->dst;
S  x4 = config->idx;
S  x5 = config->rld;
S
S  base = (volatile Uint32 *)((hEdma&0x0000FFFF)+_EDMA_PRAM_START);
S  base[_EDMA_OPT_OFFSET] = 0x00000000;
S  base[_EDMA_SRC_OFFSET] = x1;
S  base[_EDMA_CNT_OFFSET] = x2;
S  base[_EDMA_DST_OFFSET] = x3;
S  base[_EDMA_IDX_OFFSET] = x4;
S  base[_EDMA_RLD_OFFSET] = x5;
S  base[_EDMA_OPT_OFFSET] = x0;
S  IRQ_globalRestore(gie);
S  }
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_configArgs(EDMA_Handle hEdma, Uint32 opt, Uint32 src,
S  Uint32 cnt, Uint32 dst, Uint32 idx, Uint32 rld) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S /* Test if QDMA handle was passed with CCS 1.2 */
S if ( hEdma == (0x20000000) || hEdma == (0x10000000)) {
S        EDMA_qdmaConfigArgs(opt, src, cnt, dst, idx);
S }else{
S  gie = IRQ_globalDisable();
S  base = (volatile Uint32*)((hEdma&0x0000FFFF)+_EDMA_PRAM_START);
S  base[_EDMA_OPT_OFFSET] = 0x00000000;
S  base[_EDMA_SRC_OFFSET] = src;
S  base[_EDMA_CNT_OFFSET] = cnt;
S  base[_EDMA_DST_OFFSET] = dst;
S  base[_EDMA_IDX_OFFSET] = idx;
S  base[_EDMA_RLD_OFFSET] = rld;
S  base[_EDMA_OPT_OFFSET] = opt;
S  IRQ_globalRestore(gie);
S }
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_getConfig(EDMA_Handle hEdma, EDMA_Config *config) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S  register Uint32 x0,x1,x2,x3,x4,x5;
S
S  gie = IRQ_globalDisable();
S
S  base = (volatile Uint32 *)((hEdma&0x0000FFFF)+_EDMA_PRAM_START);
S  x0 = base[_EDMA_OPT_OFFSET];
S  x1 = base[_EDMA_SRC_OFFSET];
S  x2 = base[_EDMA_CNT_OFFSET];
S  x3 = base[_EDMA_DST_OFFSET];
S  x4 = base[_EDMA_IDX_OFFSET];
S  x5 = base[_EDMA_RLD_OFFSET];
S
S  config->opt = x0;
S  config->src = x1;
S  config->cnt = x2;
S  config->dst = x3;
S  config->idx = x4;
S  config->rld = x5;
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_qdmaConfig(EDMA_Config *config) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S  register Uint32 x0,x1,x2,x3,x4;
S
S  gie = IRQ_globalDisable();
S
S  x0 = config->opt;
S  x1 = config->src;
S  x2 = config->cnt;
S  x3 = config->dst;
S  x4 = config->idx;
S
S  base = (volatile Uint32 *)(_EDMA_QOPT_ADDR);
S  base[_EDMA_QSRC_OFFSET]  = x1;
S  base[_EDMA_QCNT_OFFSET]  = x2;
S  base[_EDMA_QDST_OFFSET]  = x3;
S  base[_EDMA_QIDX_OFFSET]  = x4;
S  base[_EDMA_QSOPT_OFFSET] = x0;
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_qdmaConfigArgs(Uint32 opt, Uint32 src, Uint32 cnt, Uint32 dst,
S  Uint32 idx) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S
S  gie = IRQ_globalDisable();
S
S  base = (volatile Uint32*)(_EDMA_QOPT_ADDR);
S  base[_EDMA_QSRC_OFFSET]  = src;
S  base[_EDMA_QCNT_OFFSET]  = cnt;
S  base[_EDMA_QDST_OFFSET]  = dst;
S  base[_EDMA_QIDX_OFFSET]  = idx;
S  base[_EDMA_QSOPT_OFFSET] = opt;
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
SIDEF void EDMA_qdmaGetConfig(EDMA_Config *config) {
S
S  Uint32 gie;
S  volatile Uint32 *base;
S  volatile EDMA_Config* cfg = (volatile EDMA_Config*)config;
S  register Uint32 x0,x1,x2,x3,x4;
S
S  gie = IRQ_globalDisable();
S
S  base = (volatile Uint32 *)(_EDMA_QOPT_ADDR);
S  x0 = base[_EDMA_QOPT_OFFSET];
S  x1 = base[_EDMA_QSRC_OFFSET];
S  x2 = base[_EDMA_QCNT_OFFSET];
S  x3 = base[_EDMA_QDST_OFFSET];
S  x4 = base[_EDMA_QIDX_OFFSET];
S
S  cfg->opt = x0;
S  cfg->src = x1;
S  cfg->cnt = x2;
S  cfg->dst = x3;
S  cfg->idx = x4;
S  cfg->rld = 0x00000000;
S
S  IRQ_globalRestore(gie);
S}
S/*----------------------------------------------------------------------------*/
S#endif /* USEDEFS */
S
S
N#endif /* EDMA_SUPPORT */
N#endif /* _CSL_EDMA_H_ */
N/******************************************************************************\
N* End of csl_edma.h
X
N\******************************************************************************/
N
L 51 "../csl/csl_cache.c" 2
N
N
N#if (CACHE_SUPPORT)
X#if (( ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N/******************************************************************************\
N*                         L O C A L   S E C T I O N
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* static macro declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* static typedef declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* static function declarations
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* static variable definitions
X
N\******************************************************************************/
N
Nstatic Uint32 busyMaskCache = 0x00000000;
N
N/******************************************************************************\
N* static function definitions
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N*                        G L O B A L   S E C T I O N
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* global variable definitions
X
N\******************************************************************************/
N
N
N/******************************************************************************\
N* global function definitions
X
N\******************************************************************************/
N
Nvoid CACHE_reset() {
N
N  #if (L2CACHE_SUPPORT && C64_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && ( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S      volatile Uint32 addr;
S      #if (CHIP_6414|CHIP_6415|CHIP_6416)
S      /* Reset EMIFB regions */
S       for (addr=0x01848180u; addr<0x018481C0; addr+=4)
S            REG32(addr) = REG32(addr)&0xFFFFFFFE;
S      #endif
S      /* Reset EMIFA regions */
S       for (addr=0x01848200u; addr<0x01848300; addr+=4)
S            REG32(addr) = REG32(addr)&0xFFFFFFFE;
S
S       /* Set entire cache region as SRAM */
S       CACHE_setL2Mode(0);
N  #endif
N
N  /* Invalidate Program and Data cache */
N  CHIP_FSET(CSR,PCC,0);
X  CSR=((Uint32)((Uint32)( (((Uint32)(CSR)&~0x000000E0u)| (((Uint32)(0)<<0x00000005u) &0x000000E0u)) )));
N  CHIP_FSET(CSR,DCC,0);
X  CSR=((Uint32)((Uint32)( (((Uint32)(CSR)&~0x0000001Cu)| (((Uint32)(0)<<0x00000002u) &0x0000001Cu)) )));
N
N  #if (L2CACHE_SUPPORT && !C64_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && !( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S    CACHE_RSET(MAR0,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR1,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR2,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR3,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR4,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR5,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR6,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR7,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR8,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR9,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR10,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR11,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR12,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR13,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR14,CACHE_MAR_DEFAULT);
S    CACHE_RSET(MAR15,CACHE_MAR_DEFAULT);
S    //CACHE_RSET(CCFG,CACHE_CCFG_DEFAULT);
N  #endif /* (L2CACHE_SUPPORT) */
N}
N/*----------------------------------------------------------------------------*/
N#if (L2CACHE_SUPPORT && (CHIP_6414|CHIP_6415|CHIP_6416))
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && (0|0|0))
Svoid CACHE_resetEmifb() {
S
S    volatile Uint32 addr;
S
S    for (addr=0x01848180u; addr<0x018481C0; addr+=4) {
S      REG32(addr) = REG32(addr)&0xFFFFFFFE;
S    }
S}
N#endif
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_resetEmifa() {
N
N  volatile Uint32 addr;
N
N   #if (L2CACHE_SUPPORT && (CHIP_6414|CHIP_6415|CHIP_6416|CHIP_DM642|CHIP_DM641|CHIP_DM640|CHIP_6412|CHIP_6411|CHIP_6410|CHIP_6413|CHIP_6418))
X   #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ) && (0|0|0|0|0|0|0|0|0|0|0))
S      for (addr=0x01848200u; addr<0x01848300; addr+=4) {
S          REG32(addr) = REG32(addr)&0xFFFFFFFE;
S      }
N  #endif
N}
N/*----------------------------------------------------------------------------*/
NCACHE_L2Mode CACHE_getL2Mode() {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S    return (CACHE_L2Mode)CACHE_FGET(CCFG,L2MODE);
N  #else
N    return (CACHE_L2Mode)CACHE_0KCACHE;
N  #endif
N}
N
N/*----------------------------------------------------------------------------*/
NCACHE_L2Mode CACHE_setL2Mode(CACHE_L2Mode newMode) {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S  Uint32 gie;
S  Uint32 base_address;
S  Uint32 byte_count;
S  Uint32 curMode = CACHE_FGET(CCFG,L2MODE);
S  #if (CHIP_6211 | CHIP_6711 | CHIP_6712 | CHIP_6711C | CHIP_6712C) //....
S  Uint32 start_of_cache[] =
S  {
S    0x00010000,         /* 000 =   64K SRAM,   0K cache   */
S    0x0000C000,         /* 001 =   48K SRAM,  16K cache   */
S    0x00008000,         /* 010 =   32K SRAM,  32K cache   */
S    0x00004000,         /* 011 =   16K SRAM,  48K cache   */
S    0x00000000,         /* 100 -- undefined, treat as 111 */
S    0x00000000,         /* 101 -- undefined, treat as 111 */
S    0x00000000,         /* 110 -- undefined, treat as 111 */
S    0x00000000          /* 111 =    0K SRAM,  64K cache   */
S  };
S  #endif
S
S  #if (CHIP_6713 | CHIP_DA610)
S  Uint32 start_of_cache[] =
S  {
S    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
S    0x0003C000,         /* 001 =  240K SRAM,  16K cache   */
S    0x00038000,         /* 010 =  224K SRAM,  32K cache   */
S    0x00034000,         /* 011 =  208K SRAM,  48K cache   */
S    0x00030000,         /* 100 -- undefined, treat as 111 */
S    0x00030000,         /* 101 -- undefined, treat as 111 */
S    0x00030000,         /* 110 -- undefined, treat as 111 */
S    0x00030000          /* 111 =  192K SRAM,  64K cache   */
S  };
S  #endif
S
S  #if (CHIP_6414 | CHIP_6415 | CHIP_6416)
S  Uint32 start_of_cache[] =
S  {
S    0x00100000,         /* 000 = 1024K SRAM,   0K cache   */
S    0x000F8000,         /* 001 =  992K SRAM,  32K cache   */
S    0x000F0000,         /* 010 =  960K SRAM,  64K cache   */
S    0x000E0000,         /* 011 =  896K SRAM, 128K cache   */
S    0x000C0000,         /* 100 -- undefined, treat as 111 */
S    0x000C0000,         /* 101 -- undefined, treat as 111 */
S    0x000C0000,         /* 110 -- undefined, treat as 111 */
S    0x000C0000          /* 111 =  768K SRAM, 256K cache   */
S  };
S  #endif
S
S #if (CHIP_DM642 | CHIP_6412 | CHIP_6411)
S  Uint32 start_of_cache[] =
S  {
S    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
S    0x00038000,         /* 001 =  224K SRAM,  32K cache   */
S    0x00030000,         /* 010 =  192K SRAM,  64K cache   */
S    0x00020000,         /* 011 =  128K SRAM, 128K cache   */
S    0x00000000,         /* 100 -- undefined, treat as 111 */
S    0x00000000,         /* 101 -- undefined, treat as 111 */
S    0x00000000,         /* 110 -- undefined, treat as 111 */
S    0x00000000          /* 111 =    0K SRAM, 256K cache   */
S  };
S #endif
S
S #if (CHIP_6418)
S  Uint32 start_of_cache[] =
S  {
S    0x00080000,         /* 000 =  512K SRAM,   0K cache   */
S    0x00078000,         /* 001 =  480K SRAM,  32K cache   */
S    0x00070000,         /* 010 =  448K SRAM,  64K cache   */
S    0x00060000,         /* 011 =  384K SRAM, 128K cache   */
S    0x00040000,         /* 100 -- undefined, treat as 111 */
S    0x00040000,         /* 101 -- undefined, treat as 111 */
S    0x00040000,         /* 110 -- undefined, treat as 111 */
S    0x00040000          /* 111 =  256K SRAM, 256K cache   */
S  };
S #endif
S
S #if (CHIP_6413)
S  Uint32 start_of_cache[] =
S  {
S    0x00040000,         /* 000 =  256K SRAM,   0K cache   */
S    0x00038000,         /* 001 =  224K SRAM,  32K cache   */
S    0x00030000,         /* 010 =  192K SRAM,  64K cache   */
S    0x00020000,         /* 011 =  128K SRAM, 128K cache   */
S    0x00000000,         /* 100 -- undefined, treat as 111 */
S    0x00000000,         /* 101 -- undefined, treat as 111 */
S    0x00000000,         /* 110 -- undefined, treat as 111 */
S    0x00000000          /* 111 =  256K cache              */
S  };
S #endif
S  
S #if (CHIP_6410 | CHIP_DM641 | CHIP_DM640)
S  Uint32 start_of_cache[] =
S  {
S    0x00020000,         /* 000 =  128K SRAM,   0K cache   */
S    0x00018000,         /* 001 =  96K SRAM,   32K cache   */
S    0x00010000,         /* 010 =  64K SRAM,   64K cache   */
S    0x00000000,         /* 011 =  128K cache              */
S    0x00000000,         /* 100 -- undefined, treat as 011 */
S    0x00000000,         /* 101 -- undefined, treat as 011 */
S    0x00000000,         /* 110 -- undefined, treat as 011 */
S    0x00000000          /* 111 =  undefined, treat as 011 */
S  };
S #endif
S
S  gie = IRQ_globalDisable();
S
S
S    /* switch cache modes and flush as needed */
S    
S    /* To take care of the invalid L2MODE entries, map them to the mode with highest cache configuration   *
S     * This is just so that no unnecessary cache clean is done if an invalide mode is given.               */
S
S  #if (!(CHIP_6410|CHIP_DM640|CHIP_DM641))
S    if (newMode & 0x4) {
S       newMode = 0x7; 
S    }
S  #else
S    if (newMode & 0x4) {
S        newMode = 0x3; /* max cache size is 128 K */
S    }
S  #endif
S
S    if (newMode < curMode) {    /* increase SRAM size */
S
S      /* Clean all L2 and wait for completion */
S     CACHE_clean(CACHE_L2ALL, 0, 0);
S
S      /* Switch L2 mode */
S      CACHE_FSET(CCFG,L2MODE,newMode);
S      CACHE_FGET(CCFG,L2MODE);
S      asm(" NOP 4");
S      asm(" NOP 4");
S
S    } else if (newMode > curMode) {   /* decrease SRAM size */
S
S      /* Writeback Invalidate L1D */
S        base_address = start_of_cache[newMode];
S        byte_count   = start_of_cache[curMode] - start_of_cache[newMode];
S
S		/* If 256K Cache is to be configured */
S		if (byte_count == 0x40000)
S		{
S                    /* wbInvL1d 0xFFFF words from base_address */
S	   	    CACHE_wbInvL1d((void *)base_address, 0x3FFFC, CACHE_WAIT);
S		    /* wbInvL1d last word */
S		    CACHE_wbInvL1d((void *)((Uint32)base_address + 0x3FFFC), 0x4, CACHE_WAIT);
S		} else
S                    CACHE_wbInvL1d((void *)base_address, byte_count, CACHE_WAIT);
S
S	  //      CACHE_invalidate(CACHE_L1DALL, 0, 0);
S      /* Clean L2 and wait for completion */
S      CACHE_clean(CACHE_L2ALL, 0, 0);
S      /* Switch L2 mode */
S      CACHE_FSET(CCFG,L2MODE,newMode);
S      CACHE_FGET(CCFG,L2MODE);
S      asm(" NOP 4");
S      asm(" NOP 4");
S    }
S    IRQ_globalRestore(gie);
S
S    return (CACHE_L2Mode)(curMode);
S
N  #else
N    UNREFERENCED_PARAMETER(newMode);
X    ((newMode)=(newMode));
N    return (CACHE_L2Mode)0;
N  #endif /* (L2CACHE_SUPPORT) */
N}
N
N/*----------------------------------------------------------------------------*/
NCACHE_Pcc CACHE_setPccMode(CACHE_Pcc newMode) {
N
N  #if !(L2CACHE_SUPPORT)
X  #if !(( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
N    Uint32 curMode = CHIP_FGET(CSR,PCC);
X    Uint32 curMode = (Uint32)( (((Uint32)(CSR)&0x000000E0u) >>0x00000005u) );
N    CHIP_FSET(CSR,PCC,newMode);
X    CSR=((Uint32)((Uint32)( (((Uint32)(CSR)&~0x000000E0u)| (((Uint32)(newMode)<<0x00000005u) &0x000000E0u)) )));
N    return (CACHE_Pcc)curMode;
N  #else
S    Uint32 curMode = CHIP_FGET(CSR,PCC);
S    UNREFERENCED_PARAMETER(newMode);
S    return (CACHE_Pcc)curMode;
N  #endif
N}
N
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_flush(CACHE_Region region, void *addr, Uint32 wordCnt) {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    /* flush specified region of cache then wait for completion */
S    switch (region) {
S      case CACHE_L2:
S      {
S        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S        CACHE_RSET(L2FBAR,addr);
S        CACHE_RSET(L2FWC,wordCnt);
S
S        IRQ_globalRestore(gie);
S
S        while (CACHE_RGET(L2FWC))
S                    ;
S        break;
S      }
S
S      case CACHE_L2ALL:
S      {
S        int i = 0;
S
S        CACHE_FSET(L2FLUSH,F,1);
S        while (CACHE_FGET(L2FLUSH,F))
S                  i++;
S
S    #if     C64_SUPPORT == 0
S        UNREFERENCED_PARAMETER(i);
S    #else
S        /* -------------------------------------------------------------- */
S        /*  Workaround for L2FLUSH errata on C641x Rev 1.03 and earlier.  */
S        /*  See CACHE_clean() for description of the workaround.          */
S        /* -------------------------------------------------------------- */
S        if (i < 4)
S        {
S        CACHE_FSET(L2FLUSH,F,1);
S          while (CACHE_FGET(L2FLUSH,F))
S                        ;
S        }
S   #endif
S        break;
S      }
S
S      case CACHE_L1D:
S      {
S        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S        CACHE_RSET(L1DFBAR,addr);
S        CACHE_RSET(L1DFWC,wordCnt);
S
S        IRQ_globalRestore(gie);
S
S        while (CACHE_RGET(L1DFWC))
S                    ;
S        break;
S    }
S    }
S
N  #else
N    UNREFERENCED_PARAMETER(region);
X    ((region)=(region));
N    UNREFERENCED_PARAMETER(addr);
X    ((addr)=(addr));
N    UNREFERENCED_PARAMETER(wordCnt);
X    ((wordCnt)=(wordCnt));
N  #endif /* (L2CACHE_SUPPORT) */
N}
N
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_clean(CACHE_Region region, void *addr, Uint32 wordCnt) {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    /* clean specified region of cache then wait for completion */
S    switch (region) {
S      case CACHE_L2:
S      {
S        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S        CACHE_RSET(L2CBAR,addr);
S        asm(" NOP");
S        CACHE_RSET(L2CWC,wordCnt);
S
S        IRQ_globalRestore(gie);
S
S        while (CACHE_RGET(L2CWC))
S                   ;
S        break;
S      }
S
S      case CACHE_L2ALL:
S      {
S        int i = 0;
S
S        CACHE_FSET(L2CLEAN,C,1);
S        while (CACHE_FGET(L2CLEAN,C))  /*if executed,L2CLEAN takes a minimum of 5500 cys*/
S                  i++;                 /*if broken, L2CLEAN takes 500 cys.CPU is stalled, won't iterate*/
S
S      #if     C64_SUPPORT == 0
S        UNREFERENCED_PARAMETER(i);
S      #else
S        /* ---------------------------------------------------------------- */
S        /*  Workaround for L2CLEAN errata on C641x Rev 1.03 and earlier:    */
S        /*  Every second L2CLEAN doesn't clean L2, and thus the poll        */
S        /*  loop above won't iterate more than once or twice.  We           */
S        /*  only issue the extra clean if we detect L2CLEAN didn't work.    */
S        /*  This allows the same code to operate correctly on Rev <= 1.03   */
S        /*  and Rev > 1.03 parts with the same object code, and without     */
S        /*  unnecessary cleans.  A similar fix is required for L2FLUSH.     */
S        /*                                                                  */
S        /*  Compiler aside:  We don't need to #ifdef the i++ as the         */
S        /*  compiler will deadcode it for us.                               */
S        /* ---------------------------------------------------------------- */
S        if (i < 4)
S        {
S          CACHE_FSET(L2CLEAN,C,1);
S          while (CACHE_FGET(L2CLEAN,C))
S                       ;
S        }
S     #endif
S        break;
S    }
S    }
N  #else
N    UNREFERENCED_PARAMETER(region);
X    ((region)=(region));
N    UNREFERENCED_PARAMETER(addr);
X    ((addr)=(addr));
N    UNREFERENCED_PARAMETER(wordCnt);
X    ((wordCnt)=(wordCnt));
N  #endif /* (L2CACHE_SUPPORT) */
N}
N
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_invalidate(CACHE_Region region, void *addr, Uint32 wordCnt) {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    /* invalidate specified region of cache then wait for completion */
S    switch (region) {
S      case CACHE_L1P:
S    {
S        Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S        CACHE_RSET(L1PFBAR,addr);
S        CACHE_RSET(L1PFWC,wordCnt);
S
S        IRQ_globalRestore(gie);
S
S        while (CACHE_RGET(L1PFWC))
S                     ;
S        break;
S    }
S
S      case CACHE_L1PALL:
S     {
S        CACHE_FSET(CCFG,IP,1);
S//        while (CACHE_FGET(CCFG,IP));
S        break;
S     }
S
S/*
S      case CACHE_L1DALL:
S     {
S        CACHE_FSET(CCFG,ID,1);
S        while (CACHE_FGET(CCFG,ID));
S        break;
S    }
S*/
S  }
N  #else
N    UNREFERENCED_PARAMETER(region);
X    ((region)=(region));
N    UNREFERENCED_PARAMETER(addr);
X    ((addr)=(addr));
N    UNREFERENCED_PARAMETER(wordCnt);
X    ((wordCnt)=(wordCnt));
N  #endif /* (L2CACHE_SUPPORT) */
N}
N
N/*----------------------------------------------------------------------------*/
NUint32 CACHE_getL2SramSize() {
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    Uint32 curMode = CACHE_FGET(CCFG,L2MODE);
S    Uint32 size;
S
S   #if C11_SUPPORT
S    switch (curMode) {
S    case 7:   size = 0x00000000; break;
S    case 3:   size = 0x00004000; break;
S    case 2:   size = 0x00008000; break;
S    case 1:   size = 0x0000C000; break;
S    case 0:   size = 0x00010000; break;
S    }
S   #endif
S
S    #if (CHIP_6414|CHIP_6415|CHIP_6416)
S    switch (curMode) {
S    case 7:   size = 0x000C0000; break;
S    case 3:   size = 0x000E0000; break;
S    case 2:   size = 0x000F0000; break;
S    case 1:   size = 0x000F8000; break;
S    case 0:   size = 0x00100000; break;
S    }
S   #endif
S
S    #if (CHIP_6713 | CHIP_DA610)
S      switch (curMode) {
S    case 7:   size = 0x00030000; break; /* + 192K SRAM OFFSET*/
S    case 3:   size = 0x00034000; break;
S    case 2:   size = 0x00038000; break;
S    case 1:   size = 0x0003C000; break;
S    case 0:   size = 0x00040000; break;
S    }
S    #endif
S
S    #if (CHIP_DM642 | CHIP_6412 | CHIP_6411)
S      switch (curMode) {
S    case 7:   size = 0x00000000; break; /* 64Kbyte Bank */
S    case 3:   size = 0x00020000; break;
S    case 2:   size = 0x00030000; break;
S    case 1:   size = 0x00038000; break;
S    case 0:   size = 0x00040000; break;
S    }
S   #endif
S
S   #if (CHIP_6418)
S      switch (curMode) {
S    case 7:   size = 0x00040000; break; /* 256Kbyte Offset */
S    case 3:   size = 0x00060000; break;
S    case 2:   size = 0x00070000; break;
S    case 1:   size = 0x00078000; break; 
S    case 0:   size = 0x00080000; break;
S    }
S   #endif
S
S   #if (CHIP_6413)
S      switch (curMode) {
S    case 7:   size = 0x00000000; break; /* 256Kbyte Offset */
S    case 3:   size = 0x00020000; break;
S    case 2:   size = 0x00030000; break;
S    case 1:   size = 0x00038000; break; 
S    case 0:   size = 0x00040000; break; 
S    }
S   #endif
S
S   #if (CHIP_6410 | CHIP_DM641 | CHIP_DM640)
S      switch (curMode) {
S    case 3:   size = 0x00000000; break;
S    case 2:   size = 0x00010000; break;
S    case 1:   size = 0x00018000; break; 
S    case 0:   size = 0x00020000; break; 
S    }
S   #endif
S
S    return size;
S
N  #else
N    return (Uint32)0;
N  #endif
N}
N
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wbL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S      CACHE_RSET(L2WBAR,blockPtr);
S      CACHE_RSET(L2WWC,(byteCnt+3)>>2);
S
S	  if (wait)
S             while (CACHE_RGET(L2WWC))
S                        ;
S	  else
S	     busyMaskCache = CACHE_WAIT_L2WB;
S          
S      IRQ_globalRestore(gie);
S      
N  #else
N    UNREFERENCED_PARAMETER(blockPtr);
X    ((blockPtr)=(blockPtr));
N    UNREFERENCED_PARAMETER(byteCnt);
X    ((byteCnt)=(byteCnt));
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N/*----------------------------------------------------------------------------*/
N#if (C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
Svoid CACHE_invL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
S
S  #if (L2CACHE_SUPPORT)
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S
S      CACHE_RSET(L2IBAR,blockPtr);
S      CACHE_RSET(L2IWC,(byteCnt+3)>>2);
S
S	  if (wait)
S             while (CACHE_RGET(L2IWC))
S                          ;
S	  else
S             busyMaskCache = CACHE_WAIT_L2INV;
S
S       IRQ_globalRestore(gie);
S
S  #else
S    UNREFERENCED_PARAMETER(blockPtr);
S    UNREFERENCED_PARAMETER(byteCnt);
S    UNREFERENCED_PARAMETER(wait);
S  #endif /* (L2CACHE_SUPPORT) */
S
S}
N#endif /* C64_SUPPORT */
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wbInvL2(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
N
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S
S      CACHE_RSET(L2WIBAR,blockPtr);
S      CACHE_RSET(L2WIWC,(byteCnt+3)>>2);
S
S          if (wait)
S             while (CACHE_RGET(L2WIWC))
S                          ;
S	  else
S	     busyMaskCache = CACHE_WAIT_L2WBINV;
S          
S      IRQ_globalRestore(gie);
S
N  #else
N    UNREFERENCED_PARAMETER(blockPtr);
X    ((blockPtr)=(blockPtr));
N    UNREFERENCED_PARAMETER(byteCnt);
X    ((byteCnt)=(byteCnt));
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wbAllL2(CACHE_Wait wait){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S      int i = 0;
S	  Uint32 gie = IRQ_globalDisable();
S
S      CACHE_FSET(L2WB,C,1);
S
S      if (wait){
S         while (CACHE_FGET(L2WB,C))
S                i++;
S
S    #if     C64_SUPPORT == 0
S        UNREFERENCED_PARAMETER(i);
S    #else
S        /* -------------------------------------------------------------- */
S        /*  Workaround for L2FLUSH errata on C641x Rev 1.03 and earlier.  */
S        /*  See CACHE_clean() for description of the workaround.          */
S        /* -------------------------------------------------------------- */
S        if (i < 4)
S        {
S           CACHE_FSET(L2WB,C,1);
S           while (CACHE_FGET(L2WB,C))
S                        ;
S        }
S    #endif
S
S      }
S      else
S        busyMaskCache = CACHE_WAIT_L2WBALL;
S
S   IRQ_globalRestore(gie);
S
N  #else
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wbInvAllL2(CACHE_Wait wait){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S      int i = 0;
S
S	  Uint32 gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S      CACHE_FSET(L2WBINV,C,1);
S
S      if (wait){
S         while (CACHE_FGET(L2WBINV,C))
S		  i++;
S
S      #if     C64_SUPPORT == 0
S        UNREFERENCED_PARAMETER(i);
S      #else
S        /* ---------------------------------------------------------------- */
S        /*  Workaround for L2CLEAN errata on C641x Rev 1.03 and earlier:    */
S        /*  Every second L2CLEAN doesn't clean L2, and thus the poll        */
S        /*  loop above won't iterate more than once or twice.  We           */
S        /*  only issue the extra clean if we detect L2CLEAN didn't work.    */
S        /*  This allows the same code to operate correctly on Rev <= 1.03   */
S        /*  and Rev > 1.03 parts with the same object code, and without     */
S        /*  unnecessary cleans.  A similar fix is required for L2FLUSH.     */
S        /*                                                                  */
S        /*  Compiler aside:  We don't need to #ifdef the i++ as the         */
S        /*  compiler will deadcode it for us.                               */
S        /* ---------------------------------------------------------------- */
S        if (i < 4)
S        {
S          CACHE_FSET(L2WBINV,C,1);
S          while (CACHE_FGET(L2WBINV,C))
S             ;
S        }
S     #endif
S      }
S       else
S         busyMaskCache = CACHE_WAIT_L2WBINVALL;
S
S     IRQ_globalRestore(gie);
S
N  #else
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N/*----------------------------------------------------------------------------*/
N#if (C64_SUPPORT)
X#if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*1) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
Svoid CACHE_invL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
S
S  #if (L2CACHE_SUPPORT)
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S      CACHE_RSET(L1DIBAR,blockPtr);
S      CACHE_RSET(L1DIWC,(byteCnt+3)>>2);
S
S      if (wait)
S         while (CACHE_RGET(L1DIWC))
S                      ;
S      else
S        busyMaskCache = CACHE_WAIT_L1DINV;
S
S      IRQ_globalRestore(gie);
S      
S  #else
S    UNREFERENCED_PARAMETER(blockPtr);
S    UNREFERENCED_PARAMETER(byteCnt);
S    UNREFERENCED_PARAMETER(wait);
S  #endif /* (L2CACHE_SUPPORT) */
S
S}
N#endif /* C64_SUPPORT */
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wbInvL1d(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S      CACHE_RSET(L1DWIBAR,blockPtr);
S      CACHE_RSET(L1DWIWC,(byteCnt+3)>>2);
S
S      if (wait)
S         while (CACHE_RGET(L1DWIWC))
S                      ;
S      else
S        busyMaskCache = CACHE_WAIT_L1DWBINV;
S      
S      IRQ_globalRestore(gie);
S      
N  #else
N    UNREFERENCED_PARAMETER(blockPtr);
X    ((blockPtr)=(blockPtr));
N    UNREFERENCED_PARAMETER(byteCnt);
X    ((byteCnt)=(byteCnt));
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT && C64_SUPPORT) */
N
N}
N/*----------------------------------------------------------------------------*/
N/*
Nvoid CACHE_invAllL1d(){
N
N  #if (L2CACHE_SUPPORT)
N
N      Uint32 gie = IRQ_globalDisable();
N
N      CACHE_FSET(CCFG,ID,1);
N
N      IRQ_globalRestore(gie);
N
N  #endif
N
N}
N*/
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_invL1p(void *blockPtr,Uint32 byteCnt,CACHE_Wait wait){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S	  Uint32 gie;
S
S	  CACHE_wait();
S	  gie = IRQ_globalDisable(); /* treat BAR/WC as crit section */
S
S      CACHE_RSET(L1PIBAR,blockPtr);
S      CACHE_RSET(L1PIWC,(byteCnt+3)>>2);
S
S      if (wait)
S         while (CACHE_RGET(L1PIWC))
S                      ;
S      else
S         busyMaskCache = CACHE_WAIT_L1PINV;
S      
S      IRQ_globalRestore(gie);
S      
N  #else
N    UNREFERENCED_PARAMETER(blockPtr);
X    ((blockPtr)=(blockPtr));
N    UNREFERENCED_PARAMETER(byteCnt);
X    ((byteCnt)=(byteCnt));
N    UNREFERENCED_PARAMETER(wait);
X    ((wait)=(wait));
N  #endif /* (L2CACHE_SUPPORT && C64_SUPPORT) */
N
N
N}
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_invAllL1p(){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S      Uint32 gie = IRQ_globalDisable();
S
S      CACHE_FSET(CCFG,IP,1);
S
S      IRQ_globalRestore(gie);
N  #endif
N
N}
N/*----------------------------------------------------------------------------*/
Nvoid CACHE_wait(){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    switch (busyMaskCache) {
S      case CACHE_WAIT_L2WB:
S	  {
S        while(CACHE_RGET(L2WWC));
S
S        break;
S	  }
S    #if(C64_SUPPORT)
S	  case CACHE_WAIT_L2INV:
S	  {
S             while(CACHE_RGET(L2IWC))
S                         ;
S             break;
S	  }
S    #endif /* C64_SUPPORT */
S	  case CACHE_WAIT_L2WBINV:
S	  {
S             while(CACHE_RGET(L2WIWC))
S                         ;
S             break;
S	  }
S      case CACHE_WAIT_L2WBALL:
S	  {
S             while(CACHE_RGET(L2WB))
S                         ;
S             break;
S	  }
S      case CACHE_WAIT_L2WBINVALL:
S	  {
S             while(CACHE_RGET(L2WBINV))
S                         ;
S             break;
S	  }
S    #if (C64_SUPPORT)
S      case CACHE_WAIT_L1DINV:
S	  {
S             while(CACHE_RGET(L1DIWC))
S                         ;
S             break;
S	  }
S    #endif /* C64_SUPPORT */
S      case CACHE_WAIT_L1DWBINV:
S	  {
S             while(CACHE_RGET(L1DWIWC))
S                         ;
S             break;
S	  }
S      case CACHE_WAIT_L1PINV:
S	  {
S             while(CACHE_RGET(L1PIWC))
S                         ;
S             break;
S	  }
S   }
S
S	busyMaskCache = 0;
S
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N
N/*----------------------------------------------------------------------------*/
N
Nvoid CACHE_wbInvL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize){
N
N  #if(L2CACHE_SUPPORT)
X  #if(( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S    Uint32 i, loopCnt, wordCnt, remCnt;
S    Uint32 gie;
S
S    CACHE_wait();
S 
S    /* convert to words, round up to next word */ 
S
S    blockSize = (blockSize + 3) >> 2;       
S    wordCnt   = (byteCnt   + 3) >> 2;       
S
S    /* number of full blocks to operate on */
S
S    loopCnt = wordCnt / blockSize; 
S
S    /* number of words in last block  */    
S
S    remCnt = wordCnt - blockSize*loopCnt;           
S
S    if (remCnt) loopCnt++;
S
S    for(i = 0; i < loopCnt; i++) {
S
S       /* treat BAR/WC as critical section */
S
S       gie = IRQ_globalDisable();
S
S       /* calculate the starting address for each consecutive block */
S
S       CACHE_RSET(L2WIBAR,(void *)((Uint32)blockPtr + 4*blockSize*i));  
S
S       if (remCnt && (i==(loopCnt-1)))
S          CACHE_RSET(L2WIWC, remCnt);
S       else
S          CACHE_RSET(L2WIWC, blockSize);
S                
S       IRQ_globalRestore(gie);
S        
S       while (CACHE_RGET(L2WIWC));
S
S    }
N  #else 
N      UNREFERENCED_PARAMETER(blockPtr);
X      ((blockPtr)=(blockPtr));
N      UNREFERENCED_PARAMETER(byteCnt);
X      ((byteCnt)=(byteCnt));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N
N
N/*----------------------------------------------------------------------------*/
N
Nvoid CACHE_wbL2_blocks(void *blockPtr,Uint32 byteCnt, Uint32 blockSize){
N
N  #if (L2CACHE_SUPPORT)
X  #if (( ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 0*0) | ( 1*0) | ( 0*1) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0)| ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 1*0) | ( 0*0) ))
S
S    Uint32 i, loopCnt, wordCnt, remCnt;
S    Uint32 gie;
S
S    CACHE_wait();
S 
S    /* convert to words, round up to next word */ 
S
S    blockSize = (blockSize + 3) >> 2;       
S    wordCnt   = (byteCnt   + 3) >> 2;       
S
S    /* number of full blocks to operate on */
S
S    loopCnt = wordCnt / blockSize; 
S
S    /* number of words in last block  */    
S
S    remCnt = wordCnt - blockSize*loopCnt;           
S
S    if (remCnt) loopCnt++;
S
S    for(i = 0; i < loopCnt; i++) {
S
S       /* treat BAR/WC as critical section */
S
S       gie = IRQ_globalDisable();
S
S       /* calculate the starting address for each consecutive block */
S
S       CACHE_RSET(L2WBAR,(void *)((Uint32)blockPtr + 4*blockSize*i));  
S
S       if (remCnt && (i==(loopCnt-1)))
S          CACHE_RSET(L2WWC, remCnt);
S       else
S          CACHE_RSET(L2WWC, blockSize);
S                
S       IRQ_globalRestore(gie);
S        
S       while (CACHE_RGET(L2WWC));
S
S    }
N  #else 
N      UNREFERENCED_PARAMETER(blockPtr);
X      ((blockPtr)=(blockPtr));
N      UNREFERENCED_PARAMETER(byteCnt);
X      ((byteCnt)=(byteCnt));
N  #endif /* (L2CACHE_SUPPORT) */
N
N}
N
N/*----------------------------------------------------------------------------*/
N
N#endif /* CACHE_SUPPORT */
N/******************************************************************************\
N* End of csl_cache.c
X
N\******************************************************************************/
N
W "../csl/csl_cache.c" 78 15 variable "busyMaskCache" was declared but never referenced
